# MonArch - Groundwork Monitor Architect
# MonarchFile.pm
#
###############################################################################
# Release 1.2
# 6-Jul-2006
###############################################################################
# Author: Scott Parris
#
# Copyright (C) 2005 Groundwork Open Source Solutions
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

use strict;
use MonarchStorProc;
use MonarchConf;
use DBI;

package Files;

my ($dbhost, $database,$user,$passwd) = undef;
if (-e '/usr/local/groundwork/config/db.properties') {
	open(FILE, "< /usr/local/groundwork/config/db.properties");
	while (my $line = <FILE>) {
		if ($line =~ /\s*monarch\.dbhost\s*=\s*(\S+)/) { $dbhost = $1 }
		if ($line =~ /\s*monarch\.database\s*=\s*(\S+)/) { $database = $1 }
		if ($line =~ /\s*monarch\.username\s*=\s*(\S+)/) { $user = $1 }
		if ($line =~ /\s*monarch\.password\s*=\s*(\S+)/) { $passwd = $1 }
	}
	close(FILE);
} else {
	($dbhost, $database,$user,$passwd) = Conf->get_dbauth();
}
my $dsn = "DBI:mysql:$database:$dbhost";
my $dbh = DBI->connect($dsn, $user, $passwd, {'RaiseError' => 1});

my %obj_id = ('commands' => 'command_id',
'contact_templates' => 'contacttemplate_id',
'contactgroups' => 'contactgroup_id',
'contacts' => 'contact_id',
'escalation_templates' => 'template_id',
'extended_host_info_templates' => 'hostextinfo_id',
'extended_service_info_templates' => 'serviceextinfo_id',
'host_dependencies' => 'host_id',
'host_templates' => 'hosttemplate_id',
'hostgroups' => 'hostgroup_id',
'hosts' => 'host_id',
'service_dependency' => 'id',
'service_dependency_templates' => 'id',
'service_templates' => 'servicetemplate_id',
'servicegroups' => 'servicegroup_id',
'services' => 'service_id',
'time_periods' => 'timeperiod_id');

my %obj_template = ('contacts' => 'contact_templates',
'service_templates' => 'service_templates',
'services' => 'service_templates',
'service_dependency' => 'service_dependency_templates',
'hosts' => 'host_templates');

sub nagios_cfg(@) {
	my $userid = $_[1];
	my $preflight = $_[2];
	my $nagios_home = $_[3];
	my $workspace = $_[4];
	my $destination = $nagios_home;
	if ($preflight) { $destination = $workspace }
	my $usefiles = $_[5];
	my %use_file = %{$usefiles};
	my %version = StorProc->fetch_one('setup','name','nagios_version');
	my %nagkeys = StorProc->nagios_defaults($version{'value'},'');
	my %files = ();
	my @errors = ();
	my @nagprops = ('log_file','object_cache_file','resource_file','temp_file','status_file',
'aggregate_status_updates','status_update_interval','nagios_user','nagios_group','enable_notifications','execute_service_checks',
'accept_passive_service_checks','execute_host_checks','accept_passive_host_checks','enable_event_handlers','log_rotation_method','log_archive_path',
'check_external_commands','command_check_interval','command_file','downtime_file','comment_file','lock_file','retain_state_information','state_retention_file',
'retention_update_interval','use_retained_program_state','use_retained_scheduling_info','use_syslog','log_notifications','log_service_retries','log_host_retries',
'log_event_handlers','log_initial_states','log_external_commands','log_passive_service_checks','log_passive_checks','global_host_event_handler',
'global_service_event_handler','sleep_time','inter_check_delay_method','service_inter_check_delay_method','max_service_check_spread','service_interleave_factor',
'max_concurrent_checks','service_reaper_frequency','host_inter_check_delay_method','max_host_check_spread','interval_length','auto_reschedule_checks',
'auto_rescheduling_interval','auto_rescheduling_window','use_agressive_host_checking','enable_flap_detection','low_service_flap_threshold','high_service_flap_threshold',
'low_host_flap_threshold','high_host_flap_threshold','soft_state_dependencies','service_check_timeout','host_check_timeout','event_handler_timeout',
'notification_timeout','ocsp_timeout','ochp_timeout','perfdata_timeout','obsess_over_services','ocsp_command','obsess_over_hosts','ochp_command',
'process_performance_data','host_perfdata_command','service_perfdata_command','host_perfdata_file','service_perfdata_file','host_perfdata_file_template',
'service_perfdata_file_template','host_perfdata_file_mode','service_perfdata_file_mode','host_perfdata_file_processing_interval',
'service_perfdata_file_processing_interval','host_perfdata_file_processing_command','service_perfdata_file_processing_command',
'check_for_orphaned_services','check_service_freshness','freshness_check_interval','check_host_freshness','host_freshness_check_interval',		
'date_format','illegal_object_name_chars','illegal_macro_output_chars','admin_email','admin_pager','p1_file');

	my $sqlstmt = "select file_id, name, path from files where file_id in (select distinct file_id from file_host)";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while(my @values = $sth->fetchrow_array()) { $files{$values[1]} = $values[2] }
	$sth->finish;			

	$sqlstmt = "select file_id, name, path from files where file_id in (select distinct file_id from file_service)";
	$sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while(my @values = $sth->fetchrow_array()) { $files{$values[1]} = $values[2] }
	$sth->finish;			
	
	my $sqlstmt = "select name, path from files where file_id in (select value from setup where type = 'file')";
	$sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while(my @values = $sth->fetchrow_array()) { $files{$values[0]} = $values[1] }
	$sth->finish;			

	use File::Copy;
	my $dt = StorProc->datetime();
	my $outfile = qq(##########GROUNDWORK#############################################################################################
#GW
#GW\tnagios.cfg generated $dt by $userid from monarch.cgi nagios $version{'value'}
#GW
##########GROUNDWORK#############################################################################################
);
	foreach my $prop (@nagprops) {
		my $title = "\U$prop";
		$title =~ s/_/ /g;
		my %where = ('type' => 'nagios','name' => $prop);
		my %val = StorProc->fetch_one_where('setup',\%where);
		my $comment = undef;
		if ($prop =~ /aggregate_status_updates|enable_notifications|execute_service_checks|accept_passive_service_checks|execute_host_checks|accept_passive_host_checks|enable_event_handlers|use_retained_program_state|use_retained_scheduling_info|use_syslog|log_notifications|log_service_retries|log_host_retries|log_event_handlers|log_initial_states|log_external_commands|log_passive_checks|check_service_freshness|check_host_freshness/) {
			unless ($val{'value'}) { $val{'value'} = '0'}
		}
		if ($val{'value'} eq '') { $comment = '# ' }
		if ($prop eq 'resource_file') {
			my $file = undef;
			if ($val{'value'} =~ /.*\/(.*\.cfg)/) { $file = $1 }
			if ($preflight eq '1') {
				$outfile .= "\n# $title\n$comment$prop=$workspace/$file\n";
			} else {
				$outfile .= "\n# $title\n$comment$prop=$val{'value'}\n";
				copy("$workspace/$file","$val{'value'}") || 
					push @errors, "Error: Unable to copy $workspace/$file to $val{'value'} $!";
			}
		} elsif (defined $nagkeys{$prop}) {
			$outfile .= "\n# $title\n$comment$prop=$val{'value'}\n";
		}
		if ($prop eq 'log_file') {
			$outfile .= "\n# OBJECT CONFIGURATION FILE(S)\n";
			foreach my $file (sort keys %files) {
				if ($use_file{$file}) {
					if ($version{'value'} eq '2.x') {
						if ($preflight eq '1') {
							$outfile .= "cfg_file=$workspace/$file\n";
						} else {
							$outfile .= "cfg_file=$nagios_home/$file\n";
						}
					} else {
						unless ($file =~ /extended/i) {
							if ($preflight eq '1') {
								$outfile .= "cfg_file=$workspace/$file\n";
							} else {
								$outfile .= "cfg_file=$nagios_home/$file\n";
							}
						}
					}
					unless ($preflight) {		
						copy("$workspace/$file","$destination/$file") || 
							push @errors, "Error: Unable to copy $workspace/$file to $destination/$file $!";
					}
				}
			}
		}	
	}	
	open(FILE, "> $destination/nagios.cfg") || push @errors, "Error: Unable to open $destination/nagios.cfg $!";
	print FILE $outfile;
	close (FILE);
	return @errors;
}


sub build_files(@) {
	my $userid = $_[1];
	my $destination = $_[2];
	my $proplist = $_[3];
	my $nagios_ver = $_[4];
	my %proplist = %{$proplist};
	my @errors = ();
	my %use_file = ();
	my %version = StorProc->fetch_one('setup','name','nagios_version');
	my @types = ('commands','misccommands','time_periods','contact_templates','contactgroups','contacts',
'extended_host_info_templates','extended_host_info','extended_service_info_templates','extended_service_info',
'host_dependencies','host_templates','hostgroups','hosts','service_dependency',
'service_dependency_templates','service_templates','services','escalation_templates');
	if ($nagios_ver eq '2.x') { push @types, 'servicegroups' }
	foreach my $type (@types) {
		my $outfile = undef;
		# get files
		my %files = ();
		if ($type eq 'hosts') {
			my $sqlstmt = "select file_id, name from files where file_id in (select distinct file_id from file_host)";
			my $sth = $dbh->prepare ($sqlstmt);
			$sth->execute;
			while(my @values = $sth->fetchrow_array()) { $files{$values[1]} = $values[0] }
			$sth->finish;
			unless ($files{'hosts.cfg'}) {
				my %hf = StorProc->fetch_one('files','name','hosts.cfg');
				$files{'hosts.cfg'} = $hf{'file_id'};
			}
		} elsif ($type eq 'services') {
			my $sqlstmt = "select file_id, name from files where file_id in (select distinct file_id from file_service)";
			my $sth = $dbh->prepare ($sqlstmt);
			$sth->execute;
			while(my @values = $sth->fetchrow_array()) { $files{$values[1]} = $values[0] }
			$sth->finish;			
			unless ($files{'services.cfg'}) {
				my %hf = StorProc->fetch_one('files','name','services.cfg');
				$files{'services.cfg'} = $hf{'file_id'};
			}
		} else {
			my $sqlstmt = "select files.file_id, files.name from files left join setup on setup.value = files.file_id where setup.name = '$type'";
			my ($id,$file) = $dbh->selectrow_array($sqlstmt);
			$files{$file} = $id;
		}

		foreach my $file (keys %files) {
			my $dt = StorProc->datetime();
			my $got_output = undef;
			my $outfile = qq(##########GROUNDWORK#############################################################################################
#GW
#GW\t$file generated $dt by $userid from monarch.cgi nagios v $version{'value'}
#GW
##########GROUNDWORK#############################################################################################
);
			my @objects = ();
			if ($type eq 'hosts') {
				my %w = ('file_id' => $files{$file});
				@objects = StorProc->fetch_list_where('file_host','host_id',\%w);
				if ($file eq 'hosts.cfg') {
					my $sqlstmt = "select host_id from hosts where host_id not in (select host_id from file_host)";
					my $sth = $dbh->prepare ($sqlstmt);
					$sth->execute;
					while(my @values = $sth->fetchrow_array()) { push @objects, $values[0] }
					$sth->finish;			
				}
			} elsif ($type eq 'services') {
				my %w = ('file_id' => $files{$file});
				@objects = StorProc->fetch_list_where('file_service','service_id',\%w);
				if ($file eq 'services.cfg') {
					my $sqlstmt = "select service_id from services where service_id not in (select service_id from file_service)";
					my $sth = $dbh->prepare ($sqlstmt);
					$sth->execute;
					while(my @values = $sth->fetchrow_array()) { push @objects, $values[0] }
					$sth->finish;			
				}
			} elsif ($type eq 'misccommands') {
				my $sqlstmt = "select command_id from commands where type = 'notify' or type = 'other'";
				my $sth = $dbh->prepare ($sqlstmt);
				$sth->execute;
				while(my @values = $sth->fetchrow_array()) { push @objects, $values[0] }
				$sth->finish;	
				$type = 'commands';
			} elsif ($type eq 'commands') {
				my %w = ('type' => 'check');
				@objects = StorProc->fetch_list_where('commands','command_id',\%w,'name');
			} elsif ($type eq 'extended_host_info') {
				@objects = StorProc->fetch_list('hosts','host_id');
			} elsif ($type eq 'extended_service_info') {
				@objects = StorProc->fetch_list('services','service_id');
			} elsif ($type eq 'host_dependencies') {
				my $sqlstmt = "select * from host_dependencies";
				my $sth = $dbh->prepare ($sqlstmt);
				$sth->execute;
				while(my @values = $sth->fetchrow_array()) { push @objects, "$values[0]-$values[1]" }
				$sth->finish;			
			} else {
				my $orderby = undef;
				unless ($type =~ /depend/) { $orderby = 'name' }
				@objects = StorProc->fetch_list($type,$obj_id{$type},$orderby);
			}
			foreach my $obj (@objects) {
				my @props = split(/,/, $proplist{$type});
				my $name = $obj_id{$type};
				my %object = ();
				my $objtype = $type;
				unless ($type =~ /host_dependencies|_info$/) { %object = StorProc->fetch_one($type,$name,$obj) }
				if ($type eq 'escalation_templates') {
					if ($object{'type'} eq 'hostgroup') { $objtype = 'hostgroupescalation' }
					if ($object{'type'} eq 'host') { $objtype = 'hostescalation' }
					if ($object{'type'} eq 'service') { 
						$objtype = 'serviceescalation';
						my %s = StorProc->fetch_one('service_names','servicename_id',$object{'servicename_id'});
						$object{'service_description'} = $s{'name'};
					}
					if ($object{'escalation_options'} eq 'all') { delete $object{'escalation_options'} }
					my $etype = "$object{'type'}\_escalation_templates";
					my $sqlstmt = "select contactgroups.name from contactgroups where contactgroup_id in ".
						"(select contactgroup_id from contactgroup_assign where type = '$etype' and object = '$obj')";
					my $sth = $dbh->prepare ($sqlstmt);
					$sth->execute;
					my @members = ();
					while(my @values = $sth->fetchrow_array()) { push @members, $values[0] }
					$sth->finish;			
					foreach (@members) { $object{'contact_groups'} .= "$_," }
					chop $object{'contact_groups'};
					my %cp = StorProc->fetch_one('time_periods','timeperiod_id',$object{'escalation_period'});
					$object{'escalation_period'} = $cp{'name'};
					if ($version{'value'} eq '1.x') {
						delete $object{'escalation_period'};
						delete $object{'escalation_options'};
					}
				}
				if ($type =~ /command/) {
					delete $object{'type'};
				}
				if ($type eq 'extended_host_info') {
					my %host = StorProc->fetch_one('hosts','host_id',$obj);
					if (!$host{'hostextinfo_id'}) { next }
					my %coords = StorProc->fetch_one('extended_info_coords','host_id',$obj);
					my %ext_info = StorProc->fetch_one('extended_host_info_templates','hostextinfo_id',$host{'hostextinfo_id'});
					$object{'2d_coords'} = $coords{'2d_coords'};
					$object{'3d_coords'} = $coords{'3d_coords'};
					$object{'host_name'} = $host{'name'};
					$object{'template'} = $ext_info{'name'};
					$object{'notes_url'} = $ext_info{'notes_url'};
					$object{'notes_url'} =~ s/\$HOSTNAME\$/$object{'host_name'}/g;
					$object{'notes_url'} =~ s/\$HOSTADDRESS\$/$host{'address'}/g;
					$objtype = 'hostextinfo';
				}
				if ($type eq 'extended_service_info') {
					my %service = StorProc->fetch_one('services','service_id',$obj);
					if ($service{'serviceextinfo_id'}) {
						my %sn = StorProc->fetch_one('service_names','servicename_id',$service{'servicename_id'}); 
						my %host = StorProc->fetch_one('hosts','host_id',$service{'host_id'}); 
						my %ext_info = StorProc->fetch_one('extended_service_info_templates','serviceextinfo_id',$service{'serviceextinfo_id'});
						$object{'host_name'} = $host{'name'};
						$object{'template'} = $ext_info{'name'};
						$object{'service_description'} = $sn{'name'};
						$object{'notes_url'} = $ext_info{'notes_url'};
						$object{'notes_url'} =~ s/\$HOSTNAME\$/$object{'host_name'}/g;
						$object{'notes_url'} =~ s/\$HOSTADDRESS\$/$host{'address'}/g;
						$object{'notes_url'} =~ s/\$SERVICENAME\$|\$SERVICEDESC\$|\$SERVICEDESCRIPTION\$/$object{'service_description'}/g;
						$objtype = 'serviceextinfo';
					} else {
						next;
					}
				}
				if ($type eq 'host_dependencies') {
					my @host_parent = split(/-/, $obj);
					my %w = ('host_id' => $host_parent[0],'parent_id' => $host_parent[1]);
					%object = StorProc->fetch_one_where('host_dependencies',\%w);
					my %dh = StorProc->fetch_one('hosts','host_id',$host_parent[0]); 
					my %ph = StorProc->fetch_one('hosts','host_id',$host_parent[1]); 
					$object{'dependent_host'} = $dh{'name'};
					$object{'parent_host'} = $ph{'name'};
					$objtype = 'hostdependency';
				}
				if ($type eq 'contact_templates') {
					my %hp = StorProc->fetch_one('time_periods','timeperiod_id',$object{'host_notification_period'});
					my %sp = StorProc->fetch_one('time_periods','timeperiod_id',$object{'service_notification_period'});	
					$object{'host_notification_period'} = $sp{'name'};
					$object{'service_notification_period'} = $sp{'name'};
					my $sqlstmt = "select name from commands where command_id in ".
						"(select command_id from contact_command where contacttemplate_id = '$obj' and type = 'host')";
					my $sth = $dbh->prepare ($sqlstmt);
					$sth->execute;
					my @members = ();
					while(my @values = $sth->fetchrow_array()) { push @members, $values[0] }
					$sth->finish;			
					foreach (@members) { $object{'host_notification_commands'} .= "$_," }
					chop $object{'host_notification_commands'};

					$sqlstmt = "select name from commands where command_id in ".
						"(select command_id from contact_command where contacttemplate_id = '$obj' and type = 'service')";
					$sth = $dbh->prepare ($sqlstmt);
					$sth->execute;
					@members = ();
					while(my @values = $sth->fetchrow_array()) { push @members, $values[0] }
					$sth->finish;			
					foreach (@members) { $object{'service_notification_commands'} .= "$_," }
					chop $object{'service_notification_commands'};
				}

				if ($type eq 'service_templates' or $type eq 'host_templates') {
					my %cp = StorProc->fetch_one('time_periods','timeperiod_id',$object{'check_period'});
					my %np = StorProc->fetch_one('time_periods','timeperiod_id',$object{'notification_period'});	
					$object{'check_period'} = $cp{'name'};
					$object{'notification_period'} = $np{'name'};
					my %c = StorProc->fetch_one('commands','command_id',$object{'check_command'});
					my %e = StorProc->fetch_one('commands','command_id',$object{'event_handler'});
					$object{'check_command'} = $c{'name'};				
					$object{'event_handler'} = $e{'name'};	
					my $sqlstmt = "select contactgroups.name from contactgroups where contactgroup_id in ".
						"(select contactgroup_id from contactgroup_assign where type = '$type' and object = '$obj')";
					my $sth = $dbh->prepare ($sqlstmt);
					$sth->execute;
					my @members = ();
					while(my @values = $sth->fetchrow_array()) { push @members, $values[0] }
					$sth->finish;			
					foreach (@members) { $object{'contact_groups'} .= "$_," }
					chop $object{'contact_groups'};				
				}
				if ($type =~ /_info_templates/) {
					if ($object{'notes_url'} =~ /\$HOSTNAME\$|\$HOSTADDRESS\$|\$SERVICENAME\$|\$SERVICEDESC\$|\$SERVICEDESCRIPTION\$/) { delete $object{'notes_url'} }
				}
				if ($type eq 'hostgroups') {
					my $sqlstmt = "select hosts.name from hosts where host_id in (select host_id from hostgroup_host where hostgroup_id = '$obj')";
					my $sth = $dbh->prepare ($sqlstmt);
					$sth->execute;
					my @members = ();
					while(my @values = $sth->fetchrow_array()) { push @members, $values[0] }
					$sth->finish;			
					foreach (@members) { $object{'members'} .= "$_," }
					chop $object{'members'};
					if ($version{'value'} eq '1.x') {
						$sqlstmt = "select contactgroups.name from contactgroups where contactgroup_id in ".
							"(select contactgroup_id from contactgroup_assign where type = '$type' and object = '$obj')";
						$sth = $dbh->prepare ($sqlstmt);
						$sth->execute;
						my @contactgroups = ();
						while(my @values = $sth->fetchrow_array()) { push @contactgroups, $values[0] }
						$sth->finish;			
						foreach (@contactgroups) { $object{'contact_groups'} .= "$_," }
						chop $object{'contact_groups'};
					} else {
						delete $object{'contact_groups'};
					}
					unless (@members) { %object = () }
				}
				if ($type eq 'host_templates') {
					if ($object{'command_line'} =~ /GWHOSTNAME/ ) {
						delete $object{'command_line'};
					}
				}
				if ($type eq 'hosts') {
					my $sqlstmt = "select hosts.name from hosts where host_id in (select parent_id from host_parent where host_id = '$obj')";
					my $sth = $dbh->prepare ($sqlstmt);
					$sth->execute;
					my @members = ();
					while(my @values = $sth->fetchrow_array()) { push @members, $values[0] }
					$sth->finish;			
					foreach (@members) { $object{'parents'} .= "$_," }
					chop $object{'parents'};
					my %overrides = StorProc->fetch_one('host_overrides','host_id',$obj);
					if ($overrides{'status'}) {
						foreach my $name (keys %overrides) {
							$object{$name} = $overrides{$name};
						}
					}
					my %t = StorProc->fetch_one('host_templates','hosttemplate_id',$object{'hosttemplate_id'}); 
					if ($t{'command_line'}) {
						$t{'command_line'} =~ s/\[GWHOSTNAME\]/$object{'name'}/g;
						$object{'check_command'} =  $t{'command_line'};
					}
					my %overrides = StorProc->fetch_one('host_overrides','host_id',$obj);
					foreach my $name (keys %overrides) {
						if ($version{'value'} eq '2.x' && $name eq 'check_period') {
							my %cp = StorProc->fetch_one('time_periods','timeperiod_id',$overrides{$name});
							$object{'check_period'} = $cp{'name'};
						} elsif ($name eq 'notification_period') {
							my %np = StorProc->fetch_one('time_periods','timeperiod_id',$overrides{$name});	
							$object{'notification_period'} = $np{'name'};			
						} elsif ($name eq 'check_command') {
							my %c = StorProc->fetch_one('commands','command_id',$overrides{$name});
							$object{'check_command'} = $c{'name'};	
						} elsif ($name eq 'event_handler') {
							my %e = StorProc->fetch_one('commands','command_id',$overrides{$name});
							$object{'event_handler'} = $e{'name'};	
						} elsif ($name eq 'check_freshness') {
							unless ($version{'value'} eq '1.x') {
								$object{$name} = $overrides{$name};
							}
						} elsif ($name eq 'freshness_threshold') {
							unless ($version{'value'} eq '1.x') {
								$object{$name} = $overrides{$name};
							}
						} elsif ($name eq 'obsess_over_host') {
							unless ($version{'value'} eq '1.x') {
								$object{$name} = $overrides{$name};
							}
						} elsif ($name eq 'active_checks_enabled') {
							unless ($version{'value'} eq '1.x') {
								$object{$name} = $overrides{$name};
							}
						} elsif ($name eq 'passive_checks_enabled') {
							unless ($version{'value'} eq '1.x') {
								$object{$name} = $overrides{$name};
							}
						} elsif ($name eq 'checks_enabled') {
							if ($version{'value'} eq '1.x') {
								$object{$name} = $overrides{$name};
							}
						} else {
							$object{$name} = $overrides{$name};
						}
					}
					if ($version{'value'} eq '2.x') {
						my %contactgroups = ();
						$sqlstmt = "select contactgroups.name from contactgroups where contactgroup_id in ".
							"(select contactgroup_id from contactgroup_assign where type = '$type' and object = '$obj')";
						$sth = $dbh->prepare ($sqlstmt);
						$sth->execute;
						while(my @values = $sth->fetchrow_array()) { $contactgroups{$values[0]} = 1 }
						$sth->finish;			
						my %w = ('host_id' => $obj);
						my @hgids = StorProc->fetch_list_where('hostgroup_host','hostgroup_id',\%w);
						foreach my $hgid (@hgids) {
							$sqlstmt = "select contactgroups.name from contactgroups where contactgroup_id in ".
								"(select contactgroup_id from contactgroup_assign where type = 'hostgroups' and object = '$hgid')";
							$sth = $dbh->prepare ($sqlstmt);
							$sth->execute;
							while(my @values = $sth->fetchrow_array()) { $contactgroups{$values[0]} = 1 }
							$sth->finish;			
						}
						foreach my $cg (sort keys %contactgroups) { $object{'contact_groups'} .= "$cg," }
						chop $object{'contact_groups'};
					}
				}
				if ($type eq 'services') {
					my $sqlstmt = "select service_names.name from service_names left join services on service_names.servicename_id = services.servicename_id".
						" where services.service_id = '$obj'";
					$object{'name'} = $dbh->selectrow_array($sqlstmt); 
					my $sqlstmt = "select hosts.name from hosts left join services on hosts.host_id = services.host_id".
						" where services.service_id = '$obj'";
					$object{'host_name'} = $dbh->selectrow_array($sqlstmt); 
					my $sqlstmt = "select service_templates.name from service_templates left join services on service_templates.servicetemplate_id = services.servicetemplate_id".
						" where services.service_id = '$obj'";
					$object{'use'} = $dbh->selectrow_array($sqlstmt); 
					my $sqlstmt = "select commands.name from commands left join services on commands.command_id = services.check_command".
						" where services.service_id = '$obj'";
					$object{'check_command'} = $dbh->selectrow_array($sqlstmt); 
					my %overrides = StorProc->fetch_one('service_overrides','service_id',$obj);
					foreach my $name (keys %overrides) {
						if ($name eq 'check_period') {
							my %cp = StorProc->fetch_one('time_periods','timeperiod_id',$overrides{$name});
							$object{'check_period'} = $cp{'name'};
						} elsif ($name eq 'notification_period') {
							my %np = StorProc->fetch_one('time_periods','timeperiod_id',$overrides{$name});	
							$object{'notification_period'} = $np{'name'};			
						} elsif ($name eq 'event_handler') {
							my %e = StorProc->fetch_one('commands','command_id',$overrides{$name});
							$object{'event_handler'} = $e{'name'};	
						} else {
							$object{$name} = $overrides{$name};
						}
					}
					my %contactgroups = ();
					$sqlstmt = "select contactgroups.name from contactgroups where contactgroup_id in ".
						"(select contactgroup_id from contactgroup_assign where type = '$type' and object = '$obj')";
					my $sth = $dbh->prepare ($sqlstmt);
					$sth->execute;
					while(my @values = $sth->fetchrow_array()) { $contactgroups{$values[0]} = 1 }
					$sth->finish;			
					foreach my $cg (sort keys %contactgroups) { $object{'contact_groups'} .= "$cg," }
					chop $object{'contact_groups'};
				}
				if ($type eq 'service_dependency_templates') {
					my $sqlstmt = "select service_names.name from service_names left join service_dependency_templates on ".
						"service_names.servicename_id = service_dependency_templates.servicename_id ".
						"where service_dependency_templates.id = '$obj'";
					$object{'service_name'} = $dbh->selectrow_array($sqlstmt); 
				}
				if ($type eq 'service_dependency') {
					my $sqlstmt = "select service_names.name from service_names where servicename_id in ".
						"(select servicename_id from services left join service_dependency on ".
						"services.service_id = service_dependency.service_id where service_dependency.id = '$obj')";
					$object{'service_name'} = $dbh->selectrow_array($sqlstmt); 
					my $sqlstmt = "select hosts.name from hosts left join service_dependency on hosts.host_id = service_dependency.host_id".
						" where service_dependency.id = '$obj'";
					$object{'host_name'} = $dbh->selectrow_array($sqlstmt); 
					my $sqlstmt = "select hosts.name from hosts left join service_dependency on hosts.host_id = service_dependency.depend_on_host_id".
						" where service_dependency.id = '$obj'";
					$object{'depend_on_host'} = $dbh->selectrow_array($sqlstmt); 
				}
				if ($type eq 'servicegroups') {
					my $sqlstmt = "select host_id, service_id from servicegroup_service where servicegroup_id = '$obj'";
					my $sth = $dbh->prepare ($sqlstmt);
					$sth->execute;
					my @members = ();
					while(my @values = $sth->fetchrow_array()) { 
						my %h = StorProc->fetch_one('hosts','host_id',$values[0]);	
						my %s = StorProc->fetch_one('services','service_id',$values[1]);	
						my %sn = StorProc->fetch_one('service_names','servicename_id',$s{'servicename_id'});	
						push @members, "$h{'name'},$sn{'name'}"; 
					}
					$sth->finish;			
					foreach (@members) { $object{'members'} .= "$_," }
					chop $object{'members'};
					push @props, 'members';
					delete $object{'escalation_id'};
					unless (@members) { %object = () }
				}
				if ($type eq 'contactgroups') {
					my $sqlstmt = "select contacts.name from contacts where contact_id in (select contact_id from contactgroup_contact where contactgroup_id = '$obj')";
					my $sth = $dbh->prepare ($sqlstmt);
					$sth->execute;
					my @members = ();
					while(my @values = $sth->fetchrow_array()) { push @members, $values[0] }
					$sth->finish;			
					foreach (@members) { $object{'members'} .= "$_," }
					chop $object{'members'};
					push @props, 'members';
				}
				if ($type eq 'contacts') {
					my $sqlstmt = "select commands.name from commands left join contact_command_overrides on commands.command_id = contact_command_overrides.command_id".
						" where contact_command_overrides.contact_id = '$obj' and contact_command_overrides.type = 'host'";
					my $sth = $dbh->prepare ($sqlstmt);
					$sth->execute;
					my @members = ();
					while(my @values = $sth->fetchrow_array()) { push @members, $values[0] }
					$sth->finish;			
					foreach (@members) { $object{'host_notification_commands'} .= "$_," }
					chop $object{'host_notification_commands'};
					push @props, 'host_notification_commands';
					my $sqlstmt = "select commands.name from commands left join contact_command_overrides on commands.command_id = contact_command_overrides.command_id".
						" where contact_command_overrides.contact_id = '$obj' and contact_command_overrides.type = 'service'";
					my $sth = $dbh->prepare ($sqlstmt);
					$sth->execute;
					my @members = ();
					while(my @values = $sth->fetchrow_array()) { push @members, $values[0] }
					$sth->finish;			
					foreach (@members) { $object{'service_notification_commands'} .= "$_," }
					chop $object{'service_notification_commands'};
					push @props, 'service_notification_commands';
					
					my %overrides = StorProc->fetch_one('contact_overrides','contact_id',$obj);
					foreach my $name (keys %overrides) {
						if ($name eq 'contact_id') { next }
						if ($name eq 'host_notification_period') {
							my %np = StorProc->fetch_one('time_periods','timeperiod_id',$overrides{$name});	
							$object{'host_notification_period'} = $np{'name'};			
						} elsif ($name eq 'service_notification_period') {
							my %np = StorProc->fetch_one('time_periods','timeperiod_id',$overrides{$name});	
							$object{'service_notification_period'} = $np{'name'};			
						} else {
							$object{$name} = $overrides{$name};
						}
						push @props, $name;
					}
					my %contactgroups = ();
					$sqlstmt = "select contactgroups.name from contactgroups where contactgroup_id in ".
						"(select contactgroup_id from contactgroup_assign where type = '$type' and object = '$obj')";
					my $sth = $dbh->prepare ($sqlstmt);
					$sth->execute;
					while(my @values = $sth->fetchrow_array()) { $contactgroups{$values[0]} = 1 }
					$sth->finish;			
					foreach my $cg (sort keys %contactgroups) { $object{'contact_groups'} .= "$cg," }
					chop $object{'contact_groups'};
				}
				$objtype =~ s/s$//;
				$objtype =~ s/_//g;
				$objtype =~ s/template|parent//g;
				my $n = undef;
				unless ($object{'comment'} =~ /\n$/) { $n = "\n"}
				my $objout = undef;
				if ($objtype eq 'hostgroupescalation' && $version{'value'} eq '2.x') {
					$objout .= qq(\n$object{'comment'}$n\define hostescalation \{);
				} elsif ($objtype eq 'extendedserviceinfo') {
					$objout .= qq(\n$object{'comment'}$n\define serviceextinfo \{);
				} elsif ($objtype eq 'extendedhostinfo') {
					$objout .= qq(\n$object{'comment'}$n\define hostextinfo \{);
				} else {
					$objout .= qq(\n$object{'comment'}$n\define $objtype \{);
				}
				my $tabs = undef;
				if ($version{'values'} eq '2.x' && $type eq 'hosts') { push @props, 'contactgroups'	}
				my $got_props = undef;
				my $prop = undef;
				foreach $prop (@props) {
					unless ($object{$prop} eq 'comment') {
						my $pname = $prop;
						if ($type eq 'contactgroups' && $prop eq 'name') { $pname = 'contactgroup_name' }
						if ($type eq 'contacts' && $prop eq 'name') { $pname = 'contact_name' }
						if ($type eq 'services' && $prop eq 'name') { $pname = 'service_description' }
						if ($type eq 'servicegroups' && $prop eq 'name') { $pname = 'servicegroup_name' }
						if ($type eq 'service_dependency' && $prop eq 'service_name') { $pname = 'dependent_service_description' }
						if ($type eq 'service_dependency_templates' && $prop eq 'service_name') { $pname = 'service_description' }
						if ($type eq 'service_dependency' && $prop eq 'host_name') { $pname = 'dependent_host_name' }
						if ($type eq 'service_dependency' && $prop eq 'depend_on_host') { $pname = 'host_name' }
						if ($type eq 'hostgroup_escalation' && $prop eq 'name') { $pname = 'hostgroup_name' }
						if ($type eq 'host_escalation' && $prop eq 'name') { $pname = 'host_name' }
						if ($type eq 'time_periods' && $prop eq 'name') { $pname = 'timeperiod_name' }
						if ($type eq 'commands' && $prop eq 'name') { $pname = 'command_name' }
						if ($type eq 'host_dependencies' && $prop eq 'parent_host') { $pname = 'host_name' }
						if ($type eq 'host_dependencies' && $prop eq 'dependent_host') { $pname = 'dependent_host_name' }
						if ($type eq 'hostgroups' && $prop eq 'name') { $pname = 'hostgroup_name' }
						if ($type eq 'hostgroups' && $prop =~ /hostgroup_escalation_id|host_escalation_id|service_escalation_id/) { next }
						if ($type eq 'hosts' && $prop eq 'name') { $pname = 'host_name' }
						if ($pname eq 'template') { 
							$pname = 'use';
							if ($type eq 'service_templates') {
								my %tname = StorProc->fetch_one('service_templates','servicetemplate_id',$object{'parent_id'});
								$object{'template'} = $tname{'name'};
								if ($object{'template'} eq '-') { delete $object{'template'} }
							} elsif ($type eq 'service_dependency') {
								my %tname = StorProc->fetch_one('service_dependency_templates','id',$object{'template'});
								$object{'template'} = $tname{'name'};
								if ($object{'template'} eq '-') { delete $object{'template'} }
							} else {
								unless ($type =~ /_info$/) {
									my %tname = StorProc->fetch_one($obj_template{$type},$obj_id{$obj_template{$type}},$object{$obj_id{$obj_template{$type}}} );
									$object{'template'} = $tname{'name'};
									if ($object{'template'} eq '-') { delete $object{'template'} }
								}
							}
						}
						unless ($type eq 'commands') {
							if ($object{'command_line'}) { 
								$object{'check_command'} = $object{'command_line'};
								delete $object{'command_line'};
							}
						}
						if ($object{$prop}) {
							$object{$prop} =~ s/-zero-/0/g;
							my $length = length($pname);
							if ($length < 8) {
								$tabs = "\t\t\t\t";
							} elsif ($length < 16) {
								$tabs = "\t\t\t";
							} elsif ($length < 24) {
								$tabs = "\t\t";
							} else {
								$tabs = "\t";
							}
							$objout .= qq(
		$pname$tabs$object{$prop});
							$got_props = 1;
						}
					}
					$tabs = undef;
				}

				if ($object{'contact_groups'}) {	
					$objout .= qq(
		contact_groups			$object{'contact_groups'})
				};
				if ($type =~ /template/) {	
					$objout .= qq(
		register			0);
				}
				$objout .= "\n}\n";
				if ($got_props) { $outfile .= $objout; $got_output = 1 }
			}
			if ($got_output) {
				$use_file{$file} = 1;
				open (FILE, "> $destination/$file") || push @errors, "$destination/$file  $!";
				print FILE $outfile;
				close (FILE);
			}
		}
	}

# build_escalations

	my $outfile = undef;
	my $sqlstmt = "select files.file_id, files.name from files left join setup on setup.value = files.file_id where setup.name = 'escalations'";
	my ($id,$file) = $dbh->selectrow_array($sqlstmt);
	my $dt = StorProc->datetime();
	my $outfile = qq(##########GROUNDWORK#############################################################################################
#GW
#GW\tescalations.cfg generated $dt by $userid from monarch.cgi nagios v $version{'value'}
#GW
##########GROUNDWORK#############################################################################################
);

# escalation trees hostgroup
	my %w = ('type' => 'hostgroup');
	my @hostgroup_trees = StorProc->fetch_list_where('escalation_trees','tree_id',\%w);
	foreach my $tree (@hostgroup_trees) {
		my %where = ('hostgroup_escalation_id' => $tree);
		my @hostgroups = StorProc->fetch_list_where('hostgroups','name',\%where);
		my $sqlstmt = "select escalation_templates.template_id, escalation_templates.name from escalation_templates where template_id in ".
			"(select template_id from escalation_tree_template where tree_id = '$tree')"; 
		my $sth = $dbh->prepare ($sqlstmt);
		$sth->execute;			
		my %templates;
		while(my @values = $sth->fetchrow_array()) { $templates{$values[0]} = $values[1] }
		$sth->finish;	
		my $hostgroups = undef;
		foreach my $hg (@hostgroups) { $hostgroups .= "$hg," }
		chop $hostgroups;
		if ($hostgroups[0]) {
			$use_file{$file} = 1;
			foreach my $template (keys %templates) {
				my $sqlstmt = "select contactgroups.name from contactgroups left join tree_template_contactgroup ".
					"on contactgroups.contactgroup_id = tree_template_contactgroup.contactgroup_id ".
					"where tree_id = '$tree' and template_id = '$template'"; 
				my $sth = $dbh->prepare ($sqlstmt);
				$sth->execute;
				my @contactgroups;
				while(my @values = $sth->fetchrow_array()) { push @contactgroups, $values[0] }
				$sth->finish;	
				my $contactgroups = undef;
				foreach my $cg (@contactgroups) { $contactgroups .= "$cg," }
				chop $contactgroups;
				if ($contactgroups) {
					$contactgroups = "\n\tcontact_groups\t\t\t$contactgroups";
				}
				if ($version{'value'} eq '1.x') {
					$outfile .= qq(
define hostgroupescalation {
	use				$templates{$template}
	hostgroup_name			$hostgroups$contactgroups
}
);			
				} else {
					$outfile .= qq(
define hostescalation {
	use				$templates{$template}
	hostgroup_name			$hostgroups$contactgroups
}
);			
				}
			}
		}
	}

# escalation trees host
	my %w = ('type' => 'host');
	my @host_trees = StorProc->fetch_list_where('escalation_trees','tree_id',\%w);
	foreach my $tree (@host_trees) {
		my $sqlstmt = "select escalation_templates.template_id, escalation_templates.name from escalation_templates where template_id in ".
			"(select template_id from escalation_tree_template where tree_id = '$tree')"; 
		my $sth = $dbh->prepare ($sqlstmt);
		$sth->execute;			
		my %templates;
		while(my @values = $sth->fetchrow_array()) { $templates{$values[0]} = $values[1] }
		$sth->finish;	
		my $hostgroups = undef;
		my %where = ('host_escalation_id' => $tree);
		my @hostgroups = StorProc->fetch_list_where('hostgroups','name',\%where);
		foreach my $hg (@hostgroups) { $hostgroups .= "$hg," }
		chop $hostgroups;
		if ($hostgroups[0]) {
			$use_file{$file} = 1;
			foreach my $template (keys %templates) {
				my $sqlstmt = "select contactgroups.name from contactgroups where contactgroup_id in ".
					"(select contactgroup_id from tree_template_contactgroup where tree_id = '$tree' and template_id = '$template')"; 
				my $sth = $dbh->prepare ($sqlstmt);
				$sth->execute;
				my @contactgroups;
				while(my @values = $sth->fetchrow_array()) { push @contactgroups, $values[0] }
				$sth->finish;	
				my $contactgroups = undef;
				foreach my $cg (@contactgroups) { $contactgroups .= "$cg," }
				chop $contactgroups;
				if ($contactgroups) {
					$contactgroups = "\n\tcontact_groups\t\t\t$contactgroups";
				}
				$outfile .= qq(
define hostescalation {
	use				$templates{$template}
	hostgroup_name			$hostgroups$contactgroups
}
);			
			}
		}
		my $hosts = undef;
		my %where = ('host_escalation_id' => $tree);
		my @hosts = StorProc->fetch_list_where('hosts','name',\%where);
		foreach my $h (@hosts) { $hosts .= "$h," }
		chop $hosts;
		if ($hosts[0]) {
			$use_file{$file} = 1;
			foreach my $template (keys %templates) {
				my $sqlstmt = "select contactgroups.name from contactgroups where contactgroup_id in ".
					"(select contactgroup_id from tree_template_contactgroup where tree_id = '$tree' and template_id = '$template')"; 
				my $sth = $dbh->prepare ($sqlstmt);
				$sth->execute;
				my @contactgroups;
				while(my @values = $sth->fetchrow_array()) { push @contactgroups, $values[0] }
				$sth->finish;	
				my $contactgroups = undef;
				foreach my $cg (@contactgroups) { $contactgroups .= "$cg," }
				chop $contactgroups;
				if ($contactgroups) {
					$contactgroups = "\n\tcontact_groups\t\t\t$contactgroups";
				}
				$outfile .= qq(
define hostescalation {
	use				$templates{$template}
	host_name			$hosts$contactgroups
}
);			
			}
		}
	}

# escalation trees service
	my %w = ('type' => 'service');
	my @service_trees = StorProc->fetch_list_where('escalation_trees','tree_id',\%w);
	foreach my $tree (@service_trees) {
		my $sqlstmt = "select escalation_templates.template_id, escalation_templates.name from escalation_templates where template_id in ".
			"(select template_id from escalation_tree_template where tree_id = '$tree')"; 
		my $sth = $dbh->prepare ($sqlstmt);
		$sth->execute;			
		my %templates = ();
		my %srv_dist = ();
		while(my @values = $sth->fetchrow_array()) { 
			$templates{$values[0]} = $values[1];
			$srv_dist{$values[0]} = $values[5];
		}
		$sth->finish;	
		my $hostgroups = undef;
		my %where = ('service_escalation_id' => $tree);
		my @hostgroups = StorProc->fetch_list_where('hostgroups','name',\%where);
		foreach my $hg (@hostgroups) { $hostgroups .= "$hg," }
		chop $hostgroups;
		if ($hostgroups[0]) {
			$use_file{$file} = 1;
			foreach my $template (keys %templates) {
				my $sqlstmt = "select contactgroups.name from contactgroups where contactgroup_id in ".
					"(select contactgroup_id from tree_template_contactgroup where tree_id = '$tree' and template_id = '$template')"; 
				my $sth = $dbh->prepare ($sqlstmt);
				$sth->execute;
				my @contactgroups;
				while(my @values = $sth->fetchrow_array()) { push @contactgroups, $values[0] }
				$sth->finish;	
				my $contactgroups = undef;
				foreach my $cg (@contactgroups) { $contactgroups .= "$cg," }
				chop $contactgroups;
				if ($contactgroups) {
					$contactgroups = "\n\tcontact_groups\t\t\t$contactgroups";
				}
				my $servicedescription = undef;
				unless ($srv_dist{$template}) {
					$servicedescription = "\n\tservice_description\t\t\t*";
				}

				$outfile .= qq(
define serviceescalation {
	use				$templates{$template}
	hostgroup_name			$hostgroups$contactgroups$servicedescription
}
);			
			}
		}
	}
# escalation trees servicegroup
	my %w = ('type' => 'service');
	my @service_trees = StorProc->fetch_list_where('escalation_trees','tree_id',\%w);
	foreach my $tree (@service_trees) {
		my $sqlstmt = "select escalation_templates.template_id, escalation_templates.name from escalation_templates where template_id in ".
			"(select template_id from escalation_tree_template where tree_id = '$tree')"; 
		my $sth = $dbh->prepare ($sqlstmt);
		$sth->execute;			
		my %templates = ();
		my %srv_dist = ();
		while(my @values = $sth->fetchrow_array()) { 
			$templates{$values[0]} = $values[1];
			$srv_dist{$values[0]} = $values[5];
		}
		$sth->finish;	
		unless ($version{'value'} eq '1.x') {
			my $servicegroups = undef;
			my %where = ('escalation_id' => $tree);
			my @servicegroup = StorProc->fetch_list_where('servicegroups','name',\%where);
			foreach my $hg (@servicegroup) { $servicegroups .= "$hg," }
			chop $servicegroups;
			if ($servicegroup[0]) {
				$use_file{$file} = 1;
				foreach my $template (keys %templates) {
					my $sqlstmt = "select contactgroups.name from contactgroups where contactgroup_id in ".
						"(select contactgroup_id from tree_template_contactgroup where tree_id = '$tree' and template_id = '$template')"; 
					my $sth = $dbh->prepare ($sqlstmt);
					$sth->execute;
					my @contactgroups = ();
					while(my @values = $sth->fetchrow_array()) { push @contactgroups, $values[0] }
					$sth->finish;	
					my $contactgroups = undef;
					foreach my $cg (@contactgroups) { $contactgroups .= "$cg," }
					chop $contactgroups;
					if ($contactgroups) {
						$contactgroups = "\n\tcontact_groups\t\t\t$contactgroups";
					}
					$outfile .= qq(
define serviceescalation {
	use				$templates{$template}
	servicegroup_name		$servicegroups$contactgroups
}
);			
				}
			}
		}
		my $hosts = undef;
		my %where = ('service_escalation_id' => $tree);
		my @hosts = StorProc->fetch_list_where('hosts','name',\%where);
		foreach my $h (@hosts) { $hosts .= "$h," }
		chop $hosts;
		if ($hosts[0]) {
			$use_file{$file} = 1;
			foreach my $template (keys %templates) {
				my $sqlstmt = "select contactgroups.name from contactgroups where contactgroup_id in ".
					"(select contactgroup_id from tree_template_contactgroup where tree_id = '$tree' and template_id = '$template')"; 
				my $sth = $dbh->prepare ($sqlstmt);
				$sth->execute;
				my @contactgroups;
				while(my @values = $sth->fetchrow_array()) { push @contactgroups, $values[0] }
				$sth->finish;	
				my $contactgroups = undef;
				foreach my $cg (@contactgroups) { $contactgroups .= "$cg," }
				chop $contactgroups;
				if ($contactgroups) {
					$contactgroups = "\n\tcontact_groups\t\t\t$contactgroups";
				}
				my $servicedescription = undef;
				unless ($srv_dist{$template}) {
					$servicedescription = "\n\tservice_description\t\t\t*";
				}

				$outfile .= qq(
define serviceescalation {
	use				$templates{$template}
	host_name			$hosts$contactgroups$servicedescription
}
);			
			}
		}
		my @services = StorProc->get_hostname_servicename('services','escalation_id',$tree);
		for my $i ( 0 .. $#services ) {
			for my $host ( keys %{ $services[$i] } ) {
				$use_file{$file} = 1;
				foreach my $template (keys %templates) {
					my $sqlstmt = "select contactgroups.name from contactgroups where contactgroup_id in ".
						"(select contactgroup_id from tree_template_contactgroup where tree_id = '$tree' and template_id = '$template')"; 
					my $sth = $dbh->prepare ($sqlstmt);
					$sth->execute;
					my @contactgroups;
					while(my @values = $sth->fetchrow_array()) { push @contactgroups, $values[0] }
					$sth->finish;	
					my $contactgroups = undef;
					foreach my $cg (@contactgroups) { $contactgroups .= "$cg," }
					chop $contactgroups;
					if ($contactgroups) {
						$contactgroups = "\n\tcontact_groups\t\t\t$contactgroups";
					}
					$outfile .= qq(
define serviceescalation {
	use				$templates{$template}
	host_name			$host$contactgroups
	service_description		$services[$i]{$host}
}
);			
				}
			}
		}
	}
	if ($outfile) {
		open (FILE, "> $destination/escalations.cfg") || push @errors, " $destination/escalations.cfg  $!";
		print FILE $outfile;
		close (FILE);
		$use_file{'escalations.cfg'} = 1;
	}
	$outfile = undef;

# build_resource

	$dt = StorProc->datetime();
	$outfile = qq(##########GROUNDWORK#############################################################################################
#GW
#GW\tresource.cfg generated $dt by $userid from monarch.cgi nagios v $version{'value'}
#GW
##########GROUNDWORK#############################################################################################
);
	my %where = ('type' => 'resource');
	my %user_props = StorProc->fetch_list_hash_array('setup',\%where);
	for (my $i = 1; $i <= 32; $i++) {
		my $key = "user$i";
		if ($user_props{$key}[2]) {
			$outfile .= qq(
\$USER$i\$=$user_props{$key}[2]
);
		}					
	}
	my %where = ('type' => 'nagios','name' => 'resource_file');
	my %val = StorProc->fetch_one_where('setup',\%where);
	my $resourcef = undef;
	if ($val{'value'} =~ /.*\/(.*\.cfg)/) { $resourcef = $1 }
	open(FILE, "> $destination/$resourcef") || push @errors, "Error: Unable to open $destination/$resourcef $!";
	print FILE $outfile;
	close (FILE);
	$use_file{$resourcef} = 1;
	return \@errors,\%use_file;
}

sub nagios_cgi(@) {
	my $userid = $_[1];
	my $export = $_[2];
	my $nagios_home = $_[3];
	my $destination = $_[4];
	my %nagios_cfg = StorProc->fetch_one('setup','name','nagios_etc');
	my %version = StorProc->fetch_one('setup','name','nagios_version');
	my @cgiprops = ('physical_html_path','url_html_path','show_context_help','nagios_check_command','use_authentication','default_user_name',
'authorized_for_system_information','authorized_for_system_commands','authorized_for_configuration_information','authorized_for_all_services','authorized_for_all_hosts','authorized_for_all_host_commands',
'authorized_for_all_service_commands','statusmap_background_image','default_statusmap_layout','default_statuswrl_layout',
'statuswrl_include','refresh_rate','ping_syntax','host_unreachable_sound','host_down_sound','service_critical_sound','service_warning_sound',
'service_unknown_sound','ddb');
	my @errors = ();
	my $dt = StorProc->datetime();
	my $outfile = qq(##########GROUNDWORK#############################################################################################
#GW
#GW\tcgi.cfg generated $dt by $userid from monarch.cgi nagios v $version{'value'}
#GW
##########GROUNDWORK#############################################################################################
);
	my $sqlstmt = "select name, path from files where name like 'extended%'";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;			
	my %files = ();
	while(my @values = $sth->fetchrow_array()) { $files{$values[0]} = $values[1] }
	$sth->finish;	
	$outfile .= "\n# MAIN NAGIOS CONFIGURATION FILE\nmain_config_file=$nagios_cfg{'value'}/nagios.cfg\n";
	foreach my $prop (@cgiprops) {
		my $title = "\U$prop";
		$title =~ s/_/ /g;
		my %val = StorProc->fetch_one('setup','name',$prop);
		$val{'value'} =~ s/-zero-/0/g;
		my $comment = undef;
		if ($prop eq 'use_authentication') {
			unless ($val{'value'}) { $val{'value'} = '0'}
		}
		if ($val{'value'} eq '') { $comment = '# ' }
		$outfile .= "\n# $title\n$comment$prop=$val{'value'}\n";
		if ($version{'value'} eq '1.x' && $prop eq 'nagios_check_command') {
			my $sqlstmt = "select name, path from files where name like 'extended%'";
			my $sth = $dbh->prepare ($sqlstmt);
			$sth->execute;			
			my %files = ();
			while(my @values = $sth->fetchrow_array()) { $files{$values[0]} = $values[1] }
			$sth->finish;	
			$outfile .= "\n# XEDTEMPLATE CONFIG FILES\n";
			foreach my $file (sort keys %files) {
				if (-e "$destination/$file") {
					$outfile .= "xedtemplate_config_file=$files{$file}/$file\n";
				}
			}
		}
	}
	close(FILE);
	unless ($export) { $destination = $nagios_cfg{'value'} }
	open(FILE, "> $destination/cgi.cfg") || push @errors, "Error: Unable to open $destination/cgi.cfg $!";
	print FILE $outfile;
	close (FILE);
	return @errors;
}

sub backup(@) {
	my $nagios = $_[1];
	my $backup = $_[2];
	my @errors = ();
	use File::Copy;
	my @files = ("$nagios/nagios.cfg");
	my @dirs = ();
	open(FILE, "< $nagios/nagios.cfg") || push @errors, "error: cannot open $nagios/nagios.cfg to read $!";
	while (my $line = <FILE>) {
		if ($line =~ /^\s*cfg_file\s*=\s*(\S+)$/) { push @files, $1 }
		if ($line =~ /^\s*cfg_dir\s*=\s*(\S+)$/) { push @dirs, $1 }
		if ($line =~ /^\s*resource_file\s*=\s*(\S+)$/) { push @files, $1 }
	}
	close(FILE);
	open(FILE, "< $nagios/cgi.cfg") || push @errors, "error: cannot open $nagios/cgi.cfg to read $!";
	while (my $line = <FILE>) {
		if ($line =~ /^\s*xedtemplate_config_file\s*=\s*(\S+)$/) { push @files, $1 }
	}
	close(FILE);
	foreach my $dir (@dirs) {
		opendir(DIR, $dir) || push @errors, "error: cannot open $dir to read $!";
		while (my $file = readdir(DIR)) {
			if ($file =~ /^#/) { next }
			if ($file =~ /(\S+\.cfg$)/) { push @files, "$dir/$1" }
		}
		close(DIR);
	}
	my $dt = StorProc->datetime();
	$dt =~ s/\s/_/g;
	$dt =~ s/:/-/g;
	mkdir("$backup/$dt", 0770) || push @errors, "Error: Unable to create folder $backup/$dt $!";
	foreach my $file (@files) {
		my $fname = undef;
		if ($file =~ /.*\/(.*\.cfg)/) { $fname = $1 }
		copy("$file","$backup/$dt/$fname") || push @errors, "Error: Unable to copy $file to $backup/$dt/$fname $!";
	}
	my $sqlfile = "$backup/$dt/monarch-$dt.sql";
	if (-e '/usr/bin/mysqldump') {
		my $mysqldump_cmd = '/usr/bin/mysqldump';
		my $err = system("$mysqldump_cmd", "--host=$dbhost", "--user=$user", "--password=$passwd", "--quick", "--add-drop-table", "--result-file=$sqlfile", "$database");
		if ($err) { push @errors, "$mysqldump_cmd --host=$dbhost --user=$user --password=$passwd --result-file=$sqlfile db $database errno $err"  }
	} elsif (-e '/usr/local/groundwork/bin/mysqldump') {
		my $mysqldump_cmd = '/usr/local/groundwork/bin/mysqldump';
		my $err = system("$mysqldump_cmd", "--host=$dbhost", "--user=$user", "--password=$passwd", "--quick", "--add-drop-table", "--result-file=$sqlfile", "$database");
		if ($err) { push @errors, "$mysqldump_cmd --host=$dbhost --user=$user --password=$passwd --result-file=$sqlfile db $database errno $err"  }
	} else {
		push @errors, "Cannot find mysqldump in /usr/bin! Unable to backup database $database.";
	}
	return "$backup/$dt/", \@errors;
}

1;
