#!/usr/bin/perl --
# MonArch - Groundwork Monitor Architect
# monarch.cgi
#
###############################################################################
# Release 1.2
# 6-Jul-2006
###############################################################################
# Author: Scott Parris
#
# Copyright (C) 2005 Groundwork Open Source Solutions
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

use lib qq(/usr/local/groundwork/monarch/lib);
use strict;
use CGI;
use MonarchForms;
use MonarchStorProc;
use MonarchDoc;
use URI::Escape;
$|++;


#
###########################################################################################################################
# Global Declarations
#

my $debug = undef;
my $query = new CGI;
my $top_menu = $query->param('top_menu');
my $view = $query->param('view');
my $obj = $query->param('obj');
my $obj_view = $query->param('obj_view');
my $task = $query->param('task');
my $submit = $query->param('submit');
my $user_acct = $query->param('user_acct');
my $password = $query->param('password');
my $page_title = 'Monarch';
my (%auth_add, %auth_modify, %auth_delete, %properties, %authentication, %hidden) = ();
my ($nagios_ver, $nagios_bin, $nagios_etc, $monarch_home, $backup_dir, $is_portal, $upload_dir, $monarch_ver, $enable_externals) = 0;
$hidden{'selected'} = $query->param('selected'); 
$hidden{'top_menu'} = $query->param('top_menu');
$hidden{'view'} = $view;
$hidden{'obj'} = $obj;
$hidden{'user_acct'} = $user_acct;
$hidden{'nocache'} = time;
unless ($top_menu) { $top_menu = 'hosts' }
if ($task && $task ne 'No') { $hidden{'task'} = $task }
my $table = $obj;
if ($obj eq 'commands') { $table = 'commands' }
my @errors = ();
my @messages = ();
my ($body,$javascript) = undef;
my $doc_root = $ENV{'DOCUMENT_ROOT'};
my $userid = undef;
my $refresh_url = undef;
my $refresh_left = undef;
my $max_tree_nodes = undef;
my $tab = 1;


my @common_menus = ('time_periods','commands','contact_templates','contacts','contactgroups');
my @host_menus = ('hosts','hostgroups','parent_child','host_templates','extended_host_info_templates','host_dependencies');
my @service_menus = ('services','service_templates','service_dependency_templates','extended_service_info_templates');
my @tool_menus = ('export');				

my @control_menus = ('users','user_groups','setup','nagios_cgi_configuration','nagios_main_configuration','nagios_resource_macros',
'load','files','pre_flight_test','commit');


my %property_list = StorProc->property_list();
my %db_values = StorProc->db_values();


my %required = ();
$required{'commands'} = "name,type,command_line";
$required{'contactgroups'} = "name,alias,contact";
$required{'contact_templates'} = "name,host_notification_period,service_notification_period,host_notification_options,service_notification_options";
$required{'contacts'} = "name,alias,template";
$required{'contact_overrides'} = "host_notification_period,service_notification_period,host_notification_options,service_notification_options";
$required{'extended_host_info_templates'} = "name";
$required{'extended_host_info'} = "name,template";
$required{'extended_service_info'} = "name";
$required{'extended_service_info_templates'} = "name";
$required{'host_dependencies'} = "dependent_host,parent_host,notification_failure_criteria";
$required{'hostgroups'} = "name,alias,members,contactgroup";
$required{'host_templates'} = "name,max_check_attempts,notification_interval,notification_period,notification_options";
$required{'hosts'} = "name,alias,address";
$required{'service_dependency'} = "service_name";
$required{'service_dependency_templates'} = "name,service_name";
$required{'escalation_templates'} = "name,first_notification,last_notification,notification_interval";
$required{'service_templates'} = "name";
$required{'services'} = "name,host_name";
$required{'time_periods'} = "name,alias";

my %obj_id = ('commands' => 'command_id','contact_templates' => 'contacttemplate_id','contactgroups' => 'contactgroup_id','contacts' => 'contact_id',
'escalation_templates' => 'template_id','hostgroup_escalation_templates' => 'template_id','host_escalation_templates' => 'template_id',
'escalation_trees' => 'tree_id','service_escalation_templates' => 'template_id','extended_host_info_templates' => 'hostextinfo_id',
'extended_service_info_templates' => 'serviceextinfo_id','host_dependencies' => 'host_id','host_templates' => 'hosttemplate_id',
'hostgroups' => 'hostgroup_id','hosts' => 'host_id','service_dependency' => 'id','service_dependency_templates' => 'id',
'service_templates' => 'servicetemplate_id','servicegroups' =>  'servicegroup_id','services' => 'service_id','time_periods' => 'timeperiod_id');

my %obj_template = ('contacts' => 'contact_templates','service_templates' => 'service_templates','services' => 'service_templates',
'service_dependency' => 'service_dependency_templates','hosts' => 'host_templates');

my %obj_template_id = ('commands' => 'command_id',
'contact_templates' => 'contacttemplate_id',
'contactgroups' => 'contactgroup_id',
'contacts' => 'contact_id',
'escalation_templates' => 'template_id',
'escalation_trees' => 'tree_id',
'extended_host_info_templates' => 'hostextinfo_id',
'extended_service_info_templates' => 'serviceextinfo_id',
'host_dependencies' => 'host_id',
'host_templates' => 'hosttemplate_id',
'hostgroups' => 'hostgroup_id',
'hosts' => 'hosttemplate_id',
'service_dependency' => 'id',
'service_dependency_templates' => 'id',
'service_templates' => 'parent_id',
'servicegroups' =>  'servicegroup_id',
'services' =>  'servicetemplate_id',
'time_periods' => 'timeperiod_id');

my %textsize = ();
$textsize{'low_flap_threshold'} = 5;
$textsize{'high_flap_threshold'} = 5;
$textsize{'notification_interval'} = 15;
$textsize{'first_notification'} = 5;
$textsize{'last_notification'} = 5;
$textsize{'max_check_attempts'} = 5;
$textsize{'normal_check_interval'} = 15;
$textsize{'retry_check_interval'} = 15;
$textsize{'check_interval'} = 15;
$textsize{'freshness_threshold'} = 15;
$textsize{'notes_url'} = 75;
$textsize{'email'} = 75;
$textsize{'pager'} = 75;
$textsize{'name'} = 50;
$textsize{'alias'} = 75;
$textsize{'address'} = 16;
$textsize{'description'} = 75;
$textsize{'command_line'} = 75;
$textsize{'icon_image'} = 70;
$textsize{'icon_image_alt'} = 70;
$textsize{'vrml_image'} = 70;
$textsize{'statusmap_image'} = 70;
$textsize{'2d_coords'} = 7;
$textsize{'3d_coords'} = 7;
$textsize{'sunday'} = 75;
$textsize{'monday'} = 75;
$textsize{'tuesday'} = 75;
$textsize{'wednesday'} = 75;
$textsize{'thursday'} = 75;
$textsize{'friday'} = 75;
$textsize{'saturday'} = 75;
$textsize{'small'} = 5;
$textsize{'short_name'} = 20;
$textsize{'long_name'} = 70;
my $empty_data = qq(<?xml version="1.0" ?>
<data>
</data>);

# Some buttons

my %add = ('name' => 'add', 'value' => 'Add');
my %save = ('name' => 'save', 'value' => 'Save');
my %delete = ('name' => 'delete', 'value' => 'Delete');
my %select = ('name' => 'select', 'value' => 'Select');
my %cancel = ('name' => 'cancel', 'value' => 'Cancel');
my %close = ('name' => 'close','value' => 'Close');
my %next = ('name' => 'next', 'value' => 'Next >>');
my %back = ('name' => 'back', 'value' => '<< Back');
my %continue = ('name' => 'continue', 'value' => 'Continue');
my %abort = ('name' => 'abort', 'value' => 'Abort');
my %host_vitals = ('name' => 'obj_view', 'value' => 'Host Vitals');
my %service_list = ('name' => 'obj_view', 'value' => 'Service List');
my %service_detail = ('name' => 'obj_view', 'value' => 'Service Detail');
my %rename = ('name' => 'rename', 'value' => 'Rename');
my %yes = ('name' => 'yes','value' => 'Yes');
my %no = ('name' => 'no','value' => 'No');
my %assign_contactgroups = ('name' => 'assign_contactgroups', 'value' => 'Assign Contact Groups');


#
###########################################################################################################################
#	Sub to present errors
#

sub error_out($) {
	my $err = shift;
	$body .= "<h2>$err</h2><br>";
}

#
###########################################################################################################################
#	Sub to parse queries
#

sub parse_query($$) {
	my $data = shift;
	my $object = shift;
	my %data_vals = ();
	my %data_props = ();
	my @contactgroups = ();
	if ($db_values{$data} =~ /,data/) { 
		$data_vals{'data'} = "<?xml version=\"1.0\" ?>\n<data>";
	}
	my %overrides = ();
	foreach my $name ($query->param) {
		if ($name =~ /_override$/) {
			$name =~ s/_override$//;		
			$overrides{$name} = 1;
			$overrides{'template'} = 1;
		}
	}
	my %name_vals = ();
	foreach my $name ($query->param) {
		unless ($overrides{$name} || $name =~ /_override|nocache|servicename_id/) {
			my $val = $query->param($name);
			if ($val eq '0') { $val = '-zero-' }
			$name_vals{$name} = $val;
		}
	}
	if ($object eq 'host_templates' || $object eq 'host_overrides') {
		my @checks =();
		if ($nagios_ver eq '2.x') {
			@checks = ('notifications_enabled','check_freshness','obsess_over_host',
				'passive_checks_enabled','active_checks_enabled','event_handler_enabled','flap_detection_enabled',
				'process_perf_data','retain_status_information','retain_nonstatus_information');
		} else {
			@checks = ('checks_enabled','notifications_enabled','event_handler_enabled','flap_detection_enabled',
				'process_perf_data','retain_status_information','retain_nonstatus_information');
		}
		foreach my $check (@checks) {
			unless ($name_vals{$check}) { $name_vals{$check} = '-zero-'}
		}
	}
	if ($object eq 'service_templates' || $object eq 'service_overrides') {
		my @checks =  ('is_volatile','active_checks_enabled','passive_checks_enabled','parallelize_check',
			'obsess_over_service','check_freshness','event_handler_enabled','flap_detection_enabled',
			'process_perf_data','retain_status_information','retain_nonstatus_information','notifications_enabled');
		foreach my $check (@checks) {
			unless ($name_vals{$check}) { $name_vals{$check} = '-zero-'}
		}
	}
	my @db_vals = split(/,/, $db_values{$data});
	my @props = split(/,/, $property_list{$object});
	my $use_template_command = $query->param('use_template_command');
	foreach my $name (keys %name_vals) {
		if ($name =~ /check_command|^event_handler$/) {
			my %c = StorProc->fetch_one('commands','name',$name_vals{$name});
			$data_vals{$name} = $c{'command_id'};
			$properties{$name} = $name_vals{$name};
			$data_props{$name} = $data_vals{$name};
		} elsif ($name eq 'command_line') {
			unless ($use_template_command || $object eq 'service_overrides') {
				my $command_line = $name_vals{$name};
				$data_vals{'data'} .= "\n  <prop name=\"command_line\"><![CDATA[$command_line]]>\n  </prop>";
				delete $data_vals{$name};
				$properties{$name} = $name_vals{$name};
				$data_props{$name} = $name_vals{$name};
			}
		} elsif ($name =~ /service_description|service_name/) {
			$properties{$name} = $name_vals{$name};
			my %s = StorProc->fetch_one('service_names','name',$properties{$name});		
			$data_vals{'servicename_id'} = $s{'servicename_id'};
		} elsif ($name eq 'host_profile') {
			my %p = StorProc->fetch_one('profiles_host','name',$name_vals{$name});
			$data_vals{'hostprofile_id'} = $p{'hostprofile_id'};
			$properties{$name} = $name_vals{$name};
		} elsif ($name eq 'dependent_host') {
			my %h = StorProc->fetch_one('hosts','name',$name_vals{$name});
			$data_vals{'host_id'} = $h{'host_id'};
			$properties{$name} = $name_vals{$name};
		} elsif ($name eq 'parent_host') {
			my %h = StorProc->fetch_one('hosts','name',$name_vals{$name});
			$data_vals{'parent_id'} = $h{'host_id'};
			$properties{$name} = $name_vals{$name};
		} elsif ($name =~ /members|contact|contactgroup|parents/) {
			my @members = $query->param($name);
			delete $properties{$name};
			if ($name eq 'contactgroup' && $obj eq 'service_templates') { @contactgroups = sort @members }
			foreach  (@members) { $properties{$name} .= "$_," }
			chop $properties{$name};
			$data_vals{$name} = $properties{$name};
			$data_props{$name} = $properties{$name};
		} elsif ($name =~ /notification_options$|^stalking_options$|^notification_failure_criteria$|^execution_failure_criteria$|^escalation_options$/) {
			my @members = $query->param($name);
			delete $properties{$name};
			foreach  (@members) { $properties{$name} .= "$_," }
			chop $properties{$name};
		} elsif ($name =~ /notification_commands/) {
			my @members = $query->param($name);
			delete $properties{$name};
			foreach my $m (@members) { 
				if ($m) {
					$properties{$name} .= "$m,";
					my %c = StorProc->fetch_one('commands','name',$m);
					$data_vals{$name} .= "$c{'command_id'},";
				}
			}
			chop $data_vals{$name};
			chop $properties{$name};
		} elsif ($name eq 'last_notification') {
			my $last_notification = $query->param('last_notification');
			if ($last_notification eq '0') { 
				$properties{'last_notification'} = '-zero-';
			} else {
				$properties{'last_notification'} = $last_notification;
			}
		} elsif ($name =~ /period/) {
			my %t = StorProc->fetch_one('time_periods','name',$name_vals{$name});
			$data_vals{$name} = $t{'timeperiod_id'};
			$data_props{$name} = $data_vals{$name};
			$properties{$name} = $name_vals{$name};
		} elsif ($name =~ /escalation/) {
			my %t = StorProc->fetch_one('escalation_trees','name',$name_vals{$name});
			$data_vals{$name} = $t{'tree_id'};
			$properties{$name} = $name_vals{$name};
		} elsif ($name =~ /template/) {
			my $value = $name_vals{$name};
			my %t = StorProc->fetch_one($obj_template{$object},'name',$value);
			$data_vals{$obj_template_id{$obj_template{$object}}} = $t{$obj_template_id{$obj_template{$object}}};
			delete $data_vals{'template'};
			if ($obj eq 'service_templates') {
				$data_props{'parent_id'} = $t{'servicetemplate_id'};
				$data_vals{'parent_id'} = $t{'servicetemplate_id'};
			}
			delete $data_vals{'template'};
			delete $data_props{'template'};
			if (!$properties{'template'}) {$properties{'template'} = $name_vals{$name}}
		} else {
			$properties{$name} = $name_vals{$name};
			$data_props{$name} = $properties{$name};
		}
		foreach my $p (@props) {
			if ($overrides{$p}) { next }
			if ($name eq 'escalation_options') {
				$data_vals{'data'} .= "\n  <prop name=\"$name\"><![CDATA[$properties{$p}]]>\n  </prop>";  
				$data_props{$name} = $properties{$p};
			}
			unless ($name =~ /^template_id|^event_handler$|command|service_description|members|contact|contactgroup|parents|escalation|period/) {
				if ($p eq $name) { 
					my $match = undef;
					foreach my $val (@db_vals) {
						unless ($val =~ /template/) {
							if ($val eq $name) { 
								$data_vals{$val} = $name_vals{$name};
								$match = 1;
								last;
							}
						}
					}
					if (!$match && $data_vals{'data'}) {
						if ($properties{$p}) {
							$data_vals{'data'} .= "\n  <prop name=\"$name\"><![CDATA[$properties{$p}]]>\n  </prop>";  
							$data_props{$name} = $properties{$p};
						}
					}
				}
			}
		}
	}

	if ($obj eq 'service_templates') {
		if ($data_props{'parent_id'}) {
			%data_vals = ();
			$data_vals{'parent_id'} = $data_props{'parent_id'};
			$data_vals{'data'} = "<?xml version=\"1.0\" ?>\n<data>";
			my %parent = StorProc->get_template_properties($data_props{'parent_id'});
			foreach my $name (keys %data_props) {
				if ($parent{$name} eq $data_props{$name}) { delete $data_props{$name} }
			}
			delete $data_props{'template'};
			foreach my $name (keys %data_props) {
				my $got_it = 0;
				foreach my $p (@props) { 
					if ($name eq $p) { $got_it = 1 }
				}
				unless ($got_it) { delete $data_props{$name} }
			}
			foreach my $name (keys %data_props) {
				if ($name =~ /^event_handler$|parent_id|check_command|contactgroup|parent|period|name/) {
					$data_vals{$name} = $data_props{$name};
				} else {
					$data_vals{'data'} .= "\n  <prop name=\"$name\"><![CDATA[$data_props{$name}]]>\n  </prop>";  					
				}
			}
			foreach my $cg (@contactgroups) {
				my $got_group = 0;
				foreach my $pg (@{$parent{'contactgroup'}}) {
					if ($pg eq $cg) { $got_group = 1 }
				}
			}
		}	
	}
	$properties{'save'} = 1;
	if ($data_vals{'data'}) { $data_vals{'data'} .= "\n </data>" }
	return %data_vals;
}

#
###########################################################################################################################
# Check Fields - uniqueness/missing info
#

sub check_fields() {
	my @req = split(/,/, $required{$table});
	my $missing = undef;
	foreach my $r (@req) {
		if ($obj eq 'hostgroups' && $nagios_ver eq '2.x') { next }
		unless ($properties{$r}) {
			$required{$r} = 1;
			$required{'missing'} = 1;
			$missing .= "$r,";
		}
	}
	chop $missing;
	$missing .= ".";

	if ($required{'missing'}) {
		push @errors, "Missing required fields: $missing";
	} elsif ($query->param('add') || $submit eq 'Add') {
		unless ($obj =~ /host_dependencies/) {
			my %res = StorProc->fetch_one($table,'name',$properties{'name'}); 
			if ($res{'name'}) {
				my $type = $obj;
				$type =~ s/_/ /g;
				$type =~ s/s$//g;
				push @errors, "\u$type $properties{'name'} already exists.";
			} 
		}
	} 
	if (@errors) {
		return 1;
	} else {
		return 0;
	}
}

#
###########################################################################################################################
# Sub eval errors
#

sub eval_errors($) {
	my @eval = shift;
	my @errors = ();
	foreach my $err (@eval) {
		unless ($err =~ /duplicate|1/i) { 
			push @errors, $err;
		}
	}
	return @errors;
}


#
###########################################################################################################################
# Sub to build most forms
#

sub build_form() {
	my $form = undef;
	my $extended_props = undef;
	my $name = $query->param('name');
	my $source = $query->param('source');
	my $ext_props = $query->param('ext_props');

	$hidden{'source'} = $source;

#############################################################################
# Get Properties
#

	unless($properties{'save'}) { 
		if ($task eq 'new') {
			my @p_list = split(/,/, $property_list{$obj});
			foreach my $p (@p_list) { $properties{$p} = '' }			
		} elsif ($task =~ /copy|use/) { 
			$name = $source;
			%properties = StorProc->fetch_one($table,'name',$name); 
			if ($task eq 'use') { $properties{'template'} = $source }
			$properties{'name'} = undef;
		} else {
			%properties = StorProc->fetch_one($table,'name',$name); 
		}
	}

	if ($obj eq 'contacts' && $task eq 'new') { 
		$table = 'contact_templates'; 
		$task = 'use';
	}
	my @props = split(/,/, $property_list{$obj});

	if ($task eq 'modify') { $source = $name}

#############################################################################
# Top of form
#
	my $title = undef;
	my @t_parse = split(/_/, $obj);
	foreach (@t_parse) { $_ =~ s/ies/y/g; $title .= "\u$_ "; }
	$title =~ s/s\s$//;
	$title =~ s/Wizard/Extended/;
	my $form_top = undef;
	if ($title =~ /Escalation/) { $title =~ s/Template// }
	unless ($obj eq 'service_templates') {
		$form_top = Forms->form_top("$title Properties",'');
		if ($task eq 'modify'){
			$form .= Forms->display_hidden('Name:','name',$properties{'name'});
		} elsif ($obj eq 'host_dependencies') {
			my @hosts = StorProc->fetch_list('hosts','name'); 
			$form .= Forms->list_box('Dependent host name:','name',\@hosts,$properties{'name'},$required{'name'});
		} else {
			$form .= Forms->text_box('Name:','name',$properties{'name'},$textsize{'name'},$required{'name'});
		}
	}
	my %props = ();
	foreach my $property (@props) { $props{$property} = $property }
	if ($obj eq 'service_templates') {
		delete $props{'check_command'};
		delete $props{'command_line'};
	}
	if ($obj eq 'host_templates') {
		if ($nagios_ver eq '1.x') {
			delete $props{'active_checks_enabled'};
			delete $props{'passive_checks_enabled'};
			delete $props{'obsess_over_host'};
			delete $props{'check_period'};
			delete $props{'check_freshness'};
			delete $props{'check_interval'};
			delete $props{'freshness_threshold'};
			delete $props{'contactgroup'};
		} else {
			delete $props{'checks_enabled'};
		}
	}
	if ($obj =~ /escalation/ && $nagios_ver eq '1.x') {
		delete $props{'escalation_period'};
		delete $props{'escalation_options'};
	}

#############################################################################
# Body of form
#
	my %docs = Doc->properties_doc($obj,\@props);
	foreach my $property (@props) {
		$properties{$property} =~ s/-zero-/0/g;
		my $p_title = "\u$property:";
		$p_title =~ s/_/ /g;
		if ($props{$property} =~ /^is_volatile|^checks_enabled$|^notifications_enabled$|^parallelize_check$|^obsess_over_host|^obsess_over_service$|^check_freshness$|^passive_checks_enabled$|^active_checks_enabled$|^event_handler_enabled$|^flap_detection_enabled$|^process_perf_data$|^retain_status_information$|^retain_nonstatus_information$/) {
			$form .= Forms->checkbox($p_title,$property,$properties{$property},$docs{$property},'',$tab++);
		
		} elsif ($props{$property} eq 'contactgroup') {
			my @members = ();
			if ($properties{'save'}) {
				@members = split(/,/, $properties{'contactgroup'});
			} else { 
				@members = StorProc->get_contactgroups($obj,$properties{$obj_id{$obj}}); 
			}
			my @nonmembers = StorProc->fetch_list('contactgroups','name'); 
			if ($obj =~ /escalation|host_templates|service_templates/) {
				$form .= Forms->members('Contact Groups:','contactgroup',\@members,\@nonmembers,$required{$property},'10',$docs{$property},'',$tab++);
				$tab += 3;
				$form_top = Forms->form_top("$title Properties",'1');	
			} else {
				$form .= Forms->list_box_multiple('Contact Groups:','contactgroup',\@nonmembers,\@members,$required{'contactgroup'},$docs{$property},'',$tab++);
			}
		
		} elsif ($props{$property}  eq 'members') {
			my @members;
			if ($properties{'save'}) {
				@members = split(/,/, $properties{'members'});
			} elsif ($task =~ /copy|modify/) { 
				@members = StorProc->get_names_in('host_id','hosts','hostgroup_host','hostgroup_id',$properties{'hostgroup_id'});
			}
			my @nonmembers = StorProc->fetch_list('hosts','name'); 
			$form .= Forms->members($p_title,$property,\@members,\@nonmembers,$required{$property},'',$docs{$property},'',$tab++);
			$form_top = Forms->form_top("$title Properties",'1');
			$tab += 3;
		} elsif ($props{$property}  eq 'contact') {
			my @members = ();
			if ($properties{'save'}) {
				@members = split(/,/, $properties{'contact'});
			} elsif ($task =~ /copy|modify/) { 
				@members = StorProc->get_names_in('contact_id','contacts','contactgroup_contact','contactgroup_id',$properties{$obj_id{$obj}});
			}
			my @nonmembers = StorProc->fetch_list('contacts','name'); 
			$form .= Forms->members('Contacts:','contact',\@members,\@nonmembers,$required{$property},'',$docs{$property},'',$tab++);
			$form_top = Forms->form_top("$title Properties",'1');
		
		} elsif ($props{$property}  eq 'stalking_options') {
			my @opts = split(/,/, $properties{$property});
			$form .= Forms->stalking_options(\@opts,$required{$property},$docs{$property},'',$tab++);
			$tab += 3;
		} elsif ($props{$property}  =~ /notification_options|escalation_options/) {
			my @opts = split(/,/, $properties{$property});
			$form .= Forms->notification_options($obj,$property,\@opts,$required{$property},$nagios_ver,$docs{$property},'',$tab++);
			$tab += 6;
		} elsif ($props{$property}  =~ /host_name$/) {
			my @hosts = StorProc->fetch_list('hosts','name'); 
			$form .= Forms->list_box($p_title,$property,\@hosts,$properties{$property},$required{$property},$docs{$property},'',$tab++);
		
		} elsif ($props{$property}  =~ /host_escalation|service_escalation/) {
			my $type = $property;
			$type =~ s/_escalation_id//;
			my %where = ('type' => $type);
			my @list = StorProc->fetch_list_where('escalation_trees','name',\%where ); 
			%where = ('tree_id' => $properties{$property});
			my %t = StorProc->fetch_one_where('escalation_trees',\%where);
			$form .= Forms->list_box($p_title,$property,\@list,$t{'name'},'',$docs{$property},'',$tab++);
		
		} elsif ($props{$property}  =~ /service_name$|service_description/) {
			unless ($task eq 'new' || $properties{'save'}) {
				my %s = StorProc->fetch_one('service_names','servicename_id',$properties{'servicename_id'});
				$properties{$property} = $s{'name'};
			}
			my @services = ();
			if ($obj =~ /escalation/) { push @services, '*'	}
			my @svcs = StorProc->fetch_list('service_names','name'); 
			push (@services, @svcs);
			$form .= Forms->list_box($p_title,$property,\@services,$properties{$property},$required{$property},$docs{$property},'',$tab++);
		
		} elsif ($props{$property}  =~ /check_command$|event_handler$/) {
			unless ($task eq 'new' || $properties{'save'}) {
				my %p = StorProc->fetch_one('commands','command_id',$properties{$property});
				$properties{$property} = $p{'name'};
			}
			my @commands = StorProc->fetch_list('commands','name'); 
			$form .= Forms->list_box($p_title,$property,\@commands,$properties{$property},$required{$property},$docs{$property},'',$tab++);
		
		} elsif ($props{$property}  eq 'type') {
			unless ($obj =~ /escalation/) {
				my @types = ('check','notify','other'); 
				$form .= Forms->list_box($p_title,$property,\@types,$properties{$property},$required{$property},$docs{$property},'',$tab++);
			
			}
		} elsif ($props{$property}  =~ /notification_commands/) {
			my %w = ('type' => 'notify');
			my @commands = StorProc->fetch_list_where('commands','name',\%w); 
			my @selected = split(/,/, $properties{$property}); 
			unless ($task eq 'new' || $properties{'save'}) {
				if ($property =~ /(\S+)_notification_commands/) {
					my %w = ('type' => $1,'contacttemplate_id' => $properties{'contacttemplate_id'});
					@selected = StorProc->fetch_list_where('contact_command','command_id',\%w);
					my @sel;
					foreach (@selected) {
						my %p = StorProc->fetch_one('commands','command_id',$_);
						push @sel, $p{'name'};
					}
					@selected = @sel;
				}
			}
			$form .= Forms->list_box_multiple($p_title,$property,\@commands,\@selected,$required{$property},$docs{$property},'',$tab++);
		
		} elsif ($props{$property} =~ /notification_period|check_period|escalation_period/) {
			unless ($task eq 'new' || $properties{'save'}) {
				my %p = StorProc->fetch_one('time_periods','timeperiod_id',$properties{$property});
				$properties{$property} = $p{'name'};
			}
			my @periods = StorProc->fetch_list('time_periods','name'); 
			$form .= Forms->list_box($p_title,$property,\@periods,$properties{$property},$required{$property},$docs{$property},'',$tab++);
		
		} elsif ($props{$property} =~ /notification_failure_criteria|execution_failure_criteria/) {
			my @opts = split(/,/, $properties{$property});
			$form .= Forms->failure_criteria($property,\@opts,'',$required{$property},$docs{$property},'',$tab++);
		
		} elsif ($props{$property}  eq 'template' && $obj eq 'service_templates') {
			my @templates = StorProc->get_possible_parents($properties{'servicetemplate_id'}); 
			my %t = StorProc->fetch_one('service_templates','servicetemplate_id',$properties{'parent_id'});
			$properties{$property} = $t{'name'};
			$form .= Forms->list_box_submit('Use:',$property,\@templates,'',$required{$property},$docs{$property},'',$tab++);
		
		} elsif ($props{$property}  eq 'template') {
			if ($obj eq 'contacts') { $table = 'contact_templates' }
			unless ($obj eq 'service_templates' || $task eq 'new' || $properties{'save'}) {
				my %t = StorProc->fetch_one($obj_template{$obj},$obj_id{$obj_template{$obj}},$properties{$obj_id{$obj_template{$obj}}});
				$properties{$property} = $t{'name'};
			}
			my @templates = StorProc->fetch_list($table,'name'); 
			$form .= Forms->list_box('Use:',$property,\@templates,$properties{'template'},$required{$property},$docs{$property},'',$tab++);
		
		} elsif ($props{$property}  =~ /^command_line/) {
			$form .= Forms->text_area($p_title,$property,$properties{$property},'',$textsize{$property},$required{$property},$docs{$property},'',$tab++);
		
		} elsif ($props{$property}) {
			unless ($property eq 'name') {
				if ($properties{'last_notification'} eq '-zero-') { $properties{'last_notification'} = 0 }
				$form .= Forms->text_box($p_title,$property,$properties{$property},$textsize{$property},$required{$property},$docs{$property},'',$tab++);
			
			}
		}
	}
	if ($obj eq 'service_templates') {
		return $form;
	} else {
		$form .= Forms->hidden(\%hidden);
		my $auth = $obj;
		if ($auth =~ /escalation_templates/) { $auth = 'escalations' }
		if ($task eq 'modify' && $auth_delete{$auth}) {
			$form .= Forms->form_bottem_four_button(\%save,\%delete,\%rename,\%cancel,$tab++);
		} elsif ($task eq 'modify') {
			$form .= Forms->form_bottem_three_button(\%save,\%rename,\%cancel,$tab++);
		} else {
			$form .= Forms->form_bottem_two_button(\%add,\%cancel,$tab++);			
		}
		my $errstr = undef;
		if (@errors) { $errstr = Forms->form_errors(\@errors)."\n" }
		return $form_top."\n".$errstr.$form;
	}
}

sub build_host_template($) {
	my $name = shift;
	my $form = undef;
	my $select_all = undef;
	$hidden{'host_id'} = $query->param('host_id');
	my $host_template = $query->param('template');
	my $check_command = undef;
	my $command_line = undef;
	my $event_handler = undef;
	my $notification_period = undef;
	my %host = ();
	my %overrides_saved = ();
	my @override_contactgroup = ();
	my $hosttemplate_id = undef;
	if ($obj eq 'hosts') {
		%host = StorProc->fetch_one('hosts','name',$name);
		%overrides_saved = StorProc->fetch_one('host_overrides','host_id',$host{'host_id'});
		my %where = ('type' => 'hosts','object' => $host{'host_id'});
		@override_contactgroup = StorProc->fetch_list_where('contactgroup_assign','contactgroup_id',\%where);
		$hosttemplate_id = $host{'hosttemplate_id'};
	} else {
		%host = StorProc->fetch_one('profiles_host','name',$name);
		%overrides_saved = StorProc->fetch_one('hostprofile_overrides','hostprofile_id',$host{'hostprofile_id'});
		my %where = ('type' => 'host_profiles','object' => $host{'hostprofile_id'});
		@override_contactgroup = StorProc->fetch_list_where('contactgroup_assign','contactgroup_id',\%where);
		$hosttemplate_id = $host{'host_template_id'};
	}
	my %override = ();
	my %template = ();
	if ($host_template) {
		%template = StorProc->fetch_one('host_templates','name',$host_template);
		$hosttemplate_id = $template{'hosttemplate_id'};
	} else {
		%template = StorProc->fetch_one('host_templates','hosttemplate_id',$hosttemplate_id);
		$host_template = $template{'name'};
	}
	if ($overrides_saved{'notification_period'}) {
		my %np =  StorProc->fetch_one('time_periods','timeperiod_id',$overrides_saved{'notification_period'});
		$notification_period = $np{'name'};
	} else {
		my %np =  StorProc->fetch_one('time_periods','timeperiod_id',$template{'notification_period'});
		$notification_period = $np{'name'};
	}
	if ($overrides_saved{'check_command'}) {
		my %check =  StorProc->fetch_one('commands','command_id',$overrides_saved{'check_command'});
		$check_command = $check{'name'};
	} else {
		my %check =  StorProc->fetch_one('commands','command_id',$template{'check_command'});
		$check_command = $check{'name'};
	}
	if ($overrides_saved{'event_handler'}) {
		my %event =  StorProc->fetch_one('commands','command_id',$overrides_saved{'event_handler'});
		$event_handler = $event{'name'};
	} else {
		my %event =  StorProc->fetch_one('commands','command_id',$template{'event_handler'});
		$event_handler = $event{'name'};
	}

	my @contactgroups = ();
	my @override_conactgroups = ();

	unless ($nagios_ver eq '1.x') {
		my %cp =  StorProc->fetch_one('time_periods','timeperiod_id',$template{'check_period'});
		$template{'check_period'} = $cp{'name'};
		my %where = ('type' => 'host_templates','object' => $template{'hosttemplate_id'});
		my @temp_contactgroups = StorProc->fetch_list_where('contactgroup_assign','contactgroup_id',\%where);
		foreach (@temp_contactgroups) {
			my %c = StorProc->fetch_one('contactgroups','contactgroup_id',$_);
			push @contactgroups, $c{'name'};
		}
		$override{'contactgroups'} = 'checked';
		if ($obj eq 'hosts') {
			%where = ('type' => 'hosts','object' => $host{'host_id'});
		} else {
			%where = ('type' => 'host_profiles','object' => $host{'hostprofile_id'});
		}
		@override_conactgroups = StorProc->fetch_list_where('contactgroup_assign','contactgroup_id',\%where);
	}

	my @props = split(/,/, $property_list{'host_templates'});
	foreach (@props) { 
		$override{$_} = 'checked';
	} 

	if ($query->param('select_all')) {
		$select_all = 1;
	} else {
		foreach (@props) { 
			if ($overrides_saved{$_}) {
				$override{$_} = 'unchecked';
				if ($_ eq 'notification_period') {
					my %np =  StorProc->fetch_one('time_periods','timeperiod_id',$overrides_saved{'notification_period'});
					$template{$_} = $np{'name'};
				} elsif ($_ eq 'check_period') {
					my %check =  StorProc->fetch_one('time_periods','timeperiod_id',$overrides_saved{'check_period'});
					$template{$_} = $check{'name'};
				} elsif ($_ eq 'check_command') {
					my %check =  StorProc->fetch_one('commands','command_id',$overrides_saved{'check_command'});
					$template{$_} = $check{'name'};
				} elsif ($_ eq 'event_handler') {
					my %event =  StorProc->fetch_one('commands','command_id',$overrides_saved{'event_handler'});
					$template{$_} = $event{'name'};
				} else {
					$template{$_} = $overrides_saved{$_};
				}
			}
		} 
		unless ($nagios_ver eq '1.x') {
			$override{'contactgroups'} = 'checked';
			if (@override_conactgroups) {
				@contactgroups = ();
				foreach (@override_conactgroups) {
					my %c = StorProc->fetch_one('contactgroups','contactgroup_id',$_);
					push @contactgroups, $c{'name'};
				}
				$override{'contactgroups'} = 'unchecked';
			}
		}
	}
	my @timeperiods = StorProc->fetch_list('time_periods','name'); 
	my @members = StorProc->fetch_list('host_templates','name'); 
	my %docs = Doc->manage_hosts_vitals();

	$form .= Forms->list_box_submit('Host template:','template',\@members,$host_template,$required{'template'},$docs{'host_template'},$tab++);
	%docs = Doc->properties_doc('host_templates',\@props);

#	$form .= Forms->checkbox_override('Inherit all values from template','select_all',$select_all,$docs{'override'},$tab++);
	$form .= Forms->inheritance('Inherit from template',$docs{'override'},\%template);

# process_perf_data
	$form .= Forms->checkbox('Process performance data:','process_perf_data',$template{'process_perf_data'},$docs{'process_perf_data'},$override{'process_perf_data'},$tab++);

# retain_status_information
	$form .= Forms->checkbox('Retain status information:','retain_status_information',$template{'retain_status_information'},$docs{'retain_status_information'},$override{'retain_status_information'},$tab++);
# flap_detection_enabled
	$form .= Forms->checkbox('Flap detection enabled:','flap_detection_enabled',$template{'flap_detection_enabled'},$docs{'flap_detection_enabled'},$override{'flap_detection_enabled'},$tab++);
# low_flap_threshold
	$template{'low_flap_threshold'} =~ s/-zero-/0/;
	$form .= Forms->text_box('Low flap threshold:','low_flap_threshold',$template{'low_flap_threshold'},$textsize{'low_flap_threshold'},$required{'low_flap_threshold'},$docs{'low_flap_threshold'},$override{'low_flap_threshold'},$tab++);

# high_flap_threshold
	$template{'high_flap_threshold'} =~ s/-zero-/0/;
	$form .= Forms->text_box('High flap threshold:','high_flap_threshold',$template{'high_flap_threshold'},$textsize{'high_flap_threshold'},$required{'high_flap_threshold'},$docs{'high_flap_threshold'},$override{'high_flap_threshold'},$tab++);

# retain_nonstatus_information
	$form .= Forms->checkbox('Retain nonstatus information:','retain_nonstatus_information',$template{'retain_nonstatus_information'},$docs{'retain_nonstatus_information'},$override{'retain_nonstatus_information'},$tab++);
	if ($nagios_ver eq '1.x') {
# checks_enabled
		$form .= Forms->checkbox('Checks enabled:','checks_enabled',$template{'checks_enabled'},$docs{'checks_enabled'},$override{'checks_enabled'},$tab++);
	} else {
# active_checks_enabled
		$form .= Forms->checkbox('Active checks enabled:','active_checks_enabled',$template{'active_checks_enabled'},$docs{'active_checks_enabled'},$override{'active_checks_enabled'},$tab++);
		$form .= Forms->checkbox('Passive checks enabled:','passive_checks_enabled',$template{'passive_checks_enabled'},$docs{'passive_checks_enabled'},$override{'passive_checks_enabled'},$tab++);
# obsess_over_host
		$form .= Forms->checkbox('Obsess over host:','obsess_over_host',$template{'obsess_over_host'},$docs{'obsess_over_host'},$override{'obsess_over_host'},$tab++);
# check_freshness
		$form .= Forms->checkbox('Check freshness:','check_freshness',$template{'check_freshness'},$docs{'check_freshness'},$override{'check_freshness'},$tab++);
# freshness_threshold
		$template{'freshness_threshold'} =~ s/-zero-/0/;
		$form .= Forms->text_box('Freshness threshold:','freshness_threshold',$template{'freshness_threshold'},$textsize{'freshness_threshold'},$required{'freshness_threshold'},$docs{'freshness_threshold'},$override{'high_flap_threshold'},$tab++);
	}
# check_command
# command_line
	my @commands = StorProc->fetch_list('commands','name'); 
	$form .= Forms->list_box('Check command:','check_command',\@commands,$check_command,'',$docs{'check_command'},$override{'check_command'},$tab++);
	#$form .= Forms->text_area('Command line:','command_line',$template{'command_line'},'',$textsize{'command_line'},'',$docs{'command_line'},$override{'command_line'});
#	my %command = StorProc->fetch_one('commands','name',$check_command);
#
#   my @commands = StorProc->fetch_list_where('commands','name',\%where);
#	unless ($command_line) {
#		my @args = split(/ARG/i,$command{'command_line'});
#		if (@args) {
#			$command_line = $check_command;
#			my $cnt = 1;
#			pop @args;
#			foreach (@args) { $command_line .= "!ARG$cnt"; $cnt++ }
#		}
#	}
#	$form .= Forms->list_box_command('Check command:','check_command',\@commands,$check_command,$required{'template'},$docs{'check_command'},$override{'check_command'},$command{'command_line'},$command_line);


	
# max_check_attempts
	$form .= Forms->text_box('Max check attempts:','max_check_attempts',$template{'max_check_attempts'},$textsize{'max_check_attempts'},$required{'max_check_attempts'},$docs{'max_check_attempts'},$override{'max_check_attempts'},$tab++);

	if ($nagios_ver eq '2.x') {
# check_interval
		$template{'check_interval'} =~ s/-zero-/0/;
		$form .= Forms->text_box('Check interval:','check_interval',$template{'check_interval'},$textsize{'check_interval'},$required{'check_interval'},$docs{'check_interval'},$override{'check_interval'},$tab++);
	}

# event_handler_enabled
	$form .= Forms->checkbox('Event handler enabled:','event_handler_enabled',$template{'event_handler_enabled'},$docs{'event_handler_enabled'},$override{'event_handler_enabled'},$tab++);

# event_handler
	$form .= Forms->list_box('Event handler:','event_handler',\@commands,$event_handler,'',$docs{'event_handler'},$override{'event_handler'},$tab++);

# notifications_enabled
	$form .= Forms->checkbox('Notifications enabled:','notifications_enabled',$template{'notifications_enabled'},$docs{'notifications_enabled'},$override{'notifications_enabled'},$tab++);
# notification_interval
	$template{'notification_interval'} =~ s/-zero-/0/;
	$form .= Forms->text_box('Notification interval:','notification_interval',$template{'notification_interval'},$textsize{'notification_interval'},$required{'notification_interval'},$docs{'notification_interval'},$override{'notification_interval'},$tab++);

# notification_period
	$form .= Forms->list_box('Notification period:','notification_period',\@timeperiods,$notification_period,'',$docs{'notification_period'},$override{'notification_period'},$tab++);

# notification_options
	my @opts = split(/,/, $template{'notification_options'});
	$form .= Forms->notification_options('host_templates','notification_options',\@opts,$required{'notification_options'},$nagios_ver,$docs{'notification_options'},$override{'notification_options'},$tab++);

# stalking_options
	my @opts = split(/,/, $template{'stalking_options'});
	$form .= Forms->stalking_options(\@opts,$required{'stalking_options'},$docs{'stalking_options'},$override{'stalking_options'},$tab++);

	unless ($nagios_ver eq '1.x') {
# contactgroups
		my @nonmembers = StorProc->fetch_list('contactgroups','name'); 
		$form .= Forms->members('Contact groups:','contactgroup',\@contactgroups,\@nonmembers,'','10',$docs{'contactgroup'},$override{'contactgroups'},$tab++);
	}
	return $form;
}

sub build_service_detail($) {
	my $service_id = shift;
	my $form = undef;
	my $select_all = undef;
	my $service_template = $query->param('template');
	my %service = ();
	my %overrides_saved = ();
	my @override_contactgroup = ();
	my $servicetemplate_id = undef;
	if ($obj eq 'hosts') {
		unless ($service_id) { $service_id = $query->param('service_id') }
		%service = StorProc->fetch_one('services','service_id',$service_id);
		%overrides_saved = StorProc->fetch_one('service_overrides','service_id',$service_id);
		$servicetemplate_id = $service{'servicetemplate_id'};
		my %where = ('type' => 'services','object' => $service{'service_id'});
		@override_contactgroup = StorProc->fetch_list_where('contactgroup_assign','contactgroup_id',\%where);
	} elsif ($obj eq 'services') {
		unless ($service_id) { $service_id = $query->param('servicename_id') }
		%service = StorProc->fetch_one('service_names','servicename_id',$service_id);
		%overrides_saved = StorProc->fetch_one('servicename_overrides','servicename_id',$service_id);
		$servicetemplate_id = $service{'template'};
		my %where = ('type' => 'service_names','object' => $service{'servicename_id'});
		@override_contactgroup = StorProc->fetch_list_where('contactgroup_assign','contactgroup_id',\%where);
	} elsif ($obj eq 'service_templates') {
		foreach my $prop (keys %properties) {
			if ($prop =~ /parent_id/) {
				$servicetemplate_id = $properties{$prop};
			} else {
				$overrides_saved{$prop} = $properties{$prop};
			}
		}
		my %where = ('type' => 'service_templates','object' => $properties{'servicetemplate_id'});
		@override_contactgroup = StorProc->fetch_list_where('contactgroup_assign','contactgroup_id',\%where);
	}
	my %override = ();
	my %template = ();
	my @contactgroups = ();
	if ($service_template) {
		my %t = StorProc->fetch_one('service_templates','name',$service_template);
		$servicetemplate_id = $t{'servicetemplate_id'}
	} else {
		my %t = StorProc->fetch_one('service_templates','servicetemplate_id',$servicetemplate_id);
		$service_template = $t{'name'};
	}
	my @template_contactgroup = ();
	my $got_parent = 0;
	my $got_contactgroup = 0;
	my $i = 0;
	until ($got_parent || $i == 10) {
		$i++;
		my %tpl = StorProc->fetch_one('service_templates','servicetemplate_id',$servicetemplate_id);
		my @props = split(/,/, $property_list{'service_templates'});
		foreach my $t (@props) { 
			unless ($template{$t}) { 
				if ($t eq 'check_command') {
					my %c =  StorProc->fetch_one('commands','command_id',$tpl{'check_command'});
					$template{$t} = $c{'name'};
				} elsif ($t eq 'notification_period') {
					my %np =  StorProc->fetch_one('time_periods','timeperiod_id',$tpl{'notification_period'});
					$template{$t} = $np{'name'};
				} elsif ($t eq 'check_period') {
					my %cp =  StorProc->fetch_one('time_periods','timeperiod_id',$tpl{'check_period'});
					$template{$t} = $cp{'name'};
				} elsif ($t eq 'event_handler') {
					my %e =  StorProc->fetch_one('commands','command_id',$tpl{'event_handler'});
					$template{$t} = $e{'name'};
				} else { 
					$template{$t} = $tpl{$t};
				}
			}
		}
		unless ($got_contactgroup) {
			my %where = ('type' => 'service_templates','object' => $tpl{'servicetemplate_id'});
			@template_contactgroup = StorProc->fetch_list_where('contactgroup_assign','contactgroup_id',\%where);
			if (@template_contactgroup) { $got_contactgroup = 1 }
		}
		if ($tpl{'parent_id'}) { 
			$servicetemplate_id = $tpl{'parent_id'};
		} else {
			$got_parent = 1;
		}
	}
	
	my @props = split(/,/, $property_list{'service_templates'});
	foreach (@props) { 
		$override{$_} = 'checked';
		$properties{$_} = $template{$_};
	} 
	foreach (@template_contactgroup) {
		$override{'contactgroups'} = 'checked';
		my %g = StorProc->fetch_one('contactgroups','contactgroup_id',$_);
		push @contactgroups, $g{'name'};
	}
	if ($query->param('select_all')) {
		$select_all = 1;
	} else {
		foreach (@props) { 
			if ($overrides_saved{$_}) {
				$override{$_} = 'unchecked';
				if ($_ eq 'check_command') {
					my %c =  StorProc->fetch_one('commands','command_id',$overrides_saved{'check_command'});
					$properties{$_} = $c{'name'};
				} elsif ($_ eq 'notification_period') {
					my %np =  StorProc->fetch_one('time_periods','timeperiod_id',$overrides_saved{'notification_period'});
					$properties{$_} = $np{'name'};
				} elsif ($_ eq 'check_period') {
					my %cp =  StorProc->fetch_one('time_periods','timeperiod_id',$overrides_saved{'check_period'});
					$properties{$_} = $cp{'name'};
				} elsif ($_ eq 'event_handler') {
					my %e =  StorProc->fetch_one('commands','command_id',$overrides_saved{'event_handler'});
					$properties{$_} = $e{'name'};
				} else {
					$properties{$_} = $overrides_saved{$_};
				}
			}
		} 
		if (@override_contactgroup) {
			@contactgroups = ();
			$override{'contactgroups'} = 'unchecked';
			foreach (@override_contactgroup) {
				my %g = StorProc->fetch_one('contactgroups','contactgroup_id',$_);
				push @contactgroups, $g{'name'};
			}
		}
	}
	my @members = ();
	if ($obj eq 'service_templates') {
		push @members, '-no-template-';
		my @mems = StorProc->get_possible_parents($properties{'servicetemplate_id'});
		push (@members,(@mems));
	} else {
		@members = StorProc->fetch_list('service_templates','name'); 
	}
	my %docs = Doc->properties_doc('service_templates',\@props);
	$form .= Forms->list_box_submit('Service template:','template',\@members,$service_template,'',$docs{'template'});

#	$form .= Forms->checkbox_override('Inherit all values from template','select_all',$select_all,$docs{'override'});
	$form .= Forms->inheritance('Inherit from template',$docs{'override'},\%template);

# is_volatile
	$form .= Forms->checkbox('Is volatile:','is_volatile',$properties{'is_volatile'},$docs{'is_volatile'},$override{'is_volatile'});
# check_period,
	my @timeperiods = StorProc->fetch_list('time_periods','name'); 
	$form .= Forms->list_box('Check period:','check_period',\@timeperiods,$properties{'check_period'},'',$docs{'check_period'},$override{'check_period'});
# max_check_attempts,
	$form .= Forms->text_box('Max check attempts:','max_check_attempts',$properties{'max_check_attempts'},$textsize{'max_check_attempts'},$required{'max_check_attempts'},$docs{'max_check_attempts'},$override{'max_check_attempts'});
# normal_check_interval,
	$properties{'normal_check_interval'} =~ s/-zero-/0/;
	$form .= Forms->text_box('Normal check interval:','normal_check_interval',$properties{'normal_check_interval'},$textsize{'normal_check_interval'},$required{'normal_check_interval'},$docs{'normal_check_interval'},$override{'normal_check_interval'});
# retry_check_interval,".
	$properties{'retry_check_interval'} =~ s/-zero-/0/;
	$form .= Forms->text_box('Retry check interval:','retry_check_interval',$properties{'retry_check_interval'},$textsize{'retry_check_interval'},$required{'retry_check_interval'},$docs{'retry_check_interval'},$override{'retry_check_interval'});
# "active_checks_enabled,
	$form .= Forms->checkbox('Active checks enabled:','active_checks_enabled',$properties{'active_checks_enabled'},$docs{'active_checks_enabled'},$override{'active_checks_enabled'});

# passive_checks_enabled,
	$form .= Forms->checkbox('Passive checks enabled:','passive_checks_enabled',$properties{'passive_checks_enabled'},$docs{'passive_checks_enabled'},$override{'passive_checks_enabled'});

# parallelize_check,
	$form .= Forms->checkbox('Parallelize check:','parallelize_check',$properties{'parallelize_check'},$docs{'parallelize_check'},$override{'parallelize_check'});

# obsess_over_service,
	$form .= Forms->checkbox('Obsess over service:','obsess_over_service',$properties{'obsess_over_service'},$docs{'obsess_over_service'},$override{'obsess_over_service'});

# check_freshness,
	$form .= Forms->checkbox('Check freshness:','check_freshness',$properties{'check_freshness'},$docs{'check_freshness'},$override{'check_freshness'});

# freshness_threshold,
	$properties{'freshness_threshold'} =~ s/-zero-/0/;
	$form .= Forms->text_box('Freshness threshold:','freshness_threshold',$properties{'freshness_threshold'},$textsize{'freshness_threshold'},$required{'freshness_threshold'},$docs{'freshness_threshold'},$override{'freshness_threshold'});

# notifications_enabled,
	$form .= Forms->checkbox('Notifications enabled:','notifications_enabled',$properties{'notifications_enabled'},$docs{'notifications_enabled'},$override{'notifications_enabled'});

# notification_interval,".
	$properties{'notification_interval'} =~ s/-zero-/0/;
	$form .= Forms->text_box('Notification interval:','notification_interval',$properties{'notification_interval'},$textsize{'notification_interval'},$required{'notification_interval'},$docs{'notification_interval'},$override{'notification_interval'});
# "notification_period,
	$form .= Forms->list_box('Notification period:','notification_period',\@timeperiods,$properties{'notification_period'},'',$docs{'notification_period'},$override{'notification_period'});

# notification_options,
	my @opts = split(/,/, $properties{'notification_options'});
	$form .= Forms->notification_options('service_templates','notification_options',\@opts,$required{'notification_options'},$nagios_ver,$docs{'notification_options'},$override{'notification_options'});

# event_handler_enabled,
	$form .= Forms->checkbox('Event handler enabled:','event_handler_enabled',$properties{'event_handler_enabled'},$docs{'event_handler_enabled'},$override{'event_handler_enabled'});

# event_handler,
	my %w = ('type' => 'check');
	my @commands = StorProc->fetch_list_where('commands','name',\%w); 
	$form .= Forms->list_box('Event handler:','event_handler',\@commands,$properties{'event_handler'},$required{'event_handler'},$docs{'event_handler'},$override{'event_handler'});

# flap_detection_enabled,
	$form .= Forms->checkbox('Flap detection enabled:','flap_detection_enabled',$properties{'flap_detection_enabled'},$docs{'flap_detection_enabled'},$override{'flap_detection_enabled'});

# low_flap_threshold,
	$properties{'low_flap_threshold'} =~ s/-zero-/0/;
	$form .= Forms->text_box('Low flap threshold:','low_flap_threshold',$properties{'low_flap_threshold'},$textsize{'low_flap_threshold'},$required{'low_flap_threshold'},$docs{'low_flap_threshold'},$override{'low_flap_threshold'});

# high_flap_threshold,
	$properties{'high_flap_threshold'} =~ s/-zero-/0/;
	$form .= Forms->text_box('High flap threshold:','high_flap_threshold',$properties{'high_flap_threshold'},$textsize{'high_flap_threshold'},$required{'high_flap_threshold'},$docs{'high_flap_threshold'},$override{'high_flap_threshold'});

# process_perf_data,".
	$form .= Forms->checkbox('Process perf data:','process_perf_data',$properties{'process_perf_data'},$docs{'process_perf_data'},$override{'process_perf_data'});

# "retain_status_information,
	$form .= Forms->checkbox('Retain status information:','retain_status_information',$properties{'retain_status_information'},$docs{'retain_status_information'},$override{'retain_status_information'});

# retain_nonstatus_information
	$form .= Forms->checkbox('Retain nonstatus information:','retain_nonstatus_information',$properties{'retain_nonstatus_information'},$docs{'retain_nonstatus_information'},$override{'retain_nonstatus_information'});

# contactgroups
	my @nonmembers = StorProc->fetch_list('contactgroups','name'); 
	$form .= Forms->members('Contact Groups:','contactgroup',\@contactgroups,\@nonmembers,'','10',$docs{'contactgroup'},$override{'contactgroups'});

	return $form;
}

#
###########################################################################################################################
# Service Template
#

sub service_template() {
	my $form = undef;
	my $obj_view = $query->param('obj_view');
	my $name = $query->param('name');
	my $host = $query->param('host');
	if ($query->param('task') eq 'new') { $obj_view = 'new' }
	my $test_results = undef;
	my $message_applied = undef;
	%properties = StorProc->fetch_one('service_templates','name',$name);
	if ($query->param('add')) {
		if ($name) {
			unless ($properties{'name'}) {
				my @values = ('',$name,'','','','','','','','');
				my $id = StorProc->insert_obj_id('service_templates',\@values,'servicetemplate_id');
				if ($id =~ /^Error/) { 
					push @errors, $id;
					$obj_view = 'new';
				} else {
					$obj_view = 'service_detail';
					$refresh_left = "top_menu=services&selected=$name";
					delete $hidden{'task'};
				}	
			} else {
				push @errors, "Service template $name already exists";
				$obj_view = 'new';
				$required{'name'} = 1;
			}
		} else {					
			push @errors, "Service template name required";
			$obj_view = 'new';
		}
	} elsif ($query->param('close') || $query->param('cancel') || $query->param('continue')) {
		$obj_view = 'close';
	} elsif ($query->param('test_command')) {	
		my $arg_string = $query->param('arg_string');
		my $command = $query->param('command');
		my %cmd = StorProc->fetch_one('commands','name',$command);
		my %host = StorProc->fetch_one('hosts','name',$host);
		my %where = ('type' => 'resource'); 
		my %resources = StorProc->get_resources();
		foreach my $res (keys %resources) {
			if ($cmd{'command_line'} =~ /$res/i) { $cmd{'command_line'} =~ s/\$$res\$/$resources{$res}/ig } 
		}
		if ($cmd{'command_line'} =~ /\$HOSTADDRESS\$/) { 
			if ($host =~ /\d+\.\d+\.\d+\.\d+/) { $cmd{'command_line'} =~ s/\$HOSTADDRESS\$/$host/g }
			$cmd{'command_line'} =~ s/\$HOSTADDRESS\$/$host{'address'}/g; 
		}
		if ($cmd{'command_line'} =~ /\$HOSTNAME\$/) { $cmd{'command_line'} =~ s/\$HOSTNAME\$/$host/g } 
		my @args = split(/!/, $arg_string);
		my $cnt = 1;
		foreach my $a (@args) {
			$cmd{'command_line'} =~ s/\$ARG$cnt\$/$a/g;
			$cnt++;
		}
		$test_results = "$cmd{'command_line'}<br/>";
		$test_results .= StorProc->test_command($cmd{'command_line'});	
	} elsif ($query->param('save')) {
		if ($obj_view eq 'service_detail') {
			my %values = ();
			my $parent = $query->param('template'); 
			my %t = StorProc->fetch_one('service_templates','name',$parent);
			$values{'parent_id'} = $t{'servicetemplate_id'};
			my %data = parse_query('service_templates','service_templates');
			$values{'check_period'} = $data{'check_period'};
			$values{'event_handler'} = $data{'event_handler'};
			$values{'notification_period'} = $data{'notification_period'};
			$values{'data'} = $data{'data'};
			my $result = StorProc->update_obj('service_templates','name',$name,\%values);
			if ($result =~ /^Error/) { push @errors, $result }
			my %where = ('type' => 'service_templates','object' => $properties{'servicetemplate_id'});
			my $result = StorProc->delete_one_where('contactgroup_assign',\%where);
			if ($result =~ /^Error/) { push @errors, $result }
			unless ($query->param('contactgroup_override')) {
				my @mems = $query->param('contactgroup');
				foreach (@mems) {
					my %cg = StorProc->fetch_one('contactgroups','name',$_);
					my @vals = ($cg{'contactgroup_id'},'service_templates',$properties{'servicetemplate_id'});
					$result = StorProc->insert_obj('contactgroup_assign',\@vals);
					if ($result =~ /^Error/) { push @errors, $result }
				}
			}
			unless (@errors) { 
				$obj_view = 'saved';
			}
		} elsif ($obj_view eq 'service_check') {
			my $service_name = $query->param('service_name');
			my $service_id = $query->param('service_id');
			$hidden{'service_name'} = $service_name;
			$hidden{'service_id'} = $service_id;
			my $check_command = $query->param('command');
			my $command_line = $query->param('command_line');
			if ($query->param('inherit')) {
				my %vals = ('check_command' => '','command_line' => '');
				my $result = StorProc->update_obj('service_templates','servicetemplate_id',$properties{'servicetemplate_id'},\%vals);
				if ($result =~ /^Error/) { push @errors, $result }
			} else {
				my %check = StorProc->fetch_one('commands','name',$check_command);
				my %data = parse_query('service_templates','service_templates');
				my %vals = ('check_command' => $check{'command_id'},'command_line' => $command_line);
				my $result = StorProc->update_obj('service_templates','servicetemplate_id',$properties{'servicetemplate_id'},\%vals);
				if ($result =~ /^Error/) { push @errors, $result }
			}
			unless (@errors) { 
				$obj_view = 'saved';
				$hidden{'selected'} = 'service_check';
			}
		}
 	} elsif ($query->param('rename')) {
		if ($query->param('new_name')) {
			my $new_name = $query->param('new_name');
			my %n = StorProc->fetch_one('profiles_host','name',$new_name);
			if ($n{'name'}) {
				push @errors, "A host profile $new_name already exists.";
			} else {
				my %values = ('name' => $new_name);
				my $result = StorProc->update_obj('service_templates','name',$name,\%values);				
				if ($result =~ /error/i) {
					push @errors, $result;
				} else {
					$name = $new_name;
					$obj_view = 'service_detail';
					$refresh_left = "top_menu=$top_menu&selected=$name";
				}
			}
		} else {
			$obj_view = 'rename';
		}
	} elsif ($query->param('delete') || $query->param('confirm_delete')) {
		if ($query->param('confirm_delete')) {
			my $result = StorProc->delete_all('service_templates','name',$name);
			if ($result =~ /^Error/) { push @errors, $result }
			my %where = ('type' => 'service_templates','object' => $properties{'servicetemplate_id'});
			$result = StorProc->delete_one_where('contactgroup_assign',\%where);
			if ($result =~ /^Error/) { push @errors, $result }
			unless (@errors) { 
				$refresh_left = "top_menu=$top_menu";
				$obj_view = 'deleted';
			}
		} elsif ($query->param('task') eq 'No') {
			$obj_view = 'service_detail';
		} else {
			foreach my $name ($query->param) {
				unless ($name eq 'nocache') { $hidden{$name} = $query->param($name) }
			}
			delete $hidden{'task'};
			$obj_view = 'delete';
		}
	}
	my %save = ('name' => 'save','value' => 'Save');
	my %objs = ('servicename_id' => $hidden{'servicename_id'});
	my %docs = Doc->services();
	$hidden{'name'} = $name;
	$hidden{'servicename_id'} = $query->param('servicename_id');
	unless ($hidden{'servicename_id'}) { $hidden{'servicename_id'} = $properties{'servicename_id'}	}
	$objs{'servicename_id'} = $hidden{'servicename_id'};
	unless ($obj_view) { $obj_view = 'service_detail' }
	$hidden{'obj_view'} = $obj_view;
	$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
	if ($obj_view eq 'service_detail') {
		$form .= Forms->service_template_top($name,$user_acct,$obj_view,\%objs,$auth_add{'externals'},$hidden{'selected'});
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		if ($query->param('template') eq '-no-template-') {
			delete $properties{'parent_id'};
			$form .= build_form();
		} elsif ($properties{'parent_id'} || $query->param('template')) {
			$form .= build_service_detail($properties{'servicetemplate_id'});
		} else {
			$form .= build_form();
		}
		$form .= Forms->hidden(\%hidden);
		if ($auth_delete{'service_templates'}) {
			$form .= Forms->form_bottem_four_button(\%save,\%delete,\%rename,\%close,$tab++);
		} else {
			$form .= Forms->form_bottem_three_button(\%save,\%rename,\%close,$tab++);
		}
	} elsif ($obj_view eq 'service_check') {
		my %template = StorProc->fetch_one('service_templates','servicetemplate_id',$properties{'parent_id'});
		my $message = undef;
		my $inherit = 0;
		$inherit = $query->param('inherit');
		my %cmd = StorProc->fetch_one('commands','command_id', $properties{'check_command'});
		my $command = $cmd{'name'};
		my $command_save = $cmd{'name'};
		my $command_line = $properties{'command_line'};
		if ($query->param('command')) { $command = $query->param('command') }
		if ($query->param('command_save')) { $command_save = $query->param('command_save') }
		if ($query->param('command_line')) { $command_line = $query->param('command_line') }
		unless ($command eq $command_save) {
			%cmd = StorProc->fetch_one('commands','name',$command);
			$command_line = undef;
		}
		if ($inherit or !$command) {
			%cmd = StorProc->fetch_one('commands','command_id',$template{'check_command'});
			if ($cmd{'name'}) {
				$command = $cmd{'name'};
				$command_line = $template{'command_line'};
				$inherit = 1;
			} else {
				my $got_command = 0;
				my $stid = $template{'parent_id'};
				until ($got_command) {
					my %t = StorProc->fetch_one('service_templates','servicetemplate_id',$stid);
					if ($t{'check_command'}) {
						$got_command = 1;
						%cmd = StorProc->fetch_one('commands','command_id',$t{'check_command'});
						$command = $cmd{'name'};
						$command_line = $t{'command_line'};
						$got_command = 1;
					} else {
						if ($t{'parent_id'}) {
							$stid = $t{'parent_id'};
						} else {
							$got_command = 1;
							$message = ('Note: a parent template does not have a check command defined.');
							$command = undef;
							$command_line = undef;
						}
					}
				}
			}
		}
		unless ($cmd{'command_line'}) {
			%cmd = StorProc->fetch_one('commands','name',$command);
		}
		my $arg_string = $command_line;
		$arg_string =~ s/$command!//;
		my $usage = $command;
		my @args = split(/ARG/i,$cmd{'command_line'});
		my $args = undef;
		if ($args[1]) {
			$usage .= "!";
			my $cnt = 1;
			pop @args;
			foreach (@args) { 
				if ($cmd{'command_line'} =~ /ARG$cnt/i) { $args .= "ARG$cnt!" }
				$cnt++;
			}
			chop $args;
			$usage .= $args;
			unless ($command_line =~ /$command/) {
				$command_line = "$command!$args";
				$arg_string = $args;
			}
		}
		$form .= Forms->service_template_top($name,$user_acct,$obj_view,\%objs,$auth_add{'externals'},$hidden{'selected'});
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->display_hidden('Parent template:','',$template{'name'});
		if ($properties{'parent_id'}) {
			$form .= Forms->checkbox_override('Inherit check from template','inherit',$inherit,$docs{'override'});
		}
		my %where = ('type' => 'check');
		my @commands = StorProc->fetch_list_where('commands','name',\%where);
		if ($message) { $form .= Forms->form_doc($message) }
		$form .= Forms->list_box_submit('Check command:','command',\@commands,$command,'');
		$form .= Forms->display_hidden('Command definition:','',$cmd{'command_line'});
		$form .= Forms->display_hidden('Usage:','',$usage);
		$form .= Forms->text_area('Command line:','command_line',$command_line,'3','80','',$docs{'command_line'});
		$form .= Forms->command_test($test_results,$host,$arg_string);
		$hidden{'command_save'} = $command;
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%save,$tab++);
	} elsif ($obj_view eq 'delete') {
		my $message = qq(Are you sure you want to remove service $name from all hosts and profiles?);
		$form .= Forms->are_you_sure('Confirm Delete:',$message,'confirm_delete',\%hidden);
	} elsif ($obj_view eq 'deleted') {
		$form .= Forms->form_top('Deleted','');
		my @message = ("$name");
		$form .= Forms->form_message('Removed:',\@message,'row1');			
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%continue);
	} elsif ($obj_view eq 'saved') {
		my %objs = ('service_id' => $properties{'servicename_id'},'name' => $name);
		$form .= Forms->service_template_top($name,$user_acct,$obj_view,\%objs,$auth_add{'externals'},$hidden{'selected'});
		$form .= Forms->display_hidden('Saved:','',$name);
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%close);
	} elsif ($obj_view eq 'rename') {
		$form .= Forms->form_top('Rename Service','','');
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->display_hidden('Name:','name',$name,$textsize{'name'},'');
		$form .= Forms->text_box('Rename:','new_name','',$textsize{'name'},'');
		$hidden{'obj_view'} = 'rename';
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_two_button(\%rename,\%cancel,$tab++);
	} elsif ($obj_view eq 'new') {
		my $template = $query->param('template');
		$form .= Forms->form_top('New Service Template','','');
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->text_box('Name:','name',$name,$textsize{'name'},'',$docs{'name'});
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_two_button(\%add,\%cancel,$tab++);
	}
	return $form;
}

#
###########################################################################################################################
# Build Contact
#

sub build_contact() {
	my $form = undef;
	my $select_all = undef;
	my $name = $query->param('name');
	if ($task eq 'copy') { 
		$name = $query->param('source');
		delete $hidden{'task'};	
	}
	$hidden{'contact_id'} = $query->param('contact_id');
	my %contact = StorProc->fetch_one('contacts','name',$name);
	my %template = StorProc->fetch_one('contact_templates','contacttemplate_id',$contact{'contacttemplate_id'});
	$contact{'template'} = $template{'name'};
	if ($query->param('alias')) { $contact{'alias'} = $query->param('alias') }
	if ($query->param('pager')) { $contact{'pager'} = $query->param('pager') }
	if ($query->param('email')) { $contact{'email'} = $query->param('email') }
	if ($query->param('template')) { 
		%template = StorProc->fetch_one('contact_templates','name',$query->param('template'));
		$contact{'template'} = $query->param('template');
	}
	my %overrides_saved = StorProc->fetch_one('contact_overrides','contact_id',$contact{'contact_id'});
	my %override = ();
	my @props = split(/,/, $property_list{'contact_templates'});
	foreach (@props) { 
		$override{$_} = 'checked';
	} 

	my %np =  StorProc->fetch_one('time_periods','timeperiod_id',$template{'host_notification_period'});
	$template{'host_notification_period'} = $np{'name'};
	%np =  StorProc->fetch_one('time_periods','timeperiod_id',$template{'service_notification_period'});
	$template{'service_notification_period'} = $np{'name'};

	my @host_commands = StorProc->get_command_contact_template($template{'contacttemplate_id'},'host');
	my @service_commands = StorProc->get_command_contact_template($template{'contacttemplate_id'},'service');
	my @override_host_commands = StorProc->get_command_contact($contact{'contact_id'},'host');
	my @override_service_commands = StorProc->get_command_contact($contact{'contact_id'},'service');

	if ($query->param('select_all')) {
		$select_all = 1;
	} else {
		foreach (@props) { 
			if ($overrides_saved{$_}) {
				$override{$_} = 'unchecked';
				if ($_ eq 'host_notification_period') {
					%np =  StorProc->fetch_one('time_periods','timeperiod_id',$overrides_saved{'host_notification_period'});
					$template{$_} = $np{'name'};
				} elsif ($_ eq 'service_notification_period') {
					%np =  StorProc->fetch_one('time_periods','timeperiod_id',$overrides_saved{'service_notification_period'});
					$template{$_} = $np{'name'};
				} else {
					$template{$_} = $overrides_saved{$_};
				}
			}
		} 
		if (@override_host_commands) {
			@host_commands = ();
			$override{'host_notification_commands'} = 'unchecked';
			foreach (@override_host_commands) {
				push @host_commands, $_;
			}
		}
		if (@override_service_commands) {
			@service_commands = ();
			$override{'service_notification_commands'} = 'unchecked';
			foreach (@override_service_commands) {
				push @service_commands, $_;
			}
		}	
	}
	my @contact_props = split(/,/, $property_list{'contacts'});
	my %docs = Doc->properties_doc('contacts',\@contact_props);
	if (@errors) { $form .= Forms->form_errors(\@errors) }
#  contact_name 
	if ($view eq 'design') {
		$form .= Forms->text_box('Name:','name',$name,$textsize{'name'},$required{'name'},$docs{'name'});
	} else {
		$form .= Forms->display_hidden('Name:','name',$name);
	}
	$form .= Forms->text_box('Alias:','alias',$contact{'alias'},$textsize{'alias'},$required{'alias'},$docs{'alias'});
	$form .= Forms->text_box('Email:','email',$contact{'email'},$textsize{'email'},'',$docs{'email'});
	$form .= Forms->text_box('Pager:','pager',$contact{'pager'},$textsize{'pager'},'',$docs{'pager'});

	my @members = StorProc->fetch_list('contact_templates','name'); 
	$form .= Forms->list_box_submit('Contact template:','template',\@members,$template{'name'},$required{'template'},$docs{'template'});
	%docs = Doc->properties_doc('contact_templates',\@props);
#	$form .= Forms->checkbox_override('Inherit all values from template','select_all',$select_all,$docs{'override'});
	$form .= Forms->inheritance('Inherit from template',$docs{'override'},\%template);
	my @timeperiods = StorProc->fetch_list('time_periods','name');
	my %where = ('type' => 'notify');
	my @commands = StorProc->fetch_list_where('commands','name',\%where);

	%docs = Doc->properties_doc('contact_templates',\@props);

# host_notification_period
	$form .= Forms->list_box('Host notification period:','host_notification_period',\@timeperiods,$template{'host_notification_period'},$required{'host_notification_period'},$docs{'host_notification_period'},$override{'host_notification_period'});
# host_notification_options
	my @opts = split(/,/, $template{'host_notification_options'});
	$form .= Forms->notification_options('contact_templates','host_notification_options',\@opts,$required{'host_notification_options'},$nagios_ver,$docs{'host_notification_options'},$override{'host_notification_options'});
# host_notification_commands 
	$form .= Forms->list_box_multiple('Host notification commands','host_notification_commands',\@commands,\@host_commands,'',$docs{'host_notification_commands'},$override{'host_notification_commands'});
# service_notification_period
	$form .= Forms->list_box('Service notification period:','service_notification_period',\@timeperiods,$template{'service_notification_period'},$required{'service_notification_period'},$docs{'service_notification_period'},$override{'service_notification_period'});
# service_notification_options
	my @opts = split(/,/, $template{'service_notification_options'});
	$form .= Forms->notification_options('contact_templates','service_notification_options',\@opts,$required{'service_notification_options'},$nagios_ver,$docs{'service_notification_options'},$override{'service_notification_options'});
# service_notification_commands
	$form .= Forms->list_box_multiple('Service notification commands','service_notification_commands',\@commands,\@service_commands,'',$docs{'service_notification_commands'},$override{'service_notification_commands'});
# contactgroups
	my @contactgroups = $query->param('contactgroup');
	unless (@contactgroups) { @contactgroups = StorProc->get_contactgroup_contact($contact{'contact_id'}) }
	my @nonmembers = StorProc->fetch_list('contactgroups','name'); 
	$form .= Forms->members('Contact groups:','contactgroup',\@contactgroups,\@nonmembers,'','10',$docs{'contactgroup'});

	return $form;
}

#
###########################################################################################################################
# Build Command
#

sub command_wizard() {
	my $form = undef;
	my @command_props = split(/,/, $property_list{'commands'});
	push @command_props, 'usage';
	my %docs = Doc->properties_doc('commands',\@command_props);
	my %save = ('name' => 'save', 'value' => 'Save');
	my %done = ('name' => 'save', 'value' => 'Done');
	my %test = ('name' => 'test', 'value' => 'Test');
	my $command = $query->param('command');
	my $command_line = $query->param('command_line');
	my $name = $query->param('name');
	my $type = $query->param('type');
	my $host = $query->param('host');
	my $arg_string = $query->param('arg_string');
	my $results = undef;
	my $resource_sav = undef;
	my $got_form = 0;

	if ($query->param('back')) {

	} elsif ($query->param('bail')) {
		$task = 'modify';

	} elsif ($task eq 'external') {
		$task = 'modify';
	} elsif ($query->param('continue') || $query->param('cancel')) {
		$form .= Forms->header($page_title,$user_acct,$view);
		$got_form = 1;
	} elsif ($query->param('delete')) {
		$form .= delete_object('',$name);
		$got_form = 1;
	} elsif ($query->param('rename')) {
		$form .= rename_object('',$name);
		$got_form = 1;
	} elsif ($query->param('add') || $query->param('save')) {
		if ($name && $type && $command_line) {
			my $data = qq(<?xml version="1.0" ?>
<data>
  <prop name="command_line"><![CDATA[$command_line]]>
  </prop>
</data>);
			if ($query->param('add')) {
				my %command = StorProc->fetch_one('commands','name',$name);
				if ($command{'name'}) {
					push @errors, "Duplicate: Command $name already exists.";
				} else {
					my @values = ('',$name,$type,$data,'');
					my $result = StorProc->insert_obj('commands',\@values);
					if ($result =~ /^Error/) { 
						push @errors, $result;
					} else {
						$refresh_left = "top_menu=$top_menu&selected=$name";
					}	
				}
			} else {
				my %values = ('type' => $type,'data' => $data);
				my $result = StorProc->update_obj('commands','name',$name,\%values);
				if ($result =~ /^Error/) { push @errors, $result }
			}
		} else {
			$required{'name'} = 1;
			$required{'type'} = 1;
			$required{'command_line'} = 1;
			push @errors, "Missing required fields: Name, Type, Command line.";	
		}
		if ($obj eq 'commands') {
			unless (@errors) {
				$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
				$form .= Forms->form_top("Command",'');
				if ($query->param('add')) {
					my @message = ("Command $name added.");
					$form .= Forms->form_message('Saved:',\@message,'row1');	
				} else {
					my @message = ("Command $name updated.");
					$form .= Forms->form_message('Saved:',\@message,'row1');	
				}
				delete $hidden{'task'};
				$form .= Forms->hidden(\%hidden);
				$form .= Forms->form_bottem_one_button(\%continue,$tab++);						
				$got_form = 1;
			}
		}
	} elsif ($query->param('update_resource')) {
		$resource_sav = $query->param('resource');
		my $resource_value = $query->param('resource_value');
		my $comment = $query->param('comment');
		my %values = ('value' => $resource_value);
		my $result = StorProc->update_obj('setup','name',$resource_sav,\%values);
		if ($result =~ /error/i) { push @errors, $result }
		my $label = $resource_sav;
		$label =~ s/user//;
		my %values = ('value' => $comment);
		my $result = StorProc->update_obj('setup','name',"resource_label$label",\%values);
		if ($result =~ /error/i) { push @errors, $result }
	} elsif ($query->param('upload')) {
		$resource_sav = $query->param('resource');
		my %res = StorProc->fetch_one('setup','name',$resource_sav);
		my $file = $query->param('file');
		my $fname = StorProc->upload($res{'value'},$file);
		if ($fname =~ /Error/) { 
			push @errors, $fname;	
			$task = 'new_plugin';
		} else {
			$task = 'new_command';
			$hidden{'task'} = 'new_command';
			$resource_sav = uc($resource_sav);
			$command = $fname;
			$command_line = "\$$resource_sav\$/$fname";
		}
	} elsif ($query->param('test_command')) {
		my $command_test = $command_line;
		my %where = ('type' => 'resource'); 
		my %resources = StorProc->get_resources();
		my %host = StorProc->fetch_one('hosts','name',$host);
		foreach my $res (keys %resources) {
			if ($command_test =~ /$res/i) { $command_test =~ s/\$$res\$/$resources{$res}/ig } 
		}
		if ($command_line =~ /\$HOSTADDRESS\$/) { 
			if ($host =~ /\d+\.\d+\.\d+\.\d+/) { $command_line =~ s/\$HOSTADDRESS\$/$host/g }
			$command_line =~ s/\$HOSTADDRESS\$/$host{'address'}/g; 
		}
		if ($command_test =~ /\$HOSTNAME\$/) { $command_test =~ s/\$HOSTNAME\$/$host/g } 
		my @args = split(/!/, $arg_string);
		my $cnt = 1;
		foreach my $a (@args) {
			$command_test =~ s/\$ARG$cnt\$/$a/g;
			$cnt++;
		}
		$results = "$command_test<br/>";
		$results .= StorProc->test_command($command_test);
	} elsif ($query->param('new_command')) {
		$task = 'new_command';
		$hidden{'task'} = 'new_command';
	}
	unless ($got_form) {
		$form .= Forms->header($page_title,$user_acct,$view);
		if ($task eq 'new') {
			my $resource = undef;
			my $resource_value = undef;
			$form .= Forms->form_top("Command Wizard",'');
			$form .= Forms->form_doc('Select resource');
			if (@errors) { $form .= Forms->form_errors(\@errors) }
			my %resources = StorProc->get_resources();
			my %resource_doc = StorProc->get_resources_doc();
			my %selected = ();
			if ($resource_sav) {
				$resource = $resource_sav;
			} else {
				foreach my $name ($query->param) {
					if ($name =~ /^resource_(user\d+)/) { $resource = $1 }
				}
			}
			if ($resource) {
				$resource_value = $resources{$resource};
				%selected = ('name' => $resource,'value' => $resource_value);
				my %res = StorProc->fetch_one('setup','name',$resource);
				my @plugins = StorProc->get_dir($res{'value'});
				$hidden{'type'} = 'check';
				$form .= Forms->list_box('Plugin:','command',\@plugins,'','','');
			}
			$form .= Forms->resource_select(\%resources,\%resource_doc,\%selected,$top_menu);
			$hidden{'obj_view'} = 'new_command';
			$form .= Forms->hidden(\%hidden);
			my %next = ('name' => 'new_command','value' => 'Next >>');
			$form .= Forms->form_bottem_one_button(\%next,$tab++);

		} elsif ($task eq 'new_plugin') {
			my $resource = undef;
			my $resource_value = undef;
			$form .= Forms->form_top("Command Wizard",'');
			$form .= Forms->form_doc('Upload plugin');
			if (@errors) { $form .= Forms->form_errors(\@errors) }
			my %resources = StorProc->get_resources();
			my %resource_doc = StorProc->get_resources_doc();
			my %selected = ();
			if ($resource_sav) {
				$resource = $resource_sav;
			} else {
				foreach my $name ($query->param) {
					if ($name =~ /^resource_(user\d+)/) { $resource = $1 }
				}
			}
			if ($resource) {
				$resource_value = $resources{$resource};
				%selected = ('name' => $resource,'value' => $resource_value);
			}
			$form .= Forms->resource_select(\%resources,\%resource_doc,\%selected);		
			$form .= Forms->form_file();
			$form .= Forms->hidden(\%hidden);
			my %upload = ('name' => 'upload','value' => 'Upload');
			$form .= Forms->form_bottem_two_button(\%upload,\%cancel,$tab++);

		} elsif ($task =~ /new_command|copy|modify/) {
			my $arg_string = $query->param('arg_string');
			if ($obj eq 'commands') {
				$form .= Forms->form_top("Command Wizard",'');
			}
			if ($task eq 'copy') {
				my $source = $query->param('source');
				unless ($name) { $name = "Copy-of-$source" }
				my %command = StorProc->fetch_one('commands','name',$source);
				unless ($command_line) { $command_line = $command{'command_line'} }
				unless ($type) { $type = $command{'type'} }
				$form .= Forms->display_hidden("Copy:",'source',$source);
				if (@errors) { $form .= Forms->form_errors(\@errors) }
				$form .= Forms->text_box('Name:','name',$name,$textsize{'name'},$required{'name'});
			} elsif ($task eq 'modify') {
				my %command = StorProc->fetch_one('commands','name',$name);
				unless ($command_line) { $command_line = $command{'command_line'} }
				unless ($type) { $type = $command{'type'} }
				if (@errors) { $form .= Forms->form_errors(\@errors) }
				$form .= Forms->display_hidden("Name:",'name',$name);
			} else {
				if (@errors) { $form .= Forms->form_errors(\@errors) }
				$form .= Forms->text_box('Name:','name',$name,$textsize{'name'},$required{'name'});
				unless ($command_line) {
					my $command = $query->param('command');
					my $resource = $query->param('resource');
					$command_line = "\$".uc($resource)."\$/$command";
				}
			}
			my @args = split(/ARG/i,$command_line);
			my $usage = undef;
			if (@args) {
				my $cnt = 1;
				pop @args;
				my $ags = undef;
				foreach (@args) { 
					if ($command_line =~ /ARG$cnt/) { $usage .= "ARG$cnt!" }
					$cnt++;
				}
				chop $usage;
				unless ($arg_string) { $arg_string = $usage }
				if ($usage) { 
					$usage = "$name!$usage";
				} else {
					$usage = $name;
				}
			} else {
				$usage = "$name";
			}
			unless ($host) { $host = 'localhost' }
			my @types = ('check','notify','other');
			$form .= Forms->list_box('Type:','type',\@types,$type,'',$docs{'type'});
			$form .= Forms->text_area('Command line:','command_line',$command_line,'',$textsize{'command_line'},'',$docs{'command_line'});
			$form .= Forms->display_hidden("Usage:",'',$usage);
			$form .= Forms->command_test($results,$host,$arg_string);
			if ($obj eq 'commands') {
				$form .= Forms->hidden(\%hidden);
				if ($task eq 'modify') {
					if ($auth_delete{'commands'}) {
						$form .= Forms->form_bottem_four_button(\%save,\%rename,\%delete,\%cancel,$tab++);
					} else {		
						$form .= Forms->form_bottem_three_button(\%save,\%rename,\%cancel,$tab++);
					}
				} else {
					$form .= Forms->form_bottem_two_button(\%cancel,\%add,$tab++);
				}
			}
#		} else {
#			unless ($got_form) {
#				$form .= Forms->form_top("Command Wizard",'');
#				if (@errors) { $form .= Forms->form_errors(\@errors) }
#				my @commands = StorProc->fetch_list('commands','name');
#				my %selected = ();
#				$form .= Forms->command_select(\@commands,\%selected);
#				$form .= Forms->hidden(\%hidden);
#				$form .= Forms->form_bottem_one_button(\%next);
#			}
		}
	}
	return $form;
}


#
###########################################################################################################################
# Service Groups
#

sub service_group() {
	my $form = undef;
	my %save = ('name' => 'save', 'value' => 'Save');
	my %done = ('name' => 'save', 'value' => 'Done');
	my $got_form = 0;
	my $name = $query->param('name');
	my $alias = $query->param('alias');
	my %svcgrp = StorProc->fetch_one('servicegroups','name',$name);
	unless ($alias) { $alias = $svcgrp{'alias'} }
	my %docs = Doc->servicegroups();
	if ($query->param('continue') || $query->param('cancel')) {
		$form .= Forms->header($page_title,$user_acct,$view);
		$got_form = 1;
	} elsif ($task eq 'new' || $query->param('add')) {
		$got_form = 1;
		my $name = $query->param('name');
		my $alias = $query->param('alias');
		my $validate = $query->param('validate');
		if ($name && $alias) {
			if ($svcgrp{'name'}) {
				push @errors, "Duplicate: Service group $name already exists.";
			} else {
				my @values = ('',$name,$alias,'');
				my $result = StorProc->insert_obj('servicegroups',\@values);
				if ($result =~ /error/i) { 
					push @errors, $result;
				} else {
					delete $hidden{'task'};
					$refresh_left = "top_menu=services&selected=$name";
					$got_form = 0;
				}
			}
		} elsif ($validate) { 
			$required{'name'} = 1;
			$required{'alias'} = 1;
			push @errors, "Missing required fields: Name, Alias";
		}
		if ($got_form) {
			$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
			$form .= Forms->form_top("Service Group",'');
			if (@errors) { $form .= Forms->form_errors(\@errors) }
			$form .= Forms->text_box('Name:','name',$name,$textsize{'name'},$required{'name'},$docs{'name'});
			$form .= Forms->text_box('Alias:','alias',$alias,$textsize{'alias'},$required{'alias'},$docs{'alias'});
			$hidden{'validate'} = 1;
			$form .= Forms->hidden(\%hidden);
			$form .= Forms->form_bottem_two_button(\%add,\%cancel,$tab++);		
		}
	} elsif ($query->param('add_services')) {
		my $host = $query->param('host');
		my @services = $query->param('services');
		my %h = StorProc->fetch_one('hosts','name',$host);
		if (@services && $host) {
			foreach my $service (@services) {
				my %sn = StorProc->fetch_one('service_names','name',$service);
				my %where = ('host_id' => $h{'host_id'},'servicename_id' => $sn{'servicename_id'});
				my %s = StorProc->fetch_one_where('services',\%where);
				my @values = ($svcgrp{'servicegroup_id'},$h{'host_id'},$s{'service_id'});
				my $result = StorProc->insert_obj('servicegroup_service',\@values);
				if ($result =~ /^Error/ && !$result =~ /duplicate/i) { push @errors, $result }
			}
		}
	} elsif ($query->param('add_hosts')) {
		my $service = $query->param('service');
		my @hosts = $query->param('hosts');
		my %sn = StorProc->fetch_one('service_names','name',$service);
		if (@hosts && $service) {
			foreach my $host (@hosts) {
				my %h = StorProc->fetch_one('hosts','name',$host);
				my %where = ('host_id' => $h{'host_id'},'servicename_id' => $sn{'servicename_id'});
				my %s = StorProc->fetch_one_where('services',\%where);
				my @values = ($svcgrp{'servicegroup_id'},$h{'host_id'},$s{'service_id'});
				my $result = StorProc->insert_obj('servicegroup_service',\@values);
				if ($result =~ /^Error/ && !$result =~ /duplicate/i) { push @errors, $result }
			}
		}
	} elsif ($query->param('remove_service')) {
		my $host = $query->param('del_host');
		my $service = $query->param('del_service');
		my %h = StorProc->fetch_one('hosts','name',$host);
		my %sn = StorProc->fetch_one('service_names','name',$service);
		my %where = ('host_id' => $h{'host_id'},'servicename_id' => $sn{'servicename_id'});
		my %s = StorProc->fetch_one_where('services',\%where);
		%where = ('servicegroup_id' => $svcgrp{'servicegroup_id'},'service_id' => $s{'service_id'},'host_id' => $h{'host_id'});
		my $result = StorProc->delete_one_where('servicegroup_service',\%where);
		if ($result =~ /^Error/) { push @errors, $result }
	} elsif ($query->param('save')) {
		my %values = ('alias' => $alias);
		my $escalation = $query->param('escalation');
		if ($escalation) {
			my %esc = StorProc->fetch_one('escalation_trees','name',$escalation);
			$values{'escalation_id'} = $esc{'tree_id'};
		}
		my $result = StorProc->update_obj('servicegroups','name',$name,\%values);
		if ($result =~ /^Error/) { push @errors, $result }
		unless (@errors) {
			$form .= Forms->header($page_title,$user_acct,$view);
			$form .= Forms->form_top('Service Group');
			my @message = ("Changes to $name accepted.");
			$form .= Forms->form_message('Updated:',\@message,'row1');					
			$form .= Forms->hidden(\%hidden);
			$form .= Forms->form_bottem_one_button(\%continue,$tab++);
			$got_form = 1;
		}
	} elsif ($query->param('rename')) {
		$form .= rename_object('',$name);
		$got_form = 1;
	} elsif ($query->param('delete') || $query->param('confirm_delete')) {
		if ($query->param('confirm_delete')) {
			my $result = StorProc->delete_all('servicegroups','servicegroup_id',$svcgrp{'servicegroup_id'});
			if ($result =~ /^Error/) { push @errors, $result }
			unless (@errors) { 
				$refresh_left = "top_menu=$top_menu&selected=$name";
				$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
				$form .= Forms->form_top('Deleted','');
				my @message = ("$name");
				$form .= Forms->form_message('Removed:',\@message,'row1');			
				$form .= Forms->hidden(\%hidden);
				$form .= Forms->form_bottem_one_button(\%continue,$tab++);
				$name = undef;
				$got_form = 1;	
			}
		} elsif ($query->param('task') eq 'No') {
			$got_form = 0;	
		} else {
			foreach my $name ($query->param) {
				$hidden{$name} = $query->param($name);
			}
			delete $hidden{'task'};
			$hidden{'delete'} = 1;
			$form .= Forms->header($page_title,$user_acct,$view);
			my $message = qq(Are you sure you want to remove $name?);
			$form .= Forms->are_you_sure('Confirm Delete:',$message,'confirm_delete',\%hidden);
			$name = undef;
			$got_form = 1;
		}	
	} 
	unless ($got_form) {
		my $host = $query->param('host');
		my $service = $query->param('service');
		my $escalation = $query->param('escalation');
		unless($escalation) {
			my %esc = StorProc->fetch_one('escalation_trees','tree_id',$svcgrp{'escalation_id'});
			$escalation = $esc{'name'};
		}
		$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
		$form .= Forms->form_top("Service Group",'');
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->display_hidden('Name:','name',$name);
		$form .= Forms->text_box('Alias:','alias',$alias,$textsize{'alias'},$required{'alias'},$docs{'alias'});
		my %host_service = StorProc->get_servicegroup($svcgrp{'servicegroup_id'});
		my %h = StorProc->fetch_one('hosts','name',$host);
		my @host_nonmembers = StorProc->get_host_services($h{'host_id'});
		my @hosts = StorProc->fetch_list('hosts','name');
		my %s = StorProc->fetch_one('service_names','name',$service);
		my @serivce_nonmembers = StorProc->get_service_hosts($s{'servicename_id'});
		my @services = StorProc->fetch_list('service_names','name');
		$form .= Forms->service_group($user_acct,$view,$name,\%host_service,$host,\@host_nonmembers,\@hosts,$service,\@serivce_nonmembers,\@services);
		my %where = ('type' => 'service');
		my @serviceesc = StorProc->fetch_list_where('escalation_trees','name',\%where);
		$form .= Forms->list_box_submit('Service escalation:','escalation',\@serviceesc,$escalation,'','');
		if ($escalation) {
			my ($ranks, $templates) = StorProc->get_tree_detail($escalation);
			my %ranks = %{$ranks};
			my %templates = %{$templates};
			$form .= Forms->escalation_tree(\%ranks,\%templates,'escalations');
		}
		$form .= Forms->hidden(\%hidden);
		if ($auth_delete{'servicegroups'}) {
			$form .= Forms->form_bottem_three_button(\%save,\%delete,\%rename,$tab++);		
		} else {
			$form .= Forms->form_bottem_two_button(\%save,\%rename,$tab++);		
		}
	}
	return $form;
}


#
###########################################################################################################################
# Host Wizard
#

sub host_wizard() {
	my %files = StorProc->fetch_file_hash();
	my $service_next = undef;
	my $form = undef;
	my $test_results = undef;
	my $name = $query->param('name');
	my $host = $query->param('host');
	$hidden{'name'} = $name;
	my %service_detail = ();
	$service_detail{'service'} = $query->param('service');
	$service_detail{'template'} = $query->param('template');
	$service_detail{'command'} = $query->param('command');
	$service_detail{'command_line'} = $query->param('command_line');
	$service_detail{'inherit'} = $query->param('inherit');
	$service_detail{'ext_info'} = $query->param('ext_info');
	$service_detail{'escalation'} = $query->param('esclation');
	$service_detail{'dependency'} = $query->param('dependency');
	$service_detail{'command_save'} = $query->param('command_save');
	%properties = StorProc->fetch_one('hosts','name',$name);
	my %profile = StorProc->fetch_one('profiles_host','hostprofile_id',$properties{'hostprofile_id'});
	my $alias = $query->param('alias');
	my $step = $query->param('step');
	my @profiles = ();
	my @services = ();
	my %param_vals = ();
#
#############################################################################
# Check for existing session
#
	unless ($step) {
		if ($query->param('yes')) {
			my %where = ('user_acct' => $user_acct, 'type' => 'wizard_design');
			my %host = StorProc->fetch_one_where('stage_hosts',\%where);
			my %where = ('host' => $name,'user_acct' => $user_acct,'type' => $view,'status' => 'edit_added');
			my %service_list = StorProc->fetch_list_hash_array('stage_host_services',\%where);
			my @services = ();
			foreach my $svc (keys %service_list) { unless ($svc =~ /HASH/) { push @services, $svc } }
			@services = sort @services;
			$service_next = pop @services; 
			$step = $host{'info'};
			if ($service_next) {
				my %where = ('host' => $name,'user_acct' => $user_acct,'type' => $view,'status' => '3');
				my %service = StorProc->fetch_one_where('stage_host_services',\%where);
				%where = ('service_id' => $service{'service_id'});
				my %file = StorProc->fetch_one_where('file_service',\%where);
				$hidden{'file'} = $file{'file'};
				my $result = StorProc->delete_all('services','service_id',$service{'service_id'});
				if ($result =~ /^Error/) { push @errors, $result }
				$step = 'service_detail';	
			}
		} elsif ($query->param('no')) {
			my $result = StorProc->delete_all('hosts','name',$name);
			if ($result =~ /^Error/) { push @errors, $result }
			my %where = ('name' => $name, 'user_acct' => $user_acct);
			$result = StorProc->delete_one_where('stage_hosts',\%where);
			if ($result =~ /^Error/) { push @errors, $result }
			%where = ('host' => $name, 'user_acct' => $user_acct);
			$result = StorProc->delete_one_where('stage_host_services',\%where);
			if ($result =~ /^Error/) { push @errors, $result }
			$step = 'host_vitals';
		} else {
			my %where = ('user_acct' => $user_acct, 'type' => 'wizard_design');
			my %host = StorProc->fetch_one_where('stage_hosts',\%where);
			if ($host{'name'}) { 
				$hidden{'name'} = $host{'name'};
				my %h = StorProc->fetch_one('hosts','name',$host{'name'});
				$hidden{'host_id'} = $h{'host_id'};
				$hidden{'host_profile'} = $host{'hostprofile'};
				$hidden{'service_profile'} = $host{'service_profile'};
				$form .= Forms->header($page_title,$user_acct,$view);
				$form .= Forms->form_top('New Host Wizard');
				my $message = qq(An existing host wizard session exists for $host{'name'}. Do you wish to continue with this host definition? (No will delete the host).&nbsp;);
				$form .= Forms->form_doc($message);
				$form .= Forms->hidden(\%hidden);
				$form .= Forms->form_bottem_two_button(\%yes,\%no,$tab++);
			} else {
				$step = 'host_vitals';
			}
		}
#############################################################################
# Test command
#
	} elsif ($query->param('test_command')) {	
		my $arg_string = $query->param('arg_string');
		my $command = $query->param('command');
		my %cmd = StorProc->fetch_one('commands','name',$command);
		my %host = StorProc->fetch_one('hosts','name',$host);
		my %where = ('type' => 'resource'); 
		my %resources = StorProc->get_resources();
		foreach my $res (keys %resources) {
			if ($cmd{'command_line'} =~ /$res/i) { $cmd{'command_line'} =~ s/\$$res\$/$resources{$res}/ig } 
		}
		if ($cmd{'command_line'} =~ /\$HOSTADDRESS\$/) { 
			if ($host =~ /\d+\.\d+\.\d+\.\d+/) { $cmd{'command_line'} =~ s/\$HOSTADDRESS\$/$host/g }
			$cmd{'command_line'} =~ s/\$HOSTADDRESS\$/$host{'address'}/g; 
		}
		if ($cmd{'command_line'} =~ /\$HOSTNAME\$/) { $cmd{'command_line'} =~ s/\$HOSTNAME\$/$name/g } 
		my @args = split(/!/, $arg_string);
		my $cnt = 1;
		foreach my $a (@args) {
			$cmd{'command_line'} =~ s/\$ARG$cnt\$/$a/g;
			$cnt++;
		}
		$test_results = "$cmd{'command_line'}<br/>";
		$test_results .= StorProc->test_command($cmd{'command_line'});	

#############################################################################
# Save as profile
#
	} elsif ($query->param('save_as_profile')) {
		$step = 'save_as_profile'
	} elsif ($query->param('save')) {
		my $host_profile = $query->param('host_profile');
		my $service_profile = $query->param('service_profile');
		if ($service_profile && $host_profile) {
			my %h = StorProc->fetch_one('profiles_host','name',$host_profile);
			my %s = StorProc->fetch_one('profiles_service','name',$service_profile);
			if ($h{'name'}) {
				push @errors, "Duplicate: Host profile name $host_profile already in use.";
			}
			if ($s{'name'}) {
				push @errors, "Duplicate: Service profile name $service_profile already in use.";
			}
			unless (@errors) {
				my $data = "<?xml version=\"1.0\" ?>\n<data>\n</data>";
				my %host = StorProc->fetch_one('hosts','name',$name);
				my %f = StorProc->fetch_one('file_host','host_id',$host{'host_id'});
				my %w = ('host_id' => $host{'host_id'});
				my @hostgroups = StorProc->fetch_list_where('hostgroup_host','hostgroup_id',\%w);
				my @parents = StorProc->fetch_list_where('host_parent','parent_id',\%w);
				my @services = StorProc->fetch_list_where('services','servicename_id',\%w);
				my @sids = StorProc->fetch_list_where('services','service_id',\%w);
				my %s = StorProc->fetch_one('file_service','service_id',$sids[0]);
				my @values = ('',$service_profile,"saved from host wizard by $user_acct",$s{'file_id'},$data);
				my $spid = StorProc->insert_obj_id('profiles_service',\@values,'serviceprofile_id');
				if ($spid =~ /^Error/) { push @errors, $spid }
				unless (@errors) {
					foreach my $snid (@services) {
						@values = ($snid,$spid);
						my $result = StorProc->insert_obj('serviceprofile',\@values);
						if ($result =~ /^Error/) { push @errors, $result }
					}
				}
				unless (@errors) {
					@values = ('',$host_profile,"saved from host wizard by $user_acct",$host{'hosttemplate_id'},$host{'hostextinfo_id'},
						$host{'host_escalation_id'},$host{'service_escalation_id'},$f{'file_id'},$data);
					my $id = StorProc->insert_obj_id('profiles_host',\@values,'hostprofile_id');
					@values = ($id,$spid);
					my $result = StorProc->insert_obj('profile_host_profile_service',\@values);
					if ($result =~ /^Error/) { push @errors, $result }
					foreach my $hgid (@hostgroups) {
						@values = ($id,$hgid);
						my $result = StorProc->insert_obj('profile_hostgroup',\@values);
						if ($result =~ /^Error/) { push @errors, $result }
					}
					foreach my $hid (@parents) {
						@values = ($id,$hid);
						my $result = StorProc->insert_obj('profile_parent',\@values);
						if ($result =~ /^Error/) { push @errors, $result }
					}
					my %w = ('host_id' => $properties{'host_id'});
					my %u = ('hostprofile_id' => $id);
					my $result = StorProc->update_obj_where('hosts',\%u,\%w);
					if ($result =~ /^Error/) { push @errors, $result }
				}
			}
		} else {
			push @errors, "Required Host profile name, service profile name";
		}
		if (@errors) {
			my $result = StorProc->delete_all('profiles_host','name',$host_profile);
			my $result = StorProc->delete_all('profiles_service','name',$service_profile);
			if ($result =~ /^Error/) { push @errors, $result }
			$step = 'save_as_profile';
		} else {
			$step = 'saved';
		}
	} elsif ($query->param('bail')) {
		$step = 'host_vitals';
		$name = undef;
#############################################################################
# Back
#
	} elsif ($query->param('back')) {
		if ($step eq 'host_attribs') {
			$step = 'host_vitals';
			%properties = StorProc->fetch_one('stage_hosts','name',$name);
			my $result = StorProc->delete_all('stage_hosts','name',$name);
			if ($result =~ /^Error/) { push @errors, $result }
			$result = StorProc->delete_all('hosts','name',$name);
			if ($result =~ /^Error/) { push @errors, $result }
		} elsif ($step eq 'host_attribs2') {
			$step = 'host_attribs';
			my %values = ('info' => $step);
			my $result = StorProc->update_obj('stage_hosts','name',$name,\%values);
			if ($result =~ /^Error/) { push @errors, $result }
		} elsif ($step eq 'service_list') {
			$step = 'host_attribs2';
			my $result = StorProc->delete_all('stage_host_services','host',$name);
			if ($result =~ /^Error/) { push @errors, $result }
			my %values = ('info' => $step);
			$result = StorProc->update_obj('stage_hosts','name',$name,\%values);
			if ($result =~ /^Error/) { push @errors, $result }
		} elsif ($step eq 'service_detail') {
			my %values = ('user_acct' => $user_acct,'status' => '3','type' => $view,'host' => $name);
			my %service_list = StorProc->fetch_list_hash_array('stage_host_services',\%values);
			my @services = ();
			foreach my $svc (keys %service_list) { unless ($svc =~ /HASH/) { push @services, $svc } }
			@services = sort @services;
			$service_next = pop @services; 
			if ($service_next) {
				$step = 'service_detail';
				my %where = ('host' => $name,'user_acct' => $user_acct,'name' => $service_next);
				my %service_stage = StorProc->fetch_one_where('stage_host_services',\%where);
				my $sid = $service_stage{'service_id'};
				my $result = StorProc->delete_all('services','service_id',$service_stage{'service_id'});
				if ($result =~ /^Error/) { push @errors, $result }
				my %values = ('service_id' => '','status' => 'edit');
				my $result = StorProc->update_obj('stage_host_services','service_id',$service_stage{'service_id'},\%values);
				if ($result =~ /^Error/) { push @errors, $result }
			} else {
				$step = 'service_list';
				my @sids = StorProc->fetch_unique('stage_host_services','service_id','host',$name);
				foreach my $sid (@sids) {
					my $result = StorProc->delete_all('services','service_id',$sid);
					if ($result =~ /^Error/) { push @errors, $result }
				}
				my $result = StorProc->delete_all('stage_host_services','host',$name);
				if ($result =~ /^Error/) { push @errors, $result }
			}
		}
#############################################################################
# Continue
#
	} elsif ($query->param('continue')) {
		$step = 'host_vitals';

#############################################################################
# Cancel
#

	} elsif ($query->param('cancel')) {
		if ($query->param('yes')) {
			$step = 'host_vitals';
			%properties = StorProc->fetch_one('stage_hosts','name',$name);
			my $result = StorProc->delete_all('hosts','name',$name);
			if ($result =~ /^Error/) { push @errors, $result }
			my %where = ('name' => $properties{'name'}, 'user_acct' => $user_acct);
			$result = StorProc->delete_one_where('stage_hosts',\%where);
			if ($result =~ /^Error/) { push @errors, $result }
			%where = ('host' => $properties{'name'}, 'user_acct' => $user_acct);
			$result = StorProc->delete_one_where('stage_host_services',\%where);
			if ($result =~ /^Error/) { push @errors, $result }
		} elsif ($query->param('no')) {
			# nothing
		} else {
			foreach my $name ($query->param) {
				unless ($name eq 'task' || $name eq 'nocache' || $name eq 'parents' || $name eq 'hostgroups') { $hidden{$name} = $query->param($name) }
			}
			$form .= Forms->header($page_title,$user_acct,$view);
			$form .= Forms->form_top('New Host Wizard');
			my $message = qq(Are you sure you want to remove host $name?<br><br>);
			$form .= Forms->form_doc($message);
			my @parents = $query->param('parents');
			foreach my $parent (@parents) {
				my %hide = ('parents' => $parent);
				$form .= Forms->hidden(\%hide);
			}
			my @hostgroups = $query->param('hostgroups');
			foreach my $hostgroup (@hostgroups) {
				my %hide = ('hostgroups' => $hostgroup);
				$form .= Forms->hidden(\%hide);
			}
			$form .= Forms->hidden(\%hidden);
			$form .= Forms->form_bottem_two_button(\%yes,\%no,$tab++);
			$step = 'cancel';
		}

#############################################################################
# Next
#
	} elsif ($query->param('next')) {
		if ($step eq 'host_vitals') {
			$submit = 'Add';
			my %data = parse_query('hosts','hosts');
			my $field_ck = check_fields();
			if ($field_ck == 0) {
				my $host_profile = $query->param('host_profile');
				my @values;
				$data{'status'} = 1;
				my @data_cols = split(/,/, $db_values{$obj});
				my $primary_key = '';
				push @values, $primary_key; 
				foreach my $val (@data_cols) {
					push @values, $data{$val};
				}
				my $id = StorProc->insert_obj_id('hosts',\@values,'host_id');
				if ($id =~ /^Error/) { push @errors, $id }
				%properties = StorProc->fetch_one('hosts','name',$name);
				%profile = StorProc->fetch_one('profiles_host','hostprofile_id',$properties{'hostprofile_id'});

				$hidden{'host_id'} = $id;
				@values = ($name,$user_acct,'wizard_design','incomplete',$data{'alias'},$data{'address'},$data{'os'},$host_profile,'','host_attribs');
				my $result = StorProc->insert_obj('stage_hosts',\@values);
				if ($result =~ /^Error/) { push @errors, $result }
				if ($hidden{'host_profile'}) {
					my %hp = StorProc->fetch_one('profiles_host','name',$hidden{'host_profile'});
					my %w = ('hostprofile_id' => $hp{'hostprofile_id'});
					my @externals = StorProc->fetch_list_where('external_host_profile','external_id',\%w);
					if ($result =~ /^Error/) { push @errors, $result }
					foreach my $ext (@externals) {
						my %e = StorProc->fetch_one('externals','external_id',$ext);
						my @vals = ($ext,$id,$e{'display'});
						$result = StorProc->insert_obj('external_host',\@vals);
						if ($result =~ /^Error/) { push @errors, $result }
					}
				}
			}
			$step = 'host_attribs';
			if (@errors || $field_ck == 1) {
				$step = 'host_vitals';
			}
		} elsif ($step eq 'host_attribs') {
			$step = 'host_attribs2';
			my %values;
			my $template = $query->param('host_template');
			my %w = ('name' => $template);
			my %t = StorProc->fetch_one_where('host_templates',\%w);
			$values{'hosttemplate_id'} = $t{'hosttemplate_id'};
			my $file = $query->param('file');
			my %f = StorProc->fetch_one('files','name',$file);
			my $file_id = $f{'file_id'};
			my @parents = $query->param('parents');
			if (!$values{'hosttemplate_id'}) { 
				$required{'host_template'} = 1;
				push @errors, "Required: host template.";
			}
			if (!$file_id) { 
				$required{'file'} = 1;
				push @errors, "Required: Assign host to a file.";
			}
			unless (@errors) {
				my $result = StorProc->update_obj('hosts','host_id',$properties{'host_id'},\%values);
				if ($result =~ /^Error/) { push @errors, $result }
				%values = ('serviceprofile' => $hidden{'service_profile'},'info' => $step);
				$result = StorProc->update_obj('stage_hosts','name',$name,\%values);
				if ($result =~ /^Error/) { push @errors, $result }
				$result = StorProc->delete_all('host_parent','host_id',$properties{'host_id'});
				if ($result =~ /^Error/) { push @errors, $result }
				foreach my $parent (@parents) { 
					if (!$parent) { next }
					my %p = StorProc->fetch_one('hosts','name',$parent);
					my @values = ($properties{'host_id'},$p{'host_id'});
					my $result = StorProc->insert_obj('host_parent',\@values);
					if ($result =~ /^Error/) { push @errors, $result }	
				}
				$result = StorProc->delete_all('file_host','host_id',$properties{'host_id'});
				if ($result =~ /^Error/) { push @errors, $result }
				my @values = ($file_id,$properties{'host_id'});
				$result = StorProc->insert_obj('file_host',\@values);
				if ($result =~ /^Error/) { push @errors, $result }	
			}
			if (@errors) {
				$step = 'host_attribs';
			}
		} elsif ($step eq 'host_attribs2') {
			$step = 'service_list';
			my %values;
			my @hostgroups = $query->param('hostgroups');
			my $coords2d = $query->param('coords2d');
			my $coords3d = $query->param('coords3d');
			my $extended_info = $query->param('extended_info');
			my %w = ('name' => $extended_info);
			my %t = StorProc->fetch_one_where('extended_host_info_templates',\%w);
			$values{'hostextinfo_id'} = $t{'hostextinfo_id'};
			my $host_escalation = $query->param('host_escalation');
			%w = ('name' => $host_escalation,'type' => 'host');
			%t = StorProc->fetch_one_where('escalation_trees',\%w);
			$values{'host_escalation_id'} = $t{'tree_id'};
			my $service_escalation = $query->param('service_escalation');
			%w = ('name' => $service_escalation,'type' => 'service');
			%t = StorProc->fetch_one_where('escalation_trees',\%w);
			$values{'service_escalation_id'} = $t{'tree_id'};
			$hidden{'service_profile'} = $query->param('service_profile');
			unless (@errors) {
				my $result = StorProc->update_obj('hosts','host_id',$properties{'host_id'},\%values);
				if ($result =~ /^Error/) { push @errors, $result }
				%values = ('serviceprofile' => $hidden{'service_profile'},'info' => $step);
				$result = StorProc->update_obj('stage_hosts','name',$name,\%values);
				if ($result =~ /^Error/) { push @errors, $result }
				$result = StorProc->delete_all('hostgroup_host','host_id',$properties{'host_id'});
				if ($result =~ /^Error/) { push @errors, $result }
				foreach my $hg (@hostgroups) { 
					if (!$hg) { next }
					my %h = StorProc->fetch_one('hostgroups','name',$hg);
					my @values = ($h{'hostgroup_id'},$properties{'host_id'});
					my $result = StorProc->insert_obj('hostgroup_host',\@values);
					if ($result =~ /^Error/) { push @errors, $result }	
				}
				if ($coords2d || $coords3d) { 
					$result = StorProc->delete_all('extended_info_coords','host_id',$properties{'host_id'});
					if ($result =~ /^Error/) { push @errors, $result }
					my $data = "<?xml version=\"1.0\" ?>\n<data>";
					if ($coords2d) { $data .= "\n  <prop name=\"2d_coords\"><![CDATA[$coords2d]]>\n  </prop>" }
					if ($coords3d) { $data .= "\n  <prop name=\"3d_coords\"><![CDATA[$coords3d]]>\n  </prop>" }
					$data .= "\n</data>";
					my @vals = ($properties{'host_id'},$data);
					$result = StorProc->insert_obj('extended_info_coords',\@vals);
					if ($result =~ /^Error/) { push @errors, $result }	
				}
			}
			if (@errors) {
				$step = 'host_attribs2';
			}
		} elsif ($step eq 'service_list') {
			my $file = $query->param('file');
			if ($file) {
				my $result = StorProc->delete_all('services','host_id',$properties{'host_id'});
				if ($result =~ /^Error/) { push @errors, $result }
				$result = StorProc->delete_all('stage_host_services','host',$name);
				if ($result =~ /^Error/) { push @errors, $result }
				foreach my $service ($query->param) {
					if ($query->param($service) eq 'add') {
						my %s = StorProc->fetch_one('service_names','name',$service);
						my @values = ('',$properties{'host_id'},$s{'servicename_id'},$s{'template'},$s{'extinfo'},$s{'escalation'},'1',$s{'check_command'},$s{'command_line'},'');
						my $id = StorProc->insert_obj_id('services',\@values,'service_id');
						if ($id =~ /^Error/) { push @errors, $id }	
						my %f = StorProc->fetch_one('files','name',$file);
						@values = ($f{'file_id'},$id);
						my $result = StorProc->insert_obj('file_service',\@values);
						if ($result =~ /^Error/) { push @errors, $result }	
						@values = ($service,$user_acct,$name,$view,'1',$id);
						$result = StorProc->insert_obj('stage_host_services',\@values);
						if ($result =~ /^Error/) { push @errors, $result }	
						if ($s{'dependency'}) {
							my %t =  StorProc->fetch_one('service_dependency_templates','name',$s{'dependency'});
							@values = ('',$id,$properties{'host_id'},$properties{'host_id'},$s{'dependency'},'');
							$result = StorProc->insert_obj('service_dependency',\@values);
							if ($result =~ /^Error/) { push @errors, $result }	
						}
						my %w = ('servicename_id' => $s{'servicename_id'});
						my @externals = StorProc->fetch_list_where('external_service_names','external_id',\%w);
						foreach my $ext (@externals) {
							my %e = StorProc->fetch_one('externals','external_id',$ext);
							my @vals = ($ext,$properties{'host_id'},$id,$e{'display'});
							$result = StorProc->insert_obj('external_service',\@vals);
							if ($result =~ /^Error/) { push @errors, $result }
						}
					} elsif ($query->param($service) eq 'edit') {
						my @values = ($service,$user_acct,$name,$view,'2','');
						my $result = StorProc->insert_obj('stage_host_services',\@values);
						if ($result =~ /^Error/) { push @errors, $result }	
					}
				}
				my $result = StorProc->delete_all('serviceprofile_host','host_id',$properties{'host_id'});
				if ($result =~ /^Error/) { push @errors, $result }
				my %profiles_name = StorProc->get_table_objects('profiles_service');
				my @profiles = $query->param('profiles');
				foreach my $profile (@profiles) {
					if ($profiles_name{$profile}) {
						my @values = ($profiles_name{$profile},$properties{'host_id'});
						my $result = StorProc->insert_obj('serviceprofile_host',\@values);
						if ($result =~ /^Error/) { push @errors, $result }	
					}
				}
				my %values = ('user_acct' => $user_acct,'status' => '2','type' => $view,'host' => $name);
				my %service_list = StorProc->fetch_list_hash_array('stage_host_services',\%values);
				my @services;
				foreach my $svc (keys %service_list) { unless ($svc =~ /HASH/) { push @services, $svc } }
				@services = sort { $b cmp $a } @services;
				$service_next = pop @services;
				if ($service_next) {
					$step = 'service_detail';
				} else {
					$step = 'completed';
					my $result = StorProc->delete_all('stage_hosts','name',$name);
					if ($result =~ /^Error/) { push @errors, $result }
					my $result = StorProc->delete_all('stage_host_services','host',$name);
					if ($result =~ /^Error/) { push @errors, $result }
				}
				if (@errors) {
					$step = 'service_list';
				}
			} else {
				push @errors, "Required: File name";
				$required{'file'} = 1;	
				$step = 'service_list';
			}
		} elsif ($step eq 'service_detail') {
			my %s = StorProc->fetch_one('service_names','name',$service_detail{'service'});
			my %t = StorProc->fetch_one('service_templates','name',$service_detail{'template'});
			my %c = StorProc->fetch_one('commands','name',$service_detail{'command'});
			my $command_line = $query->param('command_line');
			if ($service_detail{'inherit'}) {
				delete $c{'command_id'};
				delete $service_detail{'command_line'};
			}
			my %x = StorProc->fetch_one('extended_service_info_templates','name',$service_detail{'ext_info'});
			my %e = StorProc->fetch_one('escalation_trees','name',$service_detail{'escalation'});
			my @values = ('',$properties{'host_id'},$s{'servicename_id'},$t{'servicetemplate_id'},$x{'serviceextinfo_id'},$e{'tree_id'},'1',$c{'command_id'},$service_detail{'command_line'},'');
			my $id = StorProc->insert_obj_id('services',\@values,'service_id');
			if ($id =~ /^Error/) { push @errors, $id }	
			my $file = $query->param('file');
			my %f = StorProc->fetch_one('files','name',$file);
			@values = ($f{'file_id'},$id);
			my $result = StorProc->insert_obj('file_service',\@values);
			if ($result =~ /^Error/) { push @errors, $result }	
			my %where = ('name' => $service_detail{'service'},'user_acct' => $user_acct,'host' => $name);
			my %values = ('status' => '3','service_id' => $id);
			$result = StorProc->update_obj_where('stage_host_services',\%values,\%where);
			if ($result =~ /^Error/) { push @errors, $result }	
			my %w = ('servicename_id' => $s{'servicename_id'});
			my @externals = StorProc->fetch_list_where('external_service_names','external_id',\%w);
			foreach my $ext (@externals) {
				my %e = StorProc->fetch_one('externals','external_id',$ext);
				my @vals = ($ext,$properties{'host_id'},$id,$e{'display'});
				$result = StorProc->insert_obj('external_service',\@vals);
				if ($result =~ /^Error/) { push @errors, $result }
			}
			# insert dependency
			if ($service_detail{'dependency'}) {
				my %d =StorProc->fetch_one('service_dependency_templates','name',$service_detail{'dependency'});
				@values = ('',$id,$properties{'host_id'},$properties{'host_id'},$d{'id'},'');
				$result = StorProc->insert_obj('service_dependency',\@values);
				if ($result =~ /^Error/) { push @errors, $result }	
			}
			if (@errors) {
				$step = 'service_detail';
			} else {
				delete $service_detail{'service'};
				delete $service_detail{'template'};
				delete $service_detail{'command'};
				delete $service_detail{'command_line'};
				delete $service_detail{'use_template_command'};
				delete $service_detail{'ext_info'};
				delete $service_detail{'escalation'};
				delete $service_detail{'dependency'};
				delete $service_detail{'command_save'};
				my %values = ('user_acct' => $user_acct,'status' => '2','type' => $view,'host' => $name);
				my %service_list = StorProc->fetch_list_hash_array('stage_host_services',\%values);
				my @services;
				foreach my $svc (keys %service_list) { unless ($svc =~ /HASH/) { push @services, $svc } }
				@services = sort { $b cmp $a } @services;
				$service_next = pop @services;
				if ($service_next) {
					$step = 'service_detail';
				} else {
					$step = 'completed';
					my $result = StorProc->delete_all('stage_hosts','name',$name);
					if ($result =~ /^Error/) { push @errors, $result }
					$result = StorProc->delete_all('stage_host_services','host',$name);
					if ($result =~ /^Error/) { push @errors, $result }
				}
			}
		}
	} elsif ($query->param('add_profile') || $query->param('add_service')) {
		$step = 'service_list';
	}
	if ($step eq 'completed') {
		$refresh_left = "top_menu=$top_menu&selected=$name";
	}
#############################################################################
# Pages
#
	$hidden{'step'} = $step;
	if ($step eq 'host_vitals') {
		$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
		my %docs = Doc->host_wizard_vitals();
		delete $hidden{'name'};
		delete $hidden{'host'};
		$form .= Forms->form_top('New Host Wizard');
		if (!$properties{'host_profile'}) { $properties{'host_profile'} = $query->param('host_profile') }
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->form_doc('Host Vitals');
		$form .= Forms->text_box('Name:','name',$properties{'name'},$textsize{'name'},$required{'name'});
		$form .= Forms->text_box('Alias:','alias',$properties{'alias'},$textsize{'alias'},$required{'alias'},$docs{'alias'});
		$form .= Forms->text_box('Address:','address',$properties{'address'},$textsize{'address'},$required{'address'},$docs{'address'});
		my @profiles =  StorProc->fetch_list('profiles_host','name'); 
		$form .= Forms->list_box('Host profile:','host_profile',\@profiles,$properties{'host_profile'},$required{'host_profile'},$docs{'host_profile'});
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%next,\%cancel,$tab++);
	} elsif ($step eq 'host_attribs') {	
		$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
		my %docs = Doc->host_wizard_attribs_1();
		my %w = ('hosttemplate_id' => $properties{'hosttemplate_id'});
		my %t = StorProc->fetch_one_where('host_templates',\%w);
		my $template = $t{'name'};
		unless ($template) {
			my %w = ('hosttemplate_id' => $profile{'host_template_id'});
			my %t = StorProc->fetch_one_where('host_templates',\%w);
			$template = $t{'name'};
		}
		my @parents = StorProc->get_host_parent($properties{'host_id'});
		unless (@parents) {
			@parents = StorProc->get_profile_parent($profile{'hostprofile_id'});
		}
		my $file = $query->param('file');
		unless ($file) {
			$file = StorProc->get_file_name('file_host','host_id',$properties{'host_id'});
		}
		unless ($file) {
			my %f = StorProc->fetch_one('files','file_id',$profile{'file_id'});
			$file = $f{'name'};
		}
		$form .= Forms->form_top('New Host Wizard','1');
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->form_doc('Host Properties 1');
		$form .= Forms->display_hidden('Name:','name',$name);
		my @members = StorProc->fetch_list('host_templates','name'); 
		$form .= Forms->list_box('Host template:','host_template',\@members,$template,$required{'host_template'},$docs{'host_template'});
		my @nonmembers = StorProc->fetch_list('hosts','name');
		$form .= Forms->members('Parents:','parents',\@parents,\@nonmembers,'','',$docs{'parents'});
		my %where = ('type' => 'hosts');
		my @files = StorProc->fetch_list_where('files','name',\%where); 
		$form .= Forms->list_box('File:','file',\@files,$file,$required{'file'},$docs{'file'});
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_three_button(\%back,\%next,\%cancel,$tab++);
	} elsif ($step eq 'host_attribs2') {	
		$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
		my %docs = Doc->host_wizard_attribs_2();
		my $coords2d = $query->param('coords2d');
		my $coords3d = $query->param('coords3d');
		my $extended_info = $query->param('extended_info');
		unless ($extended_info) {
			my %w = ('hostextinfo_id' => $properties{'hostextinfo_id'});
			my %t = StorProc->fetch_one_where('extended_host_info_templates',\%w);
			$extended_info = $t{'name'};
			my %coords = StorProc->fetch_one('extended_info_coords','host_id',$properties{'host_id'});
			unless ($coords2d) { $coords2d = $coords{'2d_coords'} }
			unless ($coords3d) { $coords3d = $coords{'3d_coords'} }
		}
		unless ($extended_info) {
			my %w = ('hostextinfo_id' => $profile{'host_extinfo_id'});
			my %t = StorProc->fetch_one_where('extended_host_info_templates',\%w);
			$extended_info = $t{'name'};
			unless ($coords2d) { $coords2d = $t{'2d_coords'} }
			unless ($coords3d) { $coords3d = $t{'3d_coords'} }
		}
		my $host_escalation = $query->param('host_escalation');
		unless ($host_escalation) {
			my %w = ('tree_id' => $properties{'host_escalation_id'},'type' => 'host');
			my %t = StorProc->fetch_one_where('escalation_trees',\%w);
			$host_escalation = $t{'name'};
		}
		unless ($host_escalation) {
			my %w = ('tree_id' => $profile{'host_escalation_id'},'type' => 'host');
			my %t = StorProc->fetch_one_where('escalation_trees',\%w);
			$host_escalation = $t{'name'};
		}
		my $service_escalation = $query->param('service_escalation');
		unless ($service_escalation) {
			my %w = ('tree_id' => $properties{'service_escalation_id'},'type' => 'service');
			my %t = StorProc->fetch_one_where('escalation_trees',\%w);
			$service_escalation = $t{'name'};
		}
		unless ($service_escalation) {
			my %w = ('tree_id' => $profile{'service_escalation_id'},'type' => 'service');
			my %t = StorProc->fetch_one_where('escalation_trees',\%w);
			$service_escalation = $t{'name'};
		}
		my @hostgroups = $query->param('hostgroups');
		unless (@hostgroups) {
			@hostgroups = StorProc->get_hostgroup_host($name);
		}
		unless (@hostgroups) {
			@hostgroups = StorProc->get_profile_hostgroup($profile{'hostprofile_id'});
		}
		$form .= Forms->form_top('New Host Wizard','1');
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->form_doc('Host Properties 2');
		$form .= Forms->display_hidden('Name:','name',$name);
		my @nonmembers = StorProc->fetch_list('hostgroups','name'); 
		$form .= Forms->members('Hostgroups:','hostgroups',\@hostgroups,\@nonmembers,'','',$docs{'hostgroups'});
		my @members = StorProc->fetch_list('extended_host_info_templates','name'); 
		$form .= Forms->list_box_submit('Extended host info:','extended_info',\@members,$extended_info,'',$docs{'extinfo'});
		$form .= Forms->text_box('2d status map coords:','coords2d',$coords2d,'10','',$docs{'coords2d'});
		$form .= Forms->text_box('3d status map coords:','coords3d',$coords3d,'10','',$docs{'coords3d'});
		my %where = ('type' => 'host');
		@members = StorProc->fetch_list_where('escalation_trees','name',\%where); 
		$form .= Forms->list_box('Host escalation tree:','host_escalation',\@members,$host_escalation,'',$docs{'host_escalation_tree'});
		if ($host_escalation) {
			my ($ranks, $templates) = StorProc->get_tree_detail($host_escalation);
			my %ranks = %{$ranks};
			my %templates = %{$templates};
			$form .= Forms->escalation_tree(\%ranks,\%templates,'escalations');
		}
		%where = ('type' => 'service');
		@members = StorProc->fetch_list_where('escalation_trees','name',\%where); 
		$form .= Forms->list_box_submit('Service escalation tree:','service_escalation',\@members,$service_escalation,'',$docs{'service_escalation_tree'});
		if ($service_escalation) {
			my ($ranks, $templates) = StorProc->get_tree_detail($service_escalation);
			my %ranks = %{$ranks};
			my %templates = %{$templates};
			$form .= Forms->escalation_tree(\%ranks,\%templates,'escalations');
		}
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_three_button(\%back,\%next,\%cancel,$tab++);
	} elsif ($step eq 'service_list') {
		$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
		my %docs = Doc->host_wizard_select_services();
		$form .= Forms->form_top('New Host Wizard','');
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->form_doc('Select Servcies');
		$form .= Forms->display_hidden('Host name:','name',$name);
		$form .= Forms->form_doc($docs{'services'});
		my %where = ('type' => 'services');
		my @files = StorProc->fetch_list_where('files','name',\%where); 
		my @profiles = ();
		my $got_profile = 0;
		my %selected = ();
		if ($query->param('back')) {
			my %where = ('host' => $name,'user_acct' => $user_acct);
			my @services = StorProc->fetch_list_where('stage_host_services','name',\%where);			
			foreach my $service (@services) { $selected{$service} = 'add' }
		} else {
			foreach my $service ($query->param) {		
				$service = uri_unescape($service);
				if ($query->param($service) eq 'add') { 
					$selected{$service} = 'add';
				} elsif ($query->param($service) eq 'edit') { 
					$selected{$service} = 'edit';
				} elsif ($query->param($service) eq 'discard') { 
					$selected{$service} = 'discard';
				}
			}	
		}
		my %sp_selected = ();
		my $remove_profile = undef;
		my @query_profiles = $query->param('profiles');
		foreach my $profile (@query_profiles) {
			$got_profile = 1;
			if ($query->param("remove_$profile")) {
				$profile = uri_unescape($profile);
				$remove_profile = $profile;
			} elsif ($profile) {
				$profile = uri_unescape($profile);
				push @profiles, $profile; 
			}
		}
		unless ($got_profile) {
			my %sps = StorProc->get_host_profile_service_profiles($properties{'hostprofile_id'});
			foreach my $sp (sort keys %sps) { push @profiles, $sp }
		}
		$form .= Forms->profile_list($user_acct,$name,\@profiles,$tab++);
		my $file = $query->param('file');
		unless ($file) {
			$file = StorProc->get_file_name('profiles_service','name',$profiles[0]);
		}
		$form .= Forms->list_box('File:','file',\@files,$file,$required{'file'},$docs{'file'});
		my @query_services = $query->param('services');
		foreach my $service (@query_services) {
			if ($service) { $selected{$service} = 'add' }
		}
		if ($remove_profile) {
			my %sp = StorProc->fetch_one('profiles_service','name',$remove_profile);
			my %svs = StorProc->get_service_profile_services($sp{'serviceprofile_id'});
			foreach my $service (keys %svs) { delete $selected{$service} }
		}	
		my %services = ();
		foreach my $profile (@profiles) {
			$sp_selected{$profile} = 1;
			my %sp = StorProc->fetch_one('profiles_service','name',$profile);
			my %svs = StorProc->get_service_profile_services($sp{'serviceprofile_id'});
			foreach my $service (keys %svs) { 
				$services{$service} = $svs{$service};
				unless (defined $selected{$service}) { $selected{$service} = 'add' }
			}
		}
		foreach my $service (keys %selected) {
			unless (defined $services{$service}) {
				my %svc = StorProc->get_service_detail($service);
				$services{$service} = $svc{$service};
			}
		}
		$form .= Forms->service_select(\%services,\%selected);
		my @service_profiles = StorProc->fetch_list('profiles_service','name');
		my @profile_list = ();
		foreach my $s (@service_profiles) { unless ($sp_selected{$s}) { push @profile_list, $s } }
		@profile_list = sort { $a <=> $b } @profile_list;
		$form .= Forms->add_service_profile(\@profile_list);
		
		my @service_names = StorProc->fetch_list('service_names','name');
		my @service_list = ();
		foreach my $s (@service_names) { unless ($selected{$s}) { push @service_list, $s } }
		@service_list = sort { $a <=> $b } @service_list;
		$form .= Forms->add_service(\@service_list);
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_three_button(\%back,\%next,\%cancel,$tab++);
	} elsif ($step eq 'service_detail') {
		$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
		my %docs = Doc->host_wizard_service_detail();
		$hidden{'file'} = $query->param('file');
		my $service = $service_next;
		unless ($service) { $service = $service_detail{'service'} }
		my %svc = StorProc->get_service_detail($service);
		my %service = ();
		foreach my $key (keys %{$svc{$service_next}}) {
			if ($key eq 'command') {
				my @cmd = split (/!/, $svc{$service_next}{$key});
				$service{'check_command'} = $cmd[0];
				$service{'command_line'} = $svc{$service_next}{$key};
			}
			$service{$key} = $svc{$service_next}{$key};
		}
		$form .= Forms->form_top('New Host Wizard');
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->form_doc('Service Detail');
		$form .= Forms->display_hidden('Service:','service',$service);

#
# Template
#
		my $template = $service_detail{'template'};
		unless ($template) {
			$template = $service{'template'};
		}
		my @members = StorProc->fetch_list('service_templates','name'); 
		$form .= Forms->list_box_submit('Service template:','template',\@members,$template,'',$docs{'template'});
		my %template = StorProc->fetch_one('service_templates','name',$template);

#
# Service check
#

		my $message = undef;
		my $inherit = 0;
		$inherit = $service_detail{'inherit'};
		my %cmd = StorProc->fetch_one('commands','name', $service{'check_command'});
		my $command = $cmd{'name'};
		my $command_save = $cmd{'name'};
		my $command_line = $service{'command_line'};
		if ($service_detail{'command'}) { $command = $service_detail{'command'} }
		if ($service_detail{'command_save'}) { $command_save = $service_detail{'command_save'} }
		if ($service_detail{'command_line'}) { $command_line = $service_detail{'command_line'} }
		unless ($command eq $command_save) {
			%cmd = StorProc->fetch_one('commands','name',$command);
			$command_line = undef;
		}
		if ($inherit or !$command) {
			%cmd = StorProc->fetch_one('commands','command_id',$template{'check_command'});
			if ($cmd{'name'}) {
				$command = $cmd{'name'};
				$command_line = $template{'command_line'};
				$inherit = 1;
			} else {
				my $got_command = 0;
				my $stid = $template{'parent_id'};
				until ($got_command) {
					my %t = StorProc->fetch_one('service_templates','servicetemplate_id',$stid);
					if ($t{'check_command'}) {
						$got_command = 1;
						%cmd = StorProc->fetch_one('commands','command_id',$t{'check_command'});
						$command = $cmd{'name'};
						$command_line = $t{'command_line'};
						$got_command = 1;
					} else {
						if ($t{'parent_id'}) {
							$stid = $t{'parent_id'};
						} else {
							$got_command = 1;
							$message = ('Note: a parent template does not have a check command defined.');
							$required{'check_command'} = 1;
							$command = undef;
							$command_line = undef;
						}
					}
				}
			}
		}
		unless ($cmd{'command_line'}) {
			%cmd = StorProc->fetch_one('commands','name',$command);
		}
		my $arg_string = $command_line;
		$arg_string =~ s/$command!//;
		my $usage = $command;
		my @args = split(/ARG/i,$cmd{'command_line'});
		my $args = undef;
		if ($args[1]) {
			$usage .= "!";
			my $cnt = 1;
			pop @args;
			foreach (@args) { 
				if ($cmd{'command_line'} =~ /ARG$cnt/i) { $args .= "ARG$cnt!" }
				$cnt++;
			}
			chop $args;
			$usage .= $args;
			unless ($command_line =~ /$command/) {
				$command_line = "$command!$args";
				$arg_string = $args;
			}
		}
		$form .= Forms->checkbox_override('Inherit check from template','inherit',$inherit,$docs{'override'});
		my %where = ('type' => 'check');
		my @commands = StorProc->fetch_list_where('commands','name',\%where);
		$form .= Forms->list_box_submit('Check command:','command',\@commands,$command,$required{'check_command'});
		$form .= Forms->display_hidden('Command definition:','',$cmd{'command_line'});
		$form .= Forms->display_hidden('Usage:','',$usage);
		$form .= Forms->text_area('Command line:','command_line',$command_line,'3','80','',$docs{'command_line'});
		unless ($host) { $host = $name }
		$form .= Forms->command_test($test_results,$host,$arg_string);
		$hidden{'command_save'} = $command;

#
# dependency
#
		my $dependency = $service_detail{'dependency'};
		unless ($dependency) {
			$dependency = $service{'dependency'};
		}
		@members = StorProc->fetch_list('service_dependency_templates','name'); 
		$form .= Forms->list_box('Service dependency:','dependency',\@members,$dependency,'',$docs{'dependency'});
		my $ext_info = $service_detail{'ext_info'};
		unless ($ext_info) {
			$ext_info = $service{'extinfo'};
		}
#
# ext_info
#
		my @members = StorProc->fetch_list('extended_service_info_templates','name'); 
		$form .= Forms->list_box('Extended info:','ext_info',\@members,$ext_info,'',$docs{'extinfo'});
#
# escalation
#
		my $escalation = $service_detail{'escalation'};
		unless ($escalation) {
			$escalation = $service{'escalation'};
		}
		my %where = ('type' => 'service');
		@members = StorProc->fetch_list_where('escalation_trees','name',\%where); 
		$form .= Forms->list_box_submit('Escalation tree:','escalation',\@members,$escalation,'',$docs{'escalation'});
		if ($escalation) {
			my ($ranks, $templates) = StorProc->get_tree_detail($escalation);
			my %ranks = %{$ranks};
			my %templates = %{$templates};
			$form .= Forms->escalation_tree(\%ranks,\%templates,'service_detail');
		}
		my @profiles = $query->param('profiles');
		foreach my $profile (@profiles) {
			my %prof = ('profiles' => $profile);
			$form .= Forms->hidden(\%prof);
		}
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_three_button(\%back,\%next,\%cancel,$tab++);

	} elsif ($step eq 'completed') {
		$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
		$form .= Forms->form_top('New Host Wizard','');
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->display_hidden('Host name:','name',$name);
		my @services = StorProc->get_host_services($properties{'host_id'});
		$form .= Forms->form_message('Services:',\@services,'row1');
		my %save_to_profile = ('name' => 'save_as_profile','value' => 'Save As Profile');
		$form .= Forms->hidden(\%hidden);		
		$form .= Forms->form_bottem_two_button(\%save_to_profile,\%continue,$tab++);
	} elsif ($step eq 'save_as_profile') {
		$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
		$form .= Forms->form_top('New Host Wizard','');
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->display_hidden('Host name:','name',$name);
		$form .= Forms->text_box('Host profile name:','host_profile',$name,'50');
		$form .= Forms->text_box('Service profile name:','service_profile',$name,'50');
		my %bail = ('name' => 'bail','value' => 'Cancel');
		$form .= Forms->hidden(\%hidden);		
		$form .= Forms->form_bottem_two_button(\%save,\%bail,$tab++);
	} elsif ($step eq 'saved') {
		$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
		$form .= Forms->form_top('New Host Wizard','');
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->display_hidden('Profile name:','name',$name);
		$form .= Forms->hidden(\%hidden);		
		$form .= Forms->form_bottem_one_button(\%continue,$tab++);
	}
	return $form;
}

#
###########################################################################################################################
# Design
#

sub design() {
	my %files = StorProc->fetch_file_hash();
	my $page = undef;
	my $got_form = undef;
	my $title = undef;
	my @t_parse = split(/_/, $obj);
	my %required = ();
	foreach (@t_parse) { $title .= "\u$_ " }
	if ($query->param('cancel') || $query->param('close') || $query->param('continue')) { 
		$page .= Forms->header($page_title,$user_acct,$view);
		$got_form = 1;
	} else {
#
# Contacts
#
		if ($obj eq 'contacts') {
			if ($query->param('add')) {
				my $name = $query->param('name');
				$hidden{'name'} = $name;
				my %data = parse_query('contacts','contacts');
				$properties{'template'} = $query->param('template');
				my $field_ck = check_fields();
				if ($field_ck == 0) {
					my %t = StorProc->fetch_one('contact_templates','name',$properties{'template'});
					my @vals = ('',$name,$data{'alias'},$data{'email'},$data{'pager'},$t{'contacttemplate_id'},'1','');
					my $id = StorProc->insert_obj_id('contacts',\@vals,'contact_id');
					if ($id =~ /^Error/) { push @errors, $id }
					unless (@errors) {		
						$refresh_left = "top_menu=$top_menu&selected=$name";
						my @mems = $query->param('contactgroup');
						foreach (@mems) {
							my %cg = StorProc->fetch_one('contactgroups','name',$_);
							my @vals = ($cg{'contactgroup_id'},$id);
							my $result = StorProc->insert_obj('contactgroup_contact',\@vals);
							if ($result =~ /^Error/) { push @errors, $result }
						}
						unless (@errors) {
							my %where = ('contact_id' => $id);
							my $result = StorProc->delete_one_where('contact_overrides',\%where);
							if ($result =~ /^Error/) { push @errors, $result }
							my %values = ();
							my $got_override = undef;
							unless ($query->param('host_notification_period_override')) {
								my $np = $query->param('host_notification_period');
								my %np = StorProc->fetch_one('time_periods','name',$np);
								$values{'host_notification_period'} = $np{'timeperiod_id'};
								$got_override = 1;
							}
							unless ($query->param('service_notification_period_override')) {
								my $np = $query->param('service_notification_period');
								my %np = StorProc->fetch_one('time_periods','name',$np);
								my %np = StorProc->fetch_one('time_periods','name',$np);
								$values{'service_notification_period'} = $np{'timeperiod_id'};
								$got_override = 1;
							}
							my $data = undef;
							unless ($query->param('host_notification_options_override')) {
								my @no = $query->param('host_notification_options');
								my $str = undef;
								foreach (@no) { $str .= "$_," }
								chop $str;
								$data .= "\n  <prop name=\"host_notification_options\"><![CDATA[$str]]>\n  </prop>";
								$got_override = 1;
							}
							unless ($query->param('service_notification_options_override')) {
								my @no = $query->param('service_notification_options');
								my $str = undef;
								foreach (@no) { $str .= "$_," }
								chop $str;
								$data .= "\n  <prop name=\"service_notification_options\"><![CDATA[$str]]>\n  </prop>";
								$got_override = 1;
							}
							if ($data) {
								$values{'data'} = "<?xml version=\"1.0\" ?>\n<data>".$data."\n</data>";
							}
							if ($got_override) {
								my @vals = ($id,$values{'host_notification_period'},$values{'service_notification_period'},$values{'data'});
								$result = StorProc->insert_obj('contact_overrides',\@vals);
								if ($result =~ /^Error/) { push @errors, $result }					
							}
							my %where = ('contact_id' => $id);
							my $result = StorProc->delete_one_where('contact_command_overrides',\%where);
							if ($result =~ /^Error/) { push @errors, $result }
							unless ($query->param('host_notification_commands_override')) {
								my @mems = $query->param('host_notification_commands');
								foreach (@mems) {
									my %c = StorProc->fetch_one('commands','name',$_);
									my @vals = ($id,'host',$c{'command_id'});
									$result = StorProc->insert_obj('contact_command_overrides',\@vals);
									if ($result =~ /^Error/) { push @errors, $result }					
								}
							}
							unless ($query->param('service_notification_commands_override')) {
								my @mems = $query->param('service_notification_commands');
								foreach (@mems) {
									my %c = StorProc->fetch_one('commands','name',$_);
									my @vals = ($id,'service',$c{'command_id'});
									$result = StorProc->insert_obj('contact_command_overrides',\@vals);
									if ($result =~ /^Error/) { push @errors, $result }					
								}
							}
							unless (@errors) {
								$refresh_left = "top_menu=$top_menu&selected=$name";
								$page .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
								my $message = "Contact $name added.";
								$page .= Forms->success('Added:',$message,'continue',\%hidden);
								$got_form = 1;
							}
						}
					}
				}
			}
			unless ($got_form) {
				$page .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
				$page .= Forms->form_top('Contact Properties','1');
				$page .= build_contact();
				$page .= Forms->hidden(\%hidden);
				$page .= Forms->form_bottem_two_button(\%add,\%cancel,$tab++);
				$got_form = 1;
			}

#
# Externals
#
		} elsif ($obj eq 'externals') {
			$page .= externals();
			$got_form = 1;

#
# Other
#
		} elsif ($query->param('add')) {
			my $result = undef;
			$table = $obj;
			my %data = parse_query($table,$obj);
			my $field_ck = check_fields();
			if ($field_ck == 0) {
				my @values;
				unless ($obj =~ /host_dependencies/) { push @values, ''}
				$data{'name'} =~ s/ /_/g;
				my @data_cols = split(/,/, $db_values{$table});
				foreach my $val (@data_cols) {
					if ($val eq 'comment') { 
						push @values, "# $obj $properties{'name'}";
					} else {
						push @values, $data{$val};
					}
				}
				my $id = StorProc->insert_obj_id($table,\@values,$obj_id{$table});
				if ($id =~ /^Error/) { 
					if ($id =~ /duplicate/i) {
						$id =~ s/ies/y/;
						my $ltitle = $obj;
						$ltitle =~ s/_/ /g;
						$ltitle =~ s/ies$/y/;
						push @errors, "Error: \u$ltitle already exists";
					} else {
						push @errors, $id; 		
					}
				} else {
					$refresh_left = "top_menu=$top_menu&selected=$data{'name'}";
					if ($data{'members'}) {
						my @mems = split(/,/, $data{'members'});
						foreach (@mems) {
							my %m = StorProc->fetch_one('hosts','name',$_);
							my @vals = ($id,$m{'host_id'});
							$result = StorProc->insert_obj('hostgroup_host',\@vals);
							if ($result =~ /^Error/) { push @errors, $result }
						}
					}

					if ($data{'contact'}) {
						my @mems = split(/,/, $data{'contact'});
						foreach (@mems) {
							my %c = StorProc->fetch_one('contacts','name',$_);
							my @vals = ($id,$c{'contact_id'});
							$result = StorProc->insert_obj('contactgroup_contact',\@vals);
							if ($result =~ /^Error/) { push @errors, $result }
						}
					}

					if ($data{'contactgroup'}) {
						my %o = StorProc->fetch_one($table,'name',$data{'name'});
						my @mems = split(/,/, $data{'contactgroup'});
						foreach (@mems) {
							my %cg = StorProc->fetch_one('contactgroups','name',$_);
							my @vals = ($cg{'contactgroup_id'},$obj,$id);
							$result = StorProc->insert_obj('contactgroup_assign',\@vals);
							if ($result =~ /^Error/) { push @errors, $result }
						}
					}

					if ($data{'host_notification_commands'}) {
						my %o = StorProc->fetch_one($table,'name',$data{'name'});
						my %w = ('contacttemplate_id' => $o{'contacttemplate_id'},'type' => 'host');
						my $result = StorProc->delete_one_where('contact_command',\%w);
						my @c = split(/,/, $data{'host_notification_commands'});		
						foreach (@c) {
							my @vals = ($o{'contacttemplate_id'},'host',$_);
							my $result = StorProc->insert_obj('contact_command',\@vals);
							if ($result =~ /Error/) { push @errors, $result }
						}	
					}

					if ($data{'service_notification_commands'}) {
						my %o = StorProc->fetch_one($table,'name',$data{'name'});
						my %w = ('contacttemplate_id' => $o{'contacttemplate_id'},'type' => 'service');
						my $result = StorProc->delete_one_where('contact_command',\%w);
						my @c = split(/,/, $data{'service_notification_commands'});		
						foreach (@c) {
							my @vals = ($o{'contacttemplate_id'},'service',$_);
							my $result = StorProc->insert_obj('contact_command',\@vals);
							if ($result =~ /Error/) { push @errors, $result }
						}	
					}
				}
				unless (@errors) {
					$page .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
					my $message = "$data{'name'} has been saved.";
					$hidden{'name'} = undef;
					$hidden{'task'} = undef;
					$page .= Forms->success('Added',$message,'continue',\%hidden);
					$got_form = 1;
				} else {
					$page .= Forms->header($page_title,$user_acct,$view);
					$page .= build_form();
					$got_form = 1;	
				}
			} elsif ($field_ck == 1) {
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= build_form();
				$got_form = 1;
			}
		}
		if ($query->param('bail')) {
			$got_form = 0;
		}
		unless ($got_form) {
			$page .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
			my $title = undef;
			my $l_title = undef;
			my @t_parse = split(/_/, $obj);
			foreach (@t_parse) { 
				$l_title .= "$_ ";
				$title .= "\u$_ ";
			}
			$l_title =~ s/s\s$//;
			$l_title =~ s/ie$/y/;
			my $template = $query->param('source');
			if ($obj eq 'contacts' || $task || $template) {
				if (!$template && $task =~ /copy|use/) {
					$page .= Forms->form_top('Select','');
					my @templates = StorProc->fetch_list($table,'name'); 
					$page .= Forms->list_box("\u$l_title:",'source',\@templates);
					$hidden{'task'} = $task;
					$page .= Forms->hidden(\%hidden);
					$page .= Forms->form_bottem_two_button(\%next,\%cancel,$tab++);
				} else {
					$page .= build_form();
				}
			} elsif ($obj eq 'contacts') {
				$page .= Forms->form_top('Select','');
				my @templates = StorProc->fetch_list('contact_templates','name'); 
				$page .= Forms->list_box('Contact template:','source',\@templates);
				$hidden{'task'} = 'use';
				$page .= Forms->hidden(\%hidden);
				$page .= Forms->form_bottem_two_button(\%next,\%cancel,$tab++);
			}
		}
	}
	return $page;
}


#
# host_dependencies
#

sub host_dependencies() {
	my $got_form = 0;
	my $page = undef;
	my $dependent_host = $query->param('dependent_host');
	my $parent_host = $query->param('parent_host');
	unless ($dependent_host && $parent_host) {
		my $name = $query->param('name');
		my @dp = split(/::--::/, $name);
		$dependent_host = $dp[0];
		$parent_host = $dp[1];
	}
	my %d = StorProc->fetch_one('hosts','name',$dependent_host);
	my %p = StorProc->fetch_one('hosts','name',$parent_host);
	if ($query->param('cancel') || $query->param('continue')) {
		$page .= Forms->header($page_title,$user_acct,$view);
		$got_form = 1;
	} elsif ($query->param('delete') or $query->param('confirm_delete')) {
		if ($query->param('confirm_delete')) {
			my %where = ('host_id' => $d{'host_id'},'parent_id' => $p{'host_id'});
			my $result = StorProc->delete_one_where('host_dependencies',\%where); 
			if ($result =~ /^Error/) { push @errors, $result }		
			unless (@errors) {
				my $message = "Host dependency $dependent_host - $parent_host.";
				$hidden{'name'} = undef;
				$refresh_left = "top_menu=$top_menu&selected=$dependent_host-$parent_host";
				$page .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
				$page .= Forms->success('Removed',$message,'continue',\%hidden);
				$got_form = 1;
			}
		} else {
			$hidden{'dependent_host'} = $dependent_host;
			$hidden{'parent_host'} = $parent_host;
			$page .= Forms->header($page_title,$user_acct,$view);
			my $message = qq(Are you sure you want to remove the host dependency for $dependent_host and $parent_host?);
			$page .= Forms->are_you_sure('Confirm Delete:',$message,'confirm_delete',\%hidden);
			$got_form = 1;
		}
	} elsif ($query->param('save') || $query->param('add')) {
		my @execution = $query->param('execution_failure_criteria');
		my $efc = undef;
		foreach my $n (@execution) {
			$efc .= "$n,";
		}
		chop $efc;
		my $data = qq(
  <prop name="execution_failure_criteria"><![CDATA[$efc]]>
  </prop>);
		my @notify = $query->param('notification_failure_criteria');
		my $nfc = undef;
		foreach my $n (@notify) {
			$nfc .= "$n,";
		}
		chop $nfc;
		$data .= qq(
  <prop name="notification_failure_criteria"><![CDATA[$nfc]]>
  </prop>);
		my $inherits_parent = $query->param('inherits_parent');
		if ($inherits_parent) {
			$data .= qq(
  <prop name="inherits_parent"><![CDATA[$inherits_parent]]>
  </prop>);
		}
		$data = "<?xml version=\"1.0\" ?>\n<data>\n$data\n</data>";
		if ($notify[0] && $p{'host_id'} && $d{'host_id'}) {
			if ($query->param('save')) {		
				my %values = ('data' => $data);
				my %where = ('host_id' => $d{'host_id'},'parent_id' => $p{'host_id'});
				my $result = StorProc->update_obj_where('host_dependencies',\%values,\%where);
				if ($result =~ /^Error/) { push @errors, $result }
				unless (@errors) {
					my $message = "Changes to host dependency $dependent_host-$parent_host have been saved.";
					$hidden{'name'} = undef;
					$page .= Forms->header($page_title,$user_acct,$view);
					$page .= Forms->success('Updated',$message,'continue',\%hidden);
					$refresh_left = "top_menu=$top_menu&selected=$dependent_host-$parent_host";
					$got_form = 1;
				}
			} else {
				my %where = ('parent_id' => $p{'host_id'},'host_id' => $d{'host_id'});
				my %dep = StorProc->fetch_one_where('host_dependencies',\%where);
				unless ($dep{'host_id'}) {
					my @vals = ($d{'host_id'},$p{'host_id'},$data,'');
					my $result = StorProc->insert_obj('host_dependencies',\@vals);
					if ($result =~ /^Error/) { push @errors, $result }	
					unless (@errors) {
						my $message = "Added: host dependency $dependent_host - $parent_host saved.";
						$hidden{'name'} = undef;
						$refresh_left = "top_menu=$top_menu&selected=$dependent_host-$parent_host";
						$page .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
						$page .= Forms->success('Updated',$message,'continue',\%hidden);
						$got_form = 1;
					}
				} else {
					push @errors, "Already exixts: host dependency $dependent_host - $parent_host";
				}
			}
		} else {
			push @errors, "Please correct missing fields (All are required)";
		}
	}
	unless ($got_form) {
		$page .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
		my @props = ('dependent_host','parent_host','inherits_parent','execution_failure_criteria','notification_failure_criteria');
		my %docs = Doc->properties_doc('host_dependencies',\@props);
		if ($query->param('task') eq 'new') {
			my @hosts = StorProc->fetch_list('hosts','name');
			$page .= Forms->form_top('Host Dependency Properties','');
			if (@errors) { $page .= Forms->form_errors(\@errors) }
			$page .= Forms->list_box("Dependent host:",'dependent_host',\@hosts,$properties{'dependent_host'},$required{'dependent_host'},$docs{'dependent_host'});
			$page .= Forms->list_box("Parent:",'parent_host',\@hosts,$properties{'parent_host'},$required{'parent_host'},$docs{'parent_host'});
			my @opts = split(/,/, $properties{'notification_failure_criteria'});
			$page .= Forms->failure_criteria('notification_failure_criteria',\@opts,'','host_dependencies',$docs{'notification_failure_criteria'});
			$page .= Forms->hidden(\%hidden);
			$page .= Forms->form_bottem_two_button(\%add,\%cancel,$tab++);
		} else {
			my @parents = StorProc->fetch_list('hosts','name');
			my %where = ('host_id' => $d{'host_id'},'parent_id' => $p{'host_id'});
			%properties = StorProc->fetch_one_where('host_dependencies',\%where);
			$page .= Forms->form_top('Host Dependency Properties','');
			if (@errors) { $page .= Forms->form_errors(\@errors) }
			$page .= Forms->display_hidden("Host:",'dependent_host',$dependent_host);
			$page .= Forms->display_hidden("Parent:",'parent_host',$parent_host);
			if ($nagios_ver eq '2.x') {
				my @opts = split(/,/, $properties{'execution_failure_criteria'});
				$page .= Forms->failure_criteria('execution_failure_criteria',\@opts,'','host_dependencies',$docs{'execution_failure_criteria'});
				$page .= Forms->checkbox('Inherits parent::','inherits_parent',$properties{'inherits_parent'},$docs{'inherits_parent'});
			}
			my @opts = split(/,/, $properties{'notification_failure_criteria'});
			$page .= Forms->failure_criteria('notification_failure_criteria',\@opts,'','host_dependencies',$docs{'notification_failure_criteria'});
			$page .= Forms->hidden(\%hidden);
			$page .= Forms->form_bottem_three_button(\%save,\%delete,\%cancel,$tab++);
		}
	}
	return $page;
}




#
# Clone host
#

sub clone_host() {
	my $got_form = 0;
	my $page = undef;
	if ($query->param('cancel')) {
		$page .= Forms->header($page_title,$user_acct,$view);
		$got_form = 1;
	} elsif ($query->param('add_clone_host')) {
		my $host = $query->param('host');
		my $name = $query->param('name');
		my $alias = $query->param('alias');
		my $address = $query->param('address');
		my %exists = StorProc->fetch_one('hosts','name',$name);
		if ($exists{'name'}) {	
			push @errors, "Host $name already exists.";
		} else {
			if ($host && $name && $alias && $address) {
				@errors = StorProc->clone_host($host,$name,$alias,$address);
				unless ($errors[0]) {
					$obj = 'hosts';
					$refresh_left = "top_menu=$top_menu&selected=$name";
					$page .= Forms->header($page_title,$user_acct,$view,'',$refresh_url);
					$hidden{'view'} = 'manage_host';
					$page .= manage_host($name,'name');	
					$got_form = 1;
				}
			} else {
				unless ($host) { $required{'host'} = 1 }
				unless ($name) { $required{'name'} = 1 }
				unless ($alias) { $required{'alias'} = 1 }
				unless ($address) { $required{'address'} = 1 }
				push @errors, "Check required fields.";
			}
		}
	}
	unless ($got_form) {
		$page .= Forms->header($page_title,$user_acct,$view);
		my %docs = Doc->host_wizard_vitals();
		my $host = $query->param('host');
		my $name = $query->param('name');
		my $alias = $query->param('alias');
		my $address = $query->param('address');
		$page .= Forms->form_top('Clone Host','');
		if (@errors) { $page .= Forms->form_errors(\@errors) }
		my @hosts = StorProc->fetch_list('hosts','name'); 
		$page .= Forms->text_box('Name:','name',$name,'50',$required{'name'});
		$page .= Forms->text_box('Alias:','alias',$alias,'70',$required{'alias'},$docs{'alias'});
		$page .= Forms->text_box('Address:','address',$address,'20',$required{'address'},$docs{'address'});
		$page .= Forms->list_box('Host:','host',\@hosts,$host,$required{'host'});
		$hidden{'view'} = 'clone_host';
		$page .= Forms->hidden(\%hidden);
		my %clone_host = ('name' => 'add_clone_host','value' => 'Clone Host');
		$page .= Forms->form_bottem_two_button(\%clone_host,\%cancel,$tab++);
	}
	return $page;
}

#
# Parent child
#

sub parent_child() {
	my $got_form = 0;
	my $page = undef;
	my $name = $query->param('name');
	my $parent = $query->param('parent');
	if ($query->param('continue') || $query->param('cancel')) {
		$page .= Forms->header($page_title,$user_acct,$view);
		$got_form = 1;
	} elsif ($query->param('save')) {
		my @children = $query->param('children');
		my %p = StorProc->fetch_one('hosts','name',$name);
		my %w = ('parent_id' => $p{'host_id'});
		my $result = StorProc->delete_one_where('host_parent',\%w);
		if ($result =~ /^Error/) { push @errors, $result }
		%p = StorProc->fetch_one('hosts','name',$parent);
		%w = ('parent_id' => $p{'host_id'});
		$result = StorProc->delete_one_where('host_parent',\%w);
		if ($result =~ /^Error/) { push @errors, $result }
		foreach my $child (@children) {
			if (!$child) { next }
			my %h = StorProc->fetch_one('hosts','name',$child);
			my @vals = ($h{'host_id'},$p{'host_id'});
			my $result = StorProc->insert_obj('host_parent',\@vals);
			if ($result =~ /^Error/) { push @errors, $result }
		}
		if ($query->param('task') eq 'new') {
			$refresh_left = "top_menu=$top_menu&selected=$parent";
		}
		unless (@errors) { 
			$page .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
			$page .= Forms->form_top('Saved','');
			$page .= Forms->display_hidden('Parent:','name',$parent);
			$page .= Forms->hidden(\%hidden);		
			$page .= Forms->form_bottem_one_button(\%continue,$tab++);
			$got_form = 1;
		}
	} elsif ($query->param('bail')) {
		$got_form = 0;
	} elsif ($query->param('delete') or $query->param('confirm_delete')) {
		if ($query->param('confirm_delete')) {
			$page .= Forms->header($page_title,$user_acct,$view);
			my %p = StorProc->fetch_one('hosts','name',$parent);
			my %w = ('parent_id' => $p{'host_id'});
			my $result = StorProc->delete_one_where('host_parent',\%w);
			if ($result =~ /^Error/) { 
				push @errors, $result;
			} else {
				$refresh_left = "top_menu=$top_menu";
				$page .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
				$page .= Forms->form_top('Deleted','');
				my @message = ("$parent");
				$page .= Forms->form_message('Removed:',\@message,'row1');			
				$page .= Forms->hidden(\%hidden);		
				$page .= Forms->form_bottem_one_button(\%continue,$tab++);
				$got_form = 1;	
			}
		} else {	
			$page .= Forms->header($page_title,$user_acct,$view);
			my $message = qq(Are you sure you want to remove parent $name? (Note: $name will still remain as a host).);
			$hidden{'name'} = $parent;
			$hidden{'parent'} = $parent;
			$page .= Forms->are_you_sure('Confirm Delete:',$message,'confirm_delete',\%hidden);
			$got_form = 1;
		}
	}
	unless ($got_form) {
		$page .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
		my @hosts = StorProc->fetch_list('hosts','name');
		my @parents = StorProc->get_parents();
		my @poss_parents = ();
		foreach my $host (@hosts) {
			my $got_parent = 0;
			foreach my $parent (@parents) {
				if ($host eq $parent) { $got_parent = 1 }
				if ($host eq $name) { $got_parent = 0 }
			}
			unless ($got_parent) { push @poss_parents, $host }
		}
		my %pid = StorProc->fetch_one('hosts','name',$name);
		my @children = StorProc->get_children($pid{'host_id'});
		$page .= Forms->header($page_title,$user_acct,$view);
		$page .= Forms->form_top('Parent Child','1');
		if (@errors) { $page .= Forms->form_errors(\@errors) }
		$hidden{'name'} = $name;
		$page .= Forms->list_box("Parent host:",'parent',\@poss_parents,$name);
		$page .= Forms->members('Children:','children',\@children,\@hosts,'','10');
		$hidden{'name'} = $name;
		$page .= Forms->hidden(\%hidden);
		$page .= Forms->form_bottem_three_button(\%save,\%delete,\%cancel,$tab++);		
	}
	return $page;
}

###########################################################################################################################
# Externals
#

sub externals($) {
	my $type = shift;
	my $form = undef;
	unless ($type) { $type = $query->param('type') }
	$hidden{'type'} = $type;
	my $task = $query->param('task');
	$hidden{'task'} = $task;
	my $name = $query->param('name');
	my $display =  $query->param('display');
	my $got_form = 0;
	$hidden{'update_main'} = 1;

	my %external =  StorProc->fetch_one('externals','name',$name);
	

	if ($query->param('continue') || $query->param('close') || $query->param('cancel')) {
		$form .= Forms->header($page_title,$user_acct,$view);
		$got_form = 1;
	} elsif ($query->param('add')) {
		if ($name && $type) {
			unless ($external{'name'}) {
				my @vals = ('',$name,'',$type,$display,'');
				my $result = StorProc->insert_obj('externals',\@vals);
				if ($result =~ /^Error/) { push @errors, $result }
			} else {
				push @errors, "Record $name already exists.";		
			}
		} else {
			push @errors, "Please fill in the required fields";		
			$required{'name'} = 1;
			$required{'type'} = 1;
		}
		unless (@errors) { 
			$refresh_left = "top_menu=$top_menu&selected=$name";
			$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
			$form .= Forms->form_top('Saved','');
			$form .= Forms->display_hidden('External:','name',$name);
			$form .= Forms->hidden(\%hidden);		
			$form .= Forms->form_bottem_one_button(\%continue,$tab++);
			$got_form = 1;
		}
	} elsif ($query->param('save')) {
		if ($type) {
			my %values = ('type' => $type,'display' => $display);
			my $result = StorProc->update_obj('externals','name',$name,\%values);
			if ($result =~ /^Error/) { push @errors, $result }
		} else {
			push @errors, "Please fill in the required fields";		
			$required{'type'} = 1;
		}
		unless (@errors) { 
			$form .= Forms->header($page_title,$user_acct,$view,'','');
			$form .= Forms->form_top('Saved','');
			$form .= Forms->display_hidden('External:','name',$name);
			$form .= Forms->hidden(\%hidden);
			my %apply = ('name' => 'apply','value' => 'Apply to Services');
			if ($type eq 'host') {
				%apply = ('name' => 'apply','value' => 'Apply to Hosts');
			}
			$form .= Forms->form_bottem_two_button(\%apply,\%continue,$tab++);
			$got_form = 1;
		}
	} elsif ($query->param('apply')) {
		my %w = ('external_id' => $external{'external_id'});
		my %values = ('data' => $external{'display'});
		if ($type eq 'service') {
			my $result = StorProc->update_obj_where('external_service',\%values,\%w);			
			if ($result =~ /^Error/) { push @errors, $result }
		} else {
			my $result = StorProc->update_obj_where('external_host',\%values,\%w);			
			if ($result =~ /^Error/) { push @errors, $result }
		}
		unless (@errors) {
			$form .= Forms->header($page_title,$user_acct,$view);
			$form .= Forms->form_top('Applied','');
			my @message = ("$name applied to $type\s");
			$form .= Forms->form_message('Saved:',\@message,'row1');			
			$form .= Forms->hidden(\%hidden);
			$form .= Forms->form_bottem_one_button(\%continue,$tab++);
			$got_form = 1;
		}
	} elsif ($query->param('delete') || $query->param('confirm_delete')) {
		foreach my $name ($query->param) {
			unless ($name eq 'nocache') { $hidden{$name} = $query->param($name) }
		}
		if ($query->param('confirm_delete')) {
			my $result = StorProc->delete_all('externals','external_id',$external{'external_id'});
			if ($result =~ /^Error/) { push @errors, $result }
			unless (@errors) { 
				$refresh_left = "top_menu=$top_menu";
				$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
				$form .= Forms->form_top('Deleted','');
				my @message = ("$name");
				$form .= Forms->form_message('Removed:',\@message,'row1');	
				$form .= Forms->hidden(\%hidden);
				$form .= Forms->form_bottem_one_button(\%continue,$tab++);
				$name = undef;
				$got_form = 1;	
			}
		} elsif ($query->param('task') eq 'No') {
			$got_form = 0;	
		} else {
			delete $hidden{'task'};
			$hidden{'delete'} = 1;
			$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_url);
			my $message = qq(Are you sure you want to remove $name? (Warning: $name will be removed from all $type\s and profiles).);
			$form .= Forms->are_you_sure('Confirm Delete:',$message,'confirm_delete',\%hidden);
			$name = undef;
			$got_form = 1;
		}
	}
	unless ($got_form) {
		$form .= Forms->header($page_title,$user_acct,$view);
		$form .= Forms->form_top("\u$type External",'');
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		if ($task eq 'new') {
			$form .= Forms->text_box('Name:','name',$name,$textsize{'name'},$required{'name'});
		} else {
			unless ($display) { $display = $external{'display'} }
			$form .= Forms->display_hidden('Name:','name',$name);
			delete $hidden{'task'};
		}
		$form .= Forms->display_hidden("Type:",'type',$type);
		$form .= Forms->text_area('Detail:','display',$display,'30','140');
		$form .= Forms->hidden(\%hidden);
		if ($task eq 'new') {
			$form .= Forms->form_bottem_two_button(\%add,\%cancel,$tab++);
		} else {
			$form .= Forms->form_bottem_three_button(\%save,\%delete,\%cancel,$tab++);
		}

	}	
	return $form;
}


###########################################################################################################################
# Escalations
#
sub escalations() {
	my $page = undef;
	my $type = $query->param('type');
	my $name = $query->param('name');
	$table = "escalation_templates";
	$hidden{'obj'} = $obj;
	$hidden{'type'} = $type;
	my $got_form = 0;
	my $escalation = $query->param('escalation');
	if ($query->param('continue') || $query->param('close') || $query->param('cancel')) {
		$page .= Forms->header($page_title,$user_acct,$view);
		$got_form = 1;
	} elsif ($query->param('add')) {
		my %data = parse_query($table,$obj);
		my $field_ck = check_fields();
		if ($field_ck == 0) {
			my @values = ('');
			my @data_cols = split(/,/, $db_values{$table});
			foreach my $val (@data_cols) {
				if ($val eq 'comment') { 
					push @values, "# $obj $properties{'name'}";
				} else {
					push @values, $data{$val};
				}
			}
			my %esc = StorProc->fetch_one('escalation_templates','name',$properties{'name'});
			if ($esc{'name'}) {
				push @errors, "An escalation with name $properties{'name'} already exists.";
			} else {
				my $result = StorProc->insert_obj('escalation_templates',\@values,$obj_id{$table});
				if ($result =~ /^Error/) { push @errors, $result }
			}		
			unless (@errors) {
				$refresh_left = "top_menu=$top_menu&selected=$data{'name'}";
				$page .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
				my $message = "$data{'name'} has been saved.";
				$hidden{'name'} = undef;
				$hidden{'task'} = undef;
				$page .= Forms->success('Added',$message,'continue',\%hidden);
				$got_form = 1;
			} else {
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= build_form();
				$got_form = 1;
			}
		} elsif ($field_ck == 1) {
			$page .= Forms->header($page_title,$user_acct,$view);
			$page .= build_form();
			$got_form = 1;
		}
	} elsif ($query->param('save')) {
		my %values = ();
		my %data = parse_query($table,$obj);
		my $field_ck = check_fields();
		if ($field_ck == 0) {
			foreach my $key (keys %data) {
				unless ($key =~ /HASH|^name$|^contactgroup$|members$/) {
					$values{$key} = $data{$key};					
				}
			}
			my $result = StorProc->update_obj('escalation_templates','name',$data{'name'},\%values);
			if ($result =~ /^Error/) { push @errors, $result }
			unless (@errors) {
				my $message = "Changes to $name have been saved.";
				$hidden{'name'} = undef;
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= Forms->success('Updated',$message,'continue',\%hidden);
				$got_form = 1;
			} else {
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= build_form();
				$got_form = 1;
			}
		} elsif ($field_ck == 1) {
			$page .= Forms->header($page_title,$user_acct,$view);
			$page .= build_form();
			$got_form = 1;
		}
	} elsif ($query->param('rename')) {
		$page .= rename_object('',$name);
		$got_form = 1;
	} elsif ($query->param('bail')) {
		$task = 'modify';
		$hidden{'task'} = 'modify';
	} elsif ($query->param('delete') or $query->param('confirm_delete')) {
		$hidden{'task'} = 'modify';
		$page .= Forms->header($page_title,$user_acct,$view);
		$page .= delete_object('',$name);
		$got_form = 1;
	}
	unless ($got_form) {
		if ($obj =~ /escalation_templates/) {
			$page .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
			$hidden{'type'} = $type;
			$hidden{'task'} = $task;
			$page .= build_form();
		}
	}
	return $page;
}

sub escalation_trees() {
	my @views = ('detail','assign_hostgroups','assign_hosts','assign_service_groups','assign_services');
	unless ($obj_view eq 'assign_contactgroups') {
		foreach my $v (@views) {
			if ($query->param($v)) { $obj_view = $v }
		}
	}
	foreach my $name ($query->param) {
		if ($name =~ /assign_contactgroups_(\d+)/) {
			$hidden{'id'} = $1;
			$obj_view = 'assign_contactgroups';
		} 
		if ($name =~ /remove_escalation_(\d+)/) {
			$hidden{'id'} = $1;
			$obj_view = 'remove_escalation';
		} 
	}
	my $message = undef;
	my $page = undef;
	my $type = $query->param('type');
	my $name = $query->param('name');
	$hidden{'obj'} = $obj;
	$hidden{'type'} = $type;
	my %tree = StorProc->fetch_one('escalation_trees','name',$name);
	my $escalation = $query->param('escalation');
	my $got_form = 0;
	if ($query->param('continue') || $query->param('close')) {
		$page .= Forms->header($page_title,$user_acct,$view);
		$got_form = 1;
	} elsif ($query->param('cancel')) {
		$obj_view = 'detail';
	} elsif ($query->param('add')) {
		my %t = StorProc->fetch_one('escalation_trees','name',$name);
		if ($t{'name'}) {
			push @errors,  "Duplicate: Escalation tree $name already exists.";
			$name = undef;
		} else {
			my @values = ('',$name,'',$type);
			my $result = StorProc->insert_obj('escalation_trees',\@values);
			if ($result =~ /^Error/) { 
				push @errors, $result;
			} else {
				$task = undef;
				delete $hidden{'task'};
				$refresh_left = "top_menu=$top_menu&selected=$name";
				$obj_view = 'detail';
			}
		}
	} elsif ($query->param('save')) {
		if ($obj_view eq 'assign_hostgroups') {
			my %w = ("$type\_escalation_id" => $tree{'tree_id'});
			my %u = ("$type\_escalation_id" => '');
			my $result = StorProc->update_obj_where('hostgroups',\%u,\%w);
			if ($result =~ /^Error/) { push @errors, $result }
			my %hostgroup_name = StorProc->get_table_objects('hostgroups');
			my @hostgroups = $query->param('hostgroups');
			foreach my $assn (@hostgroups) {
				my %values = ("$type\_escalation_id" => $tree{'tree_id'});
				my $result = StorProc->update_obj('hostgroups','hostgroup_id',$hostgroup_name{$assn},\%values);
				if ($result =~ /^Error/) { push @errors, $result }
			}
			unless (@errors) {
				$obj_view = 'saved';
				$message = "Hostgroups assigned.";
			}

		} elsif ($obj_view eq 'assign_hosts') {
			my %w = ("$type\_escalation_id" => $tree{'tree_id'});
			my %u = ("$type\_escalation_id" => '');
			my $result = StorProc->update_obj_where('hosts',\%u,\%w);
			if ($result =~ /^Error/) { push @errors, $result }
			my %host_name = StorProc->get_table_objects('hosts');
			my @hosts = $query->param('hosts');
			foreach my $assn (@hosts) {
				my %values = ("$type\_escalation_id" => $tree{'tree_id'});
				my $result = StorProc->update_obj('hosts','host_id',$host_name{$assn},\%values);
				if ($result =~ /^Error/) { push @errors, $result }
			}
			unless (@errors) {
				$obj_view = 'saved';
				$message = "Hosts assigned.";
			}
		} elsif ($obj_view eq 'assign_service_groups') {
			my %w = ('escalation_id' => $tree{'tree_id'});
			my %u = ('escalation_id' => '');
			my $result = StorProc->update_obj_where('servicegroups',\%u,\%w);
			if ($result =~ /^Error/) { push @errors, $result }
			my %servicegroup_name = StorProc->get_table_objects('servicegroups');
			my @servicegroups = $query->param('servicegroups');
			foreach my $assn (@servicegroups) {
				my %values = ('escalation_id' => $tree{'tree_id'});
				my $result = StorProc->update_obj('servicegroups','servicegroup_id',$servicegroup_name{$assn},\%values);
				if ($result =~ /^Error/) { push @errors, $result }
			}
			unless (@errors) {
				$obj_view = 'saved';
				$message = "Service groups assigned.";
			}
		} elsif ($obj_view eq 'assign_services') {
			my %w = ('escalation' => $tree{'tree_id'});
			my %u = ('escalation' => '');
			my $result = StorProc->update_obj_where('service_names',\%u,\%w);
			if ($result =~ /^Error/) { push @errors, $result }
			my %service_name = StorProc->get_table_objects('service_names');
			my @services = $query->param('service_names');
			foreach my $assn (@services) {
				my %values = ('escalation' => $tree{'tree_id'});
				my $result = StorProc->update_obj('service_names','servicename_id',$service_name{$assn},\%values);
				if ($result =~ /^Error/) { push @errors, $result }
				if ($query->param('apply_hosts')) {
					my %values = ('escalation_id' => $tree{'tree_id'});
					my $result = StorProc->update_obj('services','servicename_id',$service_name{$assn},\%values);
					if ($result =~ /^Error/) { push @errors, $result }
				}
			}
			unless (@errors) {
				$obj_view = 'saved';
				$message = "Services assigned.";
			}
		}
	} elsif ($query->param('rename')) {
		$page .= rename_object('',$name);
		$got_form = 1;
	} elsif ($query->param('bail')) {
		$task = 'modify';
		$obj_view = 'detail'
	} elsif ($query->param('delete') or $query->param('confirm_delete')) {
		$hidden{'task'} = 'modify';
		$page .= Forms->header($page_title,$user_acct,$view);
		$page .= delete_object('',$name);
		$got_form = 1;
	} elsif ($query->param('add_escalation')) {
		my $escalation = $query->param('escalation');
		if ($escalation) {
			my %tree = StorProc->fetch_one('escalation_trees','name',$name);
			my %esc = StorProc->fetch_one('escalation_templates','name',$escalation);
			my @values = ($tree{'tree_id'},$esc{'template_id'});	
			my $result = StorProc->insert_obj('escalation_tree_template',\@values);
			if ($result =~ /^Error/) { push @errors, $result }
			unless (@errors) {	
				$obj_view = 'assign_contactgroups';
				$hidden{'id'} = $esc{'template_id'};
			} else {
				$obj_view = 'detail';
			}
		} else {
			$obj_view = 'detail';
		}
	} elsif ($query->param('assign_contactgroups')) {
		my @contactgroups = $query->param('contactgroups');
		my $id = $query->param('id');
		if ($contactgroups[0]) {
			my %w = ('template_id' => $id,'tree_id' => $tree{'tree_id'});
			my $result = StorProc->delete_one_where('tree_template_contactgroup',\%w);
			if ($result =~ /^Error/) { push @errors, $result }
			foreach my $cg (@contactgroups) {
				my %c = StorProc->fetch_one('contactgroups','name',$cg);
				my @values = ($tree{'tree_id'},$id,$c{'contactgroup_id'});
				my $result = StorProc->insert_obj('tree_template_contactgroup',\@values);
				if ($result =~ /^Error/) { push @errors, $result }
			}
		} else {
			$required{'contactgroup'} = 1;
			push @errors, "Contact group required.";
		}
		unless (@errors) { 
			$obj_view = 'detail';
		} else {
			$obj_view = 'assign_contactgroups';
		}
	} elsif ($obj_view eq 'remove_escalation') {
		my %w = ('template_id' => $hidden{'id'},'tree_id' => $tree{'tree_id'});
		my $result = StorProc->delete_one_where('escalation_tree_template',\%w);
		if ($result =~ /^Error/) { push @errors, $result }
		%w = ('template_id' => $hidden{'id'},'tree_id' => $tree{'tree_id'});
		$result = StorProc->delete_one_where('tree_template_contactgroup',\%w);
		if ($result =~ /^Error/) { push @errors, $result }
		$obj_view = 'detail';
	}
	
	unless ($got_form) {
		my %docs = Doc->escalations();
		$page .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
		if ($obj_view eq 'new') {
			$page .= Forms->form_top('Escalation Tree','');
			if (@errors) { $page .= Forms->form_errors(\@errors) }
			$page .= Forms->text_box('Name:','name',$name,$textsize{'name'});
			$page .= Forms->display_hidden('Type:','type',$type);
			$page .= Forms->hidden(\%hidden);
			$page .= Forms->form_bottem_two_button(\%add,\%cancel,$tab++);		
		} elsif ($obj_view eq 'detail') {
			$page .= Forms->escalation_top($name,$user_acct,$obj_view,$type,$nagios_ver);
			my %tree = StorProc->fetch_one('escalation_trees','name',$name);
			my ($order, $first_notify, $members) = StorProc->get_tree_templates($tree{'tree_id'});
			my %members = %{$members};
			my %first_notify = %{$first_notify};
			my %order = %{$order};
			my %w = ('type' => $tree{'type'});
			my @templates = StorProc->fetch_list_where('escalation_templates','name',\%w);
			my @nonmembers = ();
			my %mems = ();
			my %ids = ();
			my %contact_groups = ();
			my @members = ();
			foreach my $oid ( sort keys %order) {
				my $id = $order{$oid};
				my %cg = ();
				my $type = "$tree{'type'}_escalation_templates";
				my @cgs = StorProc->get_tree_template_contactgroup($tree{'tree_id'},$id);
				foreach (@cgs) { 
					$contact_groups{$id} .= "$_, ";
				}
				$contact_groups{$id} =~ s/,\s$//;
				push @members, $members{$id}[1];
				$mems{$members{$id}[1]} = 1;
				$ids{$members{$id}[1]} = $id;
			}
			foreach my $temp (@templates) {
				unless ($mems{$temp}) {push @nonmembers, $temp} 
			}
			if (@errors) { $page .= Forms->form_errors(\@errors) }
			$page .= Forms->wizard_doc('Managing Escalations',$docs{'escalation_tree'});
			$page .= Forms->manage_escalation_tree($user_acct,$view,$type,$name,$tree{'tree_id'},\%ids,\@members,\@nonmembers,\%contact_groups, $first_notify);
			$page .= Forms->hidden(\%hidden);
			if ($auth_delete{'escalations'}) {
				$page .= Forms->form_bottem_three_button(\%delete,\%rename,\%close,$tab++);		
			} else {
				$page .= Forms->form_bottem_two_button(\%rename,\%close,$tab++);		
			}
		} elsif ($obj_view eq 'assign_contactgroups') {
			$page .= Forms->escalation_top($name,$user_acct,$obj_view,$type,$nagios_ver);
			unless ($hidden{'id'}) { $hidden{'id'} = $query->param('id') }
			my %esc = StorProc->fetch_one('escalation_templates','template_id',$hidden{'id'});
			$hidden{'obj_view'} = 'assign_contactgroups';
			$hidden{'type'} = $type;
			if (@errors) { $page .= Forms->form_errors(\@errors) }
			$page .= Forms->display_hidden('Escalation:','escalation',$esc{'name'});
			my @contact_groups = StorProc->fetch_list('contactgroups','name');
			my @assigned = StorProc->get_tree_template_contactgroup($tree{'tree_id'},$esc{'template_id'});
			$page .= Forms->wizard_doc('Assign Contact Groups',$docs{'contactgroup'});
			$page .= Forms->members('Contact groups:','contactgroups',\@assigned,\@contact_groups,$required{'contactgroup'},'','');
			$page .= Forms->hidden(\%hidden);
			$page .= Forms->form_bottem_one_button(\%assign_contactgroups,$tab++);
		} elsif ($obj_view eq 'assign_hostgroups') {
			my $doc = $docs{'host_hostgroup'};
			if ($type eq 'service') { $doc = $docs{'service_hostgroup'} }
			my @assigned = StorProc->get_escalation_assigned($tree{'tree_id'},$type,'hostgroups');
			my @unassigned = StorProc->fetch_list('hostgroups','name');
			$page .= Forms->escalation_top($name,$user_acct,$obj_view,$type,$nagios_ver);
			$page .= Forms->wizard_doc('Assign Hostgroups',$doc);
			$page .= Forms->members('Host groups:','hostgroups',\@assigned,\@unassigned,'','','');
			$hidden{'assign_hostgroups'} = 1;
			$page .= Forms->hidden(\%hidden);
			$page .= Forms->form_bottem_two_button(\%save,\%cancel,$tab++);		
		} elsif ($obj_view eq 'assign_hosts') {
			my $doc = $docs{'host_host'};
			if ($type eq 'service') { $doc = $docs{'service_host'} }
			my @assigned = StorProc->get_escalation_assigned($tree{'tree_id'},$type,'hosts');
			my @unassigned = StorProc->fetch_list('hosts','name');
			$page .= Forms->escalation_top($name,$user_acct,$obj_view,$type,$nagios_ver);
			$page .= Forms->wizard_doc('Assign Hosts',$doc);
			$page .= Forms->members('Hosts:','hosts',\@assigned,\@unassigned,'','','');
			$hidden{'assign_hosts'} = 1;
			$page .= Forms->hidden(\%hidden);
			$page .= Forms->form_bottem_two_button(\%save,\%cancel,$tab++);		
		} elsif ($obj_view eq 'assign_service_groups') {
			my @assigned = StorProc->get_escalation_assigned($tree{'tree_id'},'service','servicegroups');
			my @unassigned = StorProc->fetch_list('servicegroups','name');
			$page .= Forms->escalation_top($name,$user_acct,$obj_view,$type,$nagios_ver);
			$page .= Forms->wizard_doc('Assign Service Groups',$docs{'servicegroup'});
			$page .= Forms->members('Servie groups:','servicegroups',\@assigned,\@unassigned,'','','');
			$hidden{'assign_service_groups'} = 1;
			$page .= Forms->hidden(\%hidden);
			$page .= Forms->form_bottem_two_button(\%save,\%cancel,$tab++);		
		} elsif ($obj_view eq 'assign_services') {
			my @assigned = StorProc->get_escalation_assigned($tree{'tree_id'},'service','services');
			my @unassigned = StorProc->fetch_list('service_names','name');
			$page .= Forms->escalation_top($name,$user_acct,$obj_view,$type,$nagios_ver);
			$page .= Forms->wizard_doc('Assign Services',$docs{'service'});
			$page .= Forms->checkbox_left('Apply to hosts','apply_hosts');
			$page .= Forms->members('Services:','service_names',\@assigned,\@unassigned,'','','');
			$hidden{'assign_services'} = 1;
			$page .= Forms->hidden(\%hidden);
			$page .= Forms->form_bottem_two_button(\%save,\%cancel,$tab++);		
		} elsif ($obj_view eq 'saved') {
			$page .= Forms->escalation_top($name,$user_acct,$obj_view,$type,$nagios_ver);
			$page .= Forms->display_hidden('Saved','',$message);				
			$page .= Forms->hidden(\%hidden);
			$page .= Forms->form_bottem_one_button(\%close,$tab++);		
		
		}
	}
	return $page;
}

#
###########################################################################################################################
# Profiles
#
###############################################
# Host profile
#	

sub host_profile() {
	my $name = $query->param('name');
	unless ($obj_view) { $obj_view = 'host_detail' }
	$hidden{'obj_view'} = $obj_view;
	my %profile = StorProc->fetch_one('profiles_host','name',$name);
	my $form = undef;
	if ($query->param('add')) {
		my $description = $query->param('description');
		my $template = $query->param('template');
		my $file = $query->param('file');
		unless ($profile{'name'} || $name eq '') {
			if ($template && $file) {
				$profile{'description'} = $description;
				my %t = StorProc->fetch_one('host_templates','name',$template);
				$profile{'hosttemplate_id'} = $t{'hosttemplate_id'};
				my %f = StorProc->fetch_one('files','name',$file);
				$profile{'file_id'} = $f{'file_id'};
				my @values = ('',$name,$description,$t{'hosttemplate_id'},'','','',$f{'file_id'},'');
				my $id = StorProc->insert_obj_id('profiles_host',\@values,'hostprofile_id');
				if ($id =~ /^Error/) { 
					push @errors, $id;
					%profile = ();
				} else {
					$profile{'hostprofile_id'} = $id;
					$refresh_left = "top_menu=$top_menu&selected=$name";
				}
			} else {
				push @errors, "Check required fields.";
				$required{'template'} = 1;
				$required{'file'} = 1;
			}
		} else {
			push @errors, "Check name field. It's either blank or a record already exists";
		}
		unless (@errors) { $obj_view = 'host_detail' }
	} elsif ($query->param('close') || $query->param('cancel') || $query->param('continue')) {
		$form .= Forms->header($page_title,$user_acct,$view);
		$obj_view = 'close';
	} elsif ($query->param('export')) {
		use MonarchProfileExport;
		my $results = ProfileExporter->host_profile($profile{'hostprofile_id'});
		if ($results =~ /^Error/) { 
			push @errors, $results;
		} else {
			$obj_view = 'exported';
		}
	} elsif ($query->param('save')) {
		if ($obj_view eq 'host_detail') {
			my %data = parse_query('host_templates','host_overrides');
			my $host_template = $query->param('template');
			my $extinfo = $query->param('extended_info');
			my $file = $query->param('file');
			if ($host_template && $file) {
				my %values = ();
				my %t = StorProc->fetch_one('host_templates','name',$host_template);
				$values{'host_template_id'} = $t{'hosttemplate_id'};
				my %e = StorProc->fetch_one('extended_host_info_templates','name',$extinfo);
				$values{'host_extinfo_id'} = $e{'hostextinfo_id'};
				my %f = StorProc->fetch_one('files','name',$file);
				$values{'file_id'} = $f{'file_id'};
				my $result = StorProc->update_obj('profiles_host','hostprofile_id',$profile{'hostprofile_id'},\%values);
				if ($result =~ /^Error/) { push @errors, $result }
				my %where = ('hostprofile_id' => $profile{'hostprofile_id'});
				my $result = StorProc->delete_one_where('hostprofile_overrides',\%where);
				if ($result =~ /^Error/) { push @errors, $result }
				if ($data{'check_period'} || $data{'notification_period'}|| $data{'check_command'} || $data{'event_handler'} || $data{'data'}) {
					my @values = ($profile{'hostprofile_id'},$data{'check_period'},$data{'notification_period'},$data{'check_command'},$data{'event_handler'},$data{'data'});
					my $result = StorProc->insert_obj('hostprofile_overrides',\@values);
					if ($result =~ /^Error/) { push @errors, $result }
				}
				if ($query->param('contactgroup_override')) {
					my %where = ('type' => 'host_profiles','object' => $profile{'hostprofile_id'});
					my $result = StorProc->delete_one_where('contactgroup_assign',\%where);
					if ($result =~ /^Error/) { push @errors, $result }
				} else {
					my %where = ('type' => 'host_profiles','object' => $profile{'hostprofile_id'});
					my $result = StorProc->delete_one_where('contactgroup_assign',\%where);
					if ($result =~ /^Error/) { push @errors, $result }
					my @mems = $query->param('contactgroup');
					foreach (@mems) {
						my %cg = StorProc->fetch_one('contactgroups','name',$_);
						my @vals = ($cg{'contactgroup_id'},'host_profiles',$profile{'hostprofile_id'});
						$result = StorProc->insert_obj('contactgroup_assign',\@vals);
						if ($result =~ /^Error/) { push @errors, $result }
					}
				}
			} else {
				push @errors, "Error: missing required fields.";
				$required{'file'} = 1;
				$required{'template'} = 1;
			}
			unless (@errors) { $obj_view = 'saved' }
		} elsif ($obj_view eq 'parents') {
			my @parents = $query->param('parents');
			my $result = StorProc->delete_all('profile_parent','hostprofile_id',$profile{'hostprofile_id'});
			if ($result =~ /^Error/) { push @errors, $result }
			foreach my $p (@parents) {
				if (!$p) { next }
				my %h = StorProc->fetch_one('hosts','name',$p);
				my @vals = ($profile{'hostprofile_id'},$h{'host_id'});
				$result = StorProc->insert_obj('profile_parent',\@vals);
				if ($result =~ /^Error/) { push @errors, $result }
			}
			unless (@errors) { 
				$obj_view = 'saved';
				$hidden{'obj_view'} = 'parents';
				$hidden{'apply'} = 0;
			}

		} elsif ($obj_view eq 'hostgroups') {
			my @hostgroups = $query->param('hostgroups');
			my $result = StorProc->delete_all('profile_hostgroup','hostprofile_id',$profile{'hostprofile_id'});
			if ($result =~ /^Error/) { push @errors, $result }
				foreach my $hg (@hostgroups) {
				if (!$hg) { next }
				my %h = StorProc->fetch_one('hostgroups','name',$hg);
				my @vals = ($profile{'hostprofile_id'},$h{'hostgroup_id'});
				$result = StorProc->insert_obj('profile_hostgroup',\@vals);
				if ($result =~ /^Error/) { push @errors, $result }
			}
			unless (@errors) { 
				$obj_view = 'saved';
				$hidden{'obj_view'} = 'hostgroups';
				$hidden{'apply'} = 0;
			}

		} elsif ($obj_view eq 'escalations') {
			my %values = ();
			my $host_escalation = $query->param('host_escalation');
			my %w = ('name' => $host_escalation,'type' => 'host');
			my %t = StorProc->fetch_one_where('escalation_trees',\%w);
			$values{'host_escalation_id'} = $t{'tree_id'};
			my $service_escalation = $query->param('service_escalation');
			%w = ('name' => $service_escalation,'type' => 'service');
			%t = StorProc->fetch_one_where('escalation_trees',\%w);
			$values{'service_escalation_id'} = $t{'tree_id'};
			my $result = StorProc->update_obj('profiles_host','name',$name,\%values);
			if ($result =~ /^Error/) { 
				push @errors, $result;
			} else {
				$obj_view = 'saved';
			}
		} elsif ($obj_view eq 'externals') {
			my @externals = $query->param('externals');
			my $result = StorProc->delete_all('external_host_profile','hostprofile_id',$profile{'hostprofile_id'});
			if ($result =~ /^Error/) { push @errors, $result }
			foreach my $e (@externals) {
				my %e = StorProc->fetch_one('externals','name',$e);
				my @vals = ($e{'external_id'},$profile{'hostprofile_id'});
				$result = StorProc->insert_obj('external_host_profile',\@vals);
				if ($result =~ /^Error/) { push @errors, $result }
			}
			unless (@errors) { 
				$obj_view = 'saved';
				$hidden{'obj_view'} = 'externals';
				$hidden{'apply'} = 1;
			}

		} elsif ($obj_view eq 'service_profiles') {
			my @service_profile = $query->param('service_profiles');
			my $result = StorProc->delete_all('profile_host_profile_service','hostprofile_id',$profile{'hostprofile_id'});
			if ($result =~ /^Error/) { push @errors, $result }
			foreach my $p (@service_profile) {
				my %s = StorProc->fetch_one('profiles_service','name',$p);
				my @vals = ($profile{'hostprofile_id'},$s{'serviceprofile_id'});
				$result = StorProc->insert_obj('profile_host_profile_service',\@vals);
				if ($result =~ /^Error/) { push @errors, $result }
			}
			unless (@errors) { $obj_view = 'saved' }
		} elsif ($obj_view eq 'assign_hosts') {
			my @hosts = $query->param('hosts');
			my %values = ('hostprofile_id' => '0');
			my $result = StorProc->update_obj('hosts','hostprofile_id',$profile{'hostprofile_id'},\%values);
			foreach my $host (@hosts) {
				my %values = ('hostprofile_id' => $profile{'hostprofile_id'});
				my $result = StorProc->update_obj('hosts','name',$host,\%values);
				if ($result =~ /^Error/) { push @errors, $result }
			}
			unless (@errors) { $obj_view = 'saved' }

		} elsif ($obj_view eq 'assign_hostgroups') {
			my @hostgroups = $query->param('hostgroups');
			my %values = ('hostprofile_id' => '0');
			my $result = StorProc->update_obj('hostgroups','hostprofile_id',$profile{'hostprofile_id'},\%values);
			foreach my $hostgroup (@hostgroups) {
				my %values = ('hostprofile_id' => $profile{'hostprofile_id'});
				my $result = StorProc->update_obj('hostgroups','name',$hostgroup,\%values);
				if ($result =~ /^Error/) { push @errors, $result }
			}
			unless (@errors) { $obj_view = 'saved' }
		}
	} elsif ($query->param('apply')) {
		my $hostgroups_select = $query->param('hostgroups_select');
		my $hosts_select = $query->param('hosts_select');
		my $apply_parents = $query->param('apply_parents');
		my $apply_hostgroups = $query->param('apply_hostgroups');
		my $apply_contactgroups = $query->param('apply_contactgroups');
		my $apply_escalations = $query->param('apply_escalations');
		my $apply_detail = $query->param('apply_detail');
		my $apply_services = $query->param('apply_services');

		my $data = "<?xml version=\"1.0\" ?>\n<data>";
		if ($hostgroups_select) { 
			$data .= qq(
  <prop name="hostgroups_select"><![CDATA[checked]]>
  </prop>);
		}
		if ($hosts_select) { 
			$data .= qq(
  <prop name="hosts_select"><![CDATA[checked]]>
  </prop>);
		}

		my %where = ('hostprofile_id' => $profile{'hostprofile_id'});
		my @profiles = StorProc->fetch_list_where('profile_host_profile_service','serviceprofile_id',\%where);
		my @hosts = ();
		if ($hostgroups_select) {
		my %where = ('hostprofile_id' => $profile{'hostprofile_id'});
			my @hgs = StorProc->fetch_list_where('hostgroups','hostgroup_id',\%where);
			foreach my $hg (@hgs) {
				%where = ('hostgroup_id' => $hg);
				my @hs = StorProc->fetch_list_where('hostgroup_host','host_id',\%where);
				push (@hosts,@hs);
			}
		}
		if ($hosts_select) {
			my %where = ('hostprofile_id' => $profile{'hostprofile_id'});
			my @hs  = StorProc->fetch_list_where('hosts','host_id',\%where);
			push (@hosts,@hs);
		}
		my %host = ();
		foreach my $hid (@hosts) {
			unless ($host{$hid}) {
				$host{$hid} = 1;
				if ($apply_parents) { 
					my $result = StorProc->delete_all('host_parent','host_id',$hid);
					if ($result =~ /^Error/) { push @errors, $result }
					my %w = ('hostprofile_id' => $profile{'hostprofile_id'});
					my @parents = StorProc->fetch_list_where('profile_parent','host_id',\%w);
					foreach my $pid (@parents) {
						my @vals = ($hid,$pid);
						my $result = StorProc->insert_obj('host_parent',\@vals);
						if ($result =~ /^Error/) { push @errors, $result }
					}
				}
				if ($apply_hostgroups) { 
					my $result = StorProc->delete_all('hostgroup_host','host_id',$hid);
					if ($result =~ /^Error/) { push @errors, $result }
					my %w = ('hostprofile_id' => $profile{'hostprofile_id'});
					my @hostgroups = StorProc->fetch_list_where('profile_hostgroup','hostgroup_id',\%w);
					foreach my $hgid (@hostgroups) {
						my @vals = ($hgid,$hid);
						my $result = StorProc->insert_obj('hostgroup_host',\@vals);
						if ($result =~ /^Error/) { push @errors, $result }
					}
				}
				if ($apply_contactgroups) { 
					my %w = ('type' => 'hosts','object' => $hid);
					my $result = StorProc->delete_one_where('contactgroup_assign',\%w);
					if ($result =~ /^Error/) { push @errors, $result }
					%w = ('type' => 'host_profiles','object' => $profile{'hostprofile_id'});
					my @contactgroups = StorProc->fetch_list_where('contactgroup_assign','contactgroup_id',\%w);
					foreach my $cgid (@contactgroups) {
						my @vals = ($cgid,'hosts',$hid);
						my $result = StorProc->insert_obj('contactgroup_assign',\@vals);
						if ($result =~ /^Error/) { push @errors, $result }
					}
				}
				if ($apply_services eq 'replace') {
					my $result = StorProc->delete_all('serviceprofile_host','host_id',$hid);							
					if ($result =~ /^Error/) { push @errors, $result }
					foreach my $spid (@profiles) {
						my @vals = ($spid,$hid);
						my $result = StorProc->insert_obj('serviceprofile_host',\@vals);
						if ($result =~ /^Error/) { push @errors, $result }
					}
				} else {
					foreach my $spid (@profiles) {
						my %p = StorProc->fetch_one('serviceprofile_host','serviceprofile_id',$spid);
						unless ($p{'host_id'}) {
							my @vals = ($spid,$hid);
							my $result = StorProc->insert_obj('serviceprofile_host',\@vals);
							if ($result =~ /^Error/) { push @errors, $result }
						}
					}
				}			
			}
		}
		if ($apply_parents) { 
			$data .= qq(
  <prop name="apply_parents"><![CDATA[checked]]>
  </prop>);
		}
		if ($apply_hostgroups) { 
			$data .= qq(
  <prop name="apply_hostgroups"><![CDATA[checked]]>
  </prop>);
		}
		if ($apply_contactgroups) { 
			$data .= qq(
  <prop name="apply_contactgroups"><![CDATA[checked]]>
  </prop>);
		}
		if ($apply_escalations) { 
			$data .= qq(
  <prop name="apply_escalations"><![CDATA[checked]]>
  </prop>);
			my %vals = ('host_escalation_id' => $profile{'host_escalation_id'});
			my $result = StorProc->update_obj('hosts','hostprofile_id',$profile{'hostprofile_id'},\%vals);
			if ($result =~ /^Error/) { push @errors, $result }
			%vals = ('service_escalation_id' => $profile{'service_escalation_id'});
			$result = StorProc->update_obj('hosts','hostprofile_id',$profile{'hostprofile_id'},\%vals);
			if ($result =~ /^Error/) { push @errors, $result }
		}
		if ($apply_detail) { 
			$data .= qq(
  <prop name="apply_detail"><![CDATA[checked]]>
  </prop>);
			my %vals = ('hosttemplate_id' => $profile{'host_template_id'});
			my $result = StorProc->update_obj('hosts','hostprofile_id',$profile{'hostprofile_id'},\%vals);
			if ($result =~ /^Error/) { push @errors, $result }
			my @errs = StorProc->host_profile_apply($profile{'hostprofile_id'},\@hosts);
			if (@errs) { push (@errors,@errs) }
			my %vals = ('hostextinfo_id' => $profile{'host_extinfo_id'});
			my $result = StorProc->update_obj('hosts','hostprofile_id',$profile{'hostprofile_id'},\%vals);
			if ($result =~ /^Error/) { push @errors, $result }
		}
		$data .= qq(
  <prop name="apply_services"><![CDATA[$apply_services]]>
  </prop>);

		my ($cnt, $err) = StorProc->service_profile_apply(\@profiles,$apply_services,\@hosts);
		if ($err) { push (@errors, @{$err}) }

		$data .= "\n</data>\n";
		my %value = ('data' => $data);
		my $result = StorProc->update_obj('profiles_host','name',$name,\%value);
		if ($result =~ /^Error/) { push @errors, $result }
		if ($enable_externals) {
			my %where = ('hostprofile_id' => $profile{'hostprofile_id'});
			my @externals = StorProc->fetch_list_where('external_host_profile','external_id',\%where);
			foreach my $hid (@hosts) {
				my $result = StorProc->delete_all('external_host','host_id',$hid);
				if ($result =~ /^Error/) { push @errors, $result }
				foreach my $ext (@externals) {
					my %e = StorProc->fetch_one('externals','external_id',$ext);
					my @vals = ($ext,$hid,$e{'display'});
					$result = StorProc->insert_obj('external_host',\@vals);
					if ($result =~ /^Error/) { push @errors, $result }
				}
			}
		}
		unless (@errors) { $obj_view = 'applied' }
	} elsif ($query->param('add_file')) {
		my $new_file = $query->param('new_file');
		my $type = $query->param('type');
		my $path = $query->param('path');
		if ($new_file && $path) {
			unless ($new_file =~ /cfg$/) { $new_file .= ".cfg" }
			my %f = StorProc->fetch_one('files','name',$new_file);
			unless ($f{'name'}) {
				my @vals = ('',$new_file,$path,$type);
				my $result = StorProc->insert_obj('files',\@vals);
				if ($result =~ /error/i) { 
					push @errors, $result;
				} else {
					$hidden{'file'} = $new_file;
					$hidden{'new_file'} = $new_file;
				}
			} else {
				push @errors, "File $new_file already exists.";
			}
		} else {
			push @errors, "Required: File name and path.";
			$required{'new_file'} = 1;
		}
	} elsif ($query->param('rename')) {
		if ($query->param('new_name')) {
			my $new_name = $query->param('new_name');
			my %n = StorProc->fetch_one('profiles_host','name',$new_name);
			if ($n{'name'}) {
				push @errors, "A host profile $new_name already exists.";
			} else {
				my %values = ('name' => $new_name);
				my $result = StorProc->update_obj('profiles_host','name',$name,\%values);				
				if ($result =~ /error/i) {
					push @errors, $result;
				} else {
					$name = $new_name;
					$refresh_left = "top_menu=$top_menu&selected=$name";
					$obj_view = 'host_detail';
				}
			}
		} else {
			$obj_view = 'rename';
		}
	} elsif ($query->param('delete') || $query->param('confirm_delete')) {
		my %w = ('hostprofile_id' => $profile{'hostprofile_id'});
		my @hids = StorProc->fetch_list_where('hosts','host_id',\%w);
		if ($query->param('confirm_delete')) {
			my %values = ('hostprofile_id' => '0');
			my $result = StorProc->update_obj('hosts','hostprofile_id',$profile{'hostprofile_id'},\%values);
			if ($result =~ /^Error/) { push @errors, $result }
			$result = StorProc->update_obj('hostgroups','hostprofile_id',$profile{'hostprofile_id'},\%values);
			if ($result =~ /^Error/) { push @errors, $result }
			$result = StorProc->delete_all('profiles_host','hostprofile_id',$profile{'hostprofile_id'});
			if ($result =~ /^Error/) { push @errors, $result }
			unless (@errors) { 
				$refresh_left = "top_menu=$top_menu";
				$obj_view = 'deleted';
			}
		} elsif ($query->param('task') eq 'No') {
			$obj_view = 'host_detail';
		} else {
			foreach my $name ($query->param) {
				unless ($name eq 'nocache') { $hidden{$name} = $query->param($name) }
			}
			delete $hidden{'task'};
			$obj_view = 'delete';
		}
	}
	my %docs = Doc->host_profile();
	my %save = ('name' => 'save','value' => 'Save');
	my %apply = ('name' => 'apply','value' => 'Apply');
	my %export = ('name' => 'export','value' => 'Export');
	my %obj = ();
	$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
	if ($obj_view eq 'host_detail') {
		$form .= Forms->host_profile_top($name,$user_acct,$obj_view,$auth_add{'externals'},\%obj);
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->text_box('Description:','description',$profile{'description'},$textsize{'description'},'',$docs{'description'});
		$form .= build_host_template($name);
		my @members = StorProc->fetch_list('extended_host_info_templates','name'); 
		my %w = ('hostextinfo_id' => $profile{'host_extinfo_id'});
		my %t = StorProc->fetch_one_where('extended_host_info_templates',\%w);
		$form .= Forms->list_box('Extended host info template:','extended_info',\@members,$t{'name'},'',$docs{'extended_host_info_template'});
		my %file = StorProc->fetch_one('files','file_id',$profile{'file_id'});
		my %where = ('type' => 'hosts');
		my @files = StorProc->fetch_list_where('files','name',\%where); 
		unless ($hidden{'file'}) { $hidden{'file'} = $file{'name'} }
		my $file = $file{'name'};
		if ($hidden{'new_file'}) {
			delete $hidden{'new_file'};
			$file = $hidden{'file'};
		}
		my $path = $query->param('path');
		$form .= Forms->add_file($file,'hosts',$path,\@files,'',$required{'new_file'},$docs{'file'});
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_five_button(\%save,\%delete,\%rename,\%export,\%close,$tab++);
	} elsif ($obj_view eq 'parents') {
		my @parents = StorProc->get_profile_parent($profile{'hostprofile_id'});
		$form .= Forms->host_profile_top($name,$user_acct,$obj_view,$auth_add{'externals'},\%obj);
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->wizard_doc('Parents',$docs{'parents'});
		my @nonmembers = StorProc->fetch_list('hosts','name');
		$form .= Forms->members('Parents:','parents',\@parents,\@nonmembers,'','',$docs{'parents'});
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%save,$tab++);
	} elsif ($obj_view eq 'hostgroups') {
		my @hostgroups = StorProc->get_profile_hostgroup($profile{'hostprofile_id'});
		$form .= Forms->host_profile_top($name,$user_acct,$obj_view,$auth_add{'externals'},\%obj);
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->wizard_doc('Hostgroups',$docs{'hostgroups'});
		my @nonmembers = StorProc->fetch_list('hostgroups','name'); 
		$form .= Forms->members('Hostgroups:','hostgroups',\@hostgroups,\@nonmembers,'','',$docs{'hostgroups'});
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%save,$tab++);
	} elsif ($obj_view eq 'escalations') {
		my $host_escalation = $query->param('host_escalation');
		my $service_escalation = $query->param('service_escalation');
		$form .= Forms->host_profile_top($name,$user_acct,$obj_view,$auth_add{'externals'},\%obj);
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->wizard_doc('Escalations',$docs{'escalations'});
		unless ($host_escalation) {
			my %w = ('tree_id' => $profile{'host_escalation_id'});
			my %t = StorProc->fetch_one_where('escalation_trees',\%w);
			$host_escalation = $t{'name'};
		}
		my %where = ('type' => 'host');
		my @members = StorProc->fetch_list_where('escalation_trees','name',\%where);
		$form .= Forms->list_box_submit('Host escalation:','host_escalation',\@members,$host_escalation,'',$docs{'host_escalation'});
		if ($host_escalation) {
			my ($ranks, $templates) = StorProc->get_tree_detail($host_escalation);
			my %ranks = %{$ranks};
			my %templates = %{$templates};
			$form .= Forms->escalation_tree(\%ranks,\%templates,'escalations');
		}
		unless ($service_escalation) {
			my %w = ('tree_id' => $profile{'service_escalation_id'});
			my %t = StorProc->fetch_one_where('escalation_trees',\%w);
			$service_escalation = $t{'name'};
		}
		my %where = ('type','service');
		@members = StorProc->fetch_list_where('escalation_trees','name',\%where);
		$form .= Forms->list_box_submit('Service escalation:','service_escalation',\@members,$service_escalation,'',$docs{'service_escalation'});
		if ($service_escalation) {
			my ($ranks, $templates) = StorProc->get_tree_detail($service_escalation);
			my %ranks = %{$ranks};
			my %templates = %{$templates};
			$form .= Forms->escalation_tree(\%ranks,\%templates,'escalations');
		}
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%save,$tab++);
	} elsif ($obj_view eq 'externals') {
		$form .= Forms->host_profile_top($name,$user_acct,$obj_view,$auth_add{'externals'},\%obj);
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		my @members = StorProc->get_profile_external($profile{'hostprofile_id'});
		my %where = ('type' => 'host');
		my @nonmembers = StorProc->fetch_list_where('externals','name',\%where);
		$form .= Forms->members('External:','externals',\@members,\@nonmembers);
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%save,$tab++);
	} elsif ($obj_view eq 'service_profiles') {
		$form .= Forms->host_profile_top($name,$user_acct,$obj_view,$auth_add{'externals'},\%obj);
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->wizard_doc('Service Profiles',$docs{'service_profiles'});
		my @members = ();
		my %w = ('hostprofile_id' => $profile{'hostprofile_id'});
		my @sids = StorProc->fetch_list_where('profile_host_profile_service','serviceprofile_id',\%w);
		foreach (@sids) {
			my %s = StorProc->fetch_one('profiles_service','serviceprofile_id',$_);
			if ($s{'name'}) { push @members, $s{'name'} }
		}
		my @nonmembers = StorProc->fetch_list('profiles_service','name');
		$form .= Forms->members('Service profiles:','service_profiles',\@members,\@nonmembers,'','20',$docs{'services'});
		$hidden{'obj_view'} = 'service_profiles';
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%save,$tab++);
	} elsif ($obj_view eq 'assign_hosts') {
		$form .= Forms->host_profile_top($name,$user_acct,$obj_view,$auth_add{'externals'},\%obj);
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		my %w = ('hostprofile_id' => $profile{'hostprofile_id'});
		my @members = StorProc->fetch_list_where('hosts','name',\%w);
		my @nonmembers = StorProc->fetch_list('hosts','name');
		$form .= Forms->wizard_doc('Assign Hosts',$docs{'assign_hosts'});
		$form .= Forms->members('Hosts:','hosts',\@members,\@nonmembers,'','20',$docs{'hosts'});
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%save,$tab++);
	} elsif ($obj_view eq 'assign_hostgroups') {
		$form .= Forms->host_profile_top($name,$user_acct,$obj_view,$auth_add{'externals'},\%obj);
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		my %w = ('hostprofile_id' => $profile{'hostprofile_id'});
		my @members = StorProc->fetch_list_where('hostgroups','name',\%w);
		my @nonmembers = StorProc->fetch_list('hostgroups','name');
		$form .= Forms->wizard_doc('Assign Hostgroups',$docs{'assign_hostgroups'});
		$form .= Forms->members('Hostgroups:','hostgroups',\@members,\@nonmembers,'','20',$docs{'hostgroups'});
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%save,$tab++);
	} elsif ($obj_view eq 'apply') {
		$form .= Forms->host_profile_top($name,$user_acct,$obj_view,$auth_add{'externals'},\%obj);
		$form .= Forms->wizard_doc('Apply',$docs{'apply'});
		$form .= Forms->apply_select($view,\%profile,$nagios_ver);
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%apply,$tab++);
	} elsif ($obj_view eq 'applied') {
		$form .= Forms->host_profile_top($name,$user_acct,$obj_view,$auth_add{'externals'},\%obj);
		my @message = ("Changes to $name applied to hosts.");
		$form .= Forms->form_message('Updated:',\@message,'row1');					
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%continue,$tab++);

	} elsif ($obj_view eq 'exported') {
		$form .= Forms->host_profile_top($name,$user_acct,$obj_view,$auth_add{'externals'},\%obj);
		my @message = ("Host profile $name saved to /tmp/host_profile_$name.xml.");
		$form .= Forms->form_message('Export:',\@message,'row1');					
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%continue,$tab++);
	} elsif ($obj_view eq 'saved') {
		my %apply = ('name' => 'apply','value' => 'Apply to Hosts');
		$form .= Forms->host_profile_top($name,$user_acct,$obj_view,$auth_add{'externals'},\%obj);
		
		my $message = "Changes to profile accepted.";
		$form .= Forms->display_hidden('Saved','',$message);				

		$form .= Forms->hidden(\%hidden);
		if ($hidden{'apply'}) {
			$form .= Forms->form_bottem_one_button(\%apply,$tab++);
		} else {
			$form .= Forms->form_bottem_no_button();
		}
	} elsif ($obj_view eq 'rename') {
		$form .= Forms->form_top('Rename Host Profile','');
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->display_hidden('Name:','name',$name,$textsize{'name'},'',$docs{'name'});
		$form .= Forms->text_box('Rename:','new_name','',$textsize{'name'},'',$docs{'name'});
		$hidden{'obj_view'} = 'rename';
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_two_button(\%rename,\%cancel,$tab++);
	} elsif ($obj_view eq 'delete') {
		my $message = qq(Are you sure you want to remove $name?);
		$form .= Forms->are_you_sure('Confirm Delete:',$message,'confirm_delete',\%hidden);
	} elsif ($obj_view eq 'deleted') {
		$form .= Forms->form_top('Service Profile','');
		$form .= Forms->display_hidden('Deleted:','deleted',"$name removed");
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%continue,$tab++);
	} elsif ($obj_view eq 'new') {
		my $description = $query->param('description');
		my $template = $query->param('template');
		my $file = $query->param('file');
		$form .= Forms->form_top('New Host Profile','');
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->text_box('Name:','name',$name,$textsize{'name'},'',$docs{'name'});
		$form .= Forms->text_box('Description:','description',$description,$textsize{'description'},'',$docs{'description'});
		my @templates = StorProc->fetch_list('host_templates','name');
		$form .= Forms->list_box('Host template:','template',\@templates,$template,$required{'template'},$docs{'template'});
		my $file = $query->param('file');
		unless ($hidden{'file'}) { $hidden{'file'} = $file }
		my $path = $query->param('path');
		my %where = ('type' => 'hosts');
		my @files = StorProc->fetch_list_where('files','name',\%where); 
		$form .= Forms->add_file($hidden{'file'},'hosts',$path,\@files,$hidden{'file'},$required{'new_file'},$docs{'file'});
		$hidden{'obj_view'} = 'new';
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_two_button(\%add,\%cancel,$tab++);
	}
	return $form;
}


#
###############################################
# Service profile
#		

sub service_profile() {
	my $form = undef;
	my %docs = Doc->service_profiles();
	@errors = ();
	my $obj_view = $query->param('obj_view');
	my $name = $query->param('name');
	my %profile = StorProc->fetch_one('profiles_service','name',$name);
	my @message = ();
	if ($query->param('add')) {
		my %p = StorProc->fetch_one('profiles_service','name',$name);
		my $file = $query->param('file');
		my $description = $query->param('description');
		my %file = StorProc->fetch_one('files','name',$file);
		unless ($p{'name'} || $name eq '') {
			if ($file{'file_id'}) {
				my @values = ('',$name,$description,$file{'file_id'},'');
				my $id = StorProc->insert_obj_id('profiles_service',\@values,'serviceprofile_id');
				unless ($id =~ /^Error/) { 
					$refresh_left = "top_menu=$top_menu&selected=$name";
					$profile{'file_id'} = $file{'file_id'};
					$profile{'description'} = $description;
					my @services = $query->param('services');
					foreach my $service (@services) {
						if ($service =~ /^\s+$/) { next	}
						my %s = StorProc->fetch_one('service_names','name',$service);
						my @vals = ($s{'servicename_id'},$id);
						my $result = StorProc->insert_obj('serviceprofile',\@vals);
						if ($result =~ /^Error/) { push @errors, $result }
					}
					unless (@errors) {
						$obj_view = 'services';
						$refresh_left = "top_menu=$top_menu&selected=$name";
					}
				} else {
					push @errors, $id;
				}
			} else {
				push @errors, "Required. You must select a default file name.";
				$required{'file'} = 1;
			}
		} else {
			push @errors, "Check name field. It's either blank or a record exists";
			$name = undef;
		}
	} elsif ($query->param('close') || $query->param('cancel') || $query->param('continue')) {
		$form .= Forms->header($page_title,$user_acct,$view);
		$obj_view = 'close';
	} elsif ($query->param('export')) {
		use MonarchProfileExport;
		my $results = ProfileExporter->service_profile($profile{'serviceprofile_id'});
		if ($results =~ /^Error/) { 
			push @errors, $results;
		} else {
			$obj_view = 'exported';
		}
	} elsif ($query->param('apply')) {
		my $hostgroups_select = $query->param('hostgroups_select');
		my $hosts_select = $query->param('hosts_select');
		my $services = $query->param('apply_services');

		my $data = "<?xml version=\"1.0\" ?>\n<data>";
		if ($hostgroups_select) { 
			$data .= qq(
  <prop name="hostgroups_select"><![CDATA[checked]]>
  </prop>);
		}
		if ($hosts_select) { 
			$data .= qq(
  <prop name="hosts_select"><![CDATA[checked]]>
  </prop>);
		}
		if ($services) { 
			$data .= qq(
  <prop name="apply_services"><![CDATA[$services]]>
  </prop>);
		}
		my @hosts = ();
		if ($hostgroups_select) {
			my %where = ('serviceprofile_id' => $profile{'serviceprofile_id'});
			my @hgs = StorProc->fetch_list_where('serviceprofile_hostgroup','hostgroup_id',\%where);
			foreach my $hg (@hgs) {
				%where = ('hostgroup_id' => $hg);
				my @hs = StorProc->fetch_list_where('hostgroup_host','host_id',\%where);
				push (@hosts,@hs);
			}
		}
		if ($hosts_select) {
			my %where = ('serviceprofile_id' => $profile{'serviceprofile_id'});
			my @hs = StorProc->fetch_list_where('serviceprofile_host','host_id',\%where);
			push (@hosts,@hs);
		}
		my @profiles = ($profile{'serviceprofile_id'});
		my ($cnt, $err) = StorProc->service_profile_apply(\@profiles,$services,\@hosts);
		@errors = @{$err};
		@message = ("$cnt hosts updated.");
		$data .= "\n</data>\n";
		my %value = ('data' => $data);
		my $result = StorProc->update_obj('profiles_service','name',$name,\%value);
		if ($result =~ /^Error/) { push @errors, $result }

		unless (@errors) { $obj_view = 'applied' }
	} elsif ($query->param('save')) {
		if ($obj_view eq 'services') {
			my $description = $query->param('description');
			my $file = $query->param('file');
			my %f = StorProc->fetch_one('files','name',$file);
			my %values = ();
			$values{'file_id'} = $f{'file_id'};
			$values{'description'} = $description;
			if ($values{'file_id'}) {
				my $result = StorProc->update_obj('profiles_service','name',$name,\%values);
				if ($result =~ /^Error/) { push @errors, $result}
				$result = StorProc->delete_all('serviceprofile','serviceprofile_id',$profile{'serviceprofile_id'});
				if ($result =~ /^Error/) { 
					push @errors, $result;
				} else {
					my @services = $query->param('services');
					my $cnt = 0;
					foreach my $service (@services) {
						if ($service =~ /^\s+$/) { next	}
						my %s = StorProc->fetch_one('service_names','name',$service);
						my @vals = ($s{'servicename_id'},$profile{'serviceprofile_id'});
						my $result = StorProc->insert_obj('serviceprofile',\@vals);
						if ($result =~ /^Error/) { push @errors, $result }
						$cnt++;
					}
					@message = ("$name has $cnt servies assigned.");
				}
				unless (@errors) { $obj_view = 'saved' }
				@message = ("Changes to $name accepted");
			} else {
				push @errors, "Required. You must select a default file name.";
				$required{'file'} = 1;
			}
		} elsif ($obj_view eq 'assign_hosts') {
			my @hosts = $query->param('hosts');
			my %w = ('serviceprofile_id' => $profile{'serviceprofile_id'});
			my $result = StorProc->delete_one_where('serviceprofile_host',\%w);
			if ($result =~ /Error/) {
				push @errors, $result;
			} else {
				my $cnt = 0;
				foreach my $host (@hosts) {
					if ($host =~ /^\s+$/) { next	}
					my %h = StorProc->fetch_one('hosts','name',$host);
					my @vals = ($profile{'serviceprofile_id'},$h{'host_id'});
					my $result = StorProc->insert_obj('serviceprofile_host',\@vals);
					if ($result =~ /^Error/) { push @errors, $result }
					$cnt++;
				}
				@message = ("$name has $cnt hosts assigned.");
			}
			unless (@errors) { $obj_view = 'saved' }
		} elsif ($obj_view eq 'assign_hostgroups') {
			my @hostgroups = $query->param('hostgroups');
			my %w = ('serviceprofile_id' => $profile{'serviceprofile_id'});
			my $result = StorProc->delete_one_where('serviceprofile_hostgroup',\%w);
			if ($result =~ /Error/) {
				push @errors, $result;
			} else {
				my $cnt = 0;
				foreach my $hostgroup (@hostgroups) {
					if ($hostgroup =~ /^\s+$/) { next	}
					my %h = StorProc->fetch_one('hostgroups','name',$hostgroup);
					my @vals = ($profile{'serviceprofile_id'},$h{'hostgroup_id'});
					my $result = StorProc->insert_obj('serviceprofile_hostgroup',\@vals);
					if ($result =~ /^Error/) { push @errors, $result }
					$cnt++;
				}
				@message = ("$name has $cnt hostgroups assigned.");
			}
			unless (@errors) { $obj_view = 'saved' }
		} elsif ($obj_view eq 'host_profiles') {
			my @profiles = $query->param('host_profiles');
			my %w = ('serviceprofile_id' => $profile{'serviceprofile_id'});
			my $result = StorProc->delete_one_where('profile_host_profile_service',\%w);
			if ($result =~ /Error/) {
				push @errors, $result;
			} else {
				my $cnt = 0;
				foreach my $profile (@profiles) {
					if ($profile =~ /^\s+$/) { next	}
					my %h = StorProc->fetch_one('profiles_host','name',$profile);
					my @vals = ($h{'hostprofile_id'},$profile{'serviceprofile_id'});
					my $result = StorProc->insert_obj('profile_host_profile_service',\@vals);
					if ($result =~ /^Error/) { push @errors, $result }
					$cnt++;
				}
				@message = ("$name has $cnt host profiles assigned.");
			}
			unless (@errors) { $obj_view = 'saved' }
		}
	} elsif ($query->param('add_file')) {
		my $new_file = $query->param('new_file');
		my $type = $query->param('type');
		my $path = $query->param('path');
		if ($new_file && $path) {
			unless ($new_file =~ /cfg$/) { $new_file .= ".cfg" }
			my %f = StorProc->fetch_one('files','name',$new_file);
			unless ($f{'name'}) {
				my @vals = ('',$new_file,$path,$type);
				my $result = StorProc->insert_obj('files',\@vals);
				if ($result =~ /error/i) { 
					push @errors, $result;
				} else {
					$hidden{'file'} = $new_file;
					$hidden{'new_file'} = $new_file;
				}
			} else {
				push @errors, "File $new_file already exists.";
			}
		} else {
			push @errors, "Required: File name and path.";
			$required{'new_file'} = 1;
		}
	} elsif ($query->param('rename')) {
		if ($query->param('new_name')) {
			my $new_name = $query->param('new_name');
			if ($new_name eq $name) {
				$obj_view = 'services';
			} else {
				my %n = StorProc->fetch_one('profiles_host','name',$new_name);
				if ($n{'name'}) {
					push @errors, "A host profile $new_name already exists.";
				} else {
					my %values = ('name' => $new_name);
					my $result = StorProc->update_obj('profiles_service','name',$name,\%values);				
					if ($result =~ /error/i) {
						push @errors, $result;
					} else {
						$name = $new_name;
						$refresh_left = "top_menu=$top_menu&selected=$name";
						$obj_view = 'services';
					}
				}
			}
		} else {
			$obj_view = 'rename';
		}
	} elsif ($query->param('delete') || $query->param('confirm_delete')) {
		my $id = $query->param('id');
		my %w = ('serviceprofile_id' => $id);
		my @hp = StorProc->fetch_list_where('profile_host_profile_service','hostprofile_id',\%w);
		my @hids = StorProc->fetch_list_where('serviceprofile_host','host_id',\%w);
		unless ($hids[0] || $hp[0]) {
			if ($query->param('confirm_delete')) {
				my $result = StorProc->delete_all('profiles_service','serviceprofile_id',$id);
				if ($result =~ /^Error/) { push @errors, $result }
				unless (@errors) { 
					$refresh_left = "top_menu=$top_menu";
					$obj_view = 'deleted';
				}
			} elsif ($query->param('task') eq 'No') {
				$obj_view = 'services';
			} else {
				foreach my $name ($query->param) {
					unless ($name eq 'nocache') { $hidden{$name} = $query->param($name) }
				}
				$hidden{'delete'} = 1;
				$obj_view = 'delete';
			}
		} else {
			push @errors, "Cannot delete $name while it is in use.";
		}
	} elsif ($query->param('close')) {
		$obj_view = undef;
	}	
	my %save = ('name' => 'save','value' => 'Save');
	my %apply = ('name' => 'apply','value' => 'Apply');
	my %export = ('name' => 'export','value' => 'Export');
	my %objs = ();
	unless ($obj_view) { $obj_view = 'services' }
	$hidden{'obj_view'} = $obj_view;
	$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
	if ($obj_view eq 'services') {
		my @members = ();
		$form .= Forms->service_profile_top($name,$user_acct,$obj_view,\%objs,$hidden{'selected'});
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$hidden{'id'} = $profile{'serviceprofile_id'};
		my %w = ('serviceprofile_id' => $profile{'serviceprofile_id'});
		my @sids = StorProc->fetch_list_where('serviceprofile','servicename_id',\%w);
		foreach (@sids) {
			my %s = StorProc->fetch_one('service_names','servicename_id',$_);
			if ($s{'name'}) { push @members, $s{'name'} }
		}
		my $description = $profile{'description'};
		my %f = StorProc->fetch_one('files','file_id',$profile{'file_id'});
		my $file = $f{'name'};
		$form .= Forms->text_box('Description:','description',$description,$textsize{'description'},'',$docs{'description'});
		my @nonmembers = StorProc->fetch_list('service_names','name'); 
		$form .= Forms->members('Services:','services',\@members,\@nonmembers,'','30',$docs{'services'});
		my %where = ('type' => 'services');
		my @files = StorProc->fetch_list_where('files','name',\%where); 
		unless ($hidden{'file'}) { $hidden{'file'} = $file }
		if ($hidden{'new_file'}) {
			delete $hidden{'new_file'};
			$file = $hidden{'file'};
		}
		my $path = $query->param('path');
		$form .= Forms->add_file($file,'services',$path,\@files,'',$required{'new_file'},$docs{'file'});
		$form .= Forms->hidden(\%hidden);
		if ($auth_delete{'profiles'}) {
			$form .= Forms->form_bottem_five_button(\%save,\%delete,\%rename,\%export,\%close,$tab++);
		} else {
			$form .= Forms->form_bottem_four_button(\%save,\%rename,\%export,\%close,$tab++);
		}
	} elsif ($obj_view eq 'assign_hosts') {
		$form .= Forms->service_profile_top($name,$user_acct,$obj_view,\%objs,$hidden{'selected'});
		$form .= Forms->wizard_doc('Assign Hosts',$docs{'assign_hosts'});
		my @members = ();
		my %w = ('serviceprofile_id' => $profile{'serviceprofile_id'});
		my @hids = StorProc->fetch_list_where('serviceprofile_host','host_id',\%w);
		foreach (@hids) {
			my %h = StorProc->fetch_one('hosts','host_id',$_);
			if ($h{'name'}) { push @members, $h{'name'} }
		}
		my @nonmembers = StorProc->fetch_list('hosts','name'); 
		$form .= Forms->members('Hosts:','hosts',\@members,\@nonmembers,'','30',$docs{'hosts'});
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%save,$tab++);
	} elsif ($obj_view eq 'assign_hostgroups') {
		$form .= Forms->service_profile_top($name,$user_acct,$obj_view,\%objs,$hidden{'selected'});
		$form .= Forms->wizard_doc('Assign Hostgroups',$docs{'assign_hostgroups'});
		my @members = ();
		my %w = ('serviceprofile_id' => $profile{'serviceprofile_id'});
		my @hids = StorProc->fetch_list_where('serviceprofile_hostgroup','hostgroup_id',\%w);
		foreach (@hids) {
			my %h = StorProc->fetch_one('hostgroups','hostgroup_id',$_);
			if ($h{'name'}) { push @members, $h{'name'} }
		}
		my @nonmembers = StorProc->fetch_list('hostgroups','name'); 
		$form .= Forms->members('Hostgroups:','hostgroups',\@members,\@nonmembers,'','30',$docs{'hosts'});
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%save,$tab++);
	} elsif ($obj_view eq 'host_profiles') {
		$form .= Forms->service_profile_top($name,$user_acct,$obj_view,\%objs,$hidden{'selected'});
		$form .= Forms->wizard_doc('Host Profiles',$docs{'host_profiles'});
		my @members = ();
		my %w = ('serviceprofile_id' => $profile{'serviceprofile_id'});
		my @hids = StorProc->fetch_list_where('profile_host_profile_service','hostprofile_id',\%w);
		foreach (@hids) {
			my %h = StorProc->fetch_one('profiles_host','hostprofile_id',$_);
			if ($h{'name'}) { push @members, $h{'name'} }
		}
		my @nonmembers = StorProc->fetch_list('profiles_host','name'); 
		$form .= Forms->members('Host profiles:','host_profiles',\@members,\@nonmembers,'','30',$docs{'host_profiles'});
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%save,$tab++);

	} elsif ($obj_view eq 'apply') {
		my %doc = ();
		$form .= Forms->service_profile_top($name,$user_acct,$obj_view,\%objs,$hidden{'selected'});
		$form .= Forms->wizard_doc('Apply',$docs{'apply'});
		$form .= Forms->apply_select($view,\%profile,$nagios_ver);
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%apply,$tab++);
	} elsif ($obj_view eq 'applied') {
		$form .= Forms->service_profile_top($name,$user_acct,$obj_view,\%objs,$hidden{'selected'});
		my @message = ("Changes to $name applied to hosts.");
		$form .= Forms->form_message('Updated:',\@message,'row1');					
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%continue,$tab++);
	} elsif ($obj_view eq 'exported') {
		$form .= Forms->service_profile_top($name,$user_acct,$obj_view,\%objs,$hidden{'selected'});
		my @message = ("Service profile $name saved to /tmp/service_profile_$name.xml.");
		$form .= Forms->form_message('Export:',\@message,'row1');					
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%continue,$tab++);
	} elsif ($obj_view eq 'saved') {
		$form .= Forms->service_profile_top($name,$user_acct,$obj_view,\%objs,$hidden{'selected'});
		$form .= Forms->form_message('Updated:',\@message,'row1');					
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%close);
	} elsif ($obj_view eq 'rename') {
		$form .= Forms->form_top('Rename Service Profile','');
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->display_hidden('Name:','name',$name,$textsize{'name'},'',$docs{'name'});
		$form .= Forms->text_box('Rename:','new_name','',$textsize{'name'},'',$docs{'name'});
		$hidden{'obj_view'} = 'rename';
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_two_button(\%rename,\%cancel,$tab++);
	} elsif ($obj_view eq 'delete') {
		my $message = qq(Are you sure you want to remove $name?);
		delete $hidden{'task'};
		$form .= Forms->are_you_sure('Confirm Delete:',$message,'confirm_delete',\%hidden);
	} elsif ($obj_view eq 'deleted') {
		$form .= Forms->form_top('Service Profile','');
		$form .= Forms->display_hidden('Deleted:','deleted',"$name removed");
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%continue,$tab++);
	} elsif ($obj_view eq 'new') {
		my $description = $query->param('description');
		my $file = $query->param('file');
		unless ($hidden{'file'}) { $hidden{'file'} = $file }
		my $path = $query->param('path');
		$form .= Forms->form_top('New Service Profile','');
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->text_box('Name:','name',$name,$textsize{'name'},'',$docs{'name'});
		$form .= Forms->text_box('Description:','description',$description,$textsize{'description'},'',$docs{'description'});
		my $file = $query->param('file');
		unless ($hidden{'file'}) { $hidden{'file'} = $file }
		my $path = $query->param('path');
		my %where = ('type' => 'services');
		my @files = StorProc->fetch_list_where('files','name',\%where); 
		$form .= Forms->add_file($hidden{'file'},'services',$path,\@files,$hidden{'file'},$required{'new_file'},$docs{'file'});
		$hidden{'obj_view'} = 'new';
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_two_button(\%add,\%cancel,$tab++);
	}
	return $form;
}

###############################################
# Services
#		

sub service() {
	my $host_id = $query->param('host_id');
	my $form = undef;
	my $obj_view = $query->param('obj_view');
	my $name = $query->param('name');
	my %service = StorProc->fetch_one('service_names','name',$name);
	my $host = $query->param('host');
	my $test_results = undef;
	my $message_applied = undef;
	if ($query->param('add')) {
		if ($name) {
			my %p = StorProc->fetch_one('service_names','name',$name);
			unless ($p{'name'}) {
				my $template = $query->param('template');
				if ($template) {
					my %s = StorProc->fetch_one('service_templates','name',$template);
					my @values = ('',$name,'',$s{'servicetemplate_id'},'','','','','');
					my $id = StorProc->insert_obj_id('service_names',\@values,'servicename_id');
					if ($id =~ /^Error/) { 
						push @errors, $id;
						$obj_view = 'new';
					} else {
						$obj_view = 'service_detail';
						$refresh_left = "top_menu=$top_menu&selected=$name";
					}	
				} else {
					push @errors, "Template required";
					$obj_view = 'new';
					$required{'template'} = 1;
				}
			} else {
				push @errors, "Service name $name already exists";
				$obj_view = 'new';
					$required{'name'} = 1;
					$required{'template'} = 1;
			}
		} else {					
			push @errors, "Service name required";
			$obj_view = 'new';
		}
	} elsif ($query->param('close') || $query->param('continue') || $query->param('cancel')) {
		$form .= Forms->header($page_title,$user_acct,$view);
		$obj_view = undef;
	} elsif ($query->param('apply')) {
		my $apply_check = $query->param('apply_check');
		my $apply_escalation_service = $query->param('apply_escalation_service');
		my $apply_contact_service = $query->param('apply_contact_service');
		my $apply_extinfo_service = $query->param('apply_extinfo_service');
		my $apply_dependencies = $query->param('apply_dependencies');
		my $services = $query->param('apply_services');
		my $data = "<?xml version=\"1.0\" ?>\n<data>";
		if ($services eq 'replace') { 
			$data .= qq(
  <prop name="apply_services"><![CDATA[replace]]>
  </prop>);
			my %values = ('servicetemplate_id' => $service{'template'});
			my $result = StorProc->update_obj('services','servicename_id',$service{'servicename_id'},\%values);
			if ($result =~ /Error/) { push @errors, $result }
			my @errs = StorProc->service_replace(\%service);
			if ($errs[0] =~ /applied/) {
				$message_applied = $errs[0];
			} else {
				push (@errors,@errs);
			}
		}
		if ($services eq 'merge') { 
			$data .= qq(
  <prop name="apply_services"><![CDATA[merge]]>
  </prop>);
			my %values = ('servicetemplate_id' => $service{'template'});
			my $result = StorProc->update_obj('services','servicename_id',$service{'servicename_id'},\%values);
			if ($result =~ /Error/) { push @errors, $result }
			my @errs = StorProc->service_merge(\%service);
			if ($errs[0] =~ /applied/) {
				$message_applied = $errs[0];
			} else {
				push (@errors,@errs);
			}
		}
		if ($apply_extinfo_service) { 
			$data .= qq(
  <prop name="apply_extinfo_service"><![CDATA[checked]]>
  </prop>);
			my %values = ('serviceextinfo_id' => $service{'extinfo'});
			my $result = StorProc->update_obj('services','servicename_id',$service{'servicename_id'},\%values);
			if ($result =~ /^Error/) { push @errors, $result }
		}
		if ($apply_contact_service) { 
			$data .= qq(
  <prop name="apply_contact_service"><![CDATA[checked]]>
  </prop>);
			my %where = ('servicename_id' => $service{'servicename_id'});
			my @sids = StorProc->fetch_list_where('services','service_id',\%where);
			%where = ('type' => 'service_names','object' => $service{'servicename_id'});
			my @cgids = StorProc->fetch_list_where('contactgroup_assign','contactgroup_id',\%where);
			foreach my $sid (@sids) {
				%where = ('object' => $sid,'type' => 'services');
				my $result = StorProc->delete_one_where('contactgroup_assign',\%where);
				if ($result =~ /^Error/) { push @errors, $result }
				foreach my $cgid (@cgids) {
					my @vals = ($cgid,'services',$sid);
					my $result = StorProc->insert_obj('contactgroup_assign',\@vals);
					if ($result =~ /^Error/) { push @errors, $result }
				}
			}
		}
		if ($apply_escalation_service) { 
			$data .= qq(
  <prop name="apply_escalation_service"><![CDATA[checked]]>
  </prop>);
			my %values = ('escalation_id' => $service{'escalation'});
			my $result = StorProc->update_obj('services','servicename_id',$service{'servicename_id'},\%values);
			if ($result =~ /^Error/) { push @errors, $result }
		}
		if ($apply_check) { 
			$data .= qq(
  <prop name="apply_check"><![CDATA[checked]]>
  </prop>);
			my %values = ('check_command' => $service{'check_command'},'command_line' => $service{'command_line'});
			my $result = StorProc->update_obj('services','servicename_id',$service{'servicename_id'},\%values);
			if ($result =~ /^Error/) { push @errors, $result }
		}
		if ($apply_dependencies) { 
			$data .= qq(
  <prop name="apply_dependencies"><![CDATA[checked]]>
  </prop>);
			my @result = StorProc->update_dependencies($service{'servicename_id'});
			if (@result) { push (@errors,@result) }
		}
		$data .= "\n</data>\n";
		my %value = ('data' => $data);
		my $result = StorProc->update_obj('service_names','name',$name,\%value);
		if ($result =~ /^Error/) { push @errors, $result }
		if ($enable_externals) {
			my %w = ('servicename_id' => $service{'servicename_id'});
			my @externals = StorProc->fetch_list_where('external_service_names','external_id',\%w);
			my %services = StorProc->fetch_list_hash_array('services',\%w);
			foreach my $sid (keys %services) {
				my $result = StorProc->delete_all('external_service','service_id',$sid);
				foreach my $ext (@externals) {
					my %external = StorProc->fetch_one('externals','external_id',$ext);	
					my @vals = ($ext,$services{$sid}[1],$sid,$external{'display'});
					my $result = StorProc->insert_obj('external_service',\@vals);
					if ($result =~ /^Error/) { push @errors, $result }
				}
			}
		}
		unless (@errors) { $obj_view = 'applied' }
	} elsif ($query->param('test_command')) {	
		my $arg_string = $query->param('arg_string');
		my $command = $query->param('command');
		my %cmd = StorProc->fetch_one('commands','name',$command);
		my %host = StorProc->fetch_one('hosts','name',$host);
		my %where = ('type' => 'resource'); 
		my %resources = StorProc->get_resources();
		foreach my $res (keys %resources) {
			if ($cmd{'command_line'} =~ /$res/i) { $cmd{'command_line'} =~ s/\$$res\$/$resources{$res}/ig } 
		}
		if ($cmd{'command_line'} =~ /\$HOSTADDRESS\$/) { 
			if ($host =~ /\d+\.\d+\.\d+\.\d+/) { $cmd{'command_line'} =~ s/\$HOSTADDRESS\$/$host/g }
			$cmd{'command_line'} =~ s/\$HOSTADDRESS\$/$host{'address'}/g; 
		}
		if ($cmd{'command_line'} =~ /\$HOSTNAME\$/) { $cmd{'command_line'} =~ s/\$HOSTNAME\$/$host/g } 
		my @args = split(/!/, $arg_string);
		my $cnt = 1;
		foreach my $a (@args) {
			$cmd{'command_line'} =~ s/\$ARG$cnt\$/$a/g;
			$cnt++;
		}
		$test_results = "$cmd{'command_line'}<br/>";
		$test_results .= StorProc->test_command($cmd{'command_line'});	
	} elsif ($query->param('save')) {
		if ($obj_view eq 'service_detail') {
			my %values = ();
			$hidden{'servicename_id'} = $service{'servicename_id'};
			my $ext_info = $query->param('ext_info');
			my %x = StorProc->fetch_one('extended_service_info_templates','name',$ext_info);
			my $escalation = $query->param('escalation');
			my %e = StorProc->fetch_one('escalation_trees','name',$escalation);
			my $template = $query->param('template');
			my %t = StorProc->fetch_one('service_templates','name',$template);
			if ($host_id) {
				my %values = ();
				my $service_id = $query->param('service_id');
				$hidden{'service_id'} = $service_id;
				my %s = StorProc->fetch_one('service_names','name',$name);
				my $ext_info = $query->param('ext_info');
				my %x = StorProc->fetch_one('extended_service_info_templates','name',$ext_info);
				my $escalation = $query->param('esclation');
				my %e = StorProc->fetch_one('escalation_trees','name',$escalation);
				my $template = $query->param('template');
				my %t = StorProc->fetch_one('service_templates','name',$template);
				my %data = parse_query('service_overrides','service_overrides');
				my %vals = ('serviceextinfo_id' => $x{'serviceextinfo_id'},'escalation_id' => $s{'tree_id'},'servicetemplate_id' => $t{'servicetemplate_id'});
				my $result = StorProc->update_obj('services','service_id',$service_id,\%vals);
				if ($result =~ /^Error/) { push @errors, $result }
				my %where = ('service_id' => $properties{'service_id'});
				my $result = StorProc->delete_one_where('service_overrides',\%where);
				if ($result =~ /^Error/) { push @errors, $result }
				if ($data{'check_period'} || $data{'notification_period'} || $data{'event_handler'} || $data{'data'}) {
					my @values = ($service_id,$data{'check_period'},$data{'notification_period'},$data{'event_handler'},$data{'data'});
					my $result = StorProc->insert_obj('service_overrides',\@values);
					if ($result =~ /^Error/) { push @errors, $result }
				}
				my %where = ('type' => 'services','object' => $service_id);
				my $result = StorProc->delete_one_where('contactgroup_assign',\%where);
				if ($result =~ /^Error/) { push @errors, $result }
				unless ($query->param('contactgroup_override')) {
					my @mems = $query->param('contactgroup');
					foreach (@mems) {
						my %cg = StorProc->fetch_one('contactgroups','name',$_);
						my @vals = ($cg{'contactgroup_id'},'services',$service_id);
						$result = StorProc->insert_obj('contactgroup_assign',\@vals);
						if ($result =~ /^Error/) { push @errors, $result }
					}
				}
			} else {
				my %data = parse_query('service_overrides','service_overrides');
				my %vals = ('extinfo' => $x{'serviceextinfo_id'},'escalation' => $e{'tree_id'},'template' => $t{'servicetemplate_id'});
				my $result = StorProc->update_obj('service_names','servicename_id',$service{'servicename_id'},\%vals);
				if ($result =~ /^Error/) { push @errors, $result }
				my %where = ('servicename_id' => $service{'servicename_id'});
				my $result = StorProc->delete_one_where('servicename_overrides',\%where);
				if ($result =~ /^Error/) { push @errors, $result }
				if ($data{'check_period'} || $data{'notification_period'} || $data{'event_handler'} || $data{'data'}) {
					my @values = ($service{'servicename_id'},$data{'check_period'},$data{'notification_period'},$data{'event_handler'},$data{'data'});
					my $result = StorProc->insert_obj('servicename_overrides',\@values);
					if ($result =~ /^Error/) { push @errors, $result }
				}
				my %where = ('type' => 'service_names','object' => $service{'servicename_id'});
				my $result = StorProc->delete_one_where('contactgroup_assign',\%where);
				if ($result =~ /^Error/) { push @errors, $result }
				unless ($query->param('contactgroup_override')) {
					my @mems = $query->param('contactgroup');
					foreach (@mems) {
						my %cg = StorProc->fetch_one('contactgroups','name',$_);
						my @vals = ($cg{'contactgroup_id'},'service_names',$service{'servicename_id'});
						$result = StorProc->insert_obj('contactgroup_assign',\@vals);
						if ($result =~ /^Error/) { push @errors, $result }
					}
				}
			}
			unless (@errors) { 
				$obj_view = 'saved';
			}
		} elsif ($obj_view eq 'service_check') {
			my $service_name = $query->param('service_name');
			my $service_id = $query->param('service_id');
			$hidden{'service_name'} = $service_name;
			$hidden{'service_id'} = $service_id;
			my $check_command = $query->param('command');
			my $command_line = $query->param('command_line');
			if ($host_id) {
				my $service_id = $query->param('service_id');
				my $check_command = $query->param('command');
				my $command_line = $query->param('command_line');
				if ($query->param('inherit')) {
					my $required = $query->param('required');
					unless ($required) {
						my %vals = ('check_command' => '','command_line' => '');
						my $result = StorProc->update_obj('services','service_id',$service_id,\%vals);
						if ($result =~ /^Error/) { push @errors, $result }
					} else {
						push @errors, "The service check is not defined on a template so it must be defined here.";
					}
				} else {
					my %check = StorProc->fetch_one('commands','name',$check_command);
					my %vals = ('check_command' => $check{'command_id'},'command_line' => $command_line);
					my $result = StorProc->update_obj('services','service_id',$service_id,\%vals);
					if ($result =~ /^Error/) { push @errors, $result }
				}
			} else {
				if ($query->param('inherit')) {
					my $required = $query->param('required');
					unless ($required) {
						my %vals = ('check_command' => '','command_line' => '');
						my $result = StorProc->update_obj('service_names','servicename_id',$service{'servicename_id'},\%vals);
						if ($result =~ /^Error/) { push @errors, $result }
					} else {
						push @errors, "The service check is not defined on a template so it must be defined here.";
					}
				} else {
					my %check = StorProc->fetch_one('commands','name',$check_command);
					my %vals = ('check_command' => $check{'command_id'},'command_line' => $command_line);
					my $result = StorProc->update_obj('service_names','servicename_id',$service{'servicename_id'},\%vals);
					if ($result =~ /^Error/) { push @errors, $result }
				}
			}	
			
			unless (@errors) { 
				$obj_view = 'saved';
				$hidden{'selected'} = 'service_check';
			}
		} elsif ($obj_view eq 'service_profiles') {
			my @profiles = $query->param('profiles');
			my $result = StorProc->delete_all('serviceprofile','servicename_id',$service{'servicename_id'});
			if ($result =~ /^Error/) { push @errors, $result }						
			foreach my $prof (@profiles) {
				my %p = StorProc->fetch_one('profiles_service','name',$prof);
				my @values = ($service{'servicename_id'},$p{'serviceprofile_id'});
				$result = StorProc->insert_obj('serviceprofile',\@values);
				if ($result =~ /^Error/) { push @errors, $result }						
			}
			unless (@errors) { 
				$obj_view = 'saved';
			}
		}
	} elsif ($query->param('external_add')) {
		my @externals = $query->param('external');
		my $result = StorProc->delete_all('external_service_names','servicename_id',$hidden{'service_id'});
		if ($result =~ /^Error/) { push @errors, $result }						
		foreach my $ext (@externals) {
			my %e = StorProc->fetch_one('externals','name',$ext);
			my @values = ($e{'external_id'},$service{'servicename_id'});
			$result = StorProc->insert_obj('external_service_names',\@values);
			if ($result =~ /^Error/) { push @errors, $result }						
		}
 	} elsif ($query->param('remove_dependency')) {
 		my $dep_id = $query->param('dependency_id');
		if ($host_id) {
			my $dep_id = $query->param('dependency_id');
			my $result = StorProc->delete_all('service_dependency','id',$dep_id);
			if ($result =~ /^Error/) { push @errors, $result }
		} else {
			my $result = StorProc->delete_all('servicename_dependency','id',$dep_id);
			if ($result =~ /^Error/) { push @errors, $result }	
		}
	} elsif ($query->param('add_dependency')) {
		my $dependency = $query->param('dep_template');
		my $parent = $query->param('depend_on_host');
		if ($parent && $dependency) {
			if ($host_id) {
				
				my $dep_template = $query->param('dep_template');
				my $service_name = $query->param('service_name');
				my $service_id = $query->param('service_id');
				$hidden{'service_name'} = $service_name;
				$hidden{'service_id'} = $service_id;
				my $parent = $query->param('depend_on_host');
				my %p = StorProc->fetch_one('hosts','name',$parent);
				my %d = StorProc->fetch_one('service_dependency_templates','name',$dep_template);
				my %w = ('service_id' => $service_id,'host_id' => $properties{'host_id'},'depend_on_host_id' => $p{'host_id'},'template' => $d{'id'});
				my %check_dep = StorProc->fetch_one_where('service_dependency',\%w);	
				if ($dep_template && $service_id && $parent) {
					unless ($check_dep{'id'}) {
						my @values = ('',$service_id,$properties{'host_id'},$p{'host_id'},$d{'id'},'');
						my $result = StorProc->insert_obj('service_dependency',\@values);
						if ($result =~ /^Error/) { push @errors, $result }	
					} else {
						push @errors, "Already defined. Check existing dependencies";
					}
				}

			} else {

				my	%dependency = StorProc->fetch_one('service_dependency_templates','name',$dependency);
				unless ($parent eq 'same host') {
					my %p = StorProc->fetch_one('hosts','name',$parent);
					$parent = $p{'host_id'};
				}
				my $check_dep = StorProc->check_dependency($service{'servicename_id'},$parent,$dependency{'id'});	
				unless ($check_dep) {
					my $result = StorProc->insert_dependency($service{'servicename_id'},$parent,$dependency{'id'});
					if ($result =~ /^Error/) { push @errors, $result }	
				} else {
					push @errors, "Already defined. Check existing dependencies";
				}
			}
		}
 	} elsif ($query->param('rename')) {
		if ($query->param('new_name')) {
			my $new_name = $query->param('new_name');
			if ($new_name eq $name) {
				$obj_view = 'service_detail';
			} else {
				my %n = StorProc->fetch_one('service_names','name',$new_name);
				if ($n{'name'}) {
					push @errors, "A Service $new_name already exists.";
				} else {
					my %values = ('name' => $new_name);
					my $result = StorProc->update_obj('service_names','name',$name,\%values);				
					if ($result =~ /error/i) {
						push @errors, $result;
					} else {
						$name = $new_name;
						$obj_view = 'service_detail';
						$refresh_left = "top_menu=$top_menu&selected=$name";
					}
				}
			}	
		} else {
			$obj_view = 'rename';
		}
	} elsif ($query->param('delete') || $query->param('confirm_delete')) {
		my %sn = StorProc->fetch_one('service_names','name',$name);
		my %where = ('servicename_id' => $sn{'servicename_id'});
		if ($query->param('confirm_delete')) {
			my $result = StorProc->delete_all('service_names','name',$name);
			if ($result =~ /^Error/) { push @errors, $result }
			unless (@errors) {
				$result = StorProc->delete_one_where('services',\%where); 
				if ($result =~ /^Error/) { push @errors, $result }
			}
			unless (@errors) {
				$result = StorProc->delete_one_where('serviceprofile',\%where); 
				if ($result =~ /^Error/) { push @errors, $result }
			}
			unless (@errors) {
				$result = StorProc->delete_one_where('service_dependency_templates',\%where); 
				if ($result =~ /^Error/) { push @errors, $result }
			}
			unless (@errors) { 
				$refresh_left = "top_menu=$top_menu";
				$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
				$form .= Forms->form_top('Deleted','');
				my @message = ("$name");
				$form .= Forms->form_message('Removed:',\@message,'row1');			
				$form .= Forms->hidden(\%hidden);
				$form .= Forms->form_bottem_one_button(\%continue,$tab++);
				$name = undef;
				$obj_view = undef;
				$hidden{'name'} = undef;
			}
		} elsif ($query->param('task') eq 'No') {
			$obj_view = 'service_detail';
		} else {
			foreach my $name ($query->param) {
				unless ($name eq 'nocache') { $hidden{$name} = $query->param($name) }
			}
			$obj_view = 'delete';
		}
	} else {
		foreach my $param ($query->param) {
			if ($param =~ /remove_external_(\d+)/) {
				my $eid = $1;
				my %where = ('servicename_id' => $service{'servicename_id'},'external_id' => $eid);
				my $result = StorProc->delete_one_where('external_service_names',\%where);
				if ($result =~ /^Error/) { push @errors, $result }						
			}
		}
	}
	my %save = ('name' => 'save','value' => 'Save');
	my %apply = ('name' => 'apply','value' => 'Apply');
	my %objs = ('servicename_id' => $hidden{'servicename_id'});
	my %docs = Doc->services();
	$hidden{'name'} = $name;
	$hidden{'obj_view'} = $obj_view;
	$hidden{'servicename_id'} = $query->param('servicename_id');
	unless ($hidden{'servicename_id'}) { $hidden{'servicename_id'} = $service{'servicename_id'}	}
	$objs{'servicename_id'} = $hidden{'servicename_id'};
	$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
	if ($obj_view eq 'service_detail') {
		my $ext_info = $query->param('ext_info');
		my $escalation = $query->param('escalation');
		unless ($ext_info) {
			my %w = ('serviceextinfo_id' => $service{'extinfo'});
			my %e = StorProc->fetch_one_where('extended_service_info_templates',\%w);
			$ext_info = $e{'name'};
		}
		unless ($escalation) {
			my %w = ('tree_id' => $service{'escalation'});
			my %esc = StorProc->fetch_one_where('escalation_trees',\%w);
			$escalation = $esc{'name'};
		}
		$form .= Forms->service_top($name,$user_acct,$obj_view,\%objs,$auth_add{'externals'},$hidden{'selected'});
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= build_service_detail($hidden{'servicename_id'});
		my @members = StorProc->fetch_list('extended_service_info_templates','name'); 
		$form .= Forms->list_box('Extended info template:','ext_info',\@members,$ext_info,'',$docs{'extinfo'});
		my %where = ('type' => 'service');
		@members = StorProc->fetch_list_where('escalation_trees','name',\%where); 
		$form .= Forms->list_box_submit('Escalation tree:','escalation',\@members,$escalation,'',$docs{'escalation'});
		if ($escalation) {
			my ($ranks, $templates) = StorProc->get_tree_detail($escalation);
			my %ranks = %{$ranks};
			my %templates = %{$templates};
			$form .= Forms->escalation_tree(\%ranks,\%templates,'service_detail');
		}
		$form .= Forms->hidden(\%hidden);
		if ($auth_delete{'services'}) {
			if ($host_id) {
				$form .= Forms->form_bottem_three_button(\%save,\%delete,\%close,$tab++);
			} else {
				$form .= Forms->form_bottem_four_button(\%save,\%delete,\%rename,\%close,$tab++);
			}
		} else {
			if ($host_id) {
				$form .= Forms->form_bottem_two_button(\%save,\%close,$tab++);
			} else {
				$form .= Forms->form_bottem_three_button(\%save,\%rename,\%close,$tab++);
			}
		}
	} elsif ($obj_view eq 'service_check') {
		my %template = StorProc->fetch_one('service_templates','servicetemplate_id',$service{'template'});
		my $message = undef;
		my $inherit = 0;
		$inherit = $query->param('inherit');
		my %cmd = StorProc->fetch_one('commands','command_id', $service{'check_command'});
		my $command = $cmd{'name'};
		my $command_save = $cmd{'name'};
		my $command_line = $service{'command_line'};
		if ($query->param('command')) { $command = $query->param('command') }
		if ($query->param('command_save')) { $command_save = $query->param('command_save') }
		if ($query->param('command_line')) { $command_line = $query->param('command_line') }
		unless ($command eq $command_save) {
			%cmd = StorProc->fetch_one('commands','name',$command);
			$command_line = undef;
		}
		if ($inherit or !$command) {
			%cmd = StorProc->fetch_one('commands','command_id',$template{'check_command'});
			if ($cmd{'name'}) {
				$command = $cmd{'name'};
				$command_line = $template{'command_line'};
				$inherit = 1;
			} else {
				my $got_command = 0;
				my $stid = $template{'parent_id'};
				until ($got_command) {
					my %t = StorProc->fetch_one('service_templates','servicetemplate_id',$stid);
					if ($t{'check_command'}) {
						$got_command = 1;
						%cmd = StorProc->fetch_one('commands','command_id',$t{'check_command'});
						$command = $cmd{'name'};
						$command_line = $t{'command_line'};
						$got_command = 1;
					} else {
						if ($t{'parent_id'}) {
							$stid = $t{'parent_id'};
						} else {
							$got_command = 1;
							$message = ('Note: a parent template does not have a check command defined.');
							$command = undef;
							$command_line = undef;
						}
					}
				}
			}
		}
		unless ($cmd{'command_line'}) {
			%cmd = StorProc->fetch_one('commands','name',$command);
		}
		my $arg_string = $command_line;
		$arg_string =~ s/$command!//;
		my $usage = $command;
		my @args = split(/ARG/i,$cmd{'command_line'});
		my $args = undef;
		if ($args[1]) {
			$usage .= "!";
			my $cnt = 1;
			pop @args;
			foreach (@args) { 
				if ($cmd{'command_line'} =~ /ARG$cnt/i) { $args .= "ARG$cnt!" }
				$cnt++;
			}
			chop $args;
			$usage .= $args;
			unless ($command_line =~ /$command/) {
				$command_line = "$command!$args";
				$arg_string = $args;
			}
		}
		$form .= Forms->service_top($name,$user_acct,$obj_view,\%objs,$auth_add{'externals'},$hidden{'selected'});
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->wizard_doc('Service Check',$docs{'service_check'});
		$form .= Forms->display_hidden('Service template:','',$template{'name'});
		$form .= Forms->checkbox_override('Inherit check from template','inherit',$inherit,$docs{'override'});
		my %where = ('type' => 'check');
		my @commands = StorProc->fetch_list_where('commands','name',\%where);
		$form .= Forms->list_box_submit('Check command:','command',\@commands,$command,$required{'check_command'});
		$form .= Forms->display_hidden('Command definition:','',$cmd{'command_line'});
		$form .= Forms->display_hidden('Usage:','',$usage);
		$form .= Forms->text_area('Command line:','command_line',$command_line,'3','80','',$docs{'command_line'});
		$form .= Forms->command_test($test_results,$host,$arg_string);
		$hidden{'command_save'} = $command;
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%save,$tab++);
	} elsif ($obj_view eq 'service_dependencies') {
		my $dep_template = $query->param('dep_template');
		my %dep = StorProc->fetch_one('service_dependency_templates','name',$dep_template);
		my @dep_hosts = ();
		my ($host,$hosts) = StorProc->get_dep_on_hosts($dep{'servicename_id'},'0');
		my @hosts = @{$hosts};
		if ($dep_template) {
			@dep_hosts = ('same host');
			push (@dep_hosts,@hosts);
		}
		$form .= Forms->service_top($name,$user_acct,$obj_view,\%objs,$auth_add{'externals'},$hidden{'selected'});
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->wizard_doc('Service Dependencies',$docs{'dependencies'});
		my %dependencies = StorProc->get_servicename_dependencies($service{'servicename_id'});
		$form .= Forms->dependency_list($name,'services',$service{'servicename_id'},$user_acct,\%dependencies);
		my @dep_templates = StorProc->fetch_list('service_dependency_templates','name');
		$form .= Forms->dependency_add($dep_template,\@dep_templates,\@dep_hosts,\%docs);
		$hidden{'name'} = $name;
		$form .= Forms->hidden(\%hidden);
		my %add_dependency = ('name' => 'add_dependency','value' => 'Add Dependency');
		$form .= Forms->form_bottem_one_button(\%add_dependency,$tab++);
	} elsif ($obj_view eq 'service_externals') {
		$hidden{'service_id'} = $query->param('service_id');
		$hidden{'service_name'} = $query->param('service_name');
		my %objs = ('service_id' => $hidden{'service_id'},'service_name' => $hidden{'service_name'});
		$form .= Forms->service_top($name,$user_acct,$obj_view,\%objs,$auth_add{'externals'},$hidden{'selected'});
		my %externals = ();
		my %where = ('servicename_id' => $service{'servicename_id'});
		my @externals = StorProc->fetch_list_where('external_service_names','external_id',\%where);
		foreach my $eid (@externals) {
			my %e = StorProc->fetch_one('externals','external_id',$eid);
			$externals{$e{'name'}} = $eid;
		}
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		if (@messages) { $form .= Forms->form_message('Status:',\@messages,'msg') }
		my %w = ('type' => 'service');
		my @external_names = StorProc->fetch_list_where('externals','name',\%w);
		my @external_list = ();
		foreach my $s (@external_names) { unless ($externals{$s}) { push @external_list, $s } }
		$form .= Forms->external_list($user_acct,$name,\%externals,\@external_list,'service_name',$hidden{'service_id'},$hidden{'service_name'});
		$hidden{'type'} = 'service';
		$hidden{'id'} = $properties{'host_id'};
		$hidden{'name'} = $name;
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_no_button();
	} elsif ($obj_view eq 'service_profiles') {
		$form .= Forms->service_top($name,$user_acct,$obj_view,\%objs,$auth_add{'externals'},$hidden{'selected'});
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->wizard_doc('Service Profiles',$docs{'service_profiles'});
		my @members = ();
		my %w = ('servicename_id' => $service{'servicename_id'});
		my @sids = StorProc->fetch_list_where('serviceprofile','serviceprofile_id',\%w);
		foreach (@sids) {
			my %s = StorProc->fetch_one('profiles_service','serviceprofile_id',$_);
			push @members, $s{'name'};
		}
		my @nonmembers = StorProc->fetch_list('profiles_service','name');
		$form .= Forms->members('Service profiles:','profiles',\@members,\@nonmembers,'','20',$docs{'services'});
		$hidden{'obj_view'} = 'service_profiles';
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%save,$tab++);
	} elsif ($obj_view eq 'apply_hosts') {
		$form .= Forms->service_top($name,$user_acct,$obj_view,\%objs,$auth_add{'externals'},$hidden{'selected'});
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->wizard_doc('Apply Hosts',$docs{'apply'});
		my %where = ('servicename_id' => $service{'servicename_id'});
		my @hosts = StorProc->fetch_list_where('services','host_id',\%where);
		my @host_list = ();
		my %host_name = StorProc->get_table_objects('hosts');
		foreach my $hn (sort keys %host_name) {
			foreach my $hid (@hosts) {
				if ($hid eq $host_name{$hn}) { push @host_list, $hn	}
			}
		}
		$form .= Forms->form_message('Hosts:',\@host_list,'row1');
		$form .= Forms->apply_select($view,\%service,$nagios_ver);
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%apply,$tab++);

	} elsif ($obj_view eq 'delete') {
		my $message = qq(Are you sure you want to remove service $name from all hosts and profiles?);
		$form .= Forms->are_you_sure('Confirm Delete:',$message,'confirm_delete',\%hidden);

	} elsif ($obj_view eq 'applied') {
		my %objs = ('service_id' => $service{'servicename_id'},'name' => $name);
		$form .= Forms->service_top($name,$user_acct,$obj_view,\%objs,$auth_add{'externals'},$hidden{'selected'});
		$form .= Forms->display_hidden('Applied:','',$message_applied);
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_no_button();
	
	} elsif ($obj_view eq 'saved') {
		my %objs = ('service_id' => $service{'servicename_id'},'name' => $name);
		$form .= Forms->service_top($name,$user_acct,$obj_view,\%objs,$auth_add{'externals'},$hidden{'selected'});
		$form .= Forms->display_hidden('Saved:','',$name);
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%close);
	} elsif ($obj_view eq 'rename') {
		$form .= Forms->form_top('Rename Service','');
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->display_hidden('Name:','name',$name,$textsize{'name'},'');
		$form .= Forms->text_box('Rename:','new_name','',$textsize{'name'},'');
		$hidden{'obj_view'} = 'rename';
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_two_button(\%rename,\%cancel,$tab++);
	} elsif ($obj_view eq 'new') {
		my $template = $query->param('template');
		$form .= Forms->form_top('New Service','');
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->text_box('Name:','name',$name,$textsize{'name'},'',$docs{'name'});
		my @members = StorProc->fetch_list('service_templates','name');
		$form .= Forms->list_box('Service template:','template',\@members,$template,'',$docs{'extinfo'});
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_two_button(\%add,\%cancel,$tab++);
	}
	return $form;
}

sub profile_importer() {
	my $file = $query->param('file');
	my $folder = $query->param('folder');
	my $overwrite = $query->param('overwrite');
	my $form = undef;
	my @messages = ();
	if ($query->param('close')) { $obj_view = 'close' }
	if ($file) {
		use MonarchProfileImport;
		@messages = ProfileImporter->import_profile("$folder/$file",$overwrite);
		$refresh_left = "top_menu=$top_menu";
		$obj_view = 'status';
	}
	unless ($obj_view) { $obj_view = 'get_file'	}
	my %docs = ();
	if ($query->param('close')) {
		$form .= Forms->header($page_title,$user_acct,$view);
	} elsif ($obj_view eq 'get_file') {
		$form .= Forms->header($page_title,$user_acct,$view);
		$form .= Forms->form_top("Profile Importer",'');
		unless ($folder) { $folder = '/tmp'	}
		$form .= Forms->text_box('Folder:','folder',$folder,'100','',$docs{'import_folder'});
		my @all_files = StorProc->get_dir($folder);
		my @files = ();
		foreach (@all_files) { if ($_ =~ /xml$/) { push @files, $_ } }
		$form .= Forms->form_files($folder,\@files);
		$form .= Forms->checkbox_left('Overwrite existing objects.','overwrite','');
		$form .= Forms->hidden(\%hidden);
		my %refresh = ('name' => 'refresh','value' => 'Refresh');
		my %select = ('name' => 'select','value' => 'Select');
		if (@files) {
			$form .= Forms->form_bottem_three_button(\%select,\%refresh,\%close,$tab++);
		} else {
			$form .= Forms->form_bottem_two_button(\%refresh,\%close,$tab++);
		}
	} elsif ($obj_view eq 'status') {
		$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
		$form .= Forms->form_top("Profile Importer",'');
		$form .= Forms->display_hidden('File:','',$file);
		if ($messages[0] =~ /error/i) {
			push @messages, "Please make the necessary corrections and try again.";
			$form .= Forms->form_errors(\@messages)
		} else {
			$form .= Forms->form_message('Status:',\@messages,'row1');
		}
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%close,$tab++);
	}
	return $form;
}

#
###########################################################################################################################
# Manage Host
#

sub manage_host() {
	my @views = ('host_detail','profile','parents','hostgroups','escalations','host_externals','host_external_detail','services','service_detail','service_check','service_dependencies','service_externals','service_external_detail');
	foreach my $v (@views) {
		if ($query->param($v)) { $obj_view = $v }
	}
	my $by = 'name';
	my $name = $query->param('name');
	my $address = $query->param('address');
	my $host = $query->param('host');
	unless ($name) { $by = 'address' }
	my $form = undef;
	my $message = undef;
	my $required = undef;
	my $results = undef;
	my $test_results = undef;
	if ($query->param('left_set')) {
		$form = Forms->header($page_title,$user_acct,$view);
	} else {
		my $selected = $query->param('selected');
		$form = Forms->header($page_title,$user_acct,$view,'',"top_menu=hosts&selected=$name");
	}
	$hidden{'left_set'} = 1;
	$hidden{'form_service'} = $query->param('form_service');
	%properties = StorProc->fetch_host($name,$by);
	if ($query->param('close') || $query->param('continue')) {
		$form .= Forms->header($page_title,$user_acct,$view);
		$obj_view = 'close';
	} elsif ($query->param('apply')) {
		if ($obj_view eq 'profile') {
			$refresh_left = "top_menu=$top_menu&selected=$name";
			my $host_profile = $query->param('host_profile');
			if ($host_profile eq 'no profile') {
				$hidden{'hostprofile_id'} = '';
				my %values = ('hostprofile_id' => '');
				my $result = StorProc->update_obj('hosts','host_id',$properties{'host_id'},\%values);
				if ($result =~ /^Error/) { push @errors, $result }	
			} else {
				my %w = ('name' => $host_profile);
				my %h = StorProc->fetch_one_where('profiles_host',\%w);
				my %values = ('hostprofile_id' => $h{'hostprofile_id'});
				$hidden{'hostprofile_id'} = $h{'hostprofile_id'};
				my $result = StorProc->update_obj('hosts','host_id',$properties{'host_id'},\%values);
				if ($result =~ /^Error/) { push @errors, $result }	
			}
			my @service_profiles = $query->param('service_profiles');
			my @profiles = ();
			my $result = StorProc->delete_all('serviceprofile_host','host_id',$properties{'host_id'});
			if ($result =~ /^Error/) { push @errors, $result }	
			foreach my $sp (@service_profiles) {
				my %s = StorProc->fetch_one('profiles_service','name',$sp);
				push @profiles, $s{'serviceprofile_id'};
				my @vals = ($s{'serviceprofile_id'},$properties{'host_id'});
				my $result = StorProc->insert_obj('serviceprofile_host',\@vals);
				if ($result =~ /^Error/) { push @errors, $result }	
			}
			my @hosts = ($properties{'host_id'});
			my $services = $query->param('apply_services');
			my ($cnt, $err) = StorProc->service_profile_apply(\@profiles,$services,\@hosts);
			if ($err) { push (@errors, @{$err}) }
			unless (@errors) {
				$obj_view = 'saved_apply';
				$hidden{'host_id'} = $properties{'host_id'};
				$hidden{'name'} = $name;
				$message = "Changes to $name accepted, $cnt services applied.";
			}	
		} else {
			my $host_id = $query->param('host_id');
			my $hostprofile_id = $query->param('hostprofile_id');
			my $serviceprofile_id = $query->param('serviceprofile_id');
			my %sp = StorProc->fetch_one('profiles_service','serviceprofile_id',$serviceprofile_id);
			my %w = ('hostprofile_id' => $hostprofile_id);
			my @externals = StorProc->fetch_list_where('external_host_profile','external_id',\%w);
			my $result = StorProc->delete_all('external_host','host_id',$host_id);
			if ($result =~ /^Error/) { push @errors, $result }
			foreach my $ext (@externals) {
				my %e = StorProc->fetch_one('externals','external_id',$ext);
				my @vals = ($ext,$host_id,$e{'display'});
				$result = StorProc->insert_obj('external_host',\@vals);
				if ($result =~ /^Error/) { push @errors, $result }
			}
			my %w = ('serviceprofile_id' => $serviceprofile_id);
			my @services = StorProc->fetch_list_where('serviceprofile','servicename_id',\%w);
			my $result = StorProc->delete_all('services','host_id',$host_id);
			if ($result =~ /^Error/) { push @errors, $result }
			foreach my $sid (@services) {
				my %sn = StorProc->fetch_one('service_names','servicename_id',$sid);
				%w = ('servicename_id' => $sid);
				@externals = StorProc->fetch_list_where('external_service_names','external_id',\%w);
				my @vals = ('',$host_id,$sid,$sn{'template'},$sn{'extinfo'},$sn{'escalation'},'1',$sn{'check_comand'},$sn{'command_line'},'');
				my $service_id = StorProc->insert_obj_id('services',\@vals,'service_id');
				if ($service_id =~ /^Error/) { push @errors, $service_id }
				@vals = ($sp{'file_id'},$service_id);
				$result = StorProc->insert_obj('file_service',\@vals);
				if ($result =~ /^Error/) { push @errors, $result }
				if ($sn{'dependency'}) {
					@vals = ('',$service_id,$host_id,$host_id,$sn{'template'},'');
					$result = StorProc->insert_obj('service_dependency',\@vals);
					if ($result =~ /^Error/) { push @errors, $result }
				}
				foreach my $ext (@externals) {
					my %e = StorProc->fetch_one('externals','external_id',$ext);
					@vals = ($ext,$host_id,$service_id,$e{'display'});
					$result = StorProc->insert_obj('external_service',\@vals);
					if ($result =~ /^Error/) { push @errors, $result }
				}
			}
		}
	} elsif ($query->param('test_command')) {	
		my $arg_string = $query->param('arg_string');
		my $command = $query->param('command');
		my %cmd = StorProc->fetch_one('commands','name',$command);
		my %host = StorProc->fetch_one('hosts','name',$host);
		my %where = ('type' => 'resource'); 
		my %resources = StorProc->get_resources();
		foreach my $res (keys %resources) {
			if ($cmd{'command_line'} =~ /$res/i) { $cmd{'command_line'} =~ s/\$$res\$/$resources{$res}/ig } 
		}
		if ($cmd{'command_line'} =~ /\$HOSTADDRESS\$/) { 
			if ($host =~ /\d+\.\d+\.\d+\.\d+/) { $cmd{'command_line'} =~ s/\$HOSTADDRESS\$/$host/g }
			$cmd{'command_line'} =~ s/\$HOSTADDRESS\$/$host{'address'}/g; 
		}
		if ($cmd{'command_line'} =~ /\$HOSTNAME\$/) { $cmd{'command_line'} =~ s/\$HOSTNAME\$/$host/g } 
		my @args = split(/!/, $arg_string);
		my $cnt = 1;
		foreach my $a (@args) {
			$cmd{'command_line'} =~ s/\$ARG$cnt\$/$a/g;
			$cnt++;
		}
		$test_results = "$cmd{'command_line'}<br/>";
		$test_results .= StorProc->test_command($cmd{'command_line'});	
	} elsif ($query->param('external_add')) {	
		my $external_name = $query->param('external');
		my $service_id = $query->param('service_id');
		my $host_id = $query->param('id');
		my $type = $query->param('type');
		my %w = ('name' => $external_name,'type' => $type);
		my %e = StorProc->fetch_one_where('externals',\%w);
		if ($type eq 'service') {
			my @vals = ($e{'external_id'},$host_id,$service_id,$e{'display'});
			my $result = StorProc->insert_obj('external_service',\@vals);
			if ($result =~ /^Error/) { push @errors, $result }
		} else {
			my @vals = ($e{'external_id'},$host_id,$e{'display'});
			my $result = StorProc->insert_obj('external_host',\@vals);
			if ($result =~ /^Error/) { push @errors, $result }
		}
	} elsif ($query->param('save')) {	
		if ($obj_view eq 'host_detail') {
			my %data = parse_query('host_templates','host_overrides');
			my $alias = $query->param('alias');
			my $address = $query->param('address');
			my $host_template = $query->param('template');
			my $extinfo = $query->param('extended_info');
			my $checks_enabled = $query->param('checks_enabled');
			my $coords2d = $query->param('coords2d');
			my $coords3d = $query->param('coords3d');
			my $file = $query->param('file');
			if ($alias && $address && $host_template && $file) {
				my %values = ('alias' => $alias,'address' => $address);
				my %t = StorProc->fetch_one('host_templates','name',$host_template);
				$values{'hosttemplate_id'} = $t{'hosttemplate_id'};
				my %e = StorProc->fetch_one('extended_host_info_templates','name',$extinfo);
				$values{'hostextinfo_id'} = $e{'hostextinfo_id'};
				my $result = StorProc->update_obj('hosts','host_id',$properties{'host_id'},\%values);
				if ($result =~ /^Error/) { push @errors, $result }
				$result = StorProc->delete_all('file_host','host_id',$properties{'host_id'});
				if ($result =~ /^Error/) { push @errors, $result }
				my %f =  StorProc->fetch_one('files','name',$file,);
				my @vals = ($f{'file_id'},$properties{'host_id'});
				$result = StorProc->insert_obj('file_host',\@vals);
				if ($result =~ /^Error/) { push @errors, $result }
				if ($coords2d || $coords3d) { 
 					my $data = "<?xml version=\"1.0\" ?>\n<data>";
					if ($coords2d) { $data .= "\n  <prop name=\"2d_coords\"><![CDATA[$coords2d]]>\n  </prop>" }
					if ($coords3d) { $data .= "\n  <prop name=\"3d_coords\"><![CDATA[$coords3d]]>\n  </prop>" }
					$data .= "\n</data>";
					my %w = ('host_id' => $properties{'host_id'});
					my $result = StorProc->delete_one_where('extended_info_coords',\%w);
					if ($result =~ /^Error/) { push @errors, $result }
					my @vals = ($properties{'host_id'},$data);
					$result = StorProc->insert_obj('extended_info_coords',\@vals);
					if ($result =~ /^Error/) { push @errors, $result }	
				}
				my %where = ('host_id' => $properties{'host_id'});
				my $result = StorProc->delete_one_where('host_overrides',\%where);
				if ($result =~ /^Error/) { push @errors, $result }
				if ($data{'check_period'} || $data{'notification_period'}|| $data{'check_command'} || $data{'event_handler'} || $data{'data'}) {
					my @values = ($properties{'host_id'},$data{'check_period'},$data{'notification_period'},$data{'check_command'},$data{'event_handler'},$data{'data'});
					my $result = StorProc->insert_obj('host_overrides',\@values);
					if ($result =~ /^Error/) { push @errors, $result }
				}
				if ($query->param('contactgroup_override')) {
					my %where = ('type' => 'hosts','object' => $properties{'host_id'});
					my $result = StorProc->delete_one_where('contactgroup_assign',\%where);
					if ($result =~ /^Error/) { push @errors, $result }
				} else {
					my %where = ('type' => 'hosts','object' => $properties{'host_id'});
					my $result = StorProc->delete_one_where('contactgroup_assign',\%where);
					if ($result =~ /^Error/) { push @errors, $result }
					my @mems = $query->param('contactgroup');
					foreach (@mems) {
						my %cg = StorProc->fetch_one('contactgroups','name',$_);
						my @vals = ($cg{'contactgroup_id'},'hosts',$properties{'host_id'});
						$result = StorProc->insert_obj('contactgroup_assign',\@vals);
						if ($result =~ /^Error/) { push @errors, $result }
					}
				}
				unless (@errors) {
					$obj_view = 'saved';
					$message = "Changes to $name accepted";
				}
			} else {
				push @errors, "Required: alias, address, host template, and file";
				$required = 1;
			}
		} elsif ($obj_view eq 'escalations') {
			my $host_escalation = $query->param('host_escalation');
			my $service_escalation = $query->param('service_escalation');
			my %w = ('name' => $host_escalation,'type' => 'host');
			my %he = StorProc->fetch_one_where('escalation_trees',\%w);
			%w = ('name' => $service_escalation,'type' => 'service');
			my %se = StorProc->fetch_one_where('escalation_trees',\%w);
			my %values = ('host_escalation_id' => $he{'tree_id'},'service_escalation_id' => $se{'tree_id'});
			my $result = StorProc->update_obj('hosts','host_id',$properties{'host_id'},\%values);
			if ($result =~ /^Error/) { push @errors, $result }	
			unless (@errors) {
				$obj_view = 'saved';
				$message = "Changes to $name accepted";
			}
		} elsif ($obj_view eq 'parents') {
			my @parents = $query->param('parents');
			my %w = ('host_id' => $properties{'host_id'});
			my $result = StorProc->delete_one_where('host_parent',\%w);
			if ($result =~ /^Error/) { push @errors, $result }
			my @parents = $query->param('parents');
			foreach my $parent (@parents) {
				if (!$parent) { next }
				my %h = StorProc->fetch_one('hosts','name',$parent);
				my @vals = ($properties{'host_id'},$h{'host_id'});
				my $result = StorProc->insert_obj('host_parent',\@vals);
				if ($result =~ /^Error/) { push @errors, $result }
			}
			unless (@errors) { 
				$obj_view = 'saved';
				$message = "Changes to parent accepted";
			}
		} elsif ($obj_view eq 'hostgroups') {
			my %w = ('host_id' => $properties{'host_id'});
			my $result = StorProc->delete_one_where('hostgroup_host',\%w);
			if ($result =~ /^Error/) { push @errors, $result }
			my @hostgroups = $query->param('members');
			foreach my $hg (@hostgroups) {
				if (!$hg) { next }
				my %h = StorProc->fetch_one('hostgroups','name',$hg);
				my @vals = ($h{'hostgroup_id'},$properties{'host_id'});
				my $result = StorProc->insert_obj('hostgroup_host',\@vals);
				if ($result =~ /^Error/) { push @errors, $result }
			}
			unless (@errors) { 
				$obj_view = 'saved';
				$message = "Changes to $name hostgroups accepted";
			}
		} elsif ($obj_view eq 'host_external_detail') {
			my $data = $query->param('detail');
			my $external_id = $query->param('external_id');
			my %where = ('host_id' => $properties{'host_id'},'external_id' => $external_id);
			my %values = ('data' => $data);
			my $result = StorProc->update_obj_where('external_host',\%values,\%where);
			if ($result =~ /^Error/) { push @errors, $result }	
			unless (@errors) { 
				$obj_view = 'saved';
				$message = "Changes to $name external accepted";
			}
		} elsif ($obj_view eq 'service_external_detail') {
			my $data = $query->param('detail');
			my $external_id = $query->param('external_id');
			my $service_name = $query->param('service_name');
			my $service_id = $query->param('service_id');
			$hidden{'service_name'} = $service_name;
			$hidden{'service_id'} = $service_id;
			my %values = ('data' => $data);
			my %where = ('service_id' => $service_id,'external_id' => $external_id);
			my $result = StorProc->update_obj_where('external_service',\%values,\%where);
			if ($result =~ /^Error/) { push @errors, $result }	
			unless (@errors) { 
				$obj_view = 'saved';
				$message = "Changes to $name external accepted";
				$hidden{'service_detail'} = 1;
			}
		} elsif ($obj_view eq 'service_detail') {
			my %values = ();
			my $service_name = $query->param('service_name');
			my $service_id = $query->param('service_id');
			$hidden{'service_name'} = $service_name;
			$hidden{'service_id'} = $service_id;
			my %s = StorProc->fetch_one('service_names','name',$service_name);
			my $ext_info = $query->param('ext_info');
			my %x = StorProc->fetch_one('extended_service_info_templates','name',$ext_info);
			my $escalation = $query->param('escalation');
			my %e = StorProc->fetch_one('escalation_trees','name',$escalation);
			my $template = $query->param('template');
			my %t = StorProc->fetch_one('service_templates','name',$template);
			my %data = parse_query('service_overrides','service_overrides');
			my %vals = ('serviceextinfo_id' => $x{'serviceextinfo_id'},'escalation_id' => $e{'tree_id'},'servicetemplate_id' => $t{'servicetemplate_id'});
			my $result = StorProc->update_obj('services','service_id',$service_id,\%vals);
			if ($result =~ /^Error/) { push @errors, $result }
			my %where = ('service_id' => $properties{'service_id'});
			my $result = StorProc->delete_one_where('service_overrides',\%where);
			if ($result =~ /^Error/) { push @errors, $result }
			if ($data{'check_period'} || $data{'notification_period'} || $data{'event_handler'} || $data{'data'}) {
				my @values = ($service_id,$data{'check_period'},$data{'notification_period'},$data{'event_handler'},$data{'data'});
				my $result = StorProc->insert_obj('service_overrides',\@values);
				if ($result =~ /^Error/) { push @errors, $result }
			}
			my %where = ('type' => 'services','object' => $service_id);
			my $result = StorProc->delete_one_where('contactgroup_assign',\%where);
			if ($result =~ /^Error/) { push @errors, $result }
			unless ($query->param('contactgroup_override')) {
				my @mems = $query->param('contactgroup');
				foreach (@mems) {
					my %cg = StorProc->fetch_one('contactgroups','name',$_);
					my @vals = ($cg{'contactgroup_id'},'services',$service_id);
					$result = StorProc->insert_obj('contactgroup_assign',\@vals);
					if ($result =~ /^Error/) { push @errors, $result }
				}
			}
			unless (@errors) {
				$obj_view = 'saved';
				$message = "Changes to $service_name accepted";
				$hidden{'service_detail'} = 1;
			}
		} elsif ($obj_view eq 'service_check') {
			my $service_name = $query->param('service_name');
			my $service_id = $query->param('service_id');
			$hidden{'service_name'} = $service_name;
			$hidden{'service_id'} = $service_id;
			my $check_command = $query->param('command');
			my $command_line = $query->param('command_line');
			if ($query->param('inherit')) {
				my $required = $query->param('required');
				unless ($required) {
					my %vals = ('check_command' => '','command_line' => '');
					my $result = StorProc->update_obj('services','service_id',$service_id,\%vals);
					if ($result =~ /^Error/) { push @errors, $result }
				} else {
					push @errors, "The service check is not defined on a template so it must be defined here.";
				}
			} else {
				my %check = StorProc->fetch_one('commands','name',$check_command);
				my %vals = ('check_command' => $check{'command_id'},'command_line' => $command_line);
				my $result = StorProc->update_obj('services','service_id',$service_id,\%vals);
				if ($result =~ /^Error/) { push @errors, $result }
			}
			unless (@errors) {
				$obj_view = 'saved';
				$message = "Changes to $service_name accepted";
				$hidden{'service_check'} = 1;
			}
		}	
	} elsif ($query->param('add_dependency')) {
		my $dep_template = $query->param('dep_template');
		my $service_name = $query->param('service_name');
		my $service_id = $query->param('service_id');
		$hidden{'service_name'} = $service_name;
		$hidden{'service_id'} = $service_id;
		my $parent = $query->param('depend_on_host');
		my %p = StorProc->fetch_one('hosts','name',$parent);
		my %d = StorProc->fetch_one('service_dependency_templates','name',$dep_template);
		my %w = ('service_id' => $service_id,'host_id' => $properties{'host_id'},'depend_on_host_id' => $p{'host_id'},'template' => $d{'id'});
		my %check_dep = StorProc->fetch_one_where('service_dependency',\%w);	
		if ($dep_template && $service_id && $parent) {
			unless ($check_dep{'id'}) {
				my @values = ('',$service_id,$properties{'host_id'},$p{'host_id'},$d{'id'},'');
				my $result = StorProc->insert_obj('service_dependency',\@values);
				if ($result =~ /^Error/) { push @errors, $result }	
			} else {
				push @errors, "Already defined. Check existing dependencies";
			}
		}
	} elsif ($query->param('remove_dependency')) {
		my $dep_id = $query->param('dependency_id');
		my $result = StorProc->delete_all('service_dependency','id',$dep_id);
		if ($result =~ /^Error/) { push @errors, $result }
	} elsif ($submit eq 'remove_service') {
		$refresh_left = "top_menu=$top_menu&selected=$name";
		my $service_id = $query->param('service_id');
		my $result = StorProc->delete_all('services','service_id',$service_id);
		if ($result =~ /^Error/) { push @errors, $result }
		$obj_view = 'services';
	} elsif ($query->param('rename')) {
		my $new_name = $query->param('new_name');
		if ($new_name) {
			my %n = StorProc->fetch_one('hosts','name',$new_name);
			if ($name =~ /$new_name/i) { delete $n{'name'} }
			if ($n{'name'}) {
				$obj_view = 'rename';
				push @errors, "Cannot rename. Another host has name $new_name.";
			} else {
				my %values = ('name' => $new_name);
				my $result = StorProc->update_obj('hosts','name',$name,\%values);
				if ($result =~ /^Error/) { 
					push @errors, $result;
					$obj_view = 'rename';
				} else {
					$name = $new_name;
					%properties = StorProc->fetch_host($name);
					$refresh_left = "top_menu=$top_menu&selected=$name";
					$obj_view = 'host_detail';
				}
			}
		} else {
			$obj_view = 'rename';
		}
	} elsif ($submit eq 'Add Service(s)') {
		my $file = $query->param('file');
		if ($file) {
			my @services = $query->param('add_service');
			foreach my $service (@services) {
				if ($service) {
					my %s = StorProc->fetch_one('service_names','name',$service);
					my @values = ('',$properties{'host_id'},$s{'servicename_id'},$s{'template'},$s{'extinfo'},$s{'escalation'},'1',$s{'check_command'},$s{'command_line'},'');
					my $id = StorProc->insert_obj_id('services',\@values,'service_id');
					if ($id =~ /^Error/) { push @errors, $id }	
					my $file = $query->param('file');
					my %f = StorProc->fetch_one('files','name',$file);
					@values = ($f{'file_id'},$id);
					my $result = StorProc->insert_obj('file_service',\@values);
					if ($result =~ /^Error/) { push @errors, $result }	
					if ($s{'dependency'}) {
						my %t =  StorProc->fetch_one('service_dependency_templates','name',$s{'dependency'});
						@values = ('',$id,$properties{'host_id'},$properties{'host_id'},$s{'dependency'},'');
						$result = StorProc->insert_obj('service_dependency',\@values);
						if ($result =~ /^Error/) { push @errors, $result }	
					}
				}
			}	
			$refresh_left = "top_menu=$top_menu&selected=$name";
		} else {
			push @errors, "Required: Select a file.";
			$required{'file'} = 1;
		}
	} elsif ($query->param('delete') || $query->param('confirm_delete_host') || $query->param('task') eq 'No') {
		if ($query->param('confirm_delete_host')) {
			my $result = StorProc->delete_all('hosts','name',$name);
			if ($result =~ /^Error/) { 
				push @errors, $result;
				$obj_view = 'host_detail';
			} else {
				$refresh_left = "top_menu=$top_menu";
				$obj_view = 'deleted';
			}
		} elsif ($query->param('task') eq 'No') {
			$obj_view = 'host_detail';		
		} else {
			$obj_view = 'delete_host';
		}
	} elsif ($query->param('delete') || $query->param('confirm_delete_service')) {
		my $service_name = $query->param('service_name');
		if ($query->param('confirm_delete_service')) {
			my %where = ('name' => $service_name, 'host_name' => $name);
			my %sid = StorProc->fetch_one_where('services',\%where);
			my $result = StorProc->delete_all('services','service_id',$sid{'id'});
			if ($result =~ /^Error/) { push @errors, $result }
			$obj_view = 'Service List';
		} elsif ($query->param('task') eq 'No') {
			$obj_view = 'Service List';		
		} else {
			$hidden{'name'} = $name; 
			$hidden{'obj_view'} = $obj_view;
			$hidden{'submit'} = 'service_delete';
			$hidden{'service_name'} = $service_name;
			my $message = qq(Are you sure you want to remove service $service_name from host $name?);
			$form .= Forms->are_you_sure('Confirm Delete:',$message,'confirm_delete_service',\%hidden);
		}
	} else {
		my $got_obj = 0;
		foreach my $param ($query->param) {
			if ($param =~ /remove_external_(\d+)/) {
				my $eid = $1;
				if ($obj_view eq 'host_externals') {
					my %where = ('host_id' => $properties{'host_id'},'external_id' => $eid);
					my $result = StorProc->delete_one_where('external_host',\%where);
					if ($result =~ /^Error/) { push @errors, $result }						
				} else {
					my $service_id = $query->param('service_id');
					my %where = ('service_id' => $service_id,'external_id' => $eid);
					my $result = StorProc->delete_one_where('external_service',\%where);
					if ($result =~ /^Error/) { push @errors, $result }						
				}
			} elsif ($param eq 'select_external') {
				if ($obj_view eq 'host_externals') {
					$properties{'external'} = $query->param('select_external');
					my %e = StorProc->fetch_one('externals','name',$properties{'external'});
					$properties{'external_id'} = $e{'external_id'};
					$obj_view = 'host_external_detail';
					$got_obj = 1;
				} else {
					$properties{'external'} = $query->param('select_external');
					my %e = StorProc->fetch_one('externals','name',$properties{'external'});
					$properties{'external_id'} = $e{'external_id'};
					$obj_view = 'service_external_detail';
					$got_obj = 1;
				}
			}
		}
	}
	delete $hidden{'task'};
	delete $hidden{'obj_view'};
	%save = ('name' => 'save','value' => 'Save');
	unless ($obj_view) { $obj_view = 'host_detail' }
	$hidden{'obj_view'} = $obj_view;
	$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
	if ($obj_view eq 'host_detail') {
		my %docs = Doc->manage_hosts_vitals();
		$form .= Forms->host_top($properties{'name'},$user_acct,$obj_view,$auth_add{'externals'},$hidden{'selected'},$hidden{'form_service'});
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->text_box('Alias:','alias',$properties{'alias'},$textsize{'alias'},$required,$docs{'alias'},'','1');
		$form .= Forms->text_box('Address:','address',$properties{'address'},$textsize{'address'},$required,$docs{'address'},'','2');
		$form .= build_host_template($name);
		my @members = StorProc->fetch_list('extended_host_info_templates','name'); 
		push @members, 'remove_extended_info';
		$form .= Forms->list_box('Extended host info template:','extended_info',\@members,$properties{'ext_info'},'',$docs{'extinfo'});
		$form .= Forms->text_box('2d status coords:','coords2d',$properties{'coords2d'},'10','',$docs{'coords2d'});
		$form .= Forms->text_box('3d status coords:','coords3d',$properties{'coords3d'},'15','',$docs{'coords3d'});
		my $file = StorProc->get_file_name('file_host','host_id',$properties{'host_id'});
		my %w = ('type' => 'hosts');
		my @files = StorProc->fetch_list_where('files','name',\%w);
		$form .= Forms->list_box('File:','file',\@files,$file,$required,$docs{'file'},'',$tab++);
		$hidden{'name'} = $name;
		$form .= Forms->hidden(\%hidden);		
		if ($auth_delete{$obj}) {
			$form .= Forms->form_bottem_three_button(\%save,\%delete,\%rename,$tab++);
		} else {
			$form .= Forms->form_bottem_two_button(\%save,\%rename,$tab++);
		}
	} elsif ($obj_view eq 'profile') {
		my %docs = Doc->manage_hosts_profile();
		my @service_ids = ();
		my $host_profile = $query->param('host_profile');
		my @service_profiles = $query->param('service_profiles');
		my $host_profile_save = undef;
		$form .= Forms->host_top($properties{'name'},$user_acct,$obj_view,$auth_add{'externals'},$hidden{'selected'},$hidden{'form_service'});
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->wizard_doc('Profile',$docs{'profile'});
		my $host_profile_id = undef;
		if ($host_profile) {
			my %h = StorProc->fetch_one('profiles_host','name',$host_profile);
			$host_profile_id = $h{'hostprofile_id'};
		} else {
			my %h = StorProc->fetch_one('profiles_host','hostprofile_id',$properties{'hostprofile_id'});
			$host_profile_id = $properties{'hostprofile_id'};
			$host_profile = $h{'name'};
			$host_profile_save = $host_profile;
		}
		unless ($host_profile) { $host_profile = 'no profile' }
		$hidden{'host_profile_save'} = $host_profile;
		my @host_profiles = ('no profile');
		my @profiles = StorProc->fetch_list('profiles_host','name');
		push (@host_profiles,@profiles);
		$form .= Forms->list_box('Host profile:','host_profile',\@host_profiles,$host_profile,'',$docs{'host_profile'});
		if ($host_profile eq 'no profile') {
			if (@service_profiles) {
				foreach my $sp (@service_profiles) {
					my %s = StorProc->fetch_one('profiles_service','name',$sp);
					my %w = ('serviceprofile_id' => $s{'serviceprofile_id'});
					my @sids = StorProc->fetch_list_where('serviceprofile','servicename_id',\%w);
					push (@service_ids,@sids);
				} 
			} else {
				my %w = ('host_id' => $properties{'host_id'});
				my @sids = StorProc->fetch_list_where('serviceprofile_host','serviceprofile_id',\%w);
				my %s = ();
				foreach my $sid (@sids) {
					unless ($s{$sid}) {
						my %s = StorProc->fetch_one('profiles_service','serviceprofile_id',$sid);
						push @service_profiles, $s{'name'};
						my %w = ('serviceprofile_id' => $sid);
						my @sids = StorProc->fetch_list_where('serviceprofile','servicename_id',\%w);
						push (@service_ids,@sids);
					}
				}
			}
		} elsif ($host_profile_save ne $host_profile) {
			$host_profile_save = undef;
			@service_profiles = ();
			my %w = ('hostprofile_id' => $host_profile_id);
			my @sids = StorProc->fetch_list_where('profile_host_profile_service','serviceprofile_id',\%w);
			my %s = ();
			foreach my $sid (@sids) {
				$s{$sid} =  1;
				my %s = StorProc->fetch_one('profiles_service','serviceprofile_id',$sid);
				push @service_profiles, $s{'name'};
				my %w = ('serviceprofile_id' => $sid);
				my @sids = StorProc->fetch_list_where('serviceprofile','servicename_id',\%w);
				push (@service_ids,@sids);
			}
		} elsif (@service_profiles) {
			foreach my $sp (@service_profiles) {
				my %s = StorProc->fetch_one('profiles_service','name',$sp);
				my %w = ('serviceprofile_id' => $s{'serviceprofile_id'});
				my @sids = StorProc->fetch_list_where('serviceprofile','servicename_id',\%w);
				push (@service_ids,@sids);
			}
		} else {
			my %w = ('host_id' => $properties{'host_id'});
			my @sids = StorProc->fetch_list_where('serviceprofile_host','serviceprofile_id',\%w);
			my %s = ();
			foreach my $sid (@sids) {
				unless ($s{$sid}) {
					my %s = StorProc->fetch_one('profiles_service','serviceprofile_id',$sid);
					push @service_profiles, $s{'name'};
					my %w = ('serviceprofile_id' => $sid);
					my @sids = StorProc->fetch_list_where('serviceprofile','servicename_id',\%w);
					push (@service_ids,@sids);
				}
			}
		}
		my @nonmembers = StorProc->fetch_list('profiles_service','name');
		$form .= Forms->members('Service profiles:','service_profiles',\@service_profiles,\@nonmembers,'','5',$docs{'services'});
		my @services = ();
		my %s = ();
		foreach my $sid (@service_ids) {
			unless ($s{$sid}) {
				$s{$sid} =  1;
				my %s = StorProc->fetch_one('service_names','servicename_id',$sid);
				if ($s{'name'}) { push @services, $s{'name'} }
			}
		}
		$form .= Forms->form_message('Services:',\@services,'row1');
		my %selected = ();
		$selected{'merge'} = 'checked';
		$form .= Forms->apply_select($view,\%selected,$nagios_ver);
		$hidden{'name'} = $name;
		$hidden{'host_profile_save'} = $host_profile_save;
		my %save = ('name' => 'save','value' => 'Save');
		$form .= Forms->hidden(\%hidden);
		my %refresh = ('name' => 'refresh','value' => 'Refresh');
		my %apply = ('name' => 'apply','value' => 'Apply');
		$form .= Forms->form_bottem_two_button(\%apply,\%refresh,$tab++);
	} elsif ($obj_view eq 'parents') {
		my %docs = Doc->manage_hosts_parents();
		$form .= Forms->host_top($properties{'name'},$user_acct,$obj_view,$auth_add{'externals'},$hidden{'selected'},$hidden{'form_service'});
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		my @members = @{$properties{'parents'}};
		# my @nonmembers = StorProc->get_parents();
		my @nonmembers = StorProc->fetch_list('hosts','name');
		$form .= Forms->members('Parents:','parents',\@members,\@nonmembers,'','',$docs{'parents'});
		$hidden{'name'} = $name;
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%save,$tab++);
	} elsif ($obj_view eq 'escalations') {
		my %docs = Doc->manage_hosts_escalations();
		my $host_escalation = $query->param('host_escalation');
		my $service_escalation = $query->param('service_escalation');
		unless ($host_escalation) {
			my %w = ('tree_id' => $properties{'host_escalation_id'} );
			my %t = StorProc->fetch_one_where('escalation_trees',\%w);
			$host_escalation = $t{'name'};
		}
		unless ($service_escalation) {
			my %w = ('tree_id' => $properties{'service_escalation_id'} );
			my %t = StorProc->fetch_one_where('escalation_trees',\%w);
			$service_escalation = $t{'name'};
		}
		$form .= Forms->host_top($properties{'name'},$user_acct,$obj_view,$auth_add{'externals'},$hidden{'selected'},$hidden{'form_service'});
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->wizard_doc('Escalations',$docs{'escalations'});
		my %where = ('type','host');
		my @members = ('-remove-escalation-');
		my @mems = StorProc->fetch_list_where('escalation_trees','name',\%where);
		push (@members,(@mems));
		$form .= Forms->list_box_submit('Host escalation:','host_escalation',\@members,$host_escalation,'',$docs{'host_escalation_tree'});
		if ($host_escalation) {
			my ($ranks, $templates) = StorProc->get_tree_detail($host_escalation);
			my %ranks = %{$ranks};
			my %templates = %{$templates};
			$form .= Forms->escalation_tree(\%ranks,\%templates,'escalations');
		}
		@members = ('-remove-escalation-');
		my %where = ('type','service');
		@mems = StorProc->fetch_list_where('escalation_trees','name',\%where);
		push (@members,(@mems));
		$form .= Forms->list_box_submit('Service escalation:','service_escalation',\@members,$service_escalation,'',$docs{'service_escalation_tree'});
		if ($service_escalation) {
			my ($ranks, $templates) = StorProc->get_tree_detail($service_escalation);
			my %ranks = %{$ranks};
			my %templates = %{$templates};
			$form .= Forms->escalation_tree(\%ranks,\%templates,'escalations');
		}
		$hidden{'name'} = $name;
		my %save = ('name' => 'save','value' => 'Save');
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%save,$tab++);
	} elsif ($obj_view eq 'hostgroups') {
		my %docs = Doc->manage_hosts_hostgroups();
		$form .= Forms->host_top($properties{'name'},$user_acct,$obj_view,$auth_add{'externals'},$hidden{'selected'},$hidden{'form_service'});
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		my @members = StorProc->get_hostgroup_host($name);
		my @nonmembers = StorProc->fetch_list('hostgroups','name'); 
		$form .= Forms->members('Hostgroups:','members',\@members,\@nonmembers,'','',$docs{'hostgroups'}); 
		$hidden{'name'} = $name;
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%save,$tab++);
	} elsif ($obj_view eq 'services') {
		my %docs = Doc->manage_hosts_services();
		$form .= Forms->host_top($properties{'name'},$user_acct,$obj_view,$auth_add{'externals'},$hidden{'selected'},$hidden{'form_service'});
		my %services = ();
		my %where = ('host_id' => $properties{'host_id'});
		my @services = StorProc->fetch_list_where('services','service_id',\%where);
		my $fsid = undef;
		foreach my $sid (@services) {
			my $service_name = StorProc->get_service_name($sid);
			$service_name =~ s/\s/+/g;
			$services{$service_name} = $sid;
			$fsid = $sid;
		}
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->wizard_doc('Services',$docs{'services'});
		if (@messages) { $form .= Forms->form_message('Status:',\@messages,'msg') }
		my @service_names = StorProc->fetch_list('service_names','name');
		my @service_list = ();
		foreach my $s (@service_names) { $s =~ s/\s/+/g; unless ($services{$s}) { $s =~ s/\+/ /g; push @service_list, $s } }
		@service_list = sort { $a <=> $b } @service_list;
		my $file = StorProc->get_file_name('file_service','service_id',$fsid);
		my %w = ('type' => 'services');
		my @files = StorProc->fetch_list_where('files','name',\%w);
		$form .= Forms->service_list($user_acct,$name,\%services,\@service_list,\@files,$file,$hidden{'selected'});
		$hidden{'name'} = $name;
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_no_button();
	} elsif ($obj_view eq 'host_externals') {
		$form .= Forms->host_top($properties{'name'},$user_acct,$obj_view,$auth_add{'externals'},$hidden{'selected'},$hidden{'form_service'});
		my %externals = ();
		my %where = ('host_id' => $properties{'host_id'});
		my @externals = StorProc->fetch_list_where('external_host','external_id',\%where);
		foreach my $eid (@externals) {
			my %e = StorProc->fetch_one('externals','external_id',$eid);
			$externals{$e{'name'}} = $eid;
		}
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		if (@messages) { $form .= Forms->form_message('Status:',\@messages,'msg') }
		my %w = ('type' => 'host');
		my @external_names = StorProc->fetch_list_where('externals','name',\%w);
		my @external_list = ();
		foreach my $s (@external_names) { unless ($externals{$s}) { push @external_list, $s } }
		@external_list = sort { $a <=> $b } @external_list;
		$form .= Forms->external_list($user_acct,$name,\%externals,\@external_list,'host');
		$hidden{'type'} = 'host';
		$hidden{'name'} = $name;
		$hidden{'id'} = $properties{'host_id'};
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_no_button();
	} elsif ($obj_view eq 'host_external_detail') {
		$form .= Forms->host_top($properties{'name'},$user_acct,$obj_view,$auth_add{'externals'},$hidden{'selected'},$hidden{'form_service'});
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		my $detail = $query->param('detail');
		my $host_external = $query->param('host_external');
		unless ($host_external) { $host_external = $properties{'external'} }
		my %where = ('external_id' => $properties{'external_id'},'host_id' => $properties{'host_id'});
		my %external = StorProc->fetch_one_where('external_host',\%where);
		unless ($detail) { $detail = $external{'data'} }
		$form .= Forms->display_hidden('External:','host_external',$host_external);
		$form .= Forms->text_area('Detail:','detail',$detail,'30','140');
		$hidden{'external_id'} = $external{'external_id'};
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%save,$tab++);
	} elsif ($obj_view eq 'service_detail') {
		my %docs = Doc->manage_hosts_service_detail();
		$hidden{'service_id'} = $query->param('service_id');
		my $service_name = $query->param('service_name');
		my $ext_info = $query->param('ext_info');
		my $escalation = $query->param('escalation');
		unless ($hidden{'service_name'}) { $hidden{'service_name'} = $query->param('service_name') }
		unless ($hidden{'service_id'}) { 
			my %snid =  StorProc->fetch_one('service_names','name',$hidden{'service_name'});
			my %where = ('host_id' => $properties{'host_id'},'servicename_id' => $snid{'servicename_id'});
			my %sid = StorProc->fetch_one_where('services',\%where);	
			$hidden{'service_id'} = $sid{'service_id'};
		}	
		my %p = StorProc->fetch_one('services','service_id',$hidden{'service_id'});
		my %w = ('serviceextinfo_id' => $p{'serviceextinfo_id'});
		my %e = StorProc->fetch_one_where('extended_service_info_templates',\%w);
		$ext_info = $e{'name'};
		%w = ('tree_id' => $p{'escalation_id'});
		unless ($escalation) {
			my %esc = StorProc->fetch_one_where('escalation_trees',\%w);
			$escalation = $esc{'name'};
		}
		my %objs = ('service_id' => $hidden{'service_id'},'service_name' => $service_name);
		$form .= Forms->host_top($properties{'name'},$user_acct,$obj_view,$auth_add{'externals'},$hidden{'selected'},$hidden{'form_service'});
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->display_hidden('Service name:','service_name',$service_name);
		$form .= build_service_detail($hidden{'service_id'});
		my @members = StorProc->fetch_list('extended_service_info_templates','name'); 
		$form .= Forms->list_box('Extended info template:','ext_info',\@members,$ext_info,'',$docs{'extinfo'});
		my %where = ('type' => 'service');
		@members = StorProc->fetch_list_where('escalation_trees','name',\%where); 
		$form .= Forms->list_box_submit('Escalation tree:','escalation',\@members,$escalation,'',$docs{'escalation'});
		if ($escalation) {
			my ($ranks, $templates) = StorProc->get_tree_detail($escalation);
			my %ranks = %{$ranks};
			my %templates = %{$templates};
			$form .= Forms->escalation_tree(\%ranks,\%templates,'service_detail');
		}
		$hidden{'name'} = $name;
		$form .= Forms->hidden(\%hidden);
		my %save = ('name' => 'save','value' => 'Save');
		$form .= Forms->form_bottem_one_button(\%save,$tab++);
	} elsif ($obj_view eq 'service_check') {
		my %docs = Doc->manage_hosts_service_check();
		$hidden{'service_id'} = $query->param('service_id');
		my %service = StorProc->fetch_one('services','service_id',$hidden{'service_id'});
		my $service_name = $query->param('service_name');
		my %template = StorProc->fetch_one('service_templates','servicetemplate_id',$service{'servicetemplate_id'});
		my $message = undef;
		my $inherit = 0;
		$inherit = $query->param('inherit');
		my %cmd = StorProc->fetch_one('commands','command_id', $service{'check_command'});
		my $command = $cmd{'name'};
		my $command_save = $cmd{'name'};
		my $command_line = $service{'command_line'};
		if ($query->param('command')) { $command = $query->param('command') }
		if ($query->param('command_save')) { $command_save = $query->param('command_save') }
		if ($query->param('command_line')) { $command_line = $query->param('command_line') }
		unless ($command eq $command_save) {
			%cmd = StorProc->fetch_one('commands','name',$command);
			$command_line = undef;
		}
		if ($inherit or !$command) {
			%cmd = StorProc->fetch_one('commands','command_id',$template{'check_command'});
			if ($cmd{'name'}) {
				$command = $cmd{'name'};
				$command_line = $template{'command_line'};
				$inherit = 1;
			} else {
				my $got_command = 0;
				my $stid = $template{'parent_id'};
				until ($got_command) {
					my %t = StorProc->fetch_one('service_templates','servicetemplate_id',$stid);
					if ($t{'check_command'}) {
						$got_command = 1;
						%cmd = StorProc->fetch_one('commands','command_id',$t{'check_command'});
						$command = $cmd{'name'};
						$command_line = $t{'command_line'};
						$got_command = 1;
					} else {
						if ($t{'parent_id'}) {
							$stid = $t{'parent_id'};
						} else {
							$got_command = 1;
							$message = ('Note: a parent template does not have a check command defined.');
							$required{'check_command'} = 1;
							$command = undef;
							$command_line = undef;
						}
					}
				}
			}
		}
		unless ($cmd{'command_line'}) {
			%cmd = StorProc->fetch_one('commands','name',$command);
		}
		my $arg_string = $command_line;
		$arg_string =~ s/$command!//;
		my $usage = $command;
		my @args = split(/ARG/i,$cmd{'command_line'});
		my $args = undef;
		if ($args[1]) {
			$usage .= "!";
			my $cnt = 1;
			pop @args;
			foreach (@args) { 
				if ($cmd{'command_line'} =~ /ARG$cnt/i) { $args .= "ARG$cnt!" }
				$cnt++;
			}
			chop $args;
			$usage .= $args;
			unless ($command_line =~ /$command/) {
				$command_line = "$command!$args";
				$arg_string = $args;
			}
		}
		my %objs = ('service_id' => $hidden{'service_id'},'service_name' => $service_name);
		$form .= Forms->host_top($properties{'name'},$user_acct,$obj_view,$auth_add{'externals'},$hidden{'selected'},$hidden{'form_service'});
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->wizard_doc('Service Check',$docs{'service_check'});
		$form .= Forms->display_hidden('Service name:','service_name',$service_name);
		$form .= Forms->checkbox_override('Inherit check from template','inherit',$inherit,$docs{'override'});
		my %where = ('type' => 'check');
		my @commands = StorProc->fetch_list_where('commands','name',\%where);
		$form .= Forms->list_box_submit('Check command:','command',\@commands,$command,$required{'check_command'});
		$form .= Forms->display_hidden('Command definition:','',$cmd{'command_line'});
		$form .= Forms->display_hidden('Usage:','',$usage);
		$form .= Forms->text_area('Command line:','command_line',$command_line,'3','80','',$docs{'command_line'});
		unless ($host) { $host = $name }
		$form .= Forms->command_test($test_results,$host,$arg_string);
		$hidden{'command_save'} = $command;
		$hidden{'name'} = $name;
		$form .= Forms->hidden(\%hidden);
		my %save = ('name' => 'save','value' => 'Save');
		$form .= Forms->form_bottem_one_button(\%save,$tab++);
	} elsif ($obj_view eq 'service_dependencies') {
		my %docs = Doc->manage_hosts_service_dependencies();
		$hidden{'service_id'} = $query->param('service_id');
		my $service_name = $query->param('service_name');
		my %objs = ('service_id' => $hidden{'service_id'},'service_name' => $service_name);
		$form .= Forms->host_top($properties{'name'},$user_acct,$obj_view,$auth_add{'externals'},$hidden{'selected'},$hidden{'form_service'});
		$form .= Forms->display_hidden('Service name:','service_name',$service_name);
		$form .= Forms->wizard_doc('Service Dependencies',$docs{'dependencies'});
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		my $dep_template = $query->param('dep_template');
		my %dep = StorProc->fetch_one('service_dependency_templates','name',$dep_template);
		my @dep_hosts = ();
		my ($host,$hosts) = StorProc->get_dep_on_hosts($dep{'servicename_id'},$properties{'host_id'});
		my @hosts = @{$hosts};
		if ($dep_template) {
			@dep_hosts = ($name);
			push (@dep_hosts,@hosts);
		}
		my %dependencies = StorProc->get_dependencies($hidden{'service_id'});
		$form .= Forms->dependency_list($name,'hosts',$hidden{'service_id'},$user_acct,\%dependencies);
		my @dep_templates = StorProc->fetch_list('service_dependency_templates','name');
		$form .= Forms->dependency_add($dep_template,\@dep_templates,\@dep_hosts,\%docs);
		$hidden{'name'} = $name;
		$form .= Forms->hidden(\%hidden);
		my %add_dependency = ('name' => 'add_dependency','value' => 'Add Dependency');
		$form .= Forms->form_bottem_one_button(\%add_dependency,$tab++);
	} elsif ($obj_view eq 'service_externals') {
		$hidden{'service_id'} = $query->param('service_id');
		$hidden{'service_name'} = $query->param('service_name');
		my %objs = ('service_id' => $hidden{'service_id'},'service_name' => $hidden{'service_name'});
		$form .= Forms->host_top($properties{'name'},$user_acct,$obj_view,$auth_add{'externals'},$hidden{'selected'},$hidden{'form_service'});
		$form .= Forms->display_hidden('Service name:','service_name',$hidden{'service_name'});
		my %externals = ();
		my %where = ('service_id' => $hidden{'service_id'});
		my @externals = StorProc->fetch_list_where('external_service','external_id',\%where);
		foreach my $eid (@externals) {
			my %e = StorProc->fetch_one('externals','external_id',$eid);
			$externals{$e{'name'}} = $eid;
		}
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		if (@messages) { $form .= Forms->form_message('Status:',\@messages,'msg') }
		my %w = ('type' => 'service');
		my @external_names = StorProc->fetch_list_where('externals','name',\%w);
		my @external_list = ();
		foreach my $s (@external_names) { unless ($externals{$s}) { push @external_list, $s } }
		@external_list = sort { $a <=> $b } @external_list;
		$form .= Forms->external_list($user_acct,$name,\%externals,\@external_list,'service',$hidden{'service_id'},$hidden{'service_name'});
		$hidden{'type'} = 'service';
		$hidden{'id'} = $properties{'host_id'};
		$hidden{'name'} = $name;
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_no_button();
	} elsif ($obj_view eq 'service_external_detail') {
		my $service_name = $query->param('service_name');
		$hidden{'service_id'} = $query->param('service_id');
		$form .= Forms->host_top($properties{'name'},$user_acct,$obj_view,$auth_add{'externals'},$hidden{'selected'},$hidden{'form_service'});
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		my $detail = $query->param('detail');
		my $service_external = $query->param('service_external');
		unless ($service_external) { $service_external = $properties{'external'} }
		my %where = ('external_id' => $properties{'external_id'},'service_id' => $hidden{'service_id'});
		my %external = StorProc->fetch_one_where('external_service',\%where);
		unless ($detail) { $detail = $external{'data'} }
		$form .= Forms->display_hidden('External:','service_external',$service_external);
		$form .= Forms->text_area('Detail:','detail',$detail,'30','140');
		$hidden{'external_id'} = $external{'external_id'};
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%save,$tab++);
	} elsif ($obj_view eq 'saved') {
		if ($hidden{'service_name'}) { $obj_view = 'service_detail' }
		$form .= Forms->host_top($properties{'name'},$user_acct,$obj_view,$auth_add{'externals'},$hidden{'selected'},$hidden{'form_service'});
		$form .= Forms->display_hidden('Saved:','',$message);
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%close);
	} elsif ($obj_view eq 'saved_apply') {
		$obj_view = 'profiles';
		$form .= Forms->host_top($properties{'name'},$user_acct,$obj_view,$auth_add{'externals'},$hidden{'selected'},$hidden{'form_service'});
		$form .= Forms->display_hidden('Saved:','',$message);
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_no_button();
	} elsif ($obj_view eq 'rename') {
		$form .= Forms->host_top($properties{'name'},$user_acct,$obj_view,$auth_add{'externals'},$hidden{'selected'},$hidden{'form_service'});
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->text_box('Rename to:','new_name','',$textsize{'name'},$required);
		$hidden{'name'} = $name;
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%rename,$tab++);
	} elsif ($obj_view eq 'delete_host') {
		$hidden{'name'} = $name; 
		my $message = qq(Are you sure you want to remove host $name?);
		$form .= Forms->are_you_sure('Confirm Delete:',$message,'confirm_delete_host',\%hidden);
	} elsif ($obj_view eq 'deleted') {
		$form .= Forms->form_top('Status','');
		my @message = ("Host $name.");
		$form .= Forms->form_message("Removed:",\@message ,'row1');
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%continue,$tab++);		
	}
	return $form;
}



#
##################################################
# Sub Delete Object
#

sub delete_object($$) {
	my $id = shift;
	my $name = shift;
	my $form = undef;
	my $title = undef;
	my @t_parse = split(/_/, $obj);
	foreach (@t_parse) { $_ =~ s/ies/y/g; $title .= "\u$_ "; }
	$title =~ s/s\s$//;
	my %dep_templates = ();
	$dep_templates{'time_periods'}{'host_templates'} = [ ('notification_period') ];
	$dep_templates{'time_periods'}{'service_templates'} = [ ('notification_period','check_period') ];
	$dep_templates{'time_periods'}{'contact_templates'} = [ ('host_notification_period','service_notification_period') ];	
	$dep_templates{'commands'}{'host_templates'} = [ ('check_command','event_handler') ];
	$dep_templates{'commands'}{'service_templates'} = [ ('check_command','event_handler') ];
	$dep_templates{'commands'}{'services'} = [ ('check_command') ];
	$dep_templates{'host_templates'}{'hosts'} = [ ('hosttemplate_id') ];
	$dep_templates{'host_templates'}{'profiles_host'} = [ ('host_template_id') ];
	$dep_templates{'service_templates'}{'service_templates'} = [ ('parent_id') ];
	$dep_templates{'service_templates'}{'service_names'} = [ ('template') ];
	$dep_templates{'service_templates'}{'services'} = [ ('servicetemplate_id') ];
	$dep_templates{'contact_templates'}{'contacts'} = [ ('contacttemplate_id') ];
	$dep_templates{'service_dependency_templates'}{'service_dependency'} = [ ('template') ];
	$dep_templates{'contactgroups'}{'contactgroup_assign'} = [ ('contactgroup_id') ];
	$dep_templates{'contactgroups'}{'tree_template_contactgroup'} = [ ('contactgroup_id') ];
	$dep_templates{'escalation_templates'}{'escalation_tree_template'} = [ ('template_id') ];
	$dep_templates{'extended_service_info_templates'}{'services'} = [ ('serviceextinfo_id') ];
	$dep_templates{'extended_service_info_templates'}{'service_names'} = [ ('extinfo') ];
	$dep_templates{'extended_host_info_templates'}{'hosts'} = [ ('hostextinfo_id') ];
	$dep_templates{'extended_host_info_templates'}{'profiles_host'} = [ ('host_extinfo_id') ];
	unless ($id) {
		my $otable = $obj;
		if ($otable =~ /escalation_template/) { $otable = 'escalation_templates' }
		if ($otable eq 'escalations') { $otable = 'escalation_trees' }
		my %id = StorProc->fetch_one($otable,'name',$name);
		$id = $id{$obj_id{$otable}};
	}
	$hidden{'id'} = $id;
	my $all_clear = 1;
	my %dependencies = ();
	foreach my $table (keys %{$dep_templates{$obj}}) {
		foreach my $column (@{$dep_templates{$obj}{$table}}) {
			my @dep = ();
			if ($table =~ /services|service_dependency$/) {
				my @names = StorProc->get_hostname_servicename($table,$column,$id);
				for my $i ( 0 .. $#names ) {
					for my $name ( keys %{ $names[$i] } ) {
						push @dep, "$name - $names[$i]{$name}";
					}
				}
				$dependencies{'Host - Service'} = [ @dep ];
			} elsif ($table eq 'contactgroup_assign') {
				my @names = StorProc->get_contactgroup_object($id,\%obj_id);
				for my $i ( 0 .. $#names ) {
					for my $name ( keys %{ $names[$i] } ) {
						push @dep, "$name - $names[$i]{$name}";
					}
				}
				$dependencies{'Templates/Hostgroups'} = [ @dep ];
			} elsif ($table eq 'tree_template_contactgroup') {
				@dep = StorProc->get_tree_contactgroup($id);
				$dependencies{'Escalation trees'} = [ @dep ];
			} elsif ($table eq 'contactgroup_contact') {
				@dep = StorProc->get_contact_contactgroup($id);
				$dependencies{'Contacts'} = [ @dep ];
			} else {
				my %where = ($column => $id);
				@dep = StorProc->fetch_list_where($table,'name',\%where);
				$dependencies{$table} = [ @dep ];
			}
			if ($dep[0]) { $all_clear = 0 }
		}
	}		
	if ($all_clear) {
		if ($query->param('confirm_delete')) {
			my $otable = $obj;
			if ($otable =~ /escalation_template/) { 
				$otable = 'escalation_templates';
				$obj = 'escalation_templates';
			}
			if ($otable eq 'escalations') { 
				$otable = 'escalation_trees';
				$obj = 'escalation_trees';
			}
			my $result = StorProc->delete_all($otable,$obj_id{$obj},$id);
			if ($result =~ /^Error/) { 
				push @errors, "Unable to process request $result";
				$form .= Forms->header($page_title,$user_acct,$view);
				$form .= Forms->form_top("Delete $title",'');
				$form .= Forms->form_errors(\@errors);
			} else {
				my @message = ("$name");
				$refresh_left = "top_menu=$top_menu";
				$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
				$form .= Forms->form_top("Delete $title",'');
				$form .= Forms->form_message('Removed:',\@message,'row1');			
			}
			$form .= Forms->hidden(\%hidden);
			$form .= Forms->form_bottem_one_button(\%continue,$tab++);
		} else {
			foreach my $name ($query->param) {
				unless ($name eq 'nocache') { $hidden{$name} = $query->param($name) }
			}
			delete $hidden{'task'};
			delete $hidden{'submit'};
			if ($obj =~ /escalation_template/) {
				$hidden{'task'} = 'escalation_template';		
			} elsif ($obj =~ /escalation_trees/) {
				$hidden{'task'} = 'escalation_trees';		
			} else {
				$hidden{'task'} = 'modify';
			}
			my $message = qq(Are you sure you want to remove $name?);
			$form .= Forms->header($page_title,$user_acct,$view);
			$form .= Forms->are_you_sure('Confirm Delete:',$message,'confirm_delete',\%hidden,'bail');
			$task = 'delete';
		}
	} else {
		$form .= Forms->header($page_title,$user_acct,$view);
		$form .= Forms->form_top("Delete $title",'');
		$form .= Forms->display_hidden('Name:','name',$name);
		my @message = ("Cannot delete until all dependencies are removed/reassigned.");
		$form .= Forms->form_errors(\@message);	
		foreach my $dep (keys %dependencies) {
			@message = ($dep);
			$form .= Forms->form_message('Object type:',\@message,'row1');	
			$form .= Forms->form_message('&nbsp;',\@{$dependencies{$dep}},'row1');		
		}
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%close,$tab++);
	}
	return $form;
}

sub rename_object($$) {
	my $id = shift;
	my $name = shift;
	my $form = undef;
	my $title = undef;
	my $ltitle = undef;
	my @t_parse = split(/_/, $obj);
	foreach (@t_parse) { $_ =~ s/ies/y/g; $title .= "\u$_ "; $ltitle = "$_ " }
	$title =~ s/s\s$//;
	my $new_name = $query->param('new_name');
	if ($name eq $new_name) { $new_name = undef }
	my $saved = 0;
	$table = $obj;
	if ($table =~ /escalation_template/) { $table = 'escalation_templates'}
	if ($table =~ /escalation_tree/) { $table = 'escalation_trees'}
	if ($new_name) {
		my %n = StorProc->fetch_one($table,'name',$new_name);
		if ($name =~ /^$new_name$/i) { delete $n{'name'} }
		if ($n{'name'}) {
			$obj_view = 'rename';
			$ltitle =~ s/s\s$//;
			push @errors, "Cannot rename. Another $ltitle has name $new_name.";
		} else {
			my %values = ('name' => $new_name);
			my $result = StorProc->update_obj($table,'name',$name,\%values);
			if ($result =~ /^Error/) { 
				push @errors, $result;
			} else {
				$name = $new_name;
				$saved = 1;
				$refresh_left = "top_menu=$top_menu&selected=$name";
			}
		}
	}
	$form .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
	if ($saved) {
		$form .= Forms->form_top("Rename $title",'');
		my @message = ("$name");
		$form .= Forms->form_message('Renamed:',\@message,'row1');			
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_one_button(\%continue,$tab++);
	} else {
		$form .= Forms->form_top("Rename $title",'');
		if (@errors) { $form .= Forms->form_errors(\@errors) }
		$form .= Forms->display_hidden("Name:",'name',$name);
		$form .= Forms->text_box('Rename to:','new_name','',$textsize{'name'});
		$form .= Forms->hidden(\%hidden);
		$form .= Forms->form_bottem_two_button(\%rename,\%cancel,$tab++);
	}
	return $form;
}

#
##################################################
# Manage
#

sub manage() {
	my $page = undef;
	my $name = $query->param('name');
	my $title = undef;
	my @t_parse = split(/_/, $obj);
	foreach (@t_parse) { $title .= "\u$_ " }
	if ($query->param('cancel') || $query->param('close') || $query->param('continue')) {
		$page .= Forms->header($page_title,$user_acct,$view);
	} else {
		my $task = $query->param('submit');
#
# Contacts
#
		if ($obj eq 'contacts') {
			my $name = $query->param('name');
			$hidden{'name'} = $name;
			my $got_form = undef;
			if ($query->param('delete') || $query->param('confirm_delete')) {
				unless ($query->param('bail')) {
					$page .= Forms->header($page_title,$user_acct,$view);
 					$page .= delete_object('',$name);
					$got_form = 1;
				}
			} elsif ($query->param('rename')) {
				$page .= rename_object('',$name);
				$got_form = 1;
			} elsif ($query->param('save')) {
				my %contact = StorProc->fetch_one('contacts','name',$name);
				my %data = parse_query('contacts','contacts');
				$properties{'template'} = $query->param('template');
				my $field_ck = check_fields();
				if ($field_ck == 0) {
					my %t = StorProc->fetch_one('contact_templates','name',$properties{'template'});
					my %vals = ('contacttemplate_id' => $t{'contacttemplate_id'},'alias' => $data{'alias'},'email' => $data{'email'},'pager' => $data{'pager'});
					my $result = StorProc->update_obj('contacts','name',$name,\%vals);
					if ($result =~ /^Error/) { push @errors, $result }
					my %where = ('contact_id' => $contact{'contact_id'});
					my $result = StorProc->delete_one_where('contactgroup_contact',\%where);
					if ($result =~ /^Error/) { push @errors, $result }
					my @mems = $query->param('contactgroup');
					foreach (@mems) {
						my %cg = StorProc->fetch_one('contactgroups','name',$_);
						my @vals = ($cg{'contactgroup_id'},$contact{'contact_id'});
						$result = StorProc->insert_obj('contactgroup_contact',\@vals);
						if ($result =~ /^Error/) { push @errors, $result }
					}
					unless (@errors) {
						my %where = ('contact_id' => $contact{'contact_id'});
						my $result = StorProc->delete_one_where('contact_overrides',\%where);
						my %values = ();
						my $got_override = undef;
						unless ($query->param('host_notification_period_override')) {
							my $np = $query->param('host_notification_period');
							my %np = StorProc->fetch_one('time_periods','name',$np);
							$values{'host_notification_period'} = $np{'timeperiod_id'};
							$got_override = 1;
						}
						unless ($query->param('service_notification_period_override')) {
							my $np = $query->param('service_notification_period');
							my %np = StorProc->fetch_one('time_periods','name',$np);
							my %np = StorProc->fetch_one('time_periods','name',$np);
							$values{'service_notification_period'} = $np{'timeperiod_id'};
							$got_override = 1;
						}
						my $data = undef;
						unless ($query->param('host_notification_options_override')) {
							my @no = $query->param('host_notification_options');
							my $str = undef;
							foreach (@no) { $str .= "$_," }
							chop $str;
							$data .= "\n  <prop name=\"host_notification_options\"><![CDATA[$str]]>\n  </prop>";
							$got_override = 1;
						}
						unless ($query->param('service_notification_options_override')) {
							my @no = $query->param('service_notification_options');
							my $str = undef;
							foreach (@no) { $str .= "$_," }
							chop $str;
							$data .= "\n  <prop name=\"service_notification_options\"><![CDATA[$str]]>\n  </prop>";
							$got_override = 1;
						}
						if ($data) {
							$values{'data'} = "<?xml version=\"1.0\" ?>\n<data>".$data."\n</data>";
						}
						if ($got_override) {
							my @vals = ($contact{'contact_id'},$values{'host_notification_period'},$values{'service_notification_period'},$values{'data'});
							$result = StorProc->insert_obj('contact_overrides',\@vals);
							if ($result =~ /^Error/) { push @errors, $result }					
						}
						my %where = ('contact_id' => $contact{'contact_id'});
						my $result = StorProc->delete_one_where('contact_command_overrides',\%where);
						if ($result =~ /^Error/) { push @errors, $result }
						unless ($query->param('host_notification_commands_override')) {
							my @mems = $query->param('host_notification_commands');
							foreach my $cmd (@mems) {
								if ($cmd) {
									my %c = StorProc->fetch_one('commands','name',$cmd);
									my @vals = ($contact{'contact_id'},'host',$c{'command_id'});
									$result = StorProc->insert_obj('contact_command_overrides',\@vals);
									if ($result =~ /^Error/) { push @errors, $result }					
								}
							}
						}
						unless ($query->param('service_notification_commands_override')) {
							my @mems = $query->param('service_notification_commands');
							foreach my $cmd (@mems) {
								if ($cmd) {
									my %c = StorProc->fetch_one('commands','name',$cmd);
									my @vals = ($contact{'contact_id'},'service',$c{'command_id'});
									$result = StorProc->insert_obj('contact_command_overrides',\@vals);
									if ($result =~ /^Error/) { push @errors, $result }		
								}
							}
						}
						unless (@errors) {
							$page .= Forms->header($page_title,$user_acct,$view);
							my $message = "Change to contact $name accepted.";
							$page .= Forms->success('Saved',$message,'continue',\%hidden);
							$got_form = 1;
						}
					}
				}
			}
			unless ($got_form) {
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= Forms->form_top('Contact Properties','1');
				$page .= build_contact();
				$page .= Forms->hidden(\%hidden);
				%save = ('name' => 'save','value' => 'Save');
				$page .= Forms->form_bottem_four_button(\%save,\%delete,\%rename,\%cancel,$tab++);
			}


		} elsif ($query->param('bail')) {
			my $title = undef;
			my @t_parse = split(/_/, $obj);
			foreach (@t_parse) { $title .= "\u$_ " }
			$title =~ s/s$//;
			$page .= Forms->header($page_title,$user_acct,$view);
			$page .= build_form();
		} elsif ($query->param('delete') or $query->param('confirm_delete')) {
			if ($obj =~ /escalation/) { $hidden{'type'} = $query->param('type')	}
			$page .= Forms->header($page_title,$user_acct,$view);
			$page .= delete_object('',$name);
		} elsif ($query->param('save')) {
			my %values = ();
			my %data = parse_query($table,$obj);
			my $field_ck = check_fields();
			if ($field_ck == 0) {
				foreach my $key (keys %data) {
					unless ($key =~ /HASH|^name$|members|^contact$|^contactgroup$|^parents$|notification_commands/) {
						$values{$key} = $data{$key};					
					}
				}
				my $result = StorProc->update_obj($table,'name',$data{'name'},\%values);
				if ($result =~ /^Error/) { push @errors, $result }
				
				if ($obj eq 'hostgroups') {
					my %n = StorProc->fetch_one('hostgroups','name',$name);
					my $result = StorProc->delete_all('hostgroup_host','hostgroup_id',$n{'hostgroup_id'});
					if ($result =~ /^Error/) { push @errors, $result }
					if ($data{'members'}) {
						my @mems = split(/,/, $data{'members'});
						foreach (@mems) {
							my %m = StorProc->fetch_one('hosts','name',$_);
							my @vals = ($n{'hostgroup_id'},$m{'host_id'});
							$result = StorProc->insert_obj('hostgroup_host',\@vals);
							if ($result =~ /^Error/) { push @errors, $result }
						}
					}
				} elsif ($obj eq 'contactgroups') {
					my %cg = StorProc->fetch_one('contactgroups','name',$data{'name'});
					my $result = StorProc->delete_all('contactgroup_contact','contactgroup_id',$cg{'contactgroup_id'});
					if ($result =~ /^Error/) { push @errors, $result }
					if ($data{'contact'}) {
						my @mems = split(/,/, $data{'contact'});
						foreach (@mems) {
							my %c = StorProc->fetch_one('contacts','name',$_);
							my @vals = ($cg{'contactgroup_id'},$c{'contact_id'});
							$result = StorProc->insert_obj('contactgroup_contact',\@vals);
							if ($result =~ /^Error/) { push @errors, $result }
						}
					}
				} elsif ($obj eq 'contact_templates') {
					my %o = StorProc->fetch_one($table,'name',$name);
					my %w = ('contacttemplate_id' => $o{'contacttemplate_id'},'type' => 'host');
					my $result = StorProc->delete_one_where('contact_command',\%w);
					if ($result =~ /^Error/) { push @errors, $result }	
					if ($data{'host_notification_commands'}) {
						my @c = split(/,/, $data{'host_notification_commands'});		
						foreach (@c) {
							my @vals = ($o{'contacttemplate_id'},'host',$_);
							my $result = StorProc->insert_obj('contact_command',\@vals);
							if ($result =~ /Error/) { push @errors, $result }
						}	
					}
					my %o = StorProc->fetch_one($table,'name',$name);
					my %w = ('contacttemplate_id' => $o{'contacttemplate_id'},'type' => 'service');
					my $result = StorProc->delete_one_where('contact_command',\%w);
					if ($result =~ /^Error/) { push @errors, $result }
					if ($data{'service_notification_commands'}) {
						my @c = split(/,/, $data{'service_notification_commands'});		
						foreach (@c) {
							my @vals = ($o{'contacttemplate_id'},'service',$_);
							my $result = StorProc->insert_obj('contact_command',\@vals);
							if ($result =~ /Error/) { push @errors, $result }
						}	
					}
				}
				my %o = StorProc->fetch_one($table,'name',$name);
				my %w = ('type' => $table,'object' => $o{$obj_id{$obj}});
				my $result = StorProc->delete_one_where('contactgroup_assign',\%w);
				if ($result =~ /^Error/) { push @errors, $result }
				if ($data{'contactgroup'}) {
					my @mems = split(/,/, $data{'contactgroup'});
					foreach (@mems) {
						my %cg = StorProc->fetch_one('contactgroups','name',$_);
						my @vals = ($cg{'contactgroup_id'},$obj,$o{$obj_id{$obj}});
						$result = StorProc->insert_obj('contactgroup_assign',\@vals);
						if ($result =~ /^Error/) { push @errors, $result }
					}
				}
				unless (@errors) {
					my $message = "Changes to $name have been saved.";
					$hidden{'name'} = undef;
					$page .= Forms->header($page_title,$user_acct,$view);
					$page .= Forms->success('Updated',$message,'continue',\%hidden);
				} else {
					foreach (@errors) { error_out("errors $_") }
				}
			} elsif ($field_ck == 1) {
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= build_form();
			}

		} elsif ($query->param('rename')) {
			$page .= rename_object('',$name);
		} elsif ($name) {
			my $title = undef;
			my @t_parse = split(/_/, $obj);
			foreach (@t_parse) { $title .= "\u$_ " }
			$title =~ s/s$//;
			$page .= Forms->header($page_title,$user_acct,$view);
			$page .= build_form();
		}
	}
	return $page;
}


#
##################################################
# Tools
#

sub tools() {
	my $page = undef;
	if ($obj eq 'export') {
		my @results = ();
		use MonarchFile;
		my %nagios = StorProc->fetch_one('setup','name','nagios_etc');
		my %nagios_cfg = StorProc->fetch_one('setup','name','log_file');
		my %nagios_cgi = StorProc->fetch_one('setup','name','default_user_name');
		if ($nagios_cfg{'value'} && $nagios_cgi{'type'}) {
			my ($errors, $usefiles) = Files->build_files($user_acct,"$doc_root/monarch/download",\%property_list,$nagios_ver);
			@errors = @{$errors};
			my %use_file = %{$usefiles};
			@errors = Files->nagios_cfg($user_acct,'2',$nagios{'value'},"$doc_root/monarch/download",\%use_file); 
			$use_file{'nagios.cfg'} = 1;
			@errors = Files->nagios_cgi($user_acct,'2',$nagios{'value'},"$doc_root/monarch/download"); 
			$use_file{'cgi.cfg'} = 1;
			if (@errors) { foreach (@errors) { error_out($_) } }
			my $server = $ENV{'SERVER_NAME'};
			$page .= Forms->header($page_title,$user_acct,$view);
			$page .= Forms->table_download_links($doc_root,\%use_file,$server);
		} else {
			$page .= Forms->header($page_title,$user_acct,$view);
			$page .= Forms->form_top('Export','');
			unless ($nagios_cfg{'value'}) {
				push @errors, 'Nagios main configuration has not been defined.','Use Control->Nagios main configuration to load an existing file or set defaults.';
			}
			unless ($nagios_cgi{'type'}) {
				push @errors, 'Nagios cgi configuration has not been defined.','Use Control->Nagios cgi configuration to load an existing file or set defaults.';
			}	
			$page .= Forms->form_message("Error(s):",\@errors ,'error');
			$hidden{'obj'} = undef;
			$page .= Forms->hidden(\%hidden);
			$page .= Forms->form_bottem_one_button(\%continue,$tab++);
		}
	}
	return $page;
}

#
##################################################
# Control Center
#

sub control() {
	my @menus = ();
	my %ver = StorProc->fetch_one('setup','name','nagios_version');
	my $page = undef;
	if ($query->param('close') || $query->param('continue') || $query->param('cancel')) {
		$page .= Forms->header($page_title,$user_acct,$view);
	} else {
#
##################################################
# Users
#
		if ($obj eq 'users') {
			@errors = ();
			$table = 'users';
			my $user = $query->param('mod_user');
			my $user_name = $query->param('user_name');
			my @password = $query->param('passwd');
			if ($query->param('task') eq 'No') { $task = 'modify' }
			if ($query->param('add') && $user) {
				my $id = undef;
				my %p = StorProc->fetch_one('users','user_acct',$user);
				if (!$password[1]) { $password[1] = 1 }
				unless ($authentication{'disabled'}) {
					if ($password[0] eq $password[1]) { $password[1] = undef }
				} else {
					$password[1] = undef;
				}
				unless ($p{'user_acct'} || $user eq '' || $password[1]) {
					my $now = time;
					$now = $now - 100000;
					my @saltchars = ('a'..'z','A'..'Z','0'..'9',',','/');
					srand(time() ^ ($$ + ($$ << 15)));
					my $salt = $saltchars[int(rand(64))];
					$salt .= $saltchars[int(rand(64))];
					my $newpw = crypt($password[0], $salt);
					my @values = ('',$user,$user_name,$newpw,$now);
					$id = StorProc->insert_obj_id($table,\@values,'user_id');
					if ($id =~ /^Error/) { push @errors, $id }
				} else {
					if ($p{'user_acct'}) { push @errors, "User id $user_acct already exists." }
					if ($user eq '') { push @errors, "Required: User id" }
					if ($password[1]) { push @errors, "Check password fields. They are either blank or a mismatch." }
				}
				unless (@errors) { 
					my @user_groups = $query->param('user_groups');
					my @ugs = StorProc->get_ids('usergroup_id','user_groups',\@user_groups);
					foreach (@ugs) {
						my @values = ($_,$id);
						my $result = StorProc->insert_obj('user_group',\@values);
						if ($result =~ /^Error/) { push @errors, $result }				
					}
				}
				unless (@errors) { 
					$refresh_left = "top_menu=$top_menu&selected=$user";
					$page .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
					$page .= Forms->form_top("User Account",'');
					my @message = ("User $user added.");
					$page .= Forms->form_message('Added:',\@message,'row1');	
					($task,$user) = undef;
					delete $hidden{'task'};
					$page .= Forms->hidden(\%hidden);
					$page .= Forms->form_bottem_one_button(\%continue,$tab++);
				}
			} elsif ($query->param('save')) {
				my %values = ();
				if (!$password[1]) { $password[1] = 1 }
				if ($query->param('set_password')) {
					if ($password[0] eq $password[1]) { $password[1] = undef }
				} else {
					$password[1] = undef;
				}
				unless ($password[1]) {
					$values{'user_name'} = $user_name;
					my @saltchars = ('a'..'z','A'..'Z','0'..'9',',','/');
					srand(time() ^ ($$ + ($$ << 15)));
					my $salt = $saltchars[int(rand(64))];
					$salt .= $saltchars[int(rand(64))];
					$values{'password'} = crypt($password[0], $salt);
					my $result = StorProc->update_obj('users','user_acct',$user,\%values);
					if ($result =~ /^Error/) { push @errors, $result }
					unless ($user eq 'super_user') {
						my %u = StorProc->fetch_one('users','user_acct',$user);
						$result = StorProc->delete_all('user_group','user_id',$u{'user_id'});
						if ($result =~ /^Error/) { push @errors, $result }
						my @user_groups = $query->param('user_groups');
						my @ugs = StorProc->get_ids('usergroup_id','user_groups',\@user_groups);
						foreach (@ugs) {
							my @values = ($_,$u{'user_id'});
							my $result = StorProc->insert_obj('user_group',\@values);
							if ($result =~ /^Error/) { push @errors, $result }				
						}
					}
				} else {
					push @errors, "Check password fields. It's either blank or a mismatch";
				}
				unless (@errors) { 
					$page .= Forms->header($page_title,$user_acct,$view);
					my $message = "Changes accepted.";					
					$page .= Forms->form_top("User Account",'');
					my @message = ("Changes to $user accepted.");
					$page .= Forms->form_message('Saved:',\@message,'row1');	
					($task,$user) = undef;
					delete $hidden{'task'};
					$page .= Forms->hidden(\%hidden);
					$page .= Forms->form_bottem_one_button(\%continue,$tab++);
				}
			} elsif ($query->param('confirm_delete')) {
				my $result = StorProc->delete_all('users','user_acct',$user);
				if ($result =~ /^Error/) { push @errors, $result }
				unless (@errors) { 
					$refresh_left = "top_menu=$top_menu";
					$page .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
					my $message = "User $user deleted.";					
					$page .= Forms->form_top("User Account",'');
					my @message = ("Changes to $user accepted.");
					$page .= Forms->form_message('Removed:',\@message,'row1');	
					($task,$user) = undef;
					delete $hidden{'task'};
					$page .= Forms->hidden(\%hidden);
					$page .= Forms->form_bottem_one_button(\%continue,$tab++);
				}
			} elsif ($query->param('delete')) {
				$page .= Forms->header($page_title,$user_acct,$view);
				$hidden{'mod_user'} = $user;
				my $message = qq(Are you sure you want to remove $user?);
				$page .= Forms->are_you_sure('Confirm Delete:',$message,'confirm_delete',\%hidden);
				$user = undef;
				$task = 'Delete';
			}
			if (@errors || $user || $task =~ /Add|new/i) {
				$page .= Forms->header($page_title,$user_acct,$view);
				my @members = $query->param('user_groups');
				$page .= Forms->form_top('User Account','1');
				if (@errors) { $page .= Forms->form_errors(\@errors) }
				if ($task =~ /Add|new/) {
					$page .= Forms->text_box('Userid:','mod_user',$user,$textsize{'short_name'},$required{'user_acct'});
					$page .= Forms->text_box('User name:','user_name',$user_name,$textsize{'long_name'});
					$hidden{'set_password'} = 1;
				} else {
					my %p = StorProc->fetch_one('users','user_acct',$user);
					if (!$members[0]) {
						@members = StorProc->get_names_in('usergroup_id','user_groups','user_group','user_id',$p{'user_id'}); 
					}
					$page .= Forms->display_hidden('User id:','mod_user',$user);
					$page .= Forms->text_box('User name:','user_name',$p{'user_name'},$textsize{'long_name'});
					$page .= Forms->checkbox('Set password:','set_password',$query->param('set_password'));
				}
				$page .= Forms->password_box('Password:','passwd',$textsize{'short_name'},$required{'password'});
				$page .= Forms->password_box('Password confirm:','passwd',$textsize{'short_name'},$required{'password'});
				%save = ('name' => 'save','value' => 'Save');
				unless ($user eq 'super_user') {
					my @nonmembers = StorProc->fetch_list('user_groups','name'); 
					$page .= Forms->members('User groups:','user_groups',\@members,\@nonmembers);
					$page .= Forms->hidden(\%hidden);
					if ($task =~ /Add|new/) {
						$page .= Forms->form_bottem_two_button(\%add,\%cancel,$tab++);
					} else {
						$page .= Forms->form_bottem_three_button(\%save,\%delete,\%cancel,$tab++);
					}
				} else {
					$page .= Forms->hidden(\%hidden);
					$page .= Forms->form_bottem_two_button(\%save,\%cancel,$tab++);
				}
			}

#
##################################################
# User Groups
#
		} elsif ($obj eq 'user_groups') {
			@errors = ();
			$table = 'user_groups';
			my $groupid = $query->param('groupid');
			my $description = $query->param('description');
			if ($query->param('task') eq 'No') { $task = 'access_list' }
			my $access_set = $query->param('access_set');
			if ($access_set)  { $task = 'access_list' }
			my $access_type = $query->param('access_type');
			my $update_assets = undef;
			my $saved = 0;
			my @message = ();
			if ($query->param('continue') || $query->param('close') ||  $query->param('cancel')) {
				$page .= Forms->header($page_title,$user_acct,$view);
			} elsif ($query->param('add') && $groupid) {
				my %p = StorProc->fetch_one('user_groups','name',$groupid);
				unless ($p{'name'}) {
					my @values = ('',$groupid,$description);
					my $result = StorProc->insert_obj($table,\@values);
					if ($result =~ /^Error/) { push @errors, $result }
				} else {
					push @errors, "Check name field. It's either blank or a record exists";
				}
				unless(@errors) {
					$refresh_left = "top_menu=$top_menu&selected=$groupid";
					$task = 'access_list';
				}
			} elsif ($query->param('next')) {
				my %values;
				$values{'description'} = $description;
				my $result = StorProc->update_obj($table,'name',$groupid,\%values);
				if ($result =~ /^Error/) { push @errors, $result}
				$task = 'access_list';
			} elsif ($groupid ne 'super_users' && $query->param('confirm_delete')) {
				my $result = StorProc->delete_all('user_groups','name',$groupid);
				if ($result =~ /^Error/) { push @errors, $result }
				unless (@errors) {
					$refresh_left = "top_menu=$top_menu&selected=$groupid";
					$page .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
					$page .= Forms->form_top("User Group",'');
					my @message = ("User group $groupid deleted.");
					$page .= Forms->form_message('Removed:',\@message,'row1');	
					($task,$groupid) = undef;
					delete $hidden{'task'};
					$page .= Forms->hidden(\%hidden);
					$page .= Forms->form_bottem_one_button(\%continue,$tab++);
					$task = 'delete';
				} else {
					$task = 'access_list';
				}
			} elsif ($groupid ne 'super_users' && $query->param('delete')) {
				$page .= Forms->header($page_title,$user_acct,$view);
				$hidden{'groupid'} = $groupid;
				$hidden{'task'} = 'access_list';
				my $message = qq(Are you sure you want to remove $groupid?);
				$page .= Forms->are_you_sure('Confirm Delete:',$message,'confirm_delete',\%hidden);
				$groupid = undef;
				$task = 'delete';
			} elsif ($query->param('update_access')) {
				my %g = StorProc->fetch_one('user_groups','name',$groupid);
				my $type = undef;
				my %selected = ('usergroup_id' => $g{'usergroup_id'},'type' => $access_set);
				my $result = StorProc->delete_one_where('access_list',\%selected); 
				if ($result =~ /^Error/) { push @errors, $result }
				my %assets = ();
				my %vals = ();
				my @assets = $query->param('access_list');
				if ($access_set eq 'tools' || $access_set eq 'control') {
					foreach my $asset (@assets) {
						my @values = ($asset,$access_set,$g{'usergroup_id'},'full_control');
						my $result = StorProc->insert_obj('access_list',\@values);
						if ($result =~ /^Error/) { push @errors, $result }
					}
				} else {
					foreach my $asset ($query->param) {
						if ($asset eq 'nocache') { next }
						if ($asset =~ /(^\S+)-(\S+)-(.*)/) {
							my $obj = $3;
							$assets{$obj} = $1;
							$vals{$obj} .= "$2,";
						}
					}
					foreach my $asset (keys %assets) {
						$vals{$asset} =~ s/,$//;
						my @values = ($asset,$assets{$asset},$g{'usergroup_id'},$vals{$asset});
						my $result = StorProc->insert_obj('access_list',\@values);
						if ($result =~ /^Error/) { push @errors, $result }
					}
				}
				unless (@errors) { 
					@message = ("Changes to access set $access_set accepted");	
					$saved = 1;
				}
				$task = 'access_list';
			}
			if ($task eq 'new') {
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= Forms->form_top('User Group','');
				if (@errors) { $page .= Forms->form_errors(\@errors) }
				$page .= Forms->text_box('Group id:','groupid','',$textsize{'short_name'},$required{'groupid'});
				$page .= Forms->text_box('Description:','description',$description,$textsize{'long_name'});
				delete $hidden{'task'};
				$page .= Forms->hidden(\%hidden);
				$page .= Forms->form_bottem_two_button(\%add,\%cancel,$tab++);
			} elsif ($task eq 'access_list') {
				unless ($groupid eq 'super_users') {
					$page .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
					my %access_button = ();
					if (@errors) { $page .= Forms->form_errors(\@errors) }
					my %p = StorProc->fetch_one('user_groups','name',$groupid);
					$description = $p{'description'};
					if (!$access_set) { $access_set = 'design_manage' }
					$hidden{'groupid'} = $groupid;
					$hidden{'access_set'} = $access_set;
					$page .= Forms->access_top($groupid,$user_acct,$access_set);
					my %g = StorProc->fetch_one('user_groups','name',$groupid);
					if ($saved) {
						$page .= Forms->hidden(\%hidden);
						$page .= Forms->form_message("Saved:",\@message,'row1');
					} elsif ($access_set eq 'tools') {
						$access_type = 'tools';
						my %values = ('usergroup_id' => $g{'usergroup_id'}, 'type' => 'tools');
						my @selected = ();
						my %selected_hash = StorProc->fetch_list_hash_array('access_list',\%values);
						foreach my $key (keys %selected_hash) { push @selected, $key }
						$page .= Forms->checkbox_list('Tools:','access_list', \@tool_menus,\@selected);
						$page .= Forms->hidden(\%hidden);
						$page .= Forms->submit_button('update_access','Save Access Values');
					} elsif ($access_set eq 'control') {
						$access_type = 'control';
						my %assets_selected = undef;
						my %values = ('usergroup_id' => $g{'usergroup_id'}, 'type' => 'control');
						my @selected = ();
						my %selected_hash = StorProc->fetch_list_hash_array('access_list',\%values);
						foreach my $key (keys %selected_hash) { push @selected, $key }
						$page .= Forms->checkbox_list('Control list:','access_list', \@control_menus,\@selected);
						$page .= Forms->hidden(\%hidden);
						$page .= Forms->submit_button('update_access','Save Access Values');
					} elsif ($access_set eq 'hostgroup_hosts') {
						$access_type = 'hostgroup_hosts';
						my %assets_selected = undef;
						my @hostgroups = StorProc->fetch_list('hostgroups','name'); 
						my %values = ('usergroup_id' => $g{'usergroup_id'}, 'type' => $access_type);
						my %access_list_values = StorProc->fetch_list_hash_array('access_list',\%values);
						foreach my $key (keys %access_list_values) {
							$assets_selected{$key} = $access_list_values{$key}[3];
						}
						$page .= Forms->access_list('Hostgroup-hosts:',\@hostgroups,\%assets_selected,$access_type);
						$page .= Forms->hidden(\%hidden);
						$page .= Forms->submit_button('update_access','Save Access Values');
					} else {
						my @access_list = ('time_periods','commands','contact_templates','contacts','contactgroups');
						push (@access_list, ('hosts','host_templates','extended_host_info_templates','hostgroups','parent_child','host_dependencies'));
						push @access_list, ('services','service_templates','service_dependency_templates','extended_service_info_templates');
						if ($enable_externals) { push @access_list, 'externals' }
						$access_type = 'design_manage';
						my %assets_selected = undef;
						my %values = ('usergroup_id' => $g{'usergroup_id'}, 'type' => $access_type);
						my %access_list_values = StorProc->fetch_list_hash_array('access_list',\%values);
						foreach my $key (keys %access_list_values) {
							$assets_selected{$key} = $access_list_values{$key}[3];
						}
						$page .= Forms->access_list('Design/Manage',\@access_list,\%assets_selected,$access_type);
						$page .= Forms->hidden(\%hidden);
						$page .= Forms->submit_button('update_access','Save Access Values');
					}
					$page .= Forms->form_bottem_two_button(\%delete,\%close,$tab++);
				} 
			} elsif ($task eq 'modify') {
				my $groupid = $query->param('name');
				$page .= Forms->header($page_title,$user_acct,$view);
				my %p = StorProc->fetch_one('user_groups','name',$groupid);
				$description = $p{'description'};
				$page .= Forms->form_top('User Group','');
				$page .= Forms->display_hidden('Group id:','groupid',$groupid);
				$page .= Forms->text_box('Description:','description',$description,$textsize{'long_name'});
				$page .= Forms->hidden(\%hidden);
				$page .= Forms->form_bottem_two_button(\%next,\%cancel,$tab++);					
			}
#
##################################################
# Setup
#
		} elsif ($obj eq 'setup') {
			@errors = ();
			my $saved = 0;
			if ($query->param('save')) {
				my @setup_props = ('cgi_home','login_authentication','session_timeout','nagios_etc','nagios_bin','nagios_version','upload_dir','backup_dir','max_tree_nodes','enable_externals');
				foreach my $name (@setup_props) {
					my %values = ();
					my $value = $query->param($name);
					unless ($value) { $value = 0 }
					$values{'value'} = $value;
					my $result = StorProc->update_obj('setup','name',$name,\%values);
					if ($result =~ /^Error/) { push @errors, $result }
				}
				unless (@errors) { $saved = 1 }
			}
			if ($saved) {
				$refresh_left = "top_menu=control";
				$page .= Forms->header($page_title,$user_acct,$view,'',$refresh_left);
				$page .= Forms->form_top('Setup','');
				$page .= Forms->form_doc('Updated: Changes to setup accepted.');
				$page .= Forms->hidden(\%hidden);
				$page .= Forms->form_bottem_one_button(\%close,$tab++);					
				
			} else {
				my %docs = Doc->setup();
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= Forms->form_top('Setup','');
				if (@errors) { $page .= Forms->form_errors(\@errors) }
				my %p = StorProc->fetch_one('setup','name','login_authentication');
				my @members = ('active','passive','none');
				$page .= Forms->list_box('Login authentication:','login_authentication',\@members,$p{'value'},'',$docs{'login_authentication'});
				%p = StorProc->fetch_one('setup','name','session_timeout');
				$page .= Forms->text_box('Session timeout seconds:','session_timeout',$p{'value'},'5','',$docs{'session_timeout'});
				@members = ('1.x','2.x');
				%p = StorProc->fetch_one('setup','name','nagios_version');
				$page .= Forms->list_box('Nagios version:','nagios_version',\@members,$p{'value'},'',$docs{'nagios_version'});
				$page .= Forms->text_box('Nagios etc:','nagios_etc',$nagios_etc,$textsize{'long_name'},'',$docs{'nagios_etc'});
				$page .= Forms->text_box('Nagios bin:','nagios_bin',$nagios_bin,$textsize{'long_name'},'',$docs{'nagios_bin'});		
				$page .= Forms->display_hidden('Monarch home:','monarch_home',$monarch_home,$docs{'monarch_home'});
				$page .= Forms->text_box('Backup dir:','backup_dir',$backup_dir,$textsize{'long_name'},'',$docs{'backup_dir'});
				$page .= Forms->text_box('Upload dir:','upload_dir',$upload_dir,$textsize{'long_name'},'',$docs{'upload_dir'});
				$page .= Forms->checkbox('Enable externals:','enable_externals',$enable_externals,$docs{'enable_externals'});
				$page .= Forms->text_box('Max tree nodes:','max_tree_nodes',$max_tree_nodes,'5','',$docs{'max_tree_nodes'});
				$page .= Forms->hidden(\%hidden);
				$page .= Forms->form_bottem_two_button(\%save,\%cancel,$tab++);					
			}
#
##################################################
# Nagios cgi		
#
		} elsif ($obj eq 'nagios_cgi_configuration') {
			my $step = $query->param('step');
			my %checks = ();
			if ($step eq '1') {
				%checks = ('show_context_help' => '0',
					'use_authentication' => '0');
			}
			my %layout_val = ('User-defined coordinates' => '-zero-','Depth_layers' => '1','Collapsed tree' => '2','Balanced tree' => '3','Circular (Mark Up)' => '4','Circular (Balloon)' => '5');
			my %val_layout = ('-zero-' => 'User-defined coordinates','1' => 'Depth_layers','2' => 'Collapsed tree','3' => 'Balanced tree','4' => 'Circular (Mark Up)','5' => 'Circular (Balloon)');
			if ($query->param('next')) {
				my %nagios_set = StorProc->fetch_one('setup','name','log_file');
				if ($nagios_set{'name'}) {
					foreach my $name($query->param) {
						unless ($name =~ /^next$|user_acct|^obj$|^nocache$/) {
							my $val = undef;
						#	if ($name =~ /^authorized/) {
						#		my @authlist = $query->param($name);
						#		foreach my $user (@authlist) { unless ($user eq '') {$val .= "$user," } }
						#		chop $val;
						#	} elsif (defined $checks{$name}) {
							if ($name =~ /layout/) {
								$val = $query->param($name);
								$val = $layout_val{$val};
							} elsif (defined $checks{$name}) {
								$checks{$name} = '1';
							} else {
								$val = $query->param($name);
							}
							my %values = ('value' => $val);
							my $result = StorProc->update_obj('setup','name',$name,\%values);
							if ($result =~ /^Error/) { push @errors, $result }
						}
					}
					foreach my $name (keys %checks) {
						my %values = ('value' => $checks{$name});
						my $result = StorProc->update_obj('setup','name',$name,\%values);
						if ($result =~ /^Error/) { push @errors, $result }
					}
				} else {
					my %nagios_defaults = StorProc->cgi_defaults();
					foreach my $name (keys %nagios_defaults) {
						my @values = ($name,'nagios',$nagios_defaults{$name});
						my $result = StorProc->insert_obj('setup',\@values);
						if ($result =~ /^Error/) { push @errors, $result }
					}
					$step = 0;
				}
				unless (@errors) {
					if ($step eq '0') {
						$step = 1;
					} elsif ($step eq '1') {
						$step = 2; 
					} elsif ($step eq '2') {
						$step = 'saved'; 
					}
				}
			} elsif ($query->param('back')) {
				$step = $step - 1;
			} elsif ($task eq 'set_defaults') {
				my %nagios_defaults = StorProc->cgi_defaults();
				my $result = StorProc->delete_all('setup','type','nagios_cgi');
				if ($result =~ /^Error/) { push @errors, $result }
				my $result = StorProc->delete_all('setup','type','nagios_ddb');
				if ($result =~ /^Error/) { push @errors, $result }
				foreach my $name (keys %nagios_defaults) {
					my @values = ($name,'nagios_cgi',$nagios_defaults{$name});
					my $result = StorProc->insert_obj('setup',\@values);
					if ($result =~ /^Error/) { push @errors, $result }
				}
				$step = 1;
			} elsif ($task eq 'load_nagios_cgi') {
				@errors = StorProc->load_nagios_cgi($nagios_etc); 
				$step = 1;
			} elsif ($task eq 'view_edit') {
				$step = 1;
			}
			%next = ('name' => 'next', 'value' => 'Save and Next >>');
			my %docs = Doc->cgi_cfg();
			my %where = ('type' => 'nagios_cgi');
			my %objects = StorProc->fetch_list_hash_array('setup',\%where);
			my @nonmembers = StorProc->fetch_list('contacts','name');
			if ($step eq '1') {
				$hidden{'step'} = 1;
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= Forms->form_top('Nagios CGI Configuration Page 1','');
				if (@errors) { $page .= Forms->form_errors(\@errors) }
				$page .= Forms->text_box('Physical HTML path:','physical_html_path',$objects{'physical_html_path'}[2],$textsize{'long_name'},'',$docs{'physical_html_path'});
				$page .= Forms->text_box('URL HTML path:','url_html_path',$objects{'url_html_path'}[2],$textsize{'long_name'},'',$docs{'url_html_path'});
				$page .= Forms->checkbox('Context sensitive help','show_context_help',$objects{'show_context_help'}[2],$docs{'show_context_help'});
				$page .= Forms->text_box('Nagios process check command:','nagios_check_command',$objects{'nagios_check_command'}[2],$textsize{'long_name'},'',$docs{'nagios_check_command'});
				$page .= Forms->checkbox('Use authentication:','use_authentication',$objects{'use_authentication'}[2],$docs{'use_authentication'});				
				$page .= Forms->text_box('Default user name:','default_user_name',$objects{'default_user_name'}[2],$textsize{'short_name'},'',$docs{'default_user_name'});
			#	my @members = split(/,/, $objects{'authorized_for_system_information'}[2]);
			#	push @nonmembers, '*';
			#	$page .= Forms->list_box_multiple('System/Process Information Access:','authorized_for_system_information',\@nonmembers,\@members,'',$docs{'authorized_for_system_information'});
			#	@members = split(/,/, $objects{'authorized_for_system_commands'}[2]);
			#	$page .= Forms->list_box_multiple('System/Process Command Access:','authorized_for_system_commands',\@nonmembers,\@members,'',$docs{'authorized_for_system_commands'});
			#	@members = split(/,/, $objects{'authorized_for_configuration_information'}[2]);
			#	$page .= Forms->list_box_multiple('Configuration information access:','authorized_for_configuration_information',\@nonmembers,\@members,'',$docs{'authorized_for_configuration_information'});
			#	@members = split(/,/, $objects{'authorized_for_all_hosts'}[2]);
			#	$page .= Forms->list_box_multiple('Global Host Information Access:','authorized_for_all_hosts',\@nonmembers,\@members,'',$docs{'authorized_for_all_hosts'});
				$page .= Forms->text_area('System/Process Information Access:','authorized_for_system_information',$objects{'authorized_for_system_information'}[2],'3','80','',$docs{'authorized_for_system_information'});

				$page .= Forms->text_area('System/Process Command Access:','authorized_for_system_commands',$objects{'authorized_for_system_commands'}[2],'3','80','',$docs{'authorized_for_system_commands'});
				$page .= Forms->text_area('Configuration information access:','authorized_for_configuration_information',$objects{'authorized_for_configuration_information'}[2],'3','80','',$docs{'authorized_for_configuration_information'});
				$page .= Forms->text_area('Global Host Information Access:','authorized_for_all_hosts',$objects{'authorized_for_all_hosts'}[2],'3','80','',$docs{'authorized_for_all_hosts'});

				$page .= Forms->hidden(\%hidden);
				$page .= Forms->form_bottem_one_button(\%next,$tab++);	
			} elsif ($step eq '2') {
				$hidden{'step'} = 2;
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= Forms->form_top('Nagios CGI Configuration Page 2','');
				if (@errors) { $page .= Forms->form_errors(\@errors) }
			#	my @members = split(/,/, $objects{'authorized_for_all_host_commands'}[2]);
			#	$page .= Forms->list_box_multiple('Global Host Command Access:','authorized_for_all_host_commands',\@nonmembers,\@members,'',$docs{'authorized_for_all_host_commands'});
			#	@members = split(/,/, $objects{'authorized_for_all_services'}[2]);
			#	$page .= Forms->list_box_multiple('Global Service Information Access:','authorized_for_all_services',\@nonmembers,\@members,'',$docs{'authorized_for_all_services'});
			#	@members = split(/,/, $objects{'authorized_for_all_service_commands'}[2]);
			#	$page .= Forms->list_box_multiple('Global Service Command Access:','authorized_for_all_service_commands',\@nonmembers,\@members,'',$docs{'authorized_for_all_service_commands'});
			
				$page .= Forms->text_area('Global Host Command Access:','authorized_for_all_host_commands',$objects{'authorized_for_all_host_commands'}[2],'3','80','',$docs{'authorized_for_all_host_commands'});
				$page .= Forms->text_area('Global Service Information Access:','authorized_for_all_services',$objects{'authorized_for_all_services'}[2],'3','80','',$docs{'authorized_for_all_services'});
				$page .= Forms->text_area('Global Service Command Access:','authorized_for_all_service_commands',$objects{'authorized_for_all_service_commands'}[2],'3','80','',$docs{'authorized_for_all_service_commands'});

				$page .= Forms->text_box('Statusmap background image:','statusmap_background_image',$objects{'statusmap_background_image'}[2],$textsize{'long_name'},'',$docs{'statusmap_background_image'});
				my @options = ('User-defined coordinates','Depth_layers','Collapsed tree','Balanced tree','Circular (Mark Up)','Circular (Balloon)');
				$objects{'default_statusmap_layout'}[2] = $val_layout{$objects{'default_statusmap_layout'}[2]};
				$page .= Forms->list_box('Default statusmap layout:','default_statusmap_layout',\@options,$objects{'default_statusmap_layout'}[2],'',$docs{'default_statusmap_layout'});
				my @options = ('Collapsed tree','Balanced tree','Circular');
				$objects{'default_statuswrl_layout'}[2] = $val_layout{$objects{'default_statuswrl_layout'}[2]};
				$page .= Forms->list_box('Default statuswrl layout:','default_statuswrl_layout',\@options,$objects{'default_statuswrl_layout'}[2],'',$docs{'default_statuswrl_layout'});
				$page .= Forms->text_box('Status wrl include:','statuswrl_include',$objects{'statuswrl_include'}[2],$textsize{'long_name'},'',$docs{'statuswrl_include'});
				$page .= Forms->text_box('Ping syntax:','ping_syntax',$objects{'ping_syntax'}[2],$textsize{'long_name'},'',$docs{'ping_syntax'});
				$page .= Forms->text_box('Refresh rate:','refresh_rate',$objects{'refresh_rate'}[2],'7','',$docs{'refresh_rate'});
				$page .= Forms->text_box('Host unreachable sound:','host_unreachable_sound',$objects{'host_unreachable_sound'}[2],$textsize{'long_name'},'',$docs{'sound_options'});
				$page .= Forms->text_box('Host down sound:','host_down_sound',$objects{'host_down_sound'}[2],$textsize{'long_name'},'','');
				$page .= Forms->text_box('Service critical sound:','service_critical_sound',$objects{'service_critical_sound'}[2],$textsize{'long_name'},'','');
				$page .= Forms->text_box('Service warning sound:','service_warning_sound',$objects{'service_warning_sound'}[2],$textsize{'long_name'},'','');
				$page .= Forms->text_box('Service unknown sound:','service_unknown_sound',$objects{'service_unknown_sound'}[2],$textsize{'long_name'},'','');
			#	if ($nagios_ver eq '1.x') {
			#		my %where = ('type' => 'nagios_ddb');
			#		my %ddbs = StorProc->fetch_list_hash_array('setup',\%where);
			#		my $ddbstr = undef;
			#		foreach my $ddb (keys %ddbs) { $ddbstr .= "$ddbs{$ddb}[0]=$ddbs{$ddb}[2]\n" }
			#		chomp $ddbstr;
			#		$page .= Forms->text_area('Database directives:','ddb',$ddbstr,'10','50','',$docs{'ddb'});
			#	}
				$page .= Forms->hidden(\%hidden);
				%save = ('name' => 'next', 'value' => 'Save and Done');
				$page .= Forms->form_bottem_two_button(\%back,\%save,$tab++);					
			} elsif ($step eq 'saved') {
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= Forms->form_top("Nagios configuration",'');
				my @message = ("Changes to nagios configuration completed");
				$page .= Forms->form_message('Saved:',\@message,'row1');	
				delete $hidden{'task'};
				$page .= Forms->hidden(\%hidden);
				$page .= Forms->form_bottem_one_button(\%continue,$tab++);
			}
#
##################################################
# Nagios main		
#
		} elsif ($obj eq 'nagios_main_configuration') {
			my $step = $query->param('step');
			my %checks = ();
			if ($step eq '1') {
				%checks = ('aggregate_status_updates' => '0',
					'enable_notifications' => '0',
					'execute_service_checks' => '0',
					'accept_passive_service_checks' => '0',
					'execute_host_checks' => '0',
					'accept_passive_host_checks' => '0',
					'enable_event_handlers' => '0',
					'check_external_commands' => '0',
					'retain_state_information' => '0',
					'use_retained_program_state' => '0',
					'use_retained_scheduling_info' => '0');
			} elsif ($step eq '2') {
				%checks = ('use_syslog' => '0',
					'log_notifications' => '0',
					'log_host_retries' => '0',
					'log_service_retries' => '0',
					'log_event_handlers' => '0',
					'log_initial_states' => '0',
					'log_external_commands' => '0',
					'log_passive_service_checks' => '0',
					'log_passive_checks' => '0',
					'auto_reschedule_checks' => '0');				
			} elsif ($step eq '3') {
				%checks = ('use_agressive_host_checking' => '0',
					'enable_flap_detection' => '0',
					'obsess_over_services' => '0',
					'obsess_over_hosts' => '0',
					'process_performance_data' => '0');
			} elsif ($step eq '4') {
				%checks = ('check_for_orphaned_services' => '0',
					'check_service_freshness' => '0',
					'check_host_freshness' => '0',
					'use_regexp_matching' => '0',
					'use_true_regexp_matching' => '0');
			}
			if ($query->param('next')) {
				my %nagios_set = StorProc->fetch_one('setup','name','log_file');
				if ($nagios_set{'name'}) {
					my %where = ('type' => 'nagios');
					my %nag_defined = StorProc->fetch_list_hash_array('setup',\%where);
					foreach my $name($query->param) {
						unless ($name =~ /^next$|user_acct|^obj$|^nocache$|^step$|^top_menu$|^update_main$|^view$/) {
							if ($name =~ /radio_option/) {
								my $val = $query->param($name);
								$name =~ s/radio_option_//;
								if ($val eq 'other') {
									my $other = "other_$name";
									$val = $query->param($other);
								}
								if ($nag_defined{$name}) {
									my %values = ('value' => $val);
									my $result = StorProc->update_obj('setup','name',$name,\%values);
									if ($result =~ /^Error/) { push @errors, $result }
								} else {
									my @values = ($name,'nagios',$val);
									my $result = StorProc->insert_obj('setup',\@values);
									if ($result =~ /^Error/) { push @errors, $result }
								}
							} else {
								if (defined $checks{$name}) {
									$checks{$name} = '1';
								} else {
									my $val = $query->param($name);
									if ($nag_defined{$name}) {
										my %values = ('value' => $val);
										my $result = StorProc->update_obj('setup','name',$name,\%values);
										if ($result =~ /^Error/) { push @errors, $result }
									} else {
										my @values = ($name,'nagios',$val);
										my $result = StorProc->insert_obj('setup',\@values);
										if ($result =~ /^Error/) { push @errors, $result }
									}
								}
							}
						}
					}
					foreach my $name (keys %checks) {
						if ($nag_defined{$name}) {
							my %values = ('value' => $checks{$name});
							my $result = StorProc->update_obj('setup','name',$name,\%values);
							if ($result =~ /^Error/) { push @errors, $result }
						} else {
							my @values = ($name,'nagios',$checks{$name});
							my $result = StorProc->insert_obj('setup',\@values);
							if ($result =~ /^Error/) { push @errors, $result }
						}
					}
				} else {
					my %nagios_defaults = StorProc->nagios_defaults($nagios_ver,$is_portal);
					foreach my $name (keys %nagios_defaults) {
						my @values = ($name,'nagios',$nagios_defaults{$name});
						my $result = StorProc->insert_obj('setup',\@values);
						if ($result =~ /^Error/) { push @errors, $result }
					}
					$step = 0;
				}
				unless (@errors) {
					if ($step eq '0') {
						$step = 1;
					} elsif ($step eq '1') {
						$step = 2; 
					} elsif ($step eq '2') {
						$step = 3; 
					} elsif ($step eq '3') {
						$step = 4; 
					} elsif ($step eq '4') {
						$step = 'saved'; 
					}
				}
			} elsif ($query->param('back')) {
				$step = $step - 1;
			} elsif ($task eq 'set_defaults') {
				my %nagios_defaults = StorProc->nagios_defaults($nagios_ver,$is_portal);
				my $result = StorProc->delete_all('setup','type','nagios');
				if ($result =~ /^Error/) { push @errors, $result }
				foreach my $name (keys %nagios_defaults) {
					my @values = ($name,'nagios',$nagios_defaults{$name});
					my $result = StorProc->insert_obj('setup',\@values);
					if ($result =~ /^Error/) { push @errors, $result }
				}
				$step = 1;
			} elsif ($task eq 'load_nagios_cfg') {
				@errors = StorProc->load_nagios_cfg($nagios_etc); 
				$step = 1;
			} elsif ($task eq 'view_edit') {
				$step = 1;
			}
			my %where = ('type' => 'other');
			my @commands = StorProc->fetch_list_where('commands','name',\%where);
			%next = ('name' => 'next', 'value' => 'Save and Next >>');
			my %docs = Doc->nagios_cfg();
			%where = ('type' => 'nagios');
			my %objects = StorProc->fetch_list_hash_array('setup',\%where);
			if ($step eq '1') {
				$hidden{'step'} = 1;
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= Forms->form_top('Nagios Main Configuration Page 1','');
				if (@errors) { $page .= Forms->form_errors(\@errors) }
				$page .= Forms->text_box('Log file:','log_file',$objects{'log_file'}[2],$textsize{'long_name'},'',$docs{'log_file'});
				if ($nagios_ver eq '2.x') {
					$page .= Forms->text_box('Object cache file:','object_cache_file',$objects{'object_cache_file'}[2],$textsize{'long_name'},'',$docs{'object_cache_file'});
				}
				$page .= Forms->text_box('Resource file:','resource_file',$objects{'resource_file'}[2],$textsize{'long_name'},'',$docs{'resource_file'});
				$page .= Forms->text_box('Temp file:','temp_file',$objects{'temp_file'}[2],$textsize{'long_name'},'',$docs{'temp_file'});
				
				$page .= Forms->text_box('Status file:','status_file',$objects{'status_file'}[2],$textsize{'long_name'},'',$docs{'status_file'});
				$page .= Forms->checkbox('Aggregated status updates option:','aggregate_status_updates',$objects{'aggregate_status_updates'}[2],$docs{'aggregate_status_updates'});
				$page .= Forms->text_box('Aggregated status data update interval:','status_update_interval',$objects{'status_update_interval'}[2],'7','',$docs{'status_update_interval'});
				
				$page .= Forms->text_box('Nagios user:','nagios_user',$objects{'nagios_user'}[2],$textsize{'long_name'},'',$docs{'nagios_user'});
				$page .= Forms->text_box('Nagios group:','nagios_group',$objects{'nagios_group'}[2],$textsize{'long_name'},'',$docs{'nagios_group'});

				$page .= Forms->checkbox('Enable notifications:','enable_notifications',$objects{'enable_notifications'}[2],$docs{'enable_notifications'});
				$page .= Forms->checkbox('Execute service checks:','execute_service_checks',$objects{'execute_service_checks'}[2],$docs{'execute_service_checks'});
				$page .= Forms->checkbox('Accept passive service checks:','accept_passive_service_checks',$objects{'accept_passive_service_checks'}[2],$docs{'accept_passive_service_checks'});
				if ($nagios_ver eq '2.x') {
					$page .= Forms->checkbox('Execute host checks','execute_host_checks',$objects{'execute_host_checks'}[2],$docs{'execute_host_checks'});
					$page .= Forms->checkbox('Accept passive host checks','accept_passive_host_checks',$objects{'accept_passive_host_checks'}[2],$docs{'accept_passive_host_checks'});
				}
				$page .= Forms->checkbox('Enable event handlers:','enable_event_handlers',$objects{'enable_event_handlers'}[2],$docs{'enable_event_handlers'});
				my @list = ('n','h','d','w','m');
				$page .= Forms->list_box('Log rotation method:','log_rotation_method',\@list,$objects{'log_rotation_method'}[2],'',$docs{'log_rotation_method'});
				$page .= Forms->text_box('Log archive path:','log_archive_path',$objects{'log_archive_path'}[2],$textsize{'long_name'},'',$docs{'log_archive_path'});
				
				$page .= Forms->checkbox('External command check option:','check_external_commands',$objects{'check_external_commands'}[2],$docs{'check_external_commands'});
				$page .= Forms->text_box('External command check interval:','command_check_interval',$objects{'command_check_interval'}[2],$textsize{'small'},'',$docs{'command_check_interval'});
				$page .= Forms->text_box('External command file:','command_file',$objects{'command_file'}[2],$textsize{'long_name'},'',$docs{'command_file'});
				
				$page .= Forms->text_box('Downtime file:','downtime_file',$objects{'downtime_file'}[2],$textsize{'long_name'},'',$docs{'downtime_file'});
				$page .= Forms->text_box('Comment file:','comment_file',$objects{'comment_file'}[2],$textsize{'long_name'},'',$docs{'comment_file'});
				$page .= Forms->text_box('Lock file:','lock_file',$objects{'lock_file'}[2],$textsize{'long_name'},'',$docs{'lock_file'});

				$page .= Forms->checkbox('State retention option:','retain_state_information',$objects{'retain_state_information'}[2],$docs{'retain_state_information'});
				$page .= Forms->text_box('State retention file:','state_retention_file',$objects{'state_retention_file'}[2],$textsize{'long_name'},'',$docs{'state_retention_file'});
				$page .= Forms->text_box('Retention update interval:','retention_update_interval',$objects{'retention_update_interval'}[2],$textsize{'small'},'',$docs{'retention_update_interval'});
				$page .= Forms->checkbox('Use retained program state:','use_retained_program_state',$objects{'use_retained_program_state'}[2],$docs{'use_retained_program_state'});
				if ($nagios_ver eq '2.x') {
					$page .= Forms->checkbox('Use retained scheduling info:','use_retained_scheduling_info',$objects{'use_retained_scheduling_info'}[2],$docs{'use_retained_scheduling_info'});
				}
				$page .= Forms->hidden(\%hidden);
				$page .= Forms->form_bottem_one_button(\%next,$tab++);					
			} elsif ($step eq '2') {
				$hidden{'step'} = 2;
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= Forms->form_top('Nagios Main Configuration Page 2','');
				if (@errors) { $page .= Forms->form_errors(\@errors) }
				$page .= Forms->checkbox('Syslog logging option:','use_syslog',$objects{'use_syslog'}[2],$docs{'use_syslog'});
				$page .= Forms->checkbox('Notification logging option:','log_notifications',$objects{'log_notifications'}[2],$docs{'log_notifications'});
				$page .= Forms->checkbox('Host check retry logging option','log_host_retries',$objects{'log_host_retries'}[2],$docs{'log_host_retries'});
				$page .= Forms->checkbox('Service check retry logging option:','log_service_retries',$objects{'log_service_retries'}[2],$docs{'log_service_retries'});
				$page .= Forms->checkbox('Event handler logging option','log_event_handlers',$objects{'log_event_handlers'}[2],$docs{'log_event_handlers'});
				$page .= Forms->checkbox('Initial states logging option:','log_initial_states',$objects{'log_initial_states'}[2],$docs{'log_initial_states'});
				$page .= Forms->checkbox('External command logging option:','log_external_commands',$objects{'log_external_commands'}[2],$docs{'log_external_commands'});
				if ($nagios_ver eq '1.x') {
					$page .= Forms->checkbox('Log passive service checks:','log_passive_service_checks',$objects{'log_passive_service_checks'}[2],$docs{'log_passive_service_checks'});
				} else {
					$page .= Forms->checkbox('Passive check logging option:','log_passive_checks',$objects{'log_passive_checks'}[2],$docs{'log_passive_checks'});
				}
				
				$page .= Forms->list_box('Global host event handler:','global_host_event_handler',\@commands,$objects{'global_host_event_handler'}[2],'',$docs{'global_host_event_handler'});
				$page .= Forms->list_box('Global service event handler:','global_service_event_handler',\@commands,$objects{'global_service_event_handler'}[2],'',$docs{'global_service_event_handler'});

				$page .= Forms->text_box('Sleep time:','sleep_time',$objects{'sleep_time'}[2],$textsize{'small'},'',$docs{'sleep_time'});
				if ($nagios_ver eq '1.x') {
					$page .= Forms->radio_options('Inter check delay method:','inter_check_delay_method',$objects{'inter_check_delay_method'}[2],$docs{'inter_check_delay_method'});
				} else {
					$page .= Forms->radio_options('Service inter check delay method:','service_inter_check_delay_method',$objects{'service_inter_check_delay_method'}[2],$docs{'service_inter_check_delay_method'});
					$page .= Forms->text_box('Max service check spread:','max_service_check_spread',$objects{'max_service_check_spread'}[2],$textsize{'small'},'',$docs{'max_service_check_spread'});
				}
				$page .= Forms->radio_options('Service interleave factor:','service_interleave_factor',$objects{'service_interleave_factor'}[2],$docs{'service_interleave_factor'});
				$page .= Forms->text_box('Service reaper frequency:','service_reaper_frequency',$objects{'service_reaper_frequency'}[2],$textsize{'small'},'',$docs{'service_reaper_frequency'});
				if ($nagios_ver eq '2.x') {
					$page .= Forms->radio_options('Host inter check delay method:','host_inter_check_delay_method',$objects{'host_inter_check_delay_method'}[2],$docs{'host_inter_check_delay_method'});
					$page .= Forms->text_box('Max host check spread:','max_host_check_spread',$objects{'max_host_check_spread'}[2],$textsize{'small'},'',$docs{'max_host_check_spread'});
				}
				$page .= Forms->text_box('Maximum Concurrent Service Checks:','max_concurrent_checks',$objects{'max_concurrent_checks'}[2],$textsize{'small'},'',$docs{'max_concurrent_checks'});
				$page .= Forms->text_box('Timing interval length:','interval_length',$objects{'interval_length'}[2],$textsize{'small'},'',$docs{'interval_length'});
				if ($nagios_ver eq '2.x') {
					$page .= Forms->checkbox('Auto reschedule checks:','auto_reschedule_checks',$objects{'auto_reschedule_checks'}[2],$docs{'auto_reschedule_checks'});
					$page .= Forms->text_box('Auto rescheduling interval:','auto_rescheduling_interval',$objects{'auto_rescheduling_interval'}[2],$textsize{'small'},'',$docs{'auto_rescheduling_interval'});
					$page .= Forms->text_box('Auto rescheduling window:','auto_rescheduling_window',$objects{'auto_rescheduling_window'}[2],$textsize{'small'},'',$docs{'auto_rescheduling_window'});
				}
				$page .= Forms->hidden(\%hidden);
				$page .= Forms->form_bottem_two_button(\%back,\%next,$tab++);					

				
			} elsif ($step eq '3') {
				$hidden{'step'} = 3;
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= Forms->form_top('Nagios Main Configuration Page 3','');
				if (@errors) { $page .= Forms->form_errors(\@errors) }

				$page .= Forms->checkbox('Agressive Host Checking Option:','use_agressive_host_checking',$objects{'use_agressive_host_checking'}[2],$docs{'use_agressive_host_checking'});
	
				$page .= Forms->checkbox('Flap Detection Option:','enable_flap_detection',$objects{'enable_flap_detection'}[2],$docs{'enable_flap_detection'});
				$page .= Forms->text_box('Low service flap threshold:','low_service_flap_threshold',$objects{'low_service_flap_threshold'}[2],$textsize{'small'},'',$docs{'service_flap_detection_thresholds'});
				$page .= Forms->text_box('High service flap threshold:','high_service_flap_threshold',$objects{'high_service_flap_threshold'}[2],$textsize{'small'});
				$page .= Forms->text_box('Low host flap threshold:','low_host_flap_threshold',$objects{'low_host_flap_threshold'}[2],$textsize{'small'},'',$docs{'host_flap_detection_thresholds'});
				$page .= Forms->text_box('High host flap threshold:','high_host_flap_threshold',$objects{'high_host_flap_threshold'}[2],$textsize{'small'});
				
				$page .= Forms->text_box('Soft state dependencies:','soft_state_dependencies',$objects{'soft_state_dependencies'}[2],$textsize{'small'},'',$docs{'soft_state_dependencies'});

				$page .= Forms->text_box('Service check timeout:','service_check_timeout',$objects{'service_check_timeout'}[2],$textsize{'small'},'',$docs{'timeout_values'});
				$page .= Forms->text_box('Host check timeout:','host_check_timeout',$objects{'host_check_timeout'}[2],$textsize{'small'});
				$page .= Forms->text_box('Event handler timeout:','event_handler_timeout',$objects{'event_handler_timeout'}[2],$textsize{'small'});
				$page .= Forms->text_box('Notification timeout:','notification_timeout',$objects{'notification_timeout'}[2],$textsize{'small'});
				$page .= Forms->text_box('Ocsp timeout:','ocsp_timeout',$objects{'ocsp_timeout'}[2],$textsize{'small'});
				if ($nagios_ver eq '2.x') {
					$page .= Forms->text_box('Ochp timeout:','ochp_timeout',$objects{'ochp_timeout'}[2],$textsize{'small'});
				}
				$page .= Forms->text_box('Perfdata timeout:','perfdata_timeout',$objects{'perfdata_timeout'}[2],$textsize{'small'});
	
				$page .= Forms->checkbox('Obsess over services:','obsess_over_services',$objects{'obsess_over_services'}[2],$docs{'obsess_over_services'});
				$page .= Forms->text_box('OCSP command:','ocsp_command',$objects{'ocsp_command'}[2],$textsize{'long_name'},'',$docs{'ocsp_command'});
				if ($nagios_ver eq '2.x') {
					$page .= Forms->checkbox('Obsess over hosts:','obsess_over_hosts',$objects{'obsess_over_hosts'}[2],$docs{'obsess_over_hosts'});		
					$page .= Forms->text_box('OCHP command:','ochp_command',$objects{'ochp_command'}[2],$textsize{'long_name'},'',$docs{'ochp_command'});
				}
				$page .= Forms->checkbox('Process performance data:','process_performance_data',$objects{'process_performance_data'}[2],$docs{'process_performance_data'});
				$page .= Forms->list_box('Host perfdata command:','host_perfdata_command',\@commands,$objects{'host_perfdata_command'}[2],'',$docs{'host_perfdata_command'});
				$page .= Forms->list_box('Service perfdata command:','service_perfdata_command',\@commands,$objects{'service_perfdata_command'}[2],'',$docs{'service_perfdata_command'});
				if ($nagios_ver eq '2.x') {
					$page .= Forms->text_box('Host performance data file:','host_perfdata_file',$objects{'host_perfdata_file'}[2],$textsize{'long_name'},'',$docs{'host_perfdata_file'});
					$page .= Forms->text_box('Service performance data file:','service_perfdata_file',$objects{'service_perfdata_file'}[2],$textsize{'long_name'},'',$docs{'service_perfdata_file'});
					$page .= Forms->text_box('Host performance data file template:','host_perfdata_file_template',$objects{'host_perfdata_file_template'}[2],$textsize{'long_name'},'',$docs{'host_perfdata_file_template'});
					$page .= Forms->text_box('Service performance data file template:','service_perfdata_file_template',$objects{'service_perfdata_file_template'}[2],$textsize{'long_name'},'',$docs{'service_perfdata_file_template'});
					my @opts = ('a','w');
					$page .= Forms->list_box('Host performance data file mode:','host_perfdata_file_mode',\@opts,$objects{'host_perfdata_file_mode'}[2],'',$docs{'host_perfdata_file_mode'});
					$page .= Forms->list_box('Service performance data file mode:','service_perfdata_file_mode',\@opts,$objects{'service_perfdata_file_mode'}[2],'',$docs{'service_perfdata_file_mode'});
					$page .= Forms->text_box('Host performance data file processing interval:','host_perfdata_file_processing_interval',$objects{'host_perfdata_file_processing_interval'}[2],$textsize{'small'},'',$docs{'host_perfdata_file_processing_interval'});
					$page .= Forms->text_box('Service performance data file processing interval:','service_perfdata_file_processing_interval',$objects{'service_perfdata_file_processing_interval'}[2],$textsize{'small'},'',$docs{'service_perfdata_file_processing_interval'});
					$page .= Forms->list_box('Host performance data file processing command:','host_perfdata_file_processing_command',\@commands,$objects{'host_perfdata_file_processing_command'}[2],'',$docs{'host_perfdata_file_processing_command'});
					$page .= Forms->list_box('Service performance data file processing command:','service_perfdata_file_processing_command',\@commands,$objects{'service_perfdata_file_processing_command'}[2],'',$docs{'service_perfdata_file_processing_command'});
				}
				$page .= Forms->hidden(\%hidden);
				$page .= Forms->form_bottem_two_button(\%back,\%next,$tab++);		
				
			} elsif ($step eq '4') {
				$hidden{'step'} = 4;
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= Forms->form_top('Nagios Main Configuration Page 4','');
				if (@errors) { $page .= Forms->form_errors(\@errors) }
				$page .= Forms->checkbox('Check for orphaned services:','check_for_orphaned_services',$objects{'check_for_orphaned_services'}[2],$docs{'check_for_orphaned_services'});
				
				$page .= Forms->checkbox('Check service freshness:','check_service_freshness',$objects{'check_service_freshness'}[2],$docs{'check_service_freshness'});
				$page .= Forms->text_box('Service freshness check interval:','freshness_check_interval',$objects{'freshness_check_interval'}[2],$textsize{'small'},'',$docs{'freshness_check_interval'});
				if ($nagios_ver eq '2.x') {
					$page .= Forms->checkbox('Check host freshness:','check_host_freshness',$objects{'check_host_freshness'}[2],$docs{'check_host_freshness'});
					$page .= Forms->text_box('Host freshness check interval:','host_freshness_check_interval',$objects{'host_freshness_check_interval'}[2],$textsize{'small'},'',$docs{'host_freshness_check_interval'});		
				}

				$page .= Forms->date_format($objects{'date_format'}[2],$docs{'date_format'});

				$page .= Forms->text_box('Illegal object name chars:','illegal_object_name_chars',$objects{'illegal_object_name_chars'}[2],$textsize{'long_name'},'',$docs{'illegal_object_name_chars'});
				$page .= Forms->text_box('Illegal macro output characters:','illegal_macro_output_chars',$objects{'illegal_macro_output_chars'}[2],$textsize{'long_name'},'',$docs{'illegal_macro_output_chars'});				
				$page .= Forms->text_box('Admin email:','admin_email',$objects{'admin_email'}[2],$textsize{'long_name'},'',$docs{'admin_email'});
				$page .= Forms->text_box('Admin pager:','admin_pager',$objects{'admin_pager'}[2],$textsize{'long_name'},'',$docs{'admin_pager'});
				$page .= Forms->text_box('p1.pl File:','p1_file',$objects{'p1_file'}[2],$textsize{'long_name'},'',$docs{'p1_file'});
				$page .= Forms->hidden(\%hidden);
				%save = ('name' => 'next', 'value' => 'Save and Done');
				$page .= Forms->form_bottem_two_button(\%back,\%save,$tab++);					
			} elsif ($step eq 'saved') {
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= Forms->form_top("Nagios configuration",'');
				my @message = ("Changes to nagios configuration completed");
				$page .= Forms->form_message('Saved:',\@message,'row1');	
				delete $hidden{'task'};
				$page .= Forms->hidden(\%hidden);
				$page .= Forms->form_bottem_one_button(\%continue,$tab++);
			}

#
##################################################
# Resource
#
		} elsif ($obj eq 'nagios_resource_macros') {
			my $task = $query->param('submit');
			my $saved = 0;
			my $resource = undef;
			my $resource_sav = undef;
			if ($query->param('update_resource')) {
				$resource_sav = $query->param('resource');
				my $resource_value = $query->param('resource_value');
				my $comment = $query->param('comment');
				my %values = ('value' => $resource_value);
				my $result = StorProc->update_obj('setup','name',$resource_sav,\%values);
				if ($result =~ /error/i) { push @errors, $result }
				my $label = $resource_sav;
				$label =~ s/user//;
				my %values = ('value' => $comment);
				my $result = StorProc->update_obj('setup','name',"resource_label$label",\%values);
				if ($result =~ /error/i) { 
					push @errors, $result;
				} else {
					$resource_sav = undef;
					$resource = undef;
				}
			}

			my $resource_value = undef;
			$page .= Forms->header($page_title,$user_acct,$view);
			$page .= Forms->form_top("Nagios Resource Macros",'');
			if (@errors) { $page .= Forms->form_errors(\@errors) }
			my %resources = StorProc->get_resources();
			my %resource_doc = StorProc->get_resources_doc();
			my %selected = ();
			if ($resource_sav) {
				$resource = $resource_sav;
			} else {
				foreach my $name ($query->param) {
					if ($name =~ /^resource_(user\d+)/) { $resource = $1 }
				}
			}
			if ($resource) {
				$resource_value = $resources{$resource};
				%selected = ('name' => $resource,'value' => $resource_value);
			} else {
				$page .= Forms->form_doc('Select resource');
			}
			$page .= Forms->resource_select(\%resources,\%resource_doc,\%selected,$top_menu);
			$page .= Forms->hidden(\%hidden);
			$page .= Forms->form_bottem_one_button(\%close,$tab++);

#
##################################################
# Files
#
		} elsif ($obj eq 'files') {
			$page .= Forms->header($page_title,$user_acct,$view);
			if ($query->param('cancel')) { $task = undef }
			if ($task) {
				$hidden{'task'} = $task;
				my $submit = $query->param('submit');
				if ($query->param('save')) {
					foreach my $name ($query->param) {
						if ($name eq 'nocache') { next }
						my %values = ();
						my $value = $query->param($name);
						$values{'value'} = $value;
						my $result = StorProc->update_obj('setup','name',$name,\%values);
						if ($result =~ /^Error/) { push @errors, $result }
					}
					unless (@errors) { $task = undef }
				} elsif ($query->param('add')) {
					my $file = $query->param('file');
					my $path = $query->param('path');
					my $type = $query->param('type');

					if (!$path || !$file || !$type) {
						push @errors, "Required: File, path and type.";
					} else {
						unless ($file =~ /\.cfg$/) { $file .= ".cfg" }
						my %p = StorProc->fetch_one('files','name',$file);
						if ($p{'file_id'}) {
							push @errors, "Duplicate: File $file already exists.";
						} else {
							my @values = ('',$file,$path,$type);
							my $result = StorProc->insert_obj('files',\@values);
							if ($result =~ /^Error/) { push @errors, $result }
						}
					}
					unless (@errors) { $task = undef }
				}
				if ($task eq 'file_add') {
					my %p = StorProc->fetch_one('setup','name','nagios_etc');
					$page .= Forms->form_top('Add File','');
					if (@errors) { $page .= Forms->form_errors(\@errors) }
					$page .= Forms->text_box("File name:",'file','',$textsize{'long_name'});
					$page .= Forms->text_box("File path:",'path',$p{'value'},$textsize{'long_name'});
					my @types = ('hosts','services');
					$page .= Forms->list_box("File type:",'type',\@types,$p{'type'});
					$page .= Forms->hidden(\%hidden);
					$page .= Forms->form_bottem_two_button(\%add,\%cancel,$tab++);					

				} elsif ($task eq 'file_assn') {
					$page .= Forms->form_top('Object File Associations','');
					if (@errors) { $page .= Forms->form_errors(\@errors) }
					my %where = ('type' => 'file');
					my %objects = StorProc->fetch_list_hash_array('setup',\%where);
					foreach my $name (sort keys %objects) {
						if ($name =~ /HASH|stage|^hosts$|^services$|coords/) { next }
						my %f = StorProc->fetch_one('files','file_id',$objects{$name}[2]);
						my $title = $name;
						$title =~ s/_/ /g;
						$title = "\u$title";
						$page .= Forms->display_hidden("$title:",'name',$f{'name'});
					}
					%where = ('type' => 'hosts');
					my @files = StorProc->fetch_list_where('files','name',\%where);
					foreach my $f (@files) {
						$page .= Forms->display_hidden('Host:','name',$f);
					}
					%where = ('type' => 'services');
					my @files = StorProc->fetch_list_where('files','name',\%where);
					foreach my $f (@files) {
						$page .= Forms->display_hidden('Service:','name',$f);
					}
					$page .= Forms->hidden(\%hidden);
					$page .= Forms->form_bottem_one_button(\%close,$tab++);					
				}
			}
#
##################################################
# Load
#
		} elsif ($obj eq 'load') {
			my %backup = ('name' => 'backup','value' => 'Backup');
			my %load = ('name' => 'load_options','value' => 'Load');
			if ($query->param('continue')) {
				print "Content-type: text/html \n\n";
				$page .= Forms->header($page_title,$user_acct,$view);
				if ($query->param('continue') =~ /$abort{'value'}/) {
					$page .= Forms->form_top('Load Nagios Configuration','');
					$hidden{'obj'} = undef;
					$page .= Forms->hidden(\%hidden);
					my @message = ("Load of Nagios files aborted.");
					$page .= Forms->form_message("Action Canceled:",\@message ,'');
					$page .= Forms->form_bottem_one_button(\%continue,$tab++);		
					$hidden{'obj'} = undef;
				}
			} elsif ($query->param('backup')) {
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= Forms->form_top('Load Nagios Configuration','');
				my %nagios = StorProc->fetch_one('setup','name','nagios_etc');
				my %backup_dir = StorProc->fetch_one('setup','name','backup_dir');
				my ($backup, $errs) = Files->backup($nagios{'value'},$backup_dir{'value'});
				push (@errors, @{$errs});
				if (@errors) {
					my @message = ("Problem(s) Backing up files and/or database to $backup.");
					push (@message, @errors);
					$page .= Forms->form_message("Backup error(s):",\@message ,'error');
				} else {
					my @message = ("Files backed up to $backup.");
					$page .= Forms->form_message("Backup complete:",\@message,'');
				}
				$page .= Forms->hidden(\%hidden);
				$page .= Forms->form_bottem_two_button(\%load,\%abort,$tab++);		
			} elsif ($query->param('abort')) {
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= Forms->form_top('Load Nagios Configuration','');
				$hidden{'obj'} = undef;
				$page .= Forms->hidden(\%hidden);
				my @message = ("Load of Nagios files aborted.");
				$page .= Forms->form_message("Action Canceled:",\@message ,'');
				$page .= Forms->form_bottem_one_button(\%continue,$tab++);		
				$hidden{'obj'} = undef;
			} elsif ($query->param('run_load')) {
				$page .= Forms->header($page_title,$user_acct,$view,'','','','2');
				$page .= Forms->form_top('Load Nagios Object Files','');
				$page .= Forms->wizard_doc('Load Process','Please keep this page open until status reads finished.');
				my $load_option = $query->param('load_option');
				my $purge_escalations = $query->param('purge_escalations');
				$page .= Forms->hidden(\%hidden);
				$page .= Forms->process_load($load_option,$purge_escalations,$nagios_etc,$abort{'value'},$continue{'value'});
				use CGI::Ajax;
				my $url = Forms->get_ajax_url();
				my $pjx = new CGI::Ajax( 'process_load' => $url );
				return $pjx->build_html($query,$page);
			} elsif ($query->param('load_options')) {
				$page .= Forms->header($page_title,$user_acct,$view,'');
				$page .= Forms->form_top('Load Nagios Object Files','');
				$page .= Forms->load_options();
				$load{'name'} = 'run_load';
				$page .= Forms->hidden(\%hidden);
				$page .= Forms->form_bottem_two_button(\%load,\%abort,$tab++);		
			} elsif (-e "$nagios_etc/nagios.cfg") {
				my @message = ();
				push @message, qq(You are about to update or drop all Nagios records in the database. Should you choose to continue it is strongly recommended that you first select the backup option.);
				push @message, qq(Are you sure you want to continue?);
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= Forms->form_top('Load Nagios Configuration','');
				$page .= Forms->hidden(\%hidden);
				$page .= Forms->form_message("Load from nagios.cfg:",\@message ,'');
				$page .= Forms->form_bottem_three_button(\%backup,\%load,\%abort,$tab++);		
			} else {
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= Forms->form_top('Error','');
				$hidden{'obj'} = undef;
				$page .= Forms->hidden(\%hidden);
				my @message = ("Unable to load $nagios_etc/nagios.cfg (update setup options).");
				$page .= Forms->form_message("Does not exist:",\@message ,'error');
				$page .= Forms->form_bottem_one_button(\%continue,$tab++);		
			}

#
##################################################
# Pre Flight Test
#
		} elsif ($obj eq 'pre_flight_test') {
			my @results = ();
			use MonarchFile;
			my %nagios = StorProc->fetch_one('setup','name','nagios_etc');
			my %monarch_home = StorProc->fetch_one('setup','name','monarch_home');
			my $workspace = $monarch_home{'value'}.'/workspace';
			my %nagios_cfg = StorProc->fetch_one('setup','name','log_file');
			my %nagios_cgi = StorProc->fetch_one('setup','name','default_user_name');
			if ($nagios_cfg{'value'} && $nagios_cgi{'type'}) {
				if ($query->param('refreshed')) {
					my ($errors, $usefiles) = Files->build_files($user_acct,$workspace,\%property_list,$nagios_ver);
					@errors = @{$errors};
					my %use_file = %{$usefiles};
					unless ($errors[0]) { 
						@errors = Files->nagios_cfg($user_acct,'1',$nagios{'value'},$workspace,\%use_file); 
					}
					unless ($errors[0]) { 
						@results = StorProc->pre_flight_check($nagios_bin, $monarch_home{'value'});
					}
					$page .= Forms->header($page_title,$user_acct,$view);
					$page .= Forms->form_top('Nagios Pre Flight Test','');
					if (@errors) { $page .= Forms->form_message("Error(s) building file(s):",\@errors ,'error') }
					$page .= Forms->form_message("Results:",\@results ,'row1');
					$hidden{'obj'} = undef;
					$page .= Forms->hidden(\%hidden);
					$page .= Forms->form_bottem_one_button(\%continue,$tab++);
				} else {
					my $now = time;
					$refresh_url = "?update_main=1&nocache=$now&refreshed=1";
					foreach my $name (keys %hidden) {
						$refresh_url .= qq(&$name=$hidden{$name});
					}
					$page .= Forms->header($page_title,$user_acct,$view,$refresh_url);
					$page .= Forms->form_top('Nagios Pre Flight Test','');
					$page .= Forms->form_doc('Running pre-flight check...');
					my $now = time;
					$page .= Forms->form_bottem_no_button();		
				}
			} else {
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= Forms->form_top('Nagios Pre Flight Test','');
				unless ($nagios_cfg{'value'}) {
					push @errors, 'Nagios main configuration has not been defined.','Use Control->Nagios main configuration to load an existing file or set defaults.';
				}
				unless ($nagios_cgi{'type'}) {
					push @errors, 'Nagios cgi configuration has not been defined.','Use Control->Nagios cgi configuration to load an existing file or set defaults.';
				}	
				$page .= Forms->form_message("Error(s):",\@errors ,'error');
				$hidden{'obj'} = undef;
				$page .= Forms->hidden(\%hidden);
				$page .= Forms->form_bottem_one_button(\%continue,$tab++);
			}

#
##################################################
# Commit
#
		} elsif ($obj eq 'commit') {
			my @results = ();
			my %abort = ('name' => 'abort','value' => 'Abort');
			my %backup = ('name' => 'backup','value' => 'Backup');
			my %commit = ('name' => 'commit','value' => 'Commit');
			use MonarchFile;
			my %nagios = StorProc->fetch_one('setup','name','nagios_etc');
			my %monarch_home = StorProc->fetch_one('setup','name','monarch_home');
			my $workspace = $monarch_home{'value'}.'/workspace';
			my %nagios_cfg = StorProc->fetch_one('setup','name','log_file');
			my %nagios_cgi = StorProc->fetch_one('setup','name','default_user_name');

			if ($query->param('backup')) {
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= Forms->form_top('Commit','');
				my ($backup, $errs) = Files->backup($nagios{'value'},$backup_dir);
				push (@errors, @{$errs});
				if (@errors) {
					my @message = ("Problem(s) Backing up files and/or database to $backup.");
					push (@message, @errors);
					$page .= Forms->form_message("Backup error(s):",\@message ,'error');
				} else {
					my @message = ("Files backed up to $backup.");
					$page .= Forms->form_message("Backup complete:",\@message,'');
				}
				$page .= Forms->hidden(\%hidden);
				$page .= Forms->form_bottem_two_button(\%abort,\%commit,$tab++);		
			} elsif ($query->param('abort')) {
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= Forms->form_top('Nagios Commit','');
				$hidden{'obj'} = undef;
				$page .= Forms->hidden(\%hidden);
				my @message = ("Commit aborted.");
				$page .= Forms->form_message("Action Canceled:",\@message ,'');
				$page .= Forms->form_bottem_one_button(\%continue,$tab++);		
				$hidden{'obj'} = undef;
			} elsif ($query->param('commit')) {
				if ($query->param('refreshed')) {
					my ($errors, $usefiles) = Files->build_files($user_acct,$workspace,\%property_list,$nagios_ver);
					@errors = @{$errors};
					my %use_file = %{$usefiles};
					unless ($errors[0]) { 
						@errors = Files->nagios_cfg($user_acct,'1',$nagios{'value'},$workspace,\%use_file); 
					}
					unless ($errors[0]) { 
						@results = StorProc->pre_flight_check($nagios_bin, $monarch_home);
					}
					my $res_str = pop @results;
					push @results, $res_str;
					unless ($res_str =~ /Things look okay/) {
						push @errors, "Make the necessary corrections and run pre flight check";
					}
					unless ($errors[0]) { 
						my @errs = Files->nagios_cfg($user_acct,'0',$nagios{'value'},$workspace,\%use_file); 
						push (@results, @errs);
					}		
					my %nagios_ver = StorProc->fetch_one('setup','name','nagios_version');
					unless ($errors[0] && $nagios_ver eq '1.x') { 
						my @errs = Files->nagios_cgi($user_acct,'0',$nagios{'value'},$workspace); 
						push (@results, @errs);
					}		
					unless ($errors[0]) { 
						my @commit = StorProc->commit($monarch_home);
						push (@results, @commit);
					}
					$page .= Forms->header($page_title,$user_acct,$view);
					$page .= Forms->form_top('Nagios Commit','');
					$page .= Forms->form_message("Results:",\@results ,'row1');
					if (@errors) { $page .= Forms->form_message("Error(s):",\@errors ,'error') }
					$hidden{'obj'} = undef;
					$page .= Forms->hidden(\%hidden);
					$page .= Forms->form_bottem_one_button(\%continue,$tab++);
				} else {
					my $now = time;
					$refresh_url = "?nocache=$now&refreshed=1&commit=1";
					foreach my $name (keys %hidden) {
						$refresh_url .= qq(&$name=$hidden{$name});
					}
					$page .= Forms->header($page_title,$user_acct,$view,$refresh_url);
					$page .= Forms->form_top('Commit to Nagios','');
					$page .= Forms->form_doc('Running commit...');
					my $now = time;
					$refresh_url = "?update_main=1&nocache=$now&refreshed=1&commit=1";
					$page .= Forms->form_bottem_no_button();		
				}
			} elsif ($nagios_cfg{'value'} && $nagios_cgi{'type'}) {
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= Forms->form_top('Nagios Commit','');
				$page .= Forms->hidden(\%hidden);
				my @message = ("Are you sure you want to overwrite your active Nagios configuration and restart Nagios?");
				push @message, ("Should you choose to continue it is strongly recommended that you first select the backup option.");
				$page .= Forms->form_message("Nagios commit:",\@message ,'');
				$page .= Forms->form_bottem_three_button(\%abort,\%backup,\%commit,$tab++);		

			} else {
				$page .= Forms->header($page_title,$user_acct,$view);
				$page .= Forms->form_top('Nagios Commit','');
				unless ($nagios_cfg{'value'}) {
					push @errors, 'Nagios main configuration has not been defined.','Use Control->Nagios main configuration to load an existing file or set defaults.';
				}
				unless ($nagios_cgi{'type'}) {
					push @errors, 'Nagios cgi configuration has not been defined.','Use Control->Nagios cgi configuration to load an existing file or set defaults.';
				}	
				$page .= Forms->form_message("Error(s):",\@errors ,'error');
				$hidden{'obj'} = undef;
				$page .= Forms->hidden(\%hidden);
				$page .= Forms->form_bottem_one_button(\%continue,$tab++);
			}
#
##################################################
# Run Ext info scripts
#
		} elsif ($obj eq 'run_extended_info_scripts') {
			$page .= Forms->header($page_title,$user_acct,$view);
			my $ext_info = $query->param('ext_info');
			my $type = $query->param('type');
			if ($ext_info) {
				my @results = ();
				if ($type eq 'host') {
					my %x = StorProc->fetch_one('extended_host_info_templates','name',$ext_info);
					my %w = ('hostextinfo_id' => $x{'hostextinfo_id'});
					my @hosts = StorProc->fetch_list_where('hosts','name',\%w);
					push @results, "Script $x{'script'} from $x{'name'} launched for:";
					push @results, "<hr />";
					foreach my $host (@hosts) {
						my $script = $x{'script'};
						$script =~ s/\$HOSTNAME\$/$host/g;
						system("$script &");
						push @results, "$host '$script'";
						push @results, "<br />";
					}
				} elsif ($type eq 'service') {
					my %x = StorProc->fetch_one('extended_service_info_templates','name',$ext_info);
					my %service_hosts = StorProc->fetch_service_extinfo($x{'serviceextinfo_id'});
					push @results, "Script $x{'script'} from $x{'name'} launched for:";
					push @results, "<hr />";
					foreach my $sid (keys %service_hosts) {
						my $script = $x{'script'};
						$script =~ s/\$HOSTNAME\$/$service_hosts{$sid}[0]/g;
						$script =~ s/\$SERVICENAME\$/$service_hosts{$sid}[1]/g;
						system("$script &");
						push @results, "$service_hosts{$sid}[0] $service_hosts{$sid}[1] '$script'";
						push @results, "<br />";
					}
				}
				$page .= Forms->form_top('External Script','');
				if (@errors) { $page .= Forms->form_message("Error(s):",\@errors ,'error') }
				$page .= Forms->form_message("Results:",\@results ,'row1');
				$page .= Forms->hidden(\%hidden);
				$page .= Forms->form_bottem_one_button(\%continue,$tab++);
			} else {
				my %scripts = StorProc->fetch_scripts('host');				
				$page .= Forms->table_script_links($user_acct,'host',\%scripts);
				%scripts = StorProc->fetch_scripts('service');				
				$page .= Forms->table_script_links($user_acct,'service',\%scripts);
			}
#
##################################################
# Run external scripts
#
		} elsif ($obj eq 'run_externals') {
			use MonarchExternals;	
			my @errors = Externals->build_externals($user_acct);
			my @results = ("Externals module executed.");
			$page .= Forms->header($page_title,$user_acct,$view);
			$page .= Forms->form_top('Run Externals','');
			$page .= Forms->form_message("Results:",\@results ,'row1');
			if (@errors) { $page .= Forms->form_message("Error(s):",\@errors ,'error') }
			$hidden{'obj'} = undef;
			$page .= Forms->hidden(\%hidden);
			$page .= Forms->form_bottem_one_button(\%continue,$tab++);
		}
	}
	return $page;
}


#
##################################################
# Search
#


sub search() {
	my $detail = Forms->header($page_title,$user_acct,$view);
	$detail .= Forms->form_top('Search Hosts','');
	$detail .= Forms->search($user_acct);
	$hidden{'view'} = 'manage_hosts';
	$detail .= Forms->hidden(\%hidden);
	$detail .= Forms->form_bottem_no_button();
	use CGI::Ajax;
	my $url = Forms->get_ajax_url();
	my $pjx = new CGI::Ajax( 'get_hosts' => $url );
	return $pjx->build_html($query,$detail);
}

sub search_services() {
	my $detail = Forms->header($page_title,$user_acct,$view);
	$detail .= Forms->form_top('Search Services','');
	$detail .= Forms->search($user_acct,'services');
	$hidden{'view'} = 'manage_services';
	$detail .= Forms->hidden(\%hidden);
	$detail .= Forms->form_bottem_no_button();
	use CGI::Ajax;
	my $url = Forms->get_ajax_url();
	my $pjx = new CGI::Ajax( 'get_services' => $url );
	return $pjx->build_html($query,$detail);
}


#
##################################################
# Login
#

my $login = 0;
sub login() {
	$login = 1;
	return Forms->login($page_title);
}

my @common_menus = ('time_periods','commands','contact_templates','contacts','contactgroups');
my @host_menus = ('hosts','hostgroups','parent_child','host_templates','extended_host_info_templates','host_dependencies');
my @service_menus = ('service_templates','service_dependency_templates','extended_service_info_templates');
my @tool_menus = ('export');				

my @control_menus = ('users','user_groups','setup','nagios_cgi_configuration','nagios_main_configuration','nagios_resource_macros',
'load','files','pre_flight_test','commit');



my $javascript = undef;
sub left_page() {
	my $selected = $query->param('selected');
	$selected = uri_unescape($selected);
	my $javascript = undef;
	my $opento = 0;
	my ($id,$pid) = 0;
	my @menus = ();
	my %tasks = ();
	my %tree = ();
	my %view = ();
	if ($top_menu eq 'time_periods') {
		$view{'time_periods'} = { ('new' => 'design','copy' => 'design','modify' => 'manage') };
		@menus = ('time_periods');
		my @submenus = StorProc->fetch_list('time_periods','name');
		if ($auth_add{'time_periods'}) {
			$tasks{'time_periods'} = [ ('new','copy') ];
			$tree{'time_periods'} = [ @submenus ];
		}
		if ($auth_modify{'time_periods'}) {
			push @{$tasks{'time_periods'}}, 'modify';
			$tree{'time_periods'} = [ @submenus ];
		}
		$body .= Forms->left_menu($user_acct,$top_menu,\%view,\@menus,\%tasks,\%tree,$selected);
	} elsif ($top_menu eq 'commands') {
		$view{'commands'} = { ('new' => 'commands','copy' => 'commands','modify' => 'commands') };
		@menus = ('commands');
		my @submenus = StorProc->fetch_list('commands','name');
		if ($auth_add{'commands'}) {
			$tasks{'commands'} = [ ('new','copy') ];
			$tree{'commands'} = [ @submenus ];
		}
		if ($auth_modify{'commands'}) {
			push @{$tasks{'commands'}}, 'modify';
			$tree{'commands'} = [ @submenus ];
		}
		$body .= Forms->left_menu($user_acct,$top_menu,\%view,\@menus,\%tasks,\%tree,$selected);
	} elsif ($top_menu eq 'contacts') {
		$view{'contacts'} = { ('new' => 'design','copy' => 'design','modify' => 'manage') };
		$view{'contactgroups'} = { ('new' => 'design','copy' => 'design','modify' => 'manage') };
		$view{'contact_templates'} = { ('new' => 'design','copy' => 'design','modify' => 'manage') };
		@menus = ('contacts','contactgroups','contact_templates');
		foreach my $menu (@menus) {
			my @submenus = StorProc->fetch_list($menu,'name');
			if ($auth_add{$menu}) {
				$tasks{$menu} = [ ('new','copy') ];
				$tree{$menu} = [ @submenus ];
			}
			if ($auth_modify{$menu}) {
				push @{$tasks{$menu}}, 'modify';
				$tree{$menu} = [ @submenus ];
			}
		}
		$body .= Forms->left_menu($user_acct,$top_menu,\%view,\@menus,\%tasks,\%tree,$selected);
	} elsif ($top_menu eq 'escalations') {	
		@menus = ('escalations','escalation_trees');
		my @smenus = ('hostgroup','host','service');
		foreach my $menu (@menus) {
			foreach my $smenu (@smenus) {
				if ($auth_add{'escalations'}) {
					$tasks{$menu} = [ ('new') ];
				}
				if ($auth_modify{'escalations'}) {
					push @{$tasks{$menu}}, 'modify';
					my %w = ('type' => $smenu);
					if ($menu eq 'escalations') { 
						my @submenus = StorProc->fetch_list_where('escalation_templates','name',\%w);
						$tree{"$menu\t$smenu"} = [ @submenus ];
					} else {
						my @submenus = StorProc->fetch_list_where('escalation_trees','name',\%w);
						$tree{"$menu\t$smenu"} = [ @submenus ];
					}
				}
			}
		}
		$body .= Forms->left_menu($user_acct,$top_menu,\%view,\@menus,\%tasks,\%tree,$selected);
	} elsif ($top_menu eq 'services') {	
		my @menus = ();
		if ($auth_add{'services'}) { 
			push @menus, 'new';
		}
		if ($auth_modify{'services'}) { 
			push @menus, 'search';
			push @menus, 'modify';
		}
		if ($nagios_ver eq '2.x') { push @service_menus, 'servicegroups' }
		foreach my $menu (@service_menus) {
			if ($auth_add{$menu} || $auth_modify{$menu}) { 
					push @menus, $menu;
			}
		}
		if ($auth_add{'externals'}) { 
			push @menus, 'service_externals';
		}
		my $num_services = StorProc->get_num_records('service_names');
		foreach my $menu (@menus) {
			if ($menu eq 'modify') {
				unless ($max_tree_nodes && ($num_services > $max_tree_nodes)) {
					my @services = StorProc->fetch_list('service_names','name');
					$tree{$menu} = [ @services ];
				} else {
					my  @services = ();
					$tree{$menu} = [ @services ];
				}
			} elsif ($menu eq 'new') { 
				next;
			} elsif ($menu eq 'search') { 
				$view{$menu} = 'search';
				$tasks{$menu} = [ ('one_branch') ];
			} elsif ($menu eq 'service_templates') {
				$view{$menu} = { ('new' => 'service_template','modify' => 'service_template') };
				my @submenus = StorProc->fetch_list($menu,'name');
				if ($auth_add{$menu}) {
					$tasks{$menu} = [ ('new') ];
				}
				if ($auth_modify{$menu}) {
					push @{$tasks{$menu}}, 'modify';
					$tree{$menu} = [ @submenus ];
				}
			} elsif ($menu eq 'servicegroups') {
				$view{$menu} = { ('new' => 'service_group','copy' => 'service_group','modify' => 'service_group') };
				my @submenus = StorProc->fetch_list($menu,'name');
				if ($auth_add{$menu}) {
					$tasks{$menu} = [ ('new') ];
				}
				if ($auth_modify{$menu}) {
					push @{$tasks{$menu}}, 'modify';
					$tree{$menu} = [ @submenus ];
				}
			} elsif ($menu eq 'service_externals') {
				$view{$menu} = { ('new' => 'service_externals','modify' => 'service_externals') };
				my %where = ('type' => 'service');
				my @submenus = StorProc->fetch_list_where('externals','name',\%where);
				$tasks{$menu} = [ ('new') ];
				push @{$tasks{$menu}}, 'modify';
				$tree{$menu} = [ @submenus ];
			} else {
				$view{$menu} = { ('new' => 'design','copy' => 'design','modify' => 'manage') };
				my @submenus = StorProc->fetch_list($menu,'name');
				if ($auth_add{$menu}) {
					$tasks{$menu} = [ ('new','copy') ];
				}
				if ($auth_modify{$menu}) {
					push @{$tasks{$menu}}, 'modify';
					$tree{$menu} = [ @submenus ];
				}
			}

		}
		$body .= Forms->left_menu($user_acct,$top_menu,\%view,\@menus,\%tasks,\%tree,$selected);
	} elsif ($top_menu eq 'hosts') {	
		@menus = ();
		my @empty = ();
		if ($auth_add{'hosts'}) { 
			push @menus, 'new_host_wizard';
			push @menus, 'clone_host';
		}
		if ($auth_modify{'hosts'}) { 
			push @menus, 'search';
		}

		foreach my $menu (@host_menus) {
			if ($auth_add{$menu} || $auth_modify{$menu}) { 
				if ($menu eq 'hosts') {
					push @menus, 'hosts';
				} else {
					push @menus, $menu;
				}
			}
		}
		if ($auth_add{'externals'}) { 
			push @menus, 'host_externals';
		}
		foreach my $menu (@menus) {
			if ($menu eq 'hosts') {
				$view{'hosts'} = 'hosts';
				@{$tree{'hosts'}} = ();
				@{$tree{'host_nodes'}} = ();
				@{$tree{'services'}} = ();
				$selected = $query->param('selected');
				my $num_hosts = StorProc->get_num_records('hosts');
				unless ($max_tree_nodes && $num_hosts > $max_tree_nodes) {
					my %hosts_services = StorProc->get_hosts_services();
					delete $hosts_services{'host_count'};
					foreach my $host (sort keys %hosts_services) {
						push @{$tree{'hosts'}}, $host;
						if ($host eq $selected) {
							foreach my $service (@{$hosts_services{$host}}) {
								push @{$tree{'services'}}, $service;
							}
						}
					}
				}
			} elsif ($menu eq 'new_host_wizard') { 
				$view{$menu} = 'hosts';
				$tasks{$menu} = [ ('one_branch') ];
			} elsif ($menu eq 'clone_host') { 
				$view{$menu} = 'clone_host';
				$tasks{$menu} = [ ('one_branch') ];
			} elsif ($menu eq 'search') { 
				$view{$menu} = 'search';
				$tasks{$menu} = [ ('one_branch') ];
			} elsif ($menu eq 'parent_child') { 
				$view{'parent_child'} = { ('new' => 'parent_child','modify' => 'parent_child') };
				my @submenus = StorProc->get_parents();
				if ($auth_add{$menu}) {
					$tasks{$menu} = [ ('new') ];
				}
				if ($auth_modify{$menu}) {
					push @{$tasks{$menu}}, 'modify';
					$tree{$menu} = [ @submenus ];
				}
			} elsif ($menu eq 'host_dependencies') { 
				$view{'host_dependencies'} = { ('new' => 'host_dependencies','modify' => 'host_dependencies') };
				my @submenus = StorProc->get_host_dep_parents();
				if ($auth_add{$menu}) {
					$tasks{$menu} = [ ('new') ];
				}
				if ($auth_modify{$menu}) {
					push @{$tasks{$menu}}, 'modify';
					$tree{$menu} = [ @submenus ];
				}
			} elsif ($menu eq 'host_externals') {
				$view{'host_externals'} = { ('new' => 'host_externals','modify' => 'host_externals') };
				my %where = ('type' => 'host');
				my @submenus = StorProc->fetch_list_where('externals','name',\%where);
				$tasks{$menu} = [ ('new') ];
				push @{$tasks{$menu}}, 'modify';
				$tree{$menu} = [ @submenus ];
			} else {
				$view{$menu} = { ('new' => 'design','copy' => 'design','modify' => 'manage') };
				my $table = $menu;
				if ($menu eq 'modify') { $table = 'hosts'}
				my @submenus = StorProc->fetch_list($table,'name');
				if ($auth_add{$menu}) {
					$tasks{$menu} = [ ('new','copy') ];
				}
				if ($auth_modify{$menu}) {
					push @{$tasks{$menu}}, 'modify';
					$tree{$menu} = [ @submenus ];
				}
			}
		}	
		$body .= Forms->left_menu($user_acct,$top_menu,\%view,\@menus,\%tasks,\%tree,$selected);

	} elsif ($top_menu eq 'profiles') {	
		@menus = ('host_profiles','service_profiles','profile_importer');
		$view{'host_profiles'} = 'host_profile';
		$view{'service_profiles'} = 'service_profile';
		$view{'profile_importer'} = 'profile_importer';
		foreach my $menu (@menus) {
			if ($menu eq 'host_profiles') { 
				my @submenus = StorProc->fetch_list('profiles_host','name');
				if ($auth_add{'profiles'}) {
					$tasks{$menu} = [ ('new') ];
				}
				if ($auth_modify{'profiles'}) {
					push @{$tasks{$menu}}, 'modify';
					$tree{$menu} = [ @submenus ];
				}
			} elsif ($menu eq 'service_profiles') {
				my @submenus = StorProc->fetch_list('profiles_service','name');
				if ($auth_add{'profiles'}) {
					$tasks{$menu} = [ ('new') ];
				}
				if ($auth_modify{'profiles'}) {
					push @{$tasks{$menu}}, 'modify';
					$tree{$menu} = [ @submenus ];
				}
			} 
		}
		$body .= Forms->left_menu($user_acct,$top_menu,\%view,\@menus,\%tasks,\%tree,$selected);

	} elsif ($top_menu eq 'control') {	
		my @menus = ();
		if ($is_portal) { @control_menus = ('setup','nagios_cgi_configuration','nagios_main_configuration','nagios_resource_macros',
'load','files','pre_flight_test','commit');}
		foreach my $menu (@control_menus) {
			if ($auth_add{$menu} || $auth_modify{$menu}) { 
				if ($menu eq 'resource') {
					push @menus, 'nagios_resource_macros';
					$view{'nagios_resource_macros'} = ('new' => 'control','modify' => 'control');
				} else {
					push @menus, $menu;
					$view{$menu} = ('new' => 'control','modify' => 'control');
				}
			}
		}
		if ($auth_add{'externals'}) { push @menus, 'run_externals' }
		foreach my $menu (@menus) {
			if ($menu eq 'nagios_cgi_configuration') {
				$tasks{$menu} = [ ('load_nagios_cgi','set_defaults','view_edit') ];
				$view{$menu} = { ('load_nagios_cgi' => 'control','set_defaults' => 'control','view_edit' => 'control') };
			} elsif ($menu eq 'nagios_main_configuration') {
				$tasks{$menu} = [ ('load_nagios_cfg','set_defaults','view_edit') ];
				$view{$menu} = { ('load_nagios_cfg' => 'control','set_defaults' => 'control','view_edit' => 'control') };
			} elsif ($menu eq 'files') {
				$tasks{$menu} = [ ('file_add','file_assn') ];
				$view{$menu} = { ('file_add' => 'control','file_assn' => 'control') };
			} elsif ($menu eq 'users') {
				my @submenus = StorProc->fetch_list($menu,'user_acct');
				$tasks{$menu} = [ ('new','modify') ];
				$tree{$menu} = [ @submenus ];
				$view{$menu} = { ('new' => 'control','modify' => 'control') };

			} elsif ($menu eq 'user_groups') {
				my @submenus = StorProc->fetch_list($menu,'name');
				$tasks{$menu} = [ ('new','modify') ];
				$tree{$menu} = [ @submenus ];
				$view{$menu} = { ('new' => 'control','modify' => 'control') };
			} else {
				$tasks{$menu} = [ ('one_branch') ];
				$view{$menu} = { ('one_branch' => 'control') };
			}
		}
		$body .= Forms->left_menu($user_acct,$top_menu,\%view,\@menus,\%tasks,\%tree,$selected);
	} elsif ($top_menu eq 'tools') {	

		foreach my $menu (@tool_menus) {
			if ($auth_add{$menu} || $auth_modify{$menu}) { 
				push @menus, $menu;
				$view{$menu} = 'tools';
				if ($menu eq 'export') { 
					$view{$menu} = { ('one_branch' => 'tools') };
					$tasks{$menu} = [ ('one_branch') ];
				}
			}
		}
	
		$body .= Forms->left_menu($user_acct,$top_menu,\%view,\@menus,\%tasks,\%tree,$selected);
	}
# End left menu
}


my $auth = StorProc->dbconnect();
if ($auth == 1) { 
	$user_acct = 'super_user';
}

sub get_configs() {
	my %where = ('type' => 'config');
	my %objects = StorProc->fetch_list_hash_array('setup',\%where);
	$is_portal = $objects{'is_portal'}[2];
	$nagios_ver = $objects{'nagios_version'}[2];
	$nagios_bin = $objects{'nagios_bin'}[2];
	$nagios_etc = $objects{'nagios_etc'}[2];
	$monarch_home = $objects{'monarch_home'}[2];
	$monarch_ver = $objects{'monarch_version'}[2];
	$backup_dir = $objects{'backup_dir'}[2];
	$upload_dir = $objects{'upload_dir'}[2];
	$max_tree_nodes = $objects{'max_tree_nodes'}[2];
	$enable_externals = $objects{'enable_externals'}[2];
	if ($is_portal) {
		$doc_root = '/usr/local/groundwork/apache2/htdocs';
	}
}

unless ($is_portal || $view =~ /search/ || $query->param('run_load')) {
	print "Content-type: text/html \n\n";
}
if ($view eq 'logout') {
	my %values = ('session' => '0');
	my $err = StorProc->update_obj('users','user_acct',$user_acct,\%values);
	print login();
} elsif ($user_acct) {
	$userid = StorProc->check_user($user_acct,$password);
	if ($userid =~ /^\d+/) {
		my ($auth_add, $auth_modify, $auth_delete) = StorProc->auth_matrix($userid);
		%auth_add = %{$auth_add};
		%auth_modify = %{$auth_modify};
		%auth_delete = %{$auth_delete};
	} else {
		push @errors, $userid;
		print login();
	}
} else {
	print login();
}

unless ($login) {
	get_configs();
	unless ($enable_externals) {
		delete $auth_add{'externals'};
		delete $auth_modify{'externals'};
		delete $auth_delete{'externals'};
	}
	if ($query->param('update_top')) {
		my @top_menus = ();
		if ($auth_add{'profiles'} || $auth_modify{'profiles'}) { push @top_menus, 'profiles' }
		if ($auth_add{'services'} || $auth_modify{'services'}) { push @top_menus, 'services' }
		if ($auth_add{'hosts'} || $auth_modify{'hosts'}) { push @top_menus, 'hosts' }
		if ($auth_add{'contacts'} || $auth_modify{'contacts'}) { push @top_menus, 'contacts' }
		if ($auth_add{'escalations'} || $auth_modify{'escalations'}) { push @top_menus, 'escalations' }
		if ($auth_add{'time_periods'} || $auth_modify{'time_periods'}) { push @top_menus, 'time_periods' }
		if ($auth_add{'commands'} || $auth_modify{'commands'}) { push @top_menus, 'commands' }
		if ($auth_add{'control'}) { push @top_menus, 'control' }
		push @top_menus, 'tools';
		push @top_menus, 'help';
		unless ($is_portal) {
			print Forms->top_frame($user_acct,$top_menu,\@top_menus,$auth,$monarch_ver);
		}
	} elsif ($query->param('update_left')) {
		print left_page();
	} elsif ($query->param('update_main')) {
		$hidden{'update_main'} = 1;
		if ($debug) {
			foreach my $name ($query->param) { 
				my $value = $query->param($name);
				$debug .= "$name $value<br>" }
		}
		if ($view eq 'host_wizard') {
			$body .= host_wizard();
		} elsif ($view eq 'host_profile') {
			$body .= host_profile();
		} elsif ($view eq 'service_profile') {
			$body .= service_profile();
		} elsif ($view eq 'service_template') {
			$body .= service_template();
		} elsif ($view eq 'service_group') {
			$body .= service_group();
		} elsif ($view eq 'service') {
			$body .= service();
		} elsif ($view eq 'service_externals') {
			$body .= externals('service');
		} elsif ($view eq 'host_externals') {
			$body .= externals('host');
		} elsif ($view eq 'profile_importer') {
			$body .= profile_importer();
		} elsif ($view eq 'manage_host') {
			$body .= manage_host();
		} elsif ($view eq 'search') {
			$body .= search();
		} elsif ($view eq 'search_service') {
			$body .= search_services();
		} elsif ($view eq 'clone_host') {
			$body .= clone_host();
		} elsif ($view eq 'host_dependencies') {
			$body .= host_dependencies();
		} elsif ($view eq 'parent_child') {
			$body .= parent_child();
		} elsif ($view eq 'escalations') {
			$body .= escalations();
		} elsif ($view eq 'escalation_trees') {
			$body .= escalation_trees();
		} elsif ($view eq 'commands') {
			$body .= command_wizard();
		} elsif ($view eq 'tools') {
			$body .= tools();
		} elsif ($view eq 'manage') {
			$body .= manage();
		} elsif ($view eq 'design') {
			$body .= design();
		} elsif ($view eq 'control') {
			$body .= control();
		} else {
			$body .= Forms->header($page_title,$user_acct,$view);
		}
		print $body;
		print Forms->footer($debug);
	} else {
		unless ($login) {
			my $url = undef;
			foreach my $name ($query->param) {
				my $value = $query->param($name);
				$url .= "&$name=$value";
			}
			print Forms->frame($user_acct,$top_menu,$url,'',$is_portal);
		}	
	}
}
my $result = StorProc->dbdisconnect();

